// scripts/logic/playerLoop.ts
import * as mc from "@minecraft/server";
var LISTENERS = /* @__PURE__ */ new Set();
var CALLBACK_EXTRA_ARGS_CACHE = /* @__PURE__ */ new Map();
mc.world.beforeEvents.playerLeave.subscribe(({ player }) => {
  CALLBACK_EXTRA_ARGS_CACHE.delete(player);
});
mc.world.afterEvents.worldLoad.subscribe(() => {
  mc.system.runInterval(loop, 1);
});
function loop() {
  const players = mc.world.getPlayers();
  for (let i = 0; i < players.length; i++) {
    const player = players[i];
    for (const callback of LISTENERS) {
      callback(player, getOrCreateCallbackExtraArgs(player));
    }
  }
}
function getOrCreateCallbackExtraArgs(player) {
  const cache = CALLBACK_EXTRA_ARGS_CACHE.get(player);
  if (cache) return cache;
  const healthComp = player.getComponent("health");
  const inventoryComp = player.getComponent("inventory");
  const equippableComp = player.getComponent("equippable");
  const inventoryContainer = inventoryComp.container;
  const mainhandSlot = equippableComp.getEquipmentSlot(mc.EquipmentSlot.Mainhand);
  const offhandSlot = equippableComp.getEquipmentSlot(mc.EquipmentSlot.Offhand);
  const obj = {
    healthComp,
    inventoryComp,
    inventoryContainer,
    equippableComp,
    mainhandSlot,
    offhandSlot
  };
  CALLBACK_EXTRA_ARGS_CACHE.set(player, obj);
  return obj;
}
function subscribe(callback) {
  LISTENERS.add(callback);
}

// scripts/logic/advancedItem/events.ts
import * as mc3 from "@minecraft/server";

// scripts/logic/advancedItem/AdvancedItem.ts
import "@minecraft/server";
var AdvancedItem = class {
  constructor(baseConstructorArgs) {
    this.baseConstructorArgs = baseConstructorArgs;
  }
  /**
   * Total elapsed ticks since the AdvancedItem instance is initialized.
   *
   * (The amount of ticks the player has held this item in their mainhand)
   */
  get currentTick() {
    return this.baseConstructorArgs.getCurrentTick();
  }
  get isBeingUsed() {
    return this.baseConstructorArgs.isBeingUsed();
  }
  get profile() {
    return this.baseConstructorArgs.profile;
  }
  get player() {
    return this.baseConstructorArgs.player;
  }
  /**
   * {@link mc.EntityHealthComponent} of the player
   */
  get playerHealth() {
    return this.baseConstructorArgs.playerHealth;
  }
  /**
   * {@link mc.EntityEquippableComponent } of the player
   */
  get playerEquippable() {
    return this.baseConstructorArgs.playerEquippable;
  }
  /**
   * {@link mc.ContainerSlot} of mainhand of the player
   */
  get playerMainhand() {
    return this.baseConstructorArgs.playerMainhand;
  }
  /**
   * {@link mc.ContainerSlot} of offhand of the player
   */
  get playerOffhand() {
    return this.baseConstructorArgs.playerOffhand;
  }
  get hotbarSlotIndex() {
    return this.baseConstructorArgs.hotbarSlotIndex;
  }
  isValid(mainhandItem) {
    try {
      if (this.player.selectedSlotIndex !== this.hotbarSlotIndex) return false;
      if (!mainhandItem) mainhandItem = this.playerMainhand.getItem();
      if (!mainhandItem) return false;
      if (this.profile.itemTypeId !== mainhandItem.typeId) return false;
      return true;
    } catch {
      return false;
    }
  }
};

// scripts/logic/advancedItem/profileRegistry.ts
var ENTRIES = /* @__PURE__ */ new Map();
function registerAdvancedItemProfile(profile) {
  const itemTypeId = profile.itemTypeId;
  if (ENTRIES.has(itemTypeId)) {
    throw new Error(`Advabced Item of type "${itemTypeId}" is already registered.`);
  }
  ENTRIES.set(itemTypeId, profile);
}
function getAdvancedItemProfile(itemTypeId) {
  return ENTRIES.get(itemTypeId);
}

// scripts/logic/advancedItem/events.ts
var ADVANCED_ITEM_MAP = /* @__PURE__ */ new Map();
function createAdvancedItemWrapper(player, profile, fieldOverrides, playerComponents) {
  const playerHealth = playerComponents?.health ?? player.getComponent("health");
  if (!playerHealth) throw new Error("Failed to get health component of the player.");
  const playerEquippable = playerComponents?.equippable ?? player.getComponent("equippable");
  if (!playerEquippable) throw new Error("Failed to get equippable component of the player.");
  const playerMainhand = playerComponents?.mainhandSlot ?? playerEquippable.getEquipmentSlot(mc3.EquipmentSlot.Mainhand);
  const playerOffhand = playerComponents?.offhandSlot ?? playerEquippable.getEquipmentSlot(mc3.EquipmentSlot.Offhand);
  const wrapperFields = {
    currentTick: fieldOverrides?.currentTick ?? 0,
    isBeingUsed: fieldOverrides?.isBeingUsed ?? false
  };
  const baseConstructorArgs = {
    getCurrentTick: () => wrapperFields.currentTick,
    isBeingUsed: () => wrapperFields.isBeingUsed,
    profile,
    player,
    playerHealth,
    playerEquippable,
    playerMainhand,
    playerOffhand,
    hotbarSlotIndex: player.selectedSlotIndex
  };
  const advancedItem = profile.createInstance(baseConstructorArgs);
  const advancedItemWrapper = {
    fields: wrapperFields,
    advancedItem
  };
  return advancedItemWrapper;
}
function removeAdvancedItemWrapper(player, advancedItemWrapper) {
  if (!advancedItemWrapper) {
    advancedItemWrapper = ADVANCED_ITEM_MAP.get(player);
    if (!advancedItemWrapper) return false;
  }
  try {
    advancedItemWrapper.advancedItem.onRemove();
    return true;
  } finally {
    ADVANCED_ITEM_MAP.delete(player);
  }
}
subscribe((player) => {
  let advancedItemWrapper = ADVANCED_ITEM_MAP.get(player);
  const playerHealth = advancedItemWrapper?.advancedItem.playerHealth ?? player.getComponent("health");
  if (playerHealth.currentValue <= 0) return;
  const playerEquippable = advancedItemWrapper?.advancedItem.playerEquippable ?? player.getComponent("equippable");
  const playerMainhandItemStack = playerEquippable.getEquipment(mc3.EquipmentSlot.Mainhand);
  if (!playerMainhandItemStack) {
    removeAdvancedItemWrapper(player, advancedItemWrapper);
    return;
  }
  const advancedItemProfile = getAdvancedItemProfile(playerMainhandItemStack.typeId);
  if (!advancedItemProfile) {
    removeAdvancedItemWrapper(player, advancedItemWrapper);
    return;
  }
  if (!advancedItemWrapper || !advancedItemWrapper.advancedItem.isValid(playerMainhandItemStack)) {
    removeAdvancedItemWrapper(player, advancedItemWrapper);
    advancedItemWrapper = createAdvancedItemWrapper(player, advancedItemProfile, void 0, {
      health: playerHealth,
      equippable: playerEquippable
    });
    ADVANCED_ITEM_MAP.set(player, advancedItemWrapper);
  }
  if (advancedItemWrapper.advancedItem.playerHealth.currentValue <= 0) {
    removeAdvancedItemWrapper(player);
    return;
  }
  advancedItemWrapper.advancedItem.onTick(playerMainhandItemStack);
  advancedItemWrapper.fields.currentTick++;
});
mc3.world.afterEvents.itemStartUse.subscribe((event) => {
  const advancedItemProfile = getAdvancedItemProfile(event.itemStack.typeId);
  if (!advancedItemProfile) return;
  let advancedItemWrapper = ADVANCED_ITEM_MAP.get(event.source);
  if (!advancedItemWrapper || !advancedItemWrapper.advancedItem.isValid(event.itemStack)) {
    removeAdvancedItemWrapper(event.source, advancedItemWrapper);
    advancedItemWrapper = createAdvancedItemWrapper(event.source, advancedItemProfile);
    ADVANCED_ITEM_MAP.set(event.source, advancedItemWrapper);
  }
  if (!advancedItemWrapper.advancedItem.isUsable(event)) return;
  advancedItemWrapper.fields.isBeingUsed = true;
  advancedItemWrapper.advancedItem.onStartUse(event);
});
mc3.world.afterEvents.itemStopUse.subscribe((event) => {
  if (!event.itemStack) return;
  const advancedItemWrapper = ADVANCED_ITEM_MAP.get(event.source);
  if (!advancedItemWrapper) return;
  if (!advancedItemWrapper.fields.isBeingUsed) return;
  advancedItemWrapper.fields.isBeingUsed = false;
  advancedItemWrapper.advancedItem.onStopUse(event);
});
mc3.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    const player = event.entity;
    const advancedItemWrapper = ADVANCED_ITEM_MAP.get(player);
    if (!advancedItemWrapper) return;
    advancedItemWrapper.advancedItem.onSwingArm();
  },
  {
    entityTypes: ["minecraft:player"],
    eventTypes: ["player:on_arm_swing"]
  }
);
mc3.world.afterEvents.entityHitEntity.subscribe(
  (event) => {
    const player = event.damagingEntity;
    const advancedItemWrapper = ADVANCED_ITEM_MAP.get(player);
    if (!advancedItemWrapper) return;
    advancedItemWrapper.advancedItem.onHitEntity(event);
  },
  {
    entityTypes: ["minecraft:player"]
  }
);
mc3.world.afterEvents.entityHitBlock.subscribe(
  (event) => {
    const player = event.damagingEntity;
    const advancedItemWrapper = ADVANCED_ITEM_MAP.get(player);
    if (!advancedItemWrapper) return;
    advancedItemWrapper.advancedItem.onHitBlock(event);
  },
  {
    entityTypes: ["minecraft:player"]
  }
);
mc3.world.afterEvents.entityDie.subscribe(
  (event) => {
    removeAdvancedItemWrapper(event.deadEntity);
  },
  {
    entityTypes: ["minecraft:player"]
  }
);
mc3.world.beforeEvents.playerLeave.subscribe((event) => {
  removeAdvancedItemWrapper(event.player);
});

// scripts/logic/ammo/ammo.ts
import "@minecraft/server";
var AMMO_TYPE_OBJ = {
  "556mm": "lc:scpdy_ammo_556mm",
  "9mm": "lc:scpdy_ammo_9mm",
  "50bmg": "lc:scpdy_ammo_50bmg",
  "762x51": "lc:scpdy_ammo_762x51",
  "12shell": "lc:scpdy_ammo_12shell",
  "338magnum": "lc:scpdy_ammo_338magnum"
};
var AMMO_TYPE_INFO_MAP = /* @__PURE__ */ new Map([
  [
    "9mm",
    {
      maxBatchLoad: 3,
      loadCooldown: 0,
      loadSoundId: "scpdy.gun.magazine.load_ammo"
    }
  ],
  [
    "12shell",
    {
      maxBatchLoad: 1,
      loadCooldown: 5,
      loadSoundId: "scpdy.gun.spas12.reload"
    }
  ],
  [
    "50bmg",
    {
      maxBatchLoad: 1,
      loadCooldown: 18,
      loadSoundId: "scpdy.gun.magazine.load_ammo"
    }
  ],
  [
    "338magnum",
    {
      maxBatchLoad: 1,
      loadCooldown: 9,
      loadSoundId: "scpdy.gun.magazine.load_ammo"
    }
  ],
  [
    "556mm",
    {
      maxBatchLoad: 3,
      loadCooldown: 0,
      loadSoundId: "scpdy.gun.magazine.load_ammo"
    }
  ],
  [
    "762x51",
    {
      maxBatchLoad: 3,
      loadCooldown: 5,
      loadSoundId: "scpdy.gun.magazine.load_ammo"
    }
  ]
]);
function getAmmoType(itemStack) {
  const ammoType = itemStack.getTags().find((tag) => tag.startsWith("ammo_type:"))?.split(":")[1];
  if (ammoType === void 0) return void 0;
  if (!(ammoType in AMMO_TYPE_OBJ)) return void 0;
  return ammoType;
}
function getAmmoItemType(ammoType) {
  const value = AMMO_TYPE_OBJ[ammoType];
  if (typeof value !== "string") throw new Error(`Ammo item type of ${ammoType} is invalid.`);
  return value;
}
function getAmmoTypeInfo(ammoType) {
  const value = AMMO_TYPE_INFO_MAP.get(ammoType);
  if (!value) throw new Error(`Ammo type info of ${ammoType} is missing.`);
  return value;
}
function getTotalAmmoCount(container, ammoType) {
  let ammoCount = 0;
  for (let i = 0; i < container.size; i++) {
    const slot = container.getSlot(i);
    const itemStack = slot.getItem();
    if (!itemStack) continue;
    if (getAmmoType(itemStack) !== ammoType) continue;
    if (itemStack.hasTag("scpdy:ammo")) {
      ammoCount += itemStack.amount;
    } else if (itemStack.hasTag("scpdy:ammo_pack")) {
      const durabilityComp = itemStack.getComponent("durability");
      ammoCount += durabilityComp.maxDurability - durabilityComp.damage;
    }
  }
  return ammoCount;
}
function removeAmmo(container, ammoType, maxAmount, excludePacks = false) {
  maxAmount = Math.floor(maxAmount);
  if (maxAmount <= 0) throw new RangeError("maxAmount is less than 0");
  let totalRemoved = 0;
  for (let i = 0; i < container.size; i++) {
    if (totalRemoved >= maxAmount) break;
    const slot = container.getSlot(i);
    const itemStack = slot.getItem();
    if (!itemStack) continue;
    if (getAmmoType(itemStack) !== ammoType) continue;
    if (itemStack.hasTag("scpdy:ammo")) {
      const initialStackAmount = itemStack.amount;
      let decreaseAmount = Math.min(Math.max(maxAmount - totalRemoved, 0), initialStackAmount);
      totalRemoved += decreaseAmount;
      if (itemStack.amount - decreaseAmount > 0) {
        itemStack.amount -= decreaseAmount;
        slot.setItem(itemStack);
      } else {
        slot.setItem();
      }
      continue;
    }
    if (excludePacks) continue;
    if (itemStack.hasTag("scpdy:ammo_pack")) {
      const durabilityComp = itemStack.getComponent("durability");
      const damageIncrease = Math.min(
        Math.max(maxAmount - totalRemoved, 0),
        durabilityComp.maxDurability - durabilityComp.damage
      );
      durabilityComp.damage += damageIncrease;
      totalRemoved += damageIncrease;
      slot.setItem(itemStack);
    }
  }
  return totalRemoved;
}

// scripts/logic/ammo/ammoItem.ts
import * as mc5 from "@minecraft/server";
function onUse(arg) {
  const ammoItemStack = arg.itemStack;
  if (!ammoItemStack) return;
  const ammoType = getAmmoType(ammoItemStack);
  if (!ammoType) throw new Error(`Ammo type of ${ammoItemStack.typeId} is undefined.`);
  const ammoTypeInfo = getAmmoTypeInfo(ammoType);
  const equippable = arg.source.getComponent("equippable");
  if (!equippable) return;
  const loadableItemStack = equippable.getEquipment(mc5.EquipmentSlot.Offhand);
  if (!loadableItemStack || !loadableItemStack.hasTag("scpdy:ammo_loadable")) {
    arg.source.onScreenDisplay.setActionBar({
      translate: "scpdy.actionHint.ammo.noLoadableInOffhand"
    });
    return;
  }
  const loadableAmmoType = getAmmoType(loadableItemStack);
  if (!loadableAmmoType) throw new Error(`Ammo type of ${loadableItemStack.typeId} is undefined.`);
  if (loadableAmmoType !== ammoType) {
    arg.source.onScreenDisplay.setActionBar({ translate: "scpdy.actionHint.ammo.wrongAmmoType" });
    return;
  }
  const loadableDurability = loadableItemStack.getComponent("durability");
  if (!loadableDurability) throw new Error("Durability component of loadable item is missing.");
  if (loadableDurability.damage <= 0) {
    arg.source.onScreenDisplay.setActionBar({ translate: "scpdy.actionHint.ammo.fullyLoaded" });
    return;
  }
  const loadAmount = Math.min(ammoTypeInfo.maxBatchLoad, ammoItemStack.amount, loadableDurability.damage);
  loadableDurability.damage -= loadAmount;
  equippable.setEquipment(mc5.EquipmentSlot.Offhand, loadableItemStack);
  if (ammoItemStack.amount - loadAmount > 1) {
    ammoItemStack.amount -= loadAmount;
    equippable.setEquipment(mc5.EquipmentSlot.Mainhand, ammoItemStack);
  } else {
    equippable.setEquipment(mc5.EquipmentSlot.Mainhand, void 0);
  }
  const fullyLoaded = loadableDurability.damage <= 0;
  if (fullyLoaded) {
    arg.source.onScreenDisplay.setActionBar({ translate: "scpdy.actionHint.ammo.fullyLoaded" });
  } else {
    const currentLoad = loadableDurability.maxDurability - loadableDurability.damage;
    const displayText = `${currentLoad} / ${loadableDurability.maxDurability}`;
    arg.source.onScreenDisplay.setActionBar(displayText);
  }
  arg.source.dimension.playSound(ammoTypeInfo.loadSoundId, arg.source.getHeadLocation());
}
mc5.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:ammo", {
    onUse
  });
});

// scripts/logic/ammo/AmmoPack.ts
import * as mc6 from "@minecraft/server";
var USE_DURATION = 10;
var AmmoPack = class extends AdvancedItem {
  playerInventoryContainer;
  ammoType;
  useTick = -1;
  constructor(args) {
    super(args);
    this.playerInventoryContainer = args.player.getComponent("inventory").container;
    const mainhandItemStack = args.playerMainhand.getItem();
    this.ammoType = getAmmoType(mainhandItemStack);
  }
  onTick(mainhandItemStack) {
    if (this.useTick === -1) return;
    if (this.player.getItemCooldown("scpdy_ammo_pack_usage") > 0) return;
    if (this.useTick <= USE_DURATION) {
      const actionbarStringHyphenArray = [];
      const loopCount = USE_DURATION - this.useTick;
      for (let i = 0; i < loopCount; i++) {
        actionbarStringHyphenArray.push("-");
      }
      const actionbarStringHyphens = actionbarStringHyphenArray.join("");
      const actionbarString = actionbarStringHyphens + "+" + actionbarStringHyphens;
      this.player.onScreenDisplay.setActionBar(`\xA77${actionbarString}`);
      this.useTick++;
      return;
    }
    this.useTick = 0;
    const durabilityComponent = mainhandItemStack.getComponent("durability");
    if (!durabilityComponent) {
      throw new Error(`Durability component of ${mainhandItemStack.typeId} is missing.`);
    }
    this.player.startItemCooldown("scpdy_ammo_pack_usage", 5);
    const mode = this.player.isSneaking ? "extractAmmo" : "putAmmo";
    if (mode === "putAmmo") {
      if (durabilityComponent.damage <= 0) {
        this.player.onScreenDisplay.setActionBar({
          translate: "scpdy.actionHint.ammoPack.full"
        });
        return;
      }
      const maxPutAmount = Math.min(64, durabilityComponent.damage);
      const putAmount = removeAmmo(this.playerInventoryContainer, this.ammoType, maxPutAmount, true);
      if (putAmount <= 0) {
        this.player.onScreenDisplay.setActionBar({
          translate: "scpdy.actionHint.ammoPack.noAmmoInInventory"
        });
        return;
      }
      durabilityComponent.damage -= putAmount;
      const currentLoad = durabilityComponent.maxDurability - durabilityComponent.damage;
      const textColor = durabilityComponent.damage <= 0 ? `\xA7a` : `\xA7f`;
      const displayText = `${textColor}${currentLoad} / ${durabilityComponent.maxDurability}`;
      this.player.onScreenDisplay.setActionBar(displayText);
      this.playerMainhand.setItem(mainhandItemStack);
      this.player.dimension.playSound("scpdy.gun.ammo_pack.load_ammo", this.player.getHeadLocation());
    } else if (mode === "extractAmmo") {
      const extractAmount = Math.min(64, durabilityComponent.maxDurability - durabilityComponent.damage);
      if (extractAmount <= 0) {
        this.player.onScreenDisplay.setActionBar({ translate: "scpdy.actionHint.ammoPack.empty" });
        return;
      }
      durabilityComponent.damage += extractAmount;
      this.playerMainhand.setItem(mainhandItemStack);
      const dropAmmoItemStack = new mc6.ItemStack(getAmmoItemType(this.ammoType), extractAmount);
      mc6.system.runTimeout(() => {
        this.player.dimension.spawnItem(dropAmmoItemStack, this.player.getHeadLocation());
      }, 1);
    }
  }
  onRemove() {
  }
  isUsable() {
    return true;
  }
  onStartUse(event) {
    this.useTick = 0;
  }
  onStopUse(event) {
    this.useTick = -1;
  }
  onSwingArm() {
  }
  onHitEntity(event) {
  }
  onHitBlock(event) {
  }
};
for (const ammoPackItemType of [
  "lc:scpdy_ammo_pack_9mm",
  "lc:scpdy_ammo_pack_12shell",
  "lc:scpdy_ammo_pack_50bmg",
  "lc:scpdy_ammo_pack_338magnum",
  "lc:scpdy_ammo_pack_556mm",
  "lc:scpdy_ammo_pack_762x51"
]) {
  registerAdvancedItemProfile({
    itemTypeId: ammoPackItemType,
    createInstance: (args) => new AmmoPack(args)
  });
}

// scripts/logic/blocks/construction3D/openableVent.ts
import * as mc7 from "@minecraft/server";
function onPlayerInteract(arg) {
  const isOpened = arg.block.permutation.getState("lc:is_opened") === true;
  if (isOpened) {
    arg.dimension.playSound("close.iron_trapdoor", arg.block.center());
  } else {
    arg.dimension.playSound("open.iron_trapdoor", arg.block.center());
  }
  arg.block.setPermutation(arg.block.permutation.withState("lc:is_opened", !isOpened));
}
mc7.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:openable_vent", {
    onPlayerInteract
  });
});

// scripts/lib/utils/blockUtils.ts
import * as mc8 from "@minecraft/server";
var AIR_OR_LIQUID_BLOCK_TYPE = /* @__PURE__ */ new Set([
  "minecraft:air",
  "minecraft:water",
  "minecraft:flowing_water",
  "minecraft:lava",
  "minecraft:flowing_lava"
]);
function isAirOrLiquid(value) {
  if (typeof value === "string") {
    return AIR_OR_LIQUID_BLOCK_TYPE.has(value);
  }
  return AIR_OR_LIQUID_BLOCK_TYPE.has(value.typeId);
}
function getBlockCardinalDirection(permutation) {
  const direction = permutation.getState("minecraft:cardinal_direction");
  switch (direction) {
    default:
    case "north":
      return mc8.Direction.North;
    case "south":
      return mc8.Direction.South;
    case "west":
      return mc8.Direction.West;
    case "east":
      return mc8.Direction.East;
  }
}

// scripts/logic/blocks/door/standardDoor.ts
import * as mc10 from "@minecraft/server";

// scripts/logic/blocks/door/doorSounds.ts
var MAP = /* @__PURE__ */ new Map([
  [
    "lc:scpdy_standard_door_1",
    {
      openSound: { id: "scpdy.door.standard_door.open", volume: 1.1 },
      closeSound: { id: "scpdy.door.standard_door.close", volume: 1.1 }
    }
  ],
  [
    "lc:scpdy_standard_door_2",
    {
      openSound: { id: "scpdy.door.standard_door.open", volume: 1.1 },
      closeSound: { id: "scpdy.door.standard_door.close_metal", volume: 1.1 }
    }
  ]
]);
function getDoorSoundInfo(doorBlockTypeId) {
  return MAP.get(doorBlockTypeId);
}

// scripts/logic/blocks/door/shared.ts
import * as mc9 from "@minecraft/server";
function dropDoorItem(doorTypeId, dimension, location, destroyer) {
  if (!mc9.world.gameRules.doTileDrops) return;
  if (destroyer && destroyer.getGameMode() === mc9.GameMode.Creative) return;
  const itemStack = new mc9.ItemStack(doorTypeId, 1);
  dimension.spawnItem(itemStack, location);
}

// scripts/logic/blocks/door/standardDoor.ts
var STATE_NAMES = {
  isLowerPart: "lc:is_lower_part",
  isOpened: "lc:is_opened"
};
function onPlace(arg) {
  const { block, dimension } = arg;
  const isLowerPart = block.permutation.getState(STATE_NAMES.isLowerPart);
  if (!isLowerPart) {
    const blockBelow = block.below();
    if (blockBelow && blockBelow.typeId !== block.typeId) {
      block.setType("minecraft:air");
    }
    return;
  }
  const blockAbove = block.above();
  if (!blockAbove || !isAirOrLiquid(blockAbove.typeId)) {
    dropDoorItem(block.typeId, dimension, block.center());
    block.setType("minecraft:air");
    return;
  }
  const upperPartPermutation = block.permutation.withState(STATE_NAMES.isLowerPart, false);
  blockAbove.setPermutation(upperPartPermutation);
}
function onPlayerInteract2(arg) {
  const { block, player, dimension } = arg;
  if (!player) return;
  const isLowerPart = block.permutation.getState(STATE_NAMES.isLowerPart) === true;
  let otherPartBlock;
  if (isLowerPart) {
    otherPartBlock = block.above();
  } else {
    otherPartBlock = block.below();
  }
  if (!otherPartBlock || otherPartBlock.typeId !== block.typeId || otherPartBlock.permutation.getState(STATE_NAMES.isLowerPart) === isLowerPart) {
    block.setType("minecraft:air");
    return;
  }
  const isOpened = block.permutation.getState(STATE_NAMES.isOpened) === true;
  block.setPermutation(block.permutation.withState(STATE_NAMES.isOpened, !isOpened));
  otherPartBlock.setPermutation(otherPartBlock.permutation.withState(STATE_NAMES.isOpened, !isOpened));
  if (isOpened) {
    const doorSoundInfo = getDoorSoundInfo(block.typeId);
    if (doorSoundInfo) {
      dimension.playSound(doorSoundInfo.closeSound.id, block.location, {
        pitch: doorSoundInfo.closeSound.pitch,
        volume: doorSoundInfo.closeSound.volume
      });
    }
  } else {
    const doorSoundInfo = getDoorSoundInfo(block.typeId);
    if (doorSoundInfo) {
      dimension.playSound(doorSoundInfo.openSound.id, block.location, {
        pitch: doorSoundInfo.openSound.pitch,
        volume: doorSoundInfo.openSound.volume
      });
    }
  }
}
function onPlayerBreak(arg) {
  const { block, brokenBlockPermutation, dimension, player } = arg;
  dropDoorItem(brokenBlockPermutation.type.id, dimension, block.center(), player);
  const isLowerPart = brokenBlockPermutation.getState(STATE_NAMES.isLowerPart) === true;
  let otherPartBlock;
  if (isLowerPart) {
    otherPartBlock = block.above();
  } else {
    otherPartBlock = block.below();
  }
  if (!otherPartBlock || otherPartBlock.typeId !== brokenBlockPermutation.type.id) return;
  otherPartBlock.setType("minecraft:air");
}
mc10.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:standard_door", {
    onPlace,
    onPlayerInteract: onPlayerInteract2,
    onPlayerBreak
  });
});

// scripts/logic/blocks/furniture/connecTable.ts
import * as mc11 from "@minecraft/server";
var STATE_NAME = {
  connectsN: "lc:connects_n",
  connectsS: "lc:connects_s",
  connectsE: "lc:connects_e",
  connectsW: "lc:connects_w",
  legNW: "lc:leg_nw",
  legNE: "lc:leg_ne",
  legSW: "lc:leg_sw",
  legSE: "lc:leg_se"
};
function updateConnections(block, tableTypeId, iter = 0) {
  function isConnecTable(block2) {
    if (!block2) return false;
    return block2.typeId === tableTypeId;
  }
  const n = block.north();
  const s = block.south();
  const e = block.east();
  const w = block.west();
  const nConnectable = isConnecTable(n);
  const sConnectable = isConnecTable(s);
  const eConnectable = isConnecTable(e);
  const wConnectable = isConnecTable(w);
  const nw = nConnectable || wConnectable ? n?.west() : void 0;
  const ne = nConnectable || eConnectable ? n?.east() : void 0;
  const sw = sConnectable || wConnectable ? s?.west() : void 0;
  const se = sConnectable || eConnectable ? s?.east() : void 0;
  if (isConnecTable(block)) {
    block.setPermutation(
      block.permutation.withState(STATE_NAME.connectsN, nConnectable).withState(STATE_NAME.connectsS, sConnectable).withState(STATE_NAME.connectsE, eConnectable).withState(STATE_NAME.connectsW, wConnectable).withState(
        STATE_NAME.legNW,
        !nConnectable && !wConnectable || nConnectable && wConnectable && !isConnecTable(nw)
      ).withState(
        STATE_NAME.legNE,
        !nConnectable && !eConnectable || nConnectable && eConnectable && !isConnecTable(ne)
      ).withState(
        STATE_NAME.legSW,
        !sConnectable && !wConnectable || sConnectable && wConnectable && !isConnecTable(sw)
      ).withState(
        STATE_NAME.legSE,
        !sConnectable && !eConnectable || sConnectable && eConnectable && !isConnecTable(se)
      )
    );
  }
  if (iter >= 1) return;
  if (nConnectable) updateConnections(n, tableTypeId, iter + 1);
  if (sConnectable) updateConnections(s, tableTypeId, iter + 1);
  if (eConnectable) updateConnections(e, tableTypeId, iter + 1);
  if (wConnectable) updateConnections(w, tableTypeId, iter + 1);
  if (isConnecTable(nw)) updateConnections(nw, tableTypeId, iter + 1);
  if (isConnecTable(ne)) updateConnections(ne, tableTypeId, iter + 1);
  if (isConnecTable(sw)) updateConnections(sw, tableTypeId, iter + 1);
  if (isConnecTable(se)) updateConnections(se, tableTypeId, iter + 1);
}
function onPlace2(arg) {
  updateConnections(arg.block, arg.block.typeId);
}
function onPlayerBreak2(arg) {
  updateConnections(arg.block, arg.brokenBlockPermutation.type.id);
}
mc11.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:connec_table", {
    onPlace: onPlace2,
    onPlayerBreak: onPlayerBreak2
  });
});

// scripts/logic/blocks/furniture/hangingLamp.ts
import * as mc12 from "@minecraft/server";
var HANGING_LAMP_TYPE_ID = "lc:scpdy_hanging_lamp";
function updateHangingLamp(block, isFirst = true) {
  const blockAbove = block.above();
  const blockBelow = block.below();
  if (block.typeId === HANGING_LAMP_TYPE_ID) {
    block.setPermutation(
      block.permutation.withState("lc:show_base", blockAbove?.typeId !== HANGING_LAMP_TYPE_ID).withState("lc:is_extension", blockBelow?.typeId === HANGING_LAMP_TYPE_ID)
    );
  }
  if (!isFirst) return;
  if (blockAbove) updateHangingLamp(blockAbove, false);
  if (blockBelow) updateHangingLamp(blockBelow, false);
}
function onPlace3(arg) {
  updateHangingLamp(arg.block);
}
function onPlayerBreak3(arg) {
  updateHangingLamp(arg.block);
}
mc12.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:hanging_lamp", {
    onPlace: onPlace3,
    onPlayerBreak: onPlayerBreak3
  });
});

// scripts/logic/blocks/furniture/landmine.ts
import * as mc13 from "@minecraft/server";
function onStepOn(arg) {
  const { block, dimension, entity } = arg;
  if (!mc13.world.gameRules.tntExplodes) return;
  const explode2 = block.permutation.getState("lc:explode") === true;
  if (explode2) return;
  try {
    block.setPermutation(block.permutation.withState("lc:explode", true));
  } catch {
  }
  if (entity) {
    entity.applyDamage(28, { cause: mc13.EntityDamageCause.blockExplosion });
  }
  mc13.system.run(() => {
    dimension.spawnEntity("lc:scpdy_landmine_explosion", block.center());
  });
}
mc13.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:landmine", {
    onStepOn
  });
});

// scripts/logic/blocks/furniture/laptop.ts
import * as mc14 from "@minecraft/server";
function beforeOnPlayerPlace(arg) {
  arg.permutationToPlace = arg.permutationToPlace.withState("lc:open", false);
}
function onPlayerInteract3(arg) {
  if (!arg.player) return;
  const interactionIndex = arg.block.permutation.getState("lc:interaction");
  switch (interactionIndex) {
    case 0:
      arg.dimension.playSound("scpdy.interact.laptop.open", arg.block.bottomCenter());
      arg.block.setPermutation(arg.block.permutation.withState("lc:open", true));
      break;
    case 1:
      arg.dimension.playSound("scpdy.interact.button.click", arg.block.bottomCenter(), {
        pitch: 1.2
      });
      arg.block.setPermutation(arg.block.permutation.withState("lc:on", true));
      break;
    case 2:
      arg.dimension.playSound("scpdy.interact.button.click", arg.block.bottomCenter(), {
        pitch: 0.8
      });
      arg.block.setPermutation(arg.block.permutation.withState("lc:on", false));
      break;
    case 3:
      arg.dimension.playSound("scpdy.interact.laptop.close", arg.block.bottomCenter());
      arg.block.setPermutation(arg.block.permutation.withState("lc:open", false));
      break;
  }
  arg.block.setPermutation(
    arg.block.permutation.withState("lc:interaction", interactionIndex < 3 ? interactionIndex + 1 : 0)
  );
}
mc14.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:laptop", {
    beforeOnPlayerPlace,
    onPlayerInteract: onPlayerInteract3
  });
});

// scripts/logic/blocks/furniture/razorWire.ts
import * as mc15 from "@minecraft/server";
function onStepOn2(arg) {
  const { block, dimension, entity } = arg;
  if (!entity) return;
  if (entity instanceof mc15.Player && entity.getGameMode() === mc15.GameMode.Creative) return;
  try {
    const result = entity.applyDamage(2, {
      cause: mc15.EntityDamageCause.contact
    });
    if (result) {
      entity.addEffect("slowness", 40);
      entity.teleport(entity.location);
      dimension.playSound("block.sweet_berry_bush.hurt", block.center());
    }
  } catch {
  }
}
function onStepOff(arg) {
  const { entity } = arg;
  if (!entity) return;
  if (entity instanceof mc15.Player && entity.getGameMode() === mc15.GameMode.Creative) return;
  try {
    entity.applyDamage(1, {
      cause: mc15.EntityDamageCause.override
    });
  } catch {
  }
}
mc15.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:razor_wire", {
    onStepOn: onStepOn2,
    onStepOff
  });
});

// scripts/lib/utils/scpdyUtils.ts
import * as mc16 from "@minecraft/server";

// scripts/lib/utils/vec3.ts
import { Direction as Direction2, Entity } from "@minecraft/server";
var ZERO = {
  x: 0,
  y: 0,
  z: 0
};
var UP = {
  x: 0,
  y: 1,
  z: 0
};
var DOWN = {
  x: 0,
  y: -1,
  z: 0
};
var FORWARD = {
  x: 0,
  y: 0,
  z: 1
};
function fromPartial(value) {
  return {
    x: value.x === void 0 ? ZERO.x : ZERO.x + value.x,
    y: value.y === void 0 ? ZERO.y : ZERO.y + value.y,
    z: value.z === void 0 ? ZERO.z : ZERO.z + value.z
  };
}
function add(vec1, ...vec2) {
  if (vec2.length === 1) {
    return {
      x: vec1.x + vec2[0].x,
      y: vec1.y + vec2[0].y,
      z: vec1.z + vec2[0].z
    };
  }
  const result = { x: vec1.x, y: vec1.y, z: vec1.z };
  for (const vec of vec2) {
    result.x += vec.x;
    result.y += vec.y;
    result.z += vec.z;
  }
  return result;
}
function sub(vec1, ...vec2) {
  if (vec2.length === 1) {
    return {
      x: vec1.x - vec2[0].x,
      y: vec1.y - vec2[0].y,
      z: vec1.z - vec2[0].z
    };
  }
  const result = { x: vec1.x, y: vec1.y, z: vec1.z };
  for (const vec of vec2) {
    result.x -= vec.x;
    result.y -= vec.y;
    result.z -= vec.z;
  }
  return result;
}
function mul(vec1, ...multipliers) {
  if (multipliers.length === 0) return vec1;
  else if (multipliers.length === 1) {
    const multiplier = multipliers[0];
    if (typeof multiplier === "number") {
      return {
        x: vec1.x * multiplier,
        y: vec1.y * multiplier,
        z: vec1.z * multiplier
      };
    }
    return {
      x: vec1.x * multiplier.x,
      y: vec1.y * multiplier.y,
      z: vec1.z * multiplier.z
    };
  }
  const result = { x: vec1.x, y: vec1.y, z: vec1.z };
  for (const multiplier of multipliers) {
    if (typeof multiplier === "number") {
      result.x *= multiplier;
      result.y *= multiplier;
      result.z *= multiplier;
      continue;
    }
    result.x *= multiplier.x;
    result.y *= multiplier.y;
    result.z *= multiplier.z;
  }
  return result;
}
function div(vec1, ...divisors) {
  if (divisors.length === 0) return vec1;
  else if (divisors.length === 1) {
    const divisor = divisors[0];
    if (typeof divisor === "number") {
      return {
        x: vec1.x / divisor,
        y: vec1.y / divisor,
        z: vec1.z / divisor
      };
    }
    return {
      x: vec1.x / divisor.x,
      y: vec1.y / divisor.y,
      z: vec1.z / divisor.z
    };
  }
  const result = { x: vec1.x, y: vec1.y, z: vec1.z };
  for (const divisor of divisors) {
    if (typeof divisor === "number") {
      result.x /= divisor;
      result.y /= divisor;
      result.z /= divisor;
      continue;
    }
    result.x /= divisor.x;
    result.y /= divisor.y;
    result.z /= divisor.z;
  }
  return result;
}
function scale(vec, scaler) {
  return {
    x: vec.x * scaler,
    y: vec.y * scaler,
    z: vec.z * scaler
  };
}
function distance(vec1, vec2) {
  return Math.sqrt((vec1.x - vec2.x) ** 2 + (vec1.y - vec2.y) ** 2 + (vec1.z - vec2.z) ** 2);
}
function normalize(vec) {
  const length2 = Math.sqrt(vec.x ** 2 + vec.y ** 2 + vec.z ** 2);
  if (length2 === 0) return { x: 0, y: 0, z: 0 };
  return {
    x: vec.x / length2,
    y: vec.y / length2,
    z: vec.z / length2
  };
}
function random() {
  return { x: Math.random(), y: Math.random(), z: Math.random() };
}
function rotateRad(vec, axis, radians) {
  let cos = Math.cos(radians);
  let sin = Math.sin(radians);
  let dot2 = axis.x * vec.x + axis.y * vec.y + axis.z * vec.z;
  let crossX = axis.y * vec.z - axis.z * vec.y;
  let crossY = axis.z * vec.x - axis.x * vec.z;
  let crossZ = axis.x * vec.y - axis.y * vec.x;
  let x = vec.x * cos + crossX * sin + axis.x * dot2 * (1 - cos);
  let y = vec.y * cos + crossY * sin + axis.y * dot2 * (1 - cos);
  let z = vec.z * cos + crossZ * sin + axis.z * dot2 * (1 - cos);
  return { x, y, z };
}
function rotateDeg(vec, axis, degrees) {
  return rotateRad(vec, axis, Math.PI / 180 * degrees);
}
function changeDir(vec, dir) {
  const magnitude = Math.sqrt(vec.x * vec.x + vec.y * vec.y + vec.z * vec.z);
  if (magnitude === 0) {
    return vec;
  }
  const dirMagnitude = Math.sqrt(dir.x * dir.x + dir.y * dir.y + dir.z * dir.z);
  vec.x = dir.x / dirMagnitude * magnitude;
  vec.y = dir.y / dirMagnitude * magnitude;
  vec.z = dir.z / dirMagnitude * magnitude;
  return vec;
}
function getRelativeLocation(origin, relative, direction = Direction2.North) {
  switch (direction) {
    case Direction2.South:
      return add(origin, rotateDeg(relative, UP, 180));
    case Direction2.West:
      return add(origin, rotateDeg(relative, UP, 90));
    case Direction2.East:
      return add(origin, rotateDeg(relative, UP, -90));
    case Direction2.North:
    default:
      return add(origin, relative);
  }
}
function getRelativeToHead(headLocation, viewDirection, move) {
  const forward = viewDirection;
  const up = { x: 0, y: 1, z: 0 };
  const right = normalize(cross(forward, up));
  const rightMove = move?.x ?? 0;
  const upMove = move?.y ?? 0;
  const forwardMove = move?.z ?? 0;
  const rightVec = scale(right, rightMove);
  const upVec = scale(up, upMove);
  const forwardVec = scale(forward, forwardMove);
  const moveVec = add(add(rightVec, upVec), forwardVec);
  const newPosition = add(headLocation, moveVec);
  return newPosition;
}
function lerp(vec1, vec2, t) {
  return {
    x: vec1.x + (vec2.x - vec1.x) * t,
    y: vec1.y + (vec2.y - vec1.y) * t,
    z: vec1.z + (vec2.z - vec1.z) * t
  };
}
function reflect(vec, normal) {
  return sub(vec, mul(mul(normal, 2), dot(vec, normal)));
}
function cross(vec1, vec2) {
  return {
    x: vec1.y * vec2.z - vec1.z * vec2.y,
    y: vec1.z * vec2.x - vec1.x * vec2.z,
    z: vec1.x * vec2.y - vec1.y * vec2.x
  };
}
function dot(vec1, vec2) {
  return vec1.x * vec2.x + vec1.y * vec2.y + vec1.z * vec2.z;
}
function length(vec) {
  return Math.sqrt(vec.x ** 2 + vec.y ** 2 + vec.z ** 2);
}
function midpoint(vec1, vec2) {
  return {
    x: (vec1.x + vec2.x) / 2,
    y: (vec1.y + vec2.y) / 2,
    z: (vec1.z + vec2.z) / 2
  };
}
function clamp(vec, min, max) {
  return {
    x: Math.max(min.x, Math.min(max.x, vec.x)),
    y: Math.max(min.y, Math.min(max.y, vec.y)),
    z: Math.max(min.z, Math.min(max.z, vec.z))
  };
}
function floor(vec) {
  return { x: Math.floor(vec.x), y: Math.floor(vec.y), z: Math.floor(vec.z) };
}
function ceil(vec) {
  return { x: Math.ceil(vec.x), y: Math.ceil(vec.y), z: Math.ceil(vec.z) };
}
function round(vec) {
  return { x: Math.round(vec.x), y: Math.round(vec.y), z: Math.round(vec.z) };
}
function toString(vec) {
  return `(${vec.x}, ${vec.y}, ${vec.z})`;
}
function toString2(vec) {
  return `${vec.x} ${vec.y} ${vec.z}`;
}
function chain(vec) {
  return new Chain(vec);
}
var Chain = class {
  _vec;
  constructor(vec) {
    this._vec = vec;
  }
  add(vec2) {
    this._vec = add(this._vec, vec2);
    return this;
  }
  sub(vec2) {
    this._vec = sub(this._vec, vec2);
    return this;
  }
  mul(value) {
    this._vec = mul(this._vec, value);
    return this;
  }
  div(vec2) {
    this._vec = div(this._vec, vec2);
    return this;
  }
  scale(scaler) {
    this._vec = scale(this._vec, scaler);
    return this;
  }
  lerp(vec2, t) {
    this._vec = lerp(this._vec, vec2, t);
    return this;
  }
  rotateRad(axis, radians) {
    this._vec = rotateRad(this._vec, axis, radians);
    return this;
  }
  rotateDeg(axis, degrees) {
    this._vec = rotateDeg(this._vec, axis, degrees);
    return this;
  }
  changeDir(direction) {
    this._vec = changeDir(this._vec, direction);
    return this;
  }
  reflect(normal) {
    this._vec = reflect(this._vec, normal);
    return this;
  }
  midpoint(vec2) {
    this._vec = midpoint(this._vec, vec2);
    return this;
  }
  clamp(min, max) {
    this._vec = clamp(this._vec, min, max);
    return this;
  }
  normalize() {
    this._vec = normalize(this._vec);
    return this;
  }
  floor() {
    this._vec = floor(this._vec);
    return this;
  }
  ceil() {
    this._vec = ceil(this._vec);
    return this;
  }
  round() {
    this._vec = round(this._vec);
    return this;
  }
  done() {
    return this._vec;
  }
};

// scripts/lib/utils/scpdyUtils.ts
function getClearanceLevel_itemStack(itemStack) {
  if (itemStack.hasTag("lc:keycard_o5")) return 6;
  if (itemStack.hasTag("lc:keycard_lvl5")) return 5;
  if (itemStack.hasTag("lc:keycard_lvl4")) return 4;
  if (itemStack.hasTag("lc:keycard_lvl3")) return 3;
  if (itemStack.hasTag("lc:keycard_lvl2")) return 2;
  if (itemStack.hasTag("lc:keycard_lvl1")) return 1;
  if (itemStack.hasTag("lc:keycard_lvl0")) return 0;
  if (itemStack.typeId.startsWith("sra")) {
    switch (itemStack.typeId) {
      case "scp:cf_o5_keycard":
      case "sra:classified_o5_keycard":
        return 6;
      case "scp:cf_omni_keycard":
      case "scp:cf_lvl_5_keycard":
      case "sra:classified_omni_keycard":
      case "sra:classified_lvl_5_keycard":
        return 5;
      case "scp:cf_lvl_4_keycard":
      case "sra:classified_lvl_4_keycard":
        return 4;
      case "scp:cf_lvl_3_keycard":
      case "sra:classified_lvl_3_keycard":
        return 3;
      case "scp:cf_lvl_2_keycard":
      case "sra:classified_lvl_2_keycard":
        return 2;
      case "scp:cf_lvl_1_keycard":
      case "sra:classified_lvl_1_keycard":
        return 1;
      case "scp:cf_lvl_0_keycard":
      case "sra:containment_box_keycard":
        return 0;
      default:
        return -1;
    }
  }
  return -1;
}
function getClearanceLevel_player(player) {
  const equippable = player.getComponent("equippable");
  if (!equippable) return -1;
  const mainhandItem = equippable.getEquipment(mc16.EquipmentSlot.Mainhand);
  if (mainhandItem) {
    const mainhandKeyLvl = getClearanceLevel(mainhandItem);
    if (mainhandKeyLvl >= 0) return mainhandKeyLvl;
  }
  const offhandItem = equippable.getEquipment(mc16.EquipmentSlot.Offhand);
  if (!offhandItem) return -1;
  const offhandKeyLvl = getClearanceLevel(offhandItem);
  return offhandKeyLvl;
}
function getClearanceLevel(value) {
  if (value instanceof mc16.ItemStack) {
    return getClearanceLevel_itemStack(value);
  } else {
    return getClearanceLevel_player(value);
  }
}
function isWrench(itemStack) {
  if (!itemStack) return false;
  if (itemStack.hasTag("lc:wrench")) return true;
  if (itemStack.typeId === "sra:hammer") return true;
  return false;
}
function isHoldingWrench(player) {
  const equippable = player.getComponent("minecraft:equippable");
  if (!equippable) return false;
  let itemStack = equippable.getEquipment(mc16.EquipmentSlot.Mainhand);
  if (!isWrench(itemStack)) {
    itemStack = equippable.getEquipment(mc16.EquipmentSlot.Offhand);
    if (!isWrench(itemStack)) {
      return false;
    }
  }
  return true;
}
function spawnBulletRicochetParticle(dimension, location, direction) {
  dimension.playSound("scpdy.projectile.bullet.ricochet", location);
  const dir = direction ?? ZERO;
  const particleMolangVarMap = new mc16.MolangVariableMap();
  particleMolangVarMap.setVector3("direction", mul(dir, -1));
  dimension.spawnParticle("minecraft:basic_crit_particle", location, particleMolangVarMap);
}

// scripts/logic/blocks/furniture/shelf.ts
import * as mc17 from "@minecraft/server";
function getMaxVariantIndex(shelfBlockPermutation) {
  switch (shelfBlockPermutation.type.id) {
    case "lc:scpdy_shelf_1":
      return 3;
    case "lc:scpdy_shelf_2":
      return 3;
    default:
      return 0;
  }
}
function beforeOnPlayerPlace2(args) {
  args.permutationToPlace = args.permutationToPlace.withState("lc:variant", 0);
}
function onPlayerInteract4(args) {
  if (!args.player) return;
  if (!isHoldingWrench(args.player)) return;
  const currentVariant = args.block.permutation.getState("lc:variant");
  const maxVariant = getMaxVariantIndex(args.block.permutation);
  const newVariant = currentVariant < maxVariant ? currentVariant + 1 : 0;
  args.block.setPermutation(args.block.permutation.withState("lc:variant", newVariant));
}
mc17.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:shelf", {
    beforeOnPlayerPlace: beforeOnPlayerPlace2,
    onPlayerInteract: onPlayerInteract4
  });
});

// scripts/lib/utils/miscUtils.ts
import * as mc18 from "@minecraft/server";
function rotationToDirection(rotation, ignoreX = false, ignoreY = false) {
  if (!ignoreX) {
    if (rotation.x > 50) {
      return mc18.Direction.Down;
    }
    if (rotation.x < -50) {
      return mc18.Direction.Up;
    }
  }
  if (!ignoreY) {
    const y = rotation.y;
    if (y >= -45 && y < 45) {
      return mc18.Direction.North;
    }
    if (y >= 45 && y < 135) {
      return mc18.Direction.East;
    }
    if (y >= 135 && y <= 180 || y >= -180 && y <= -135) {
      return mc18.Direction.South;
    }
    if (y >= -135 && y < -45) {
      return mc18.Direction.West;
    }
  }
  return mc18.Direction.North;
}
function directionToRotation(direction) {
  switch (direction) {
    default:
      return { x: 0, y: 0 };
    case mc18.Direction.Down:
      return { x: 90, y: 0 };
    case mc18.Direction.Up:
      return { x: -90, y: 0 };
    case mc18.Direction.South:
      return { x: 0, y: 180 };
    case mc18.Direction.West:
      return { x: 0, y: -90 };
    case mc18.Direction.East:
      return { x: 0, y: 90 };
  }
}
function reversedDirection(direction) {
  switch (direction) {
    default:
      return mc18.Direction.South;
    case mc18.Direction.Down:
      return mc18.Direction.Up;
    case mc18.Direction.Up:
      return mc18.Direction.Down;
    case mc18.Direction.South:
      return mc18.Direction.North;
    case mc18.Direction.West:
      return mc18.Direction.East;
    case mc18.Direction.East:
      return mc18.Direction.West;
  }
}
function getContainerSlot(container, condition) {
  for (let i = 0; i < container.size; i++) {
    const slot = container.getSlot(i);
    if (condition !== void 0 && !condition(slot)) continue;
    return slot;
  }
}
function stopSoundAt(dimension, location, soundId) {
  const locString = toString2(location);
  dimension.runCommand(
    `execute positioned ${locString} run stopsound @a[r=5]${soundId === void 0 ? "" : ` ${soundId}`}`
  );
}
function ensureType(value, type) {
  switch (type) {
    case "boolean":
      return typeof value === "boolean" ? value : void 0;
    case "number":
      return typeof value === "number" ? value : void 0;
    case "string":
      return typeof value === "string" ? value : void 0;
    case "Vector3": {
      if (typeof value !== "object") return void 0;
      return "x" in value && "y" in value && "z" in value ? value : void 0;
    }
  }
  return void 0;
}

// scripts/logic/blocks/furniture/sink.ts
import * as mc19 from "@minecraft/server";
var STATE_NAME2 = {
  isOn: "lc:is_on"
};
function spawnFaucetWaterParticle(block) {
  const direction = getBlockCardinalDirection(block.permutation);
  const particleLoc = getRelativeLocation(
    block.bottomCenter(),
    {
      x: 0,
      y: 1.2,
      z: 0.2
    },
    direction
  );
  block.dimension.spawnParticle("lc:scpdy_faucet_water_emitter", particleLoc);
}
function onPlayerInteract5(event) {
  const isOn = event.block.permutation.getState(STATE_NAME2.isOn) === true;
  let newPermutation = event.block.permutation;
  newPermutation = newPermutation.withState(STATE_NAME2.isOn, !isOn);
  if (isOn) {
    event.dimension.playSound("scpdy.misc.faucet_water_drip", event.block.center());
    stopSoundAt(event.dimension, event.block.center(), "scpdy.misc.faucet_water");
  } else {
    event.dimension.playSound("scpdy.misc.faucet_water", event.block.center(), {
      volume: 1.05
    });
    spawnFaucetWaterParticle(event.block);
    stopSoundAt(event.dimension, event.block.center(), "scpdy.misc.faucet_water_drip");
  }
  event.block.setPermutation(newPermutation);
}
function onPlayerBreak4(event) {
  stopSoundAt(event.dimension, event.block, "scpdy.misc.faucet_water");
  stopSoundAt(event.dimension, event.block, "scpdy.misc.faucet_water_drip");
}
function onTick(event) {
  const isOn = event.block.permutation.getState(STATE_NAME2.isOn) === true;
  if (!isOn) return;
  event.dimension.playSound("scpdy.misc.faucet_water", event.block.center(), {
    volume: 1.05
  });
  spawnFaucetWaterParticle(event.block);
}
mc19.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:sink", {
    onPlayerInteract: onPlayerInteract5,
    onPlayerBreak: onPlayerBreak4,
    onTick
  });
});

// scripts/logic/blocks/furniture/toilet.ts
import * as mc20 from "@minecraft/server";
var TOILET_RIDEABLE_ENTITY_TYPE = "lc:scpdy_toilet_rideable";
var FART_SOUND_ORDER = [
  "scpdy.misc.fart_1",
  "scpdy.misc.fart_2",
  "scpdy.misc.fart_3",
  "scpdy.misc.fart_4",
  "scpdy.misc.fart_5",
  "scpdy.misc.fart_6",
  "scpdy.misc.fart_7",
  "scpdy.misc.fart_8",
  "scpdy.misc.fart_1"
];
mc20.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    onUpdateToiletRideableEntity(event.entity);
  },
  {
    entityTypes: [TOILET_RIDEABLE_ENTITY_TYPE],
    eventTypes: ["toilet_rideable:update_script"]
  }
);
function onUpdateToiletRideableEntity(entity) {
  if (!entity.isValid) return;
  const rider = entity.getComponent("rideable").getRiders()[0];
  if (!rider) return;
  const lastRiderPitch = entity.getDynamicProperty("lastRiderPitch");
  const currentRiderPitch = rider.getRotation().x;
  const emitFart = typeof lastRiderPitch === "number" && (lastRiderPitch < 80 && currentRiderPitch > 80 || lastRiderPitch > -80 && currentRiderPitch < -80);
  if (emitFart) {
    const fartCount = entity.getDynamicProperty("fartCount") ?? 0;
    const fartSoundId = FART_SOUND_ORDER[fartCount];
    if (typeof fartSoundId === "string") {
      entity.dimension.playSound(fartSoundId, entity.location, {
        volume: 1.1
      });
    }
    if (fartCount === 4) {
      rider.removeEffect("poison");
      rider.removeEffect("fatal_poison");
    } else if (fartCount === 8) {
      rider.addEffect("regeneration", 80, {
        amplifier: 1
      });
    }
    entity.setDynamicProperty("fartCount", fartCount + 1);
  }
  entity.setDynamicProperty("lastRiderPitch", currentRiderPitch);
}
mc20.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    onToiletRideableEntityLostRider(event.entity);
  },
  {
    entityTypes: [TOILET_RIDEABLE_ENTITY_TYPE],
    eventTypes: ["toilet_rideable:on_lost_rider"]
  }
);
function onToiletRideableEntityLostRider(entity) {
  const fartCount = entity.getDynamicProperty("fartCount") ?? 0;
  if (fartCount > 0) {
    if (fartCount < 5) {
      entity.dimension.playSound("scpdy.misc.toilet_flush.normal", entity.location, {
        volume: 1.1
      });
    } else {
      entity.dimension.playSound("scpdy.misc.toilet_flush.hard", entity.location, {
        volume: 1.2
      });
    }
  }
  entity.remove();
}
mc20.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:toilet", {
    onPlayerInteract: onPlayerInteract6
  });
});
function onPlayerInteract6(arg) {
  const { block, dimension, player } = arg;
  if (!player) return;
  const center = block.center();
  if (distance(center, player.location) > 2.5) {
    player.onScreenDisplay.setActionBar({ translate: "scpdy.actionHint.furniture.toilet.tooFar" });
    return;
  }
  const isLidOpen = block.permutation.getState("lc:is_lid_open") === true;
  const sitLoc = { x: center.x, y: center.y - 0.26, z: center.z };
  const rideableEntityExists = dimension.getEntities({
    type: TOILET_RIDEABLE_ENTITY_TYPE,
    closest: 1,
    maxDistance: 0.5,
    location: sitLoc
  }).length > 0;
  if (rideableEntityExists) return;
  if (!player.isSneaking) {
    block.setPermutation(block.permutation.withState("lc:is_lid_open", !isLidOpen));
    dimension.playSound(isLidOpen ? "close.bamboo_wood_trapdoor" : "open.bamboo_wood_trapdoor", center, { pitch: 1.2 });
    if (!isLidOpen) {
      player.onScreenDisplay.setActionBar({
        translate: "scpdy.actionHint.furniture.toilet.sitHint"
      });
    }
    return;
  }
  if (!isLidOpen) return;
  const blockDirection = block.permutation.getState("minecraft:cardinal_direction");
  let rideableEntitySpawnEvent;
  switch (blockDirection) {
    case "north":
      rideableEntitySpawnEvent = `toilet_rideable:r180`;
      break;
    case "east":
      rideableEntitySpawnEvent = `toilet_rideable:r270`;
      break;
    case "west":
      rideableEntitySpawnEvent = `toilet_rideable:r90`;
      break;
    default:
      rideableEntitySpawnEvent = "minecraft:entity_spawned";
      break;
  }
  const rideableEntity = dimension.spawnEntity(TOILET_RIDEABLE_ENTITY_TYPE, sitLoc, {
    spawnEvent: rideableEntitySpawnEvent
  });
  rideableEntity.getComponent("rideable").addRider(player);
  mc20.system.runTimeout(() => {
    player.onScreenDisplay.setActionBar({
      translate: "scpdy.actionHint.furniture.toilet.fartHint"
    });
  }, 40);
}

// scripts/logic/blocks/furniture/toiletPaperHolder.ts
import * as mc21 from "@minecraft/server";
var TOILET_PAPER_ROLL_ITEM_TYPE = "lc:scpdy_toilet_paper_roll";
function onPlayerInteract7({ block, player, dimension }) {
  if (!player) return;
  const holderHasPaperRoll = block.permutation.getState("lc:has_paper_roll") === true;
  if (holderHasPaperRoll) {
    if (!mc21.world.gameRules.doTileDrops) return;
    const dropItemStack = new mc21.ItemStack(TOILET_PAPER_ROLL_ITEM_TYPE, 1);
    dimension.spawnItem(dropItemStack, block.bottomCenter());
    block.setPermutation(block.permutation.withState("lc:has_paper_roll", false));
    dimension.playSound("random.pop", block.center(), { pitch: 0.6 });
    return;
  }
  const equippableComp = player.getComponent("equippable");
  if (!equippableComp) return;
  const playerMainhandItemStack = equippableComp.getEquipment(mc21.EquipmentSlot.Mainhand);
  if (!playerMainhandItemStack) return;
  if (playerMainhandItemStack.typeId !== TOILET_PAPER_ROLL_ITEM_TYPE) return;
  if (playerMainhandItemStack.amount === 1) {
    equippableComp.setEquipment(mc21.EquipmentSlot.Mainhand, void 0);
  } else {
    playerMainhandItemStack.amount--;
    equippableComp.setEquipment(mc21.EquipmentSlot.Mainhand, playerMainhandItemStack);
  }
  block.setPermutation(block.permutation.withState("lc:has_paper_roll", true));
  dimension.playSound("step.chiseled_bookshelf", block.center(), { pitch: 0.9 });
}
mc21.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:toilet_paper_holder", {
    onPlayerInteract: onPlayerInteract7
  });
});

// scripts/logic/blocks/minigameBlocks/dashBlock.ts
import * as mc22 from "@minecraft/server";
function onTick2(arg) {
  const ticksUntilActiveAgain = arg.block.permutation.getState("lc:ticks_until_active_again");
  if (ticksUntilActiveAgain <= 0) return;
  arg.block.setPermutation(arg.block.permutation.withState("lc:ticks_until_active_again", ticksUntilActiveAgain - 1));
}
function onStepOn3(arg) {
  if (!arg.entity) return;
  const ticksUntilActiveAgain = arg.block.permutation.getState("lc:ticks_until_active_again");
  if (ticksUntilActiveAgain > 0) return;
  let dashDir;
  if (arg.entity instanceof mc22.Player) {
    const viewDir = arg.entity.getViewDirection();
    const viewDirNorm = normalize({ ...viewDir, y: 0 });
    const moveDir = arg.entity.inputInfo.getMovementVector();
    const moveDirRotated = changeDir(
      {
        x: moveDir.x,
        y: 0,
        z: moveDir.y
      },
      viewDirNorm
    );
    dashDir = normalize(moveDirRotated);
  } else {
    dashDir = normalize(arg.entity.getVelocity());
  }
  if (length(dashDir) < 5) {
    arg.entity.applyKnockback(
      {
        x: dashDir.x * 7,
        z: dashDir.z * 7
      },
      0.04
    );
  }
  arg.entity.addEffect("speed", 50, {
    amplifier: 1
  });
  arg.block.setPermutation(arg.block.permutation.withState("lc:ticks_until_active_again", 15));
}
mc22.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:dash_block", {
    onTick: onTick2,
    onStepOn: onStepOn3
  });
});

// scripts/logic/blocks/misc/alphanumericSign.ts
import * as mc23 from "@minecraft/server";
import { ModalFormData } from "@minecraft/server-ui";
async function showPlacementOptionsForm(player) {
  const response = await new ModalFormData().title({ translate: "scpdy.form.alphanumericSignOptions.title" }).textField(
    { translate: "scpdy.form.alphanumericSignOptions.textField.label" },
    { translate: "scpdy.form.alphanumericSignOptions.textField.placeholder" },
    { defaultValue: "" }
  ).slider({ translate: "scpdy.form.alphanumericSignOptions.spacing.label" }, 0, 2, {
    defaultValue: 0,
    valueStep: 1
  }).dropdown(
    { translate: "scpdy.form.alphanumericSignOptions.color.label" },
    [
      {
        translate: "scpdy.form.alphanumericSignOptions.color.white"
      },
      {
        translate: "scpdy.form.alphanumericSignOptions.color.black"
      }
    ],
    {
      defaultValueIndex: player.getDynamicProperty("lastAlphanumericSignColorSelection") ?? 0
    }
  ).submitButton({ translate: "scpdy.form.alphanumericSignOptions.submit" }).show(player);
  if (response.canceled) return;
  if (!response.formValues) return;
  const colorIdx = Number(response.formValues[2]);
  player.setDynamicProperty("lastAlphanumericSignColorSelection", colorIdx);
  const color = colorIdx === 0 ? "white" : "black";
  return {
    text: String(response.formValues[0]),
    spacing: Number(response.formValues[1]),
    color
  };
}
async function asyncPlacement(player, initialBlock, dimension, upOrDown, dir) {
  const playerMainhand = player.getComponent("equippable").getEquipmentSlot(mc23.EquipmentSlot.Mainhand);
  const options = await showPlacementOptionsForm(player);
  if (options === void 0) return;
  const error = (msg) => {
    player.sendMessage(msg);
    player.playSound("note.bass");
  };
  if (distance(player.location, initialBlock.location) > 7) {
    error({ translate: "scpdy.msg.alphanumericSignPlacement.error.tooFar" });
    return;
  }
  const text = options.text.trim().toLowerCase();
  if (text === "") {
    error({ translate: "scpdy.msg.alphanumericSignPlacement.error.empty" });
    return;
  }
  if (text.length > 20) {
    error({ translate: "scpdy.msg.alphanumericSignPlacement.error.tooLong" });
    return;
  }
  const placementSequence = [];
  let spacing = 0;
  for (let i = 0; i < text.length; i++) {
    if (spacing > 0) {
      placementSequence.push("space");
      spacing--;
      i--;
      continue;
    }
    spacing += options.spacing;
    const char = text[i];
    const permutation = getCharBlockPermutation(char);
    if (permutation instanceof mc23.BlockPermutation) {
      placementSequence.push(permutation);
    } else if (permutation === "space") {
      placementSequence.push("space");
    } else {
      error({ translate: "scpdy.msg.alphanumericSignPlacement.error.unsupportedChar" });
      return;
    }
  }
  let block = initialBlock;
  for (const sequenceElement of placementSequence) {
    if (!isAirOrLiquid(block)) {
      error({ translate: "scpdy.msg.alphanumericSignPlacement.error.obstructed" });
      return;
    }
    if (sequenceElement instanceof mc23.BlockPermutation) {
      if (player.getGameMode() !== mc23.GameMode.Creative) {
        const itemStack = playerMainhand.getItem();
        if (!itemStack || itemStack.typeId !== "lc:scpdy_alphanumeric_sign_placer") {
          error({ translate: "scpdy.msg.alphanumericSignPlacement.error.notEnoughToComplete" });
          return;
        }
        if (itemStack.amount > 1) {
          itemStack.amount--;
          playerMainhand.setItem(itemStack);
        } else {
          playerMainhand.setItem(void 0);
        }
      }
      const permutation = sequenceElement.withState("lc:color", options.color).withState("lc:dir", dir.toLowerCase()).withState("lc:updown", upOrDown);
      block.setPermutation(permutation);
    }
    let rightBlock;
    switch (dir) {
      case mc23.Direction.North:
        rightBlock = block.west();
        break;
      case mc23.Direction.West:
        rightBlock = block.south();
        break;
      case mc23.Direction.South:
        rightBlock = block.east();
        break;
      case mc23.Direction.East:
        rightBlock = block.north();
        break;
    }
    if (rightBlock === void 0) {
      error({ translate: "scpdy.msg.alphanumericSignPlacement.error.obstructed" });
      return;
    }
    block = rightBlock;
  }
}
function getCharBlockPermutation(char) {
  char = char.toLowerCase();
  if (char === " ") {
    return "space";
  }
  if (/[a-p]/.test(char)) {
    return mc23.BlockPermutation.resolve("lc:scpdy_alphabet_sign_1", {
      "lc:char": char
    });
  } else if (/[q-z]/.test(char)) {
    return mc23.BlockPermutation.resolve("lc:scpdy_alphabet_sign_2", {
      "lc:char": char
    });
  } else if (/[0-9]/.test(char)) {
    return mc23.BlockPermutation.resolve("lc:scpdy_number_sign", {
      "lc:number": +char
    });
  }
  switch (char) {
    case "+":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "plus"
      });
    case "-":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "hyphen"
      });
    case "*":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "asterisk"
      });
    case "=":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "equal"
      });
    case "/":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "slash"
      });
    case "|":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "bar"
      });
    case "\\":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "backslash"
      });
    case "@":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "at"
      });
    case "#":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "hash"
      });
    case "$":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "dollar"
      });
    case "%":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "percent"
      });
    case "^":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "caret"
      });
    case "&":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "ampersand"
      });
    case "~":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "period"
      });
    case ".":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "period"
      });
    case ",":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_1", {
        "lc:symbol": "comma"
      });
    case ":":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "colon"
      });
    case ";":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "semicolon"
      });
    case "\u2014":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "emdash"
      });
    case "_":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "underscore"
      });
    case "(":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "paren_left"
      });
    case ")":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "paren_right"
      });
    case "[":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "bracket_left"
      });
    case "]":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "bracket_right"
      });
    case "{":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "brace_left"
      });
    case "}":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "brace_right"
      });
    case "<":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "lessthan"
      });
    case ">":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "greaterthan"
      });
    case "`":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "backtick"
      });
    case "'":
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "quote"
      });
    case '"':
      return mc23.BlockPermutation.resolve("lc:scpdy_symbol_sign_2", {
        "lc:symbol": "dbquotes"
      });
  }
  return void 0;
}
function beforeOnPlayerPlace3(arg) {
  arg.cancel = true;
  mc23.system.run(() => {
    if (!arg.player) return;
    const upOrDown = arg.face === mc23.Direction.Up ? "up" : arg.face === mc23.Direction.Down ? "down" : "none";
    const dir = function() {
      const y = arg.player.getRotation().y;
      if (upOrDown !== "none") {
        if (y >= -45 && y < 45) {
          return mc23.Direction.North;
        } else if (y >= 45 && y < 135) {
          return mc23.Direction.East;
        } else if (y >= 135 || y < -135) {
          return mc23.Direction.South;
        }
        return mc23.Direction.West;
      }
      return arg.face;
    }();
    asyncPlacement(arg.player, arg.block, arg.dimension, upOrDown, dir);
  });
}
mc23.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:alphanumeric_sign", {
    beforeOnPlayerPlace: beforeOnPlayerPlace3
  });
});

// scripts/logic/cctv/cctvLinker.ts
import * as mc28 from "@minecraft/server";

// scripts/logic/cctv/cctvMonitor.ts
import * as mc27 from "@minecraft/server";
import { ActionFormData as ActionFormData2 } from "@minecraft/server-ui";

// scripts/logic/cctv/cctvServer.ts
import * as mc24 from "@minecraft/server";
import { ActionFormData, ModalFormData as ModalFormData2 } from "@minecraft/server-ui";

// scripts/logic/cctv/shared.ts
var LINKER_ITEM_TYPE = "lc:scpdy_cctv_linker";
var TABLET_ITEM_TYPE = "lc:scpdy_cctv_tablet";
var CAMERA_ENTITY_TYPE = "lc:scpdy_cctv_camera";
var MONITOR_ENTITY_TYPE = "lc:scpdy_cctv_monitor";
var SERVER_ENTITY_TYPE = "lc:scpdy_cctv_server";

// scripts/logic/cctv/cctvServer.ts
function getPassword(cctvSerer) {
  const value = cctvSerer.getDynamicProperty("password");
  return value?.toString();
}
function setPassword(cctvServer, password) {
  cctvServer.setDynamicProperty("password", password);
}
async function showPasswordForm(player, cctvServer) {
  const password = getPassword(cctvServer);
  const lastEnteredCctvPassword = ensureType(player.getDynamicProperty("lastEnteredCctvPassword"), "string");
  const response = await new ModalFormData2().title({
    translate: "scpdy.form.cctvServer.locked.title"
  }).textField(
    {
      translate: "scpdy.form.cctvServer.locked.passwordField.label"
    },
    {
      translate: "scpdy.form.cctvServer.locked.passwordField.placeholder"
    },
    { defaultValue: lastEnteredCctvPassword }
  ).submitButton({
    translate: "scpdy.form.cctvServer.locked.submitButton"
  }).show(player);
  if (response.canceled) return false;
  if (response.formValues === void 0) return false;
  const enteredPassword = response.formValues[0]?.toString();
  player.setDynamicProperty("lastEnteredCctvPassword", enteredPassword);
  if (enteredPassword === void 0 || password !== enteredPassword) {
    player.sendMessage({
      translate: "scpdy.msg.misc.wrongPassword"
    });
    return false;
  }
  return true;
}
function getCameraRefs(cctvServer) {
  const cameraIds = cctvServer.getDynamicPropertyIds().filter((propId) => propId.startsWith("cameraId_")).sort().map((propId) => cctvServer.getDynamicProperty(propId));
  const arr = [];
  for (let i = 0; i < cameraIds.length; i++) {
    const entityId = cameraIds[i];
    arr.push({
      entityId
    });
  }
  return arr;
}
function setCameraRefs(cctvServer, values) {
  const propIds = cctvServer.getDynamicPropertyIds();
  const cameraIdPropIds = propIds.filter((propId) => propId.startsWith("cameraId_"));
  for (let i = 0; i < cameraIdPropIds.length; i++) {
    const propId = cameraIdPropIds[i];
    cctvServer.setDynamicProperty(propId);
  }
  for (let i = 0; i < values.length; i++) {
    const newValue = values[i];
    cctvServer.setDynamicProperty(`cameraId_${i}`, newValue.entityId);
  }
}
function sortCameraList(values, sortMethod, cctvServer) {
  switch (sortMethod) {
    case "a-z":
      values.sort((a, b) => {
        const cameraEntityA = mc24.world.getEntity(a.entityId);
        const cameraEntityB = mc24.world.getEntity(b.entityId);
        if (!cameraEntityA || !cameraEntityB) return 0;
        return cameraEntityA.nameTag.localeCompare(cameraEntityB.nameTag);
      });
      break;
    case "z-a":
      values.sort((a, b) => {
        const cameraEntityA = mc24.world.getEntity(a.entityId);
        const cameraEntityB = mc24.world.getEntity(b.entityId);
        if (!cameraEntityA || !cameraEntityB) return 0;
        return cameraEntityB.nameTag.localeCompare(cameraEntityA.nameTag);
      });
      break;
    case "nearest":
      values.sort((a, b) => {
        if (!cctvServer) return 0;
        const cameraEntityA = mc24.world.getEntity(a.entityId);
        const cameraEntityB = mc24.world.getEntity(b.entityId);
        if (!cameraEntityA || !cameraEntityB) return 0;
        if (cameraEntityA.dimension.id !== cameraEntityB.dimension.id) return 0;
        const distA = distance(cctvServer.location, cameraEntityA.location);
        const distB = distance(cctvServer.location, cameraEntityB.location);
        return distA - distB;
      });
      break;
    case "farthest":
      values.sort((a, b) => {
        if (!cctvServer) return 0;
        const cameraEntityA = mc24.world.getEntity(a.entityId);
        const cameraEntityB = mc24.world.getEntity(b.entityId);
        if (!cameraEntityA || !cameraEntityB) return 0;
        if (cameraEntityA.dimension.id !== cameraEntityB.dimension.id) return 0;
        const distA = distance(cctvServer.location, cameraEntityA.location);
        const distB = distance(cctvServer.location, cameraEntityB.location);
        return distB - distA;
      });
      break;
  }
}
async function showMainMenu(player, cctvServer) {
  const password = getPassword(cctvServer);
  async function actionCameraList() {
    const cameraRefs = getCameraRefs(cctvServer);
    const cameraListForm = new ActionFormData().title({
      translate: "scpdy.form.cctvServer.cameraList.title"
    }).body({
      translate: "scpdy.form.cctvServer.cameraList.body",
      with: [cameraRefs.length.toString()]
    });
    const buttons = [];
    if (cameraRefs.length <= 0) {
      buttons.push({
        label: { translate: "scpdy.form.cctvServer.cameraList.button.noElement" },
        async callback() {
        }
      });
    } else {
      for (let cameraRefIndex = 0; cameraRefIndex < cameraRefs.length; cameraRefIndex++) {
        const cameraRef = cameraRefs[cameraRefIndex];
        const cameraEntity = mc24.world.getEntity(cameraRef.entityId);
        async function actionEditCamera() {
          const response2 = await new ActionFormData().title({
            translate: "scpdy.form.cctvServer.editCamera.title",
            with: [cameraRefIndex.toString()]
          }).body({ translate: "scpdy.form.cctvServer.editCamera.body" }).button({ translate: "scpdy.form.misc.no" }).button({ translate: "scpdy.form.misc.yes" }).show(player);
          if (response2.selection === 1) {
            cameraRefs.splice(cameraRefIndex, 1);
            setCameraRefs(cctvServer, cameraRefs);
          }
        }
        if (!cameraEntity || cameraEntity.dimension.id !== cctvServer.dimension.id) {
          buttons.push({
            label: {
              translate: "scpdy.form.cctvServer.cameraList.button.cameraElementError",
              with: [cameraRefIndex.toString()]
            },
            callback: actionEditCamera
          });
        } else {
          buttons.push({
            label: {
              translate: "scpdy.form.cctvServer.cameraList.button.cameraElement",
              with: [
                cameraRefIndex.toString(),
                cameraEntity.nameTag.trim() ? cameraEntity.nameTag : `${toString(round(cameraEntity.location))}`
              ]
            },
            callback: actionEditCamera
          });
        }
      }
      buttons.push({
        label: {
          translate: "scpdy.form.cctvServer.cameraList.button.sortAZ"
        },
        async callback() {
          sortCameraList(cameraRefs, "a-z");
          setCameraRefs(cctvServer, cameraRefs);
        }
      });
      buttons.push({
        label: {
          translate: "scpdy.form.cctvServer.cameraList.button.sortZA"
        },
        async callback() {
          sortCameraList(cameraRefs, "z-a");
          setCameraRefs(cctvServer, cameraRefs);
        }
      });
      buttons.push({
        label: {
          translate: "scpdy.form.cctvServer.cameraList.button.sortByNearest"
        },
        async callback() {
          sortCameraList(cameraRefs, "nearest", cctvServer);
          setCameraRefs(cctvServer, cameraRefs);
        }
      });
      buttons.push({
        label: {
          translate: "scpdy.form.cctvServer.cameraList.button.sortByFarthest"
        },
        async callback() {
          sortCameraList(cameraRefs, "farthest", cctvServer);
          setCameraRefs(cctvServer, cameraRefs);
        }
      });
      buttons.push({
        label: {
          translate: "scpdy.form.cctvServer.cameraList.button.removeAllCameras"
        },
        async callback() {
          setCameraRefs(cctvServer, []);
        }
      });
    }
    for (const button of buttons) {
      cameraListForm.button(button.label);
    }
    const response = await cameraListForm.show(player);
    if (response.canceled || response.selection === void 0) {
      mc24.system.run(() => {
        showMainMenu(player, cctvServer);
      });
      return;
    }
    await buttons[response.selection].callback();
    mc24.system.run(() => {
      actionCameraList();
    });
  }
  async function actionSetPassword() {
    const response = await new ModalFormData2().title({
      translate: password === void 0 || password.trim() === "" ? "scpdy.form.cctvServer.setPassword.title1" : "scpdy.form.cctvServer.setPassword.title2"
    }).textField(
      {
        translate: "scpdy.form.cctvServer.setPassword.passwordField.label"
      },
      {
        translate: "scpdy.form.cctvServer.setPassword.passwordField.placeholder"
      }
    ).submitButton({
      translate: "scpdy.form.cctvServer.setPassword.submitButton"
    }).show(player);
    if (response.canceled || response.formValues === void 0) {
      mc24.system.run(() => {
        showMainMenu(player, cctvServer);
      });
      return;
    }
    player.setDynamicProperty("lastEnteredCctvPassword");
    const newPassword = response.formValues[0]?.toString();
    if (newPassword === void 0 || newPassword.trim() === "") {
      player.sendMessage({
        translate: "scpdy.msg.misc.removedPassword"
      });
    } else {
      player.sendMessage({
        translate: "scpdy.msg.misc.setNewPassword"
      });
    }
    setPassword(cctvServer, newPassword?.trim());
  }
  const menuResponse = await new ActionFormData().title({
    translate: "scpdy.form.cctvServer.main.title"
  }).body({
    translate: "scpdy.form.cctvServer.main.body"
  }).button({
    translate: "scpdy.form.cctvServer.main.button.cameraList"
  }).button({
    translate: password === void 0 || password.trim() === "" ? "scpdy.form.cctvServer.main.button.setPassword" : "scpdy.form.cctvServer.main.button.changePassword"
  }).show(player);
  if (menuResponse.canceled) return;
  if (menuResponse.selection === void 0) return;
  switch (menuResponse.selection) {
    case 0:
      await actionCameraList();
      break;
    case 1:
      await actionSetPassword();
      break;
  }
}
async function onInteract(player, cctvServer) {
  const password = getPassword(cctvServer);
  if (password === void 0 || password.trim() === "") {
    showMainMenu(player, cctvServer);
    return;
  }
  const playerEnteredCorrectPassword = await showPasswordForm(player, cctvServer);
  if (!playerEnteredCorrectPassword) return;
  try {
    await showMainMenu(player, cctvServer);
  } catch (error) {
    player.sendMessage({
      translate: "scpdy.msg.misc.error",
      with: [`${error}`]
    });
  }
}
mc24.world.afterEvents.playerInteractWithEntity.subscribe((event) => {
  if (event.target.typeId !== SERVER_ENTITY_TYPE) return;
  if (event.itemStack?.typeId === LINKER_ITEM_TYPE) return;
  if (event.itemStack?.typeId === TABLET_ITEM_TYPE) return;
  if (event.player.isSneaking) return;
  onInteract(event.player, event.target);
});

// scripts/logic/cctv/tick.ts
import * as mc26 from "@minecraft/server";

// scripts/lib/utils/mathUtils.ts
function clamp2(value, min, max) {
  return Math.max(min, Math.min(max, value));
}
function randomInt(min, max) {
  return Math.floor(Math.random() * (max - min + 1)) + min;
}
function randomFloat(min, max) {
  return Math.random() * (max - min) + min;
}

// scripts/logic/cctv/cctvCamera.ts
import * as mc25 from "@minecraft/server";
function getHeadRotation(cctvCamera) {
  const x = cctvCamera.getProperty("lc:head_rotation_x");
  const y = cctvCamera.getProperty("lc:head_rotation_y");
  return { x, y };
}
function setHeadRotation(cctvCamera, value) {
  const x = clamp2(value.x, -180, 180);
  const y = clamp2(value.y, -180, 180);
  cctvCamera.setProperty("lc:head_rotation_x", x);
  cctvCamera.setProperty("lc:head_rotation_y", y);
}
mc25.world.afterEvents.entityLoad.subscribe((event) => {
  if (event.entity.typeId !== CAMERA_ENTITY_TYPE) return;
  event.entity.setProperty("lc:is_active", false);
});

// scripts/logic/cctv/tick.ts
var cctvUsageMap = /* @__PURE__ */ new Map();
function setCctvUsage(options) {
  const oldUsageData = cctvUsageMap.get(options.player);
  if (oldUsageData) {
    removeCctvUsageInternal(options.player, oldUsageData);
  }
  const newUsageData = {
    options,
    initialPlayerRotation: options.player.getRotation(),
    elapsedTicks: 0
  };
  cctvUsageMap.set(options.player, newUsageData);
}
function removeCctvUsageInternal(player, usageData) {
  cctvUsageMap.delete(player);
  usageData.options.cctvCamera.setProperty("lc:is_active", false);
  player.camera.clear();
  player.teleport(player.location, {
    rotation: usageData.initialPlayerRotation
  });
  usageData.options.onStop ? usageData.options.onStop(usageData) : 0;
}
function removeCctvUsage(player) {
  const usageData = cctvUsageMap.get(player);
  if (!usageData) return false;
  removeCctvUsageInternal(player, usageData);
  return true;
}
function onTickPlayer(player) {
  const usageData = cctvUsageMap.get(player);
  if (!usageData) return;
  const stop = usageData.options.stopCondition ? usageData.options.stopCondition(usageData) : false;
  if (stop) {
    removeCctvUsageInternal(player, usageData);
    return;
  }
  const { cctvCamera } = usageData.options;
  const cctvCameraRot = cctvCamera.getRotation();
  const cctvCameraHeadRot = getHeadRotation(cctvCamera);
  if (usageData.elapsedTicks === 0) {
    cctvCamera.setProperty("lc:is_active", true);
    const newRotation = {
      x: cctvCameraHeadRot.x,
      y: cctvCameraRot.y < 0 ? cctvCameraRot.y + cctvCameraHeadRot.y : cctvCameraRot.y > 0 ? cctvCameraRot.y - cctvCameraHeadRot.y : 0
    };
    player.teleport(player.location, { rotation: newRotation });
    player.camera.setCamera("minecraft:free", {
      rotation: player.getRotation(),
      easeOptions: {
        easeTime: 0,
        easeType: mc26.EasingType.Linear
      },
      location: {
        x: cctvCamera.location.x,
        y: cctvCamera.location.y + 0.2,
        z: cctvCamera.location.z
      }
    });
  } else {
    const playerRot = player.getRotation();
    setHeadRotation(cctvCamera, {
      x: playerRot.x,
      y: (playerRot.y - cctvCameraRot.y + 180) % 360 - 180
    });
    player.camera.setCamera("minecraft:free", {
      rotation: playerRot,
      easeOptions: {
        easeTime: 0.14,
        easeType: mc26.EasingType.InOutSine
      },
      location: {
        x: cctvCamera.location.x,
        y: cctvCamera.location.y + 0.2,
        z: cctvCamera.location.z
      }
    });
  }
  usageData.elapsedTicks++;
}
subscribe(onTickPlayer);
mc26.world.afterEvents.entityDie.subscribe(
  (event) => {
    removeCctvUsage(event.deadEntity);
  },
  {
    entityTypes: ["minecraft:player"]
  }
);
mc26.world.afterEvents.playerDimensionChange.subscribe((event) => {
  removeCctvUsage(event.player);
});
mc26.world.beforeEvents.playerLeave.subscribe((event) => {
  removeCctvUsage(event.player);
});

// scripts/logic/cctv/cctvMonitor.ts
function getCctvServerId(cctvMonitor) {
  return ensureType(cctvMonitor.getDynamicProperty("serverId"), "string");
}
function setCctvServerId(cctvMonitor, cctvServerId) {
  cctvMonitor.setDynamicProperty("serverId", cctvServerId);
}
function onSelectCameraToUse(player, cctvServer, cctvCamera, cctvMonitor) {
  function stopCondition() {
    try {
      if (player.getComponent("health").currentValue <= 0) return true;
      if (cctvServer.getComponent("health").currentValue <= 0) return true;
      if (cctvCamera.getComponent("health").currentValue <= 0) return true;
      if (cctvMonitor.getComponent("health").currentValue <= 0) return true;
      if (distance(player.location, cctvMonitor.location) > 3) return true;
      if (player.isSneaking) return true;
      return false;
    } catch {
      return true;
    }
  }
  setCctvUsage({
    player,
    cctvServer,
    cctvCamera,
    stopCondition
  });
}
async function showCameraList(player, cctvServer, cctvMonitor) {
  const cameraRefs = getCameraRefs(cctvServer);
  const cameraListForm = new ActionFormData2().title({
    translate: "scpdy.form.cctvMonitor.chooseCamera.title"
  }).body({
    translate: "scpdy.form.cctvMonitor.chooseCamera.body",
    with: [cameraRefs.length.toString()]
  });
  const buttons = [];
  for (let cameraRefIndex = 0; cameraRefIndex < cameraRefs.length; cameraRefIndex++) {
    const cameraRef = cameraRefs[cameraRefIndex];
    const cameraEntity = mc27.world.getEntity(cameraRef.entityId);
    if (!cameraEntity || cameraEntity.dimension.id !== cctvServer.dimension.id) {
      buttons.push({
        label: {
          translate: "scpdy.form.cctvMonitor.chooseCamera.button.cameraElementError",
          with: [cameraRefIndex.toString()]
        },
        async callback() {
          player.sendMessage({
            translate: "scpdy.msg.cctvMonitor.cameraUnavailable"
          });
        }
      });
    } else {
      buttons.push({
        label: {
          translate: "scpdy.form.cctvMonitor.chooseCamera.button.cameraElement",
          with: [
            cameraRefIndex.toString(),
            cameraEntity.nameTag.trim() ? cameraEntity.nameTag : `${toString(round(cameraEntity.location))}`
          ]
        },
        async callback() {
          onSelectCameraToUse(player, cctvServer, cameraEntity, cctvMonitor);
        }
      });
    }
  }
  buttons.push({
    label: {
      translate: "scpdy.form.cctvMonitor.failedToGetServer.removeLinkButton"
    },
    async callback() {
      setCctvServerId(cctvMonitor, void 0);
      removeCctvUsage(player);
      player.sendMessage({ translate: "scpdy.msg.cctvMonitor.removedServerLink" });
    }
  });
  for (const button of buttons) {
    cameraListForm.button(button.label);
  }
  const response = await cameraListForm.show(player);
  if (response.canceled || response.selection === void 0) {
    return;
  }
  await buttons[response.selection].callback();
}
function onInteract2(player, cctvMonitor) {
  if (distance(player.location, cctvMonitor.location) > 3) {
    player.sendMessage({
      translate: "scpdy.msg.cctvMonitor.tooFarFromMonitor"
    });
    return;
  }
  const cctvServerId = getCctvServerId(cctvMonitor);
  if (cctvServerId === void 0) {
    new ActionFormData2().title({ translate: "scpdy.form.cctvMonitor.notLinkedToServer.title" }).body({ translate: "scpdy.form.cctvMonitor.notLinkedToServer.body" }).button({ translate: "scpdy.form.misc.close" }).show(player);
    return;
  }
  const cctvServer = mc27.world.getEntity(cctvServerId);
  if (!cctvServer || cctvServer.typeId !== SERVER_ENTITY_TYPE) {
    new ActionFormData2().title({ translate: "scpdy.form.cctvMonitor.failedToGetServer.title" }).body({ translate: "scpdy.form.cctvMonitor.failedToGetServer.body" }).button({ translate: "scpdy.form.cctvMonitor.failedToGetServer.removeLinkButton" }).show(player).then((response) => {
      if (response.canceled) return;
      if (response.selection === 0) {
        setCctvServerId(cctvMonitor);
        player.sendMessage({ translate: "scpdy.msg.cctvMonitor.removedServerLink" });
      }
    });
    return;
  }
  showCameraList(player, cctvServer, cctvMonitor);
}
mc27.world.afterEvents.playerInteractWithEntity.subscribe((event) => {
  if (event.target.typeId !== MONITOR_ENTITY_TYPE) return;
  if (event.itemStack?.typeId === LINKER_ITEM_TYPE) return;
  if (event.itemStack?.typeId === TABLET_ITEM_TYPE) return;
  if (event.player.isSneaking) return;
  onInteract2(event.player, event.target);
});

// scripts/logic/cctv/cctvLinker.ts
function getCameraRefs2(cctvLinker) {
  const cameraIds = cctvLinker.getDynamicPropertyIds().filter((propId) => propId.startsWith("cameraId_")).sort().map((propId) => ensureType(cctvLinker.getDynamicProperty(propId), "string"));
  const arr = [];
  for (let i = 0; i < cameraIds.length; i++) {
    const entityId = cameraIds[i];
    arr.push({
      entityId
    });
  }
  return arr;
}
function setCameraRefs2(cctvLinker, values) {
  const propIds = cctvLinker.getDynamicPropertyIds();
  const cameraIdPropIds = propIds.filter((propId) => propId.startsWith("cameraId_"));
  const newItemStack = cctvLinker.clone();
  for (let i = 0; i < cameraIdPropIds.length; i++) {
    const propId = cameraIdPropIds[i];
    newItemStack.setDynamicProperty(propId);
  }
  for (let i = 0; i < values.length; i++) {
    const newValue = values[i];
    newItemStack.setDynamicProperty(`cameraId_${i}`, newValue.entityId);
  }
  return newItemStack;
}
function getMonitorRefs(cctvLinker) {
  const monitorIds = cctvLinker.getDynamicPropertyIds().filter((propId) => propId.startsWith("monitorId_")).sort().map((propId) => ensureType(cctvLinker.getDynamicProperty(propId), "string"));
  const arr = [];
  for (let i = 0; i < monitorIds.length; i++) {
    const entityId = monitorIds[i];
    arr.push({
      entityId
    });
  }
  return arr;
}
function setMonitorRefs(cctvLinker, values) {
  const propIds = cctvLinker.getDynamicPropertyIds();
  const monitorIdPropIds = propIds.filter((propId) => propId.startsWith("monitorId_"));
  const newItemStack = cctvLinker.clone();
  for (let i = 0; i < monitorIdPropIds.length; i++) {
    const propId = monitorIdPropIds[i];
    newItemStack.setDynamicProperty(propId);
  }
  for (let i = 0; i < values.length; i++) {
    const newValue = values[i];
    newItemStack.setDynamicProperty(`monitorId_${i}`, newValue.entityId);
  }
  return newItemStack;
}
function onUseCctvLinker(player, itemStack) {
  const cameraRefs = getCameraRefs2(itemStack);
  const monitorRefs = getMonitorRefs(itemStack);
  if (cameraRefs.length <= 0 && monitorRefs.length <= 0) {
    player.sendMessage({
      translate: "scpdy.msg.cctvLinker.guide"
    });
    return;
  }
  player.sendMessage({
    translate: "scpdy.msg.cctvLinker.status.cameras",
    with: [cameraRefs.length.toString()]
  });
  player.sendMessage({
    translate: "scpdy.msg.cctvLinker.status.monitors",
    with: [monitorRefs.length.toString()]
  });
}
function onInteractCctvCamera(player, cctvCamera, itemStack) {
  const mainhandSlot = player.getComponent("equippable").getEquipmentSlot(mc28.EquipmentSlot.Mainhand);
  if (mainhandSlot.typeId !== LINKER_ITEM_TYPE) return;
  const cameraRefs = getCameraRefs2(itemStack);
  const isAlreadyCollected = cameraRefs.findIndex((ref) => ref.entityId === cctvCamera.id) !== -1;
  if (isAlreadyCollected) {
    player.sendMessage({
      translate: "scpdy.msg.cctvLinker.alreadyCollectedCamera",
      with: [cctvCamera.id.toString()]
    });
    return;
  }
  cameraRefs.push({
    entityId: cctvCamera.id
  });
  const newItemStack = setCameraRefs2(itemStack, cameraRefs);
  mainhandSlot.setItem(newItemStack);
  player.sendMessage({
    translate: "scpdy.msg.cctvLinker.collectedCamera",
    with: [cctvCamera.id.toString()]
  });
}
function onInteractCctvMonitor(player, cctvMonitor, itemStack) {
  const mainhandSlot = player.getComponent("equippable").getEquipmentSlot(mc28.EquipmentSlot.Mainhand);
  if (mainhandSlot.typeId !== LINKER_ITEM_TYPE) return;
  const monitorRefs = getMonitorRefs(itemStack);
  const isAlreadyCollected = monitorRefs.findIndex((ref) => ref.entityId === cctvMonitor.id) !== -1;
  if (isAlreadyCollected) {
    player.sendMessage({
      translate: "scpdy.msg.cctvLinker.alreadyCollectedMonitor",
      with: [cctvMonitor.id.toString()]
    });
    return;
  }
  monitorRefs.push({
    entityId: cctvMonitor.id
  });
  const newItemStack = setMonitorRefs(itemStack, monitorRefs);
  mainhandSlot.setItem(newItemStack);
  player.sendMessage({
    translate: "scpdy.msg.cctvLinker.collectedMonitor",
    with: [cctvMonitor.id.toString()]
  });
}
function onInteractCctvServerAuthorized(player, cctvServer, itemStack) {
  const mainhandSlot = player.getComponent("equippable").getEquipmentSlot(mc28.EquipmentSlot.Mainhand);
  if (mainhandSlot.typeId !== LINKER_ITEM_TYPE) return;
  const linkerCameraRefs = getCameraRefs2(itemStack);
  const linkerMonitorRefs = getMonitorRefs(itemStack);
  if (linkerCameraRefs.length <= 0 && linkerMonitorRefs.length <= 0) {
    player.sendMessage({
      translate: "scpdy.msg.cctvLinker.nothingToLink"
    });
    return;
  }
  const serverCameraRefs = getCameraRefs(cctvServer);
  let linkedCameraCount = 0;
  let linkedMonitorCount = 0;
  let skippedCameraCount = 0;
  let skippedMonitorCount = 0;
  for (const cameraRefToLink of linkerCameraRefs) {
    const isAlreadyAdded = serverCameraRefs.findIndex((ref) => ref.entityId === cameraRefToLink.entityId) !== -1;
    if (isAlreadyAdded) {
      skippedCameraCount++;
      continue;
    }
    serverCameraRefs.push(cameraRefToLink);
    linkedCameraCount++;
  }
  for (const monitorRef of linkerMonitorRefs) {
    const cctvMonitor = mc28.world.getEntity(monitorRef.entityId);
    if (!cctvMonitor || cctvMonitor.typeId !== MONITOR_ENTITY_TYPE) {
      skippedMonitorCount++;
      continue;
    }
    if (getCctvServerId(cctvMonitor) !== void 0) {
      skippedMonitorCount++;
      continue;
    }
    setCctvServerId(cctvMonitor, cctvServer.id);
    linkedMonitorCount++;
  }
  setCameraRefs(cctvServer, serverCameraRefs);
  let newItemStack = setCameraRefs2(itemStack, []);
  newItemStack = setMonitorRefs(newItemStack, []);
  mainhandSlot.setItem(newItemStack);
  if (linkedCameraCount > 0) {
    player.sendMessage({
      translate: "scpdy.msg.cctvLinker.linkedCameras",
      with: [linkedCameraCount.toString()]
    });
  }
  if (linkedMonitorCount > 0) {
    player.sendMessage({
      translate: "scpdy.msg.cctvLinker.linkedMonitors",
      with: [linkedMonitorCount.toString()]
    });
  }
  if (skippedCameraCount > 0) {
    player.sendMessage({
      translate: "scpdy.msg.cctvLinker.skippedCameras",
      with: [skippedCameraCount.toString()]
    });
  }
  if (skippedMonitorCount > 0) {
    player.sendMessage({
      translate: "scpdy.msg.cctvLinker.skippedMonitors",
      with: [skippedMonitorCount.toString()]
    });
  }
}
function onInteractCctvServer(player, cctvServer, itemStack) {
  const password = getPassword(cctvServer);
  if (password === void 0 || password.trim() === "") {
    onInteractCctvServerAuthorized(player, cctvServer, itemStack);
    return;
  }
  showPasswordForm(player, cctvServer).then((isCorrect) => {
    if (!isCorrect) return;
    try {
      onInteractCctvServerAuthorized(player, cctvServer, itemStack);
    } catch (error) {
      player.sendMessage({
        translate: "scpdy.msg.misc.error",
        with: [`${error}`]
      });
    }
  });
}
function onUseItem(arg) {
  if (!arg.itemStack) return;
  onUseCctvLinker(arg.source, arg.itemStack);
}
mc28.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:cctv_linker", {
    onUse: onUseItem
  });
});
mc28.world.afterEvents.playerInteractWithEntity.subscribe((event) => {
  if (!event.itemStack) return;
  if (event.itemStack.typeId !== LINKER_ITEM_TYPE) return;
  if (event.player.isSneaking) return;
  switch (event.target.typeId) {
    case CAMERA_ENTITY_TYPE:
      onInteractCctvCamera(event.player, event.target, event.itemStack);
      break;
    case MONITOR_ENTITY_TYPE:
      onInteractCctvMonitor(event.player, event.target, event.itemStack);
      break;
    case SERVER_ENTITY_TYPE:
      onInteractCctvServer(event.player, event.target, event.itemStack);
      break;
  }
});

// scripts/logic/cctv/cctvTablet.ts
import * as mc29 from "@minecraft/server";
import { ActionFormData as ActionFormData3 } from "@minecraft/server-ui";
function getCctvServerId2(cctvTablet) {
  return ensureType(cctvTablet.getDynamicProperty("serverId"), "string");
}
function setCctvServerId2(cctvTablet, value) {
  const newItemStack = cctvTablet.clone();
  newItemStack.setDynamicProperty("serverId", value);
  return newItemStack;
}
function onSelectCameraToUse2(player, cctvServer, cctvCamera, cctvTabletItem) {
  function stopCondition() {
    try {
      const equippable = player.getComponent("equippable");
      const mainhandItem = equippable.getEquipment(mc29.EquipmentSlot.Mainhand);
      if (!mainhandItem) return true;
      if (mainhandItem.typeId !== cctvTabletItem.typeId) return true;
      if (player.getComponent("health").currentValue <= 0) return true;
      if (cctvServer.getComponent("health").currentValue <= 0) return true;
      if (cctvCamera.getComponent("health").currentValue <= 0) return true;
      return false;
    } catch {
      return true;
    }
  }
  setCctvUsage({
    player,
    cctvServer,
    cctvCamera,
    stopCondition
  });
}
async function showCameraList2(player, cctvServer, cctvTabletItem) {
  const mainhandSlot = player.getComponent("equippable").getEquipmentSlot(mc29.EquipmentSlot.Mainhand);
  if (mainhandSlot.typeId !== TABLET_ITEM_TYPE) return;
  const cameraRefs = getCameraRefs(cctvServer);
  const cameraListForm = new ActionFormData3().title({
    translate: "scpdy.form.cctvTablet.chooseCamera.title"
  }).body({
    translate: "scpdy.form.cctvTablet.chooseCamera.body",
    with: [cameraRefs.length.toString()]
  });
  const buttons = [];
  for (let cameraRefIndex = 0; cameraRefIndex < cameraRefs.length; cameraRefIndex++) {
    const cameraRef = cameraRefs[cameraRefIndex];
    const cameraEntity = mc29.world.getEntity(cameraRef.entityId);
    if (!cameraEntity || cameraEntity.dimension.id !== cctvServer.dimension.id) {
      buttons.push({
        label: {
          translate: "scpdy.form.cctvTablet.chooseCamera.button.cameraElementError",
          with: [cameraRefIndex.toString()]
        },
        async callback() {
          player.sendMessage({
            translate: "scpdy.msg.cctvTablet.cameraUnavailable"
          });
        }
      });
    } else {
      buttons.push({
        label: {
          translate: "scpdy.form.cctvTablet.chooseCamera.button.cameraElement",
          with: [
            cameraRefIndex.toString(),
            cameraEntity.nameTag.trim() ? cameraEntity.nameTag : `${toString(round(cameraEntity.location))}`
          ]
        },
        async callback() {
          onSelectCameraToUse2(player, cctvServer, cameraEntity, cctvTabletItem);
        }
      });
    }
  }
  buttons.push({
    label: {
      translate: "scpdy.form.cctvTablet.failedToGetServer.removeLinkButton"
    },
    async callback() {
      if (mainhandSlot.typeId !== TABLET_ITEM_TYPE) return;
      const newItemStack = setCctvServerId2(cctvTabletItem, void 0);
      mainhandSlot.setItem(newItemStack);
      removeCctvUsage(player);
      player.sendMessage({ translate: "scpdy.msg.cctvTablet.removedServerLink" });
    }
  });
  for (const button of buttons) {
    cameraListForm.button(button.label);
  }
  const response = await cameraListForm.show(player);
  if (response.canceled || response.selection === void 0) {
    return;
  }
  await buttons[response.selection].callback();
}
function onUseCctvTablet(player, itemStack) {
  const mainhandSlot = player.getComponent("equippable").getEquipmentSlot(mc29.EquipmentSlot.Mainhand);
  if (mainhandSlot.typeId !== TABLET_ITEM_TYPE) return;
  const cctvServerId = getCctvServerId2(itemStack);
  if (cctvServerId === void 0) {
    new ActionFormData3().title({ translate: "scpdy.form.cctvTablet.notLinkedToServer.title" }).body({ translate: "scpdy.form.cctvTablet.notLinkedToServer.body" }).button({ translate: "scpdy.form.misc.close" }).show(player);
    return;
  }
  const cctvServer = mc29.world.getEntity(cctvServerId);
  if (!cctvServer || cctvServer.typeId !== SERVER_ENTITY_TYPE) {
    new ActionFormData3().title({ translate: "scpdy.form.cctvTablet.failedToGetServer.title" }).body({ translate: "scpdy.form.cctvTablet.failedToGetServer.body" }).button({ translate: "scpdy.form.cctvTablet.failedToGetServer.removeLinkButton" }).show(player).then((response) => {
      if (mainhandSlot.typeId !== TABLET_ITEM_TYPE) return;
      if (response.canceled) return;
      if (response.selection === 0) {
        const newItemStack = setCctvServerId2(itemStack, void 0);
        mainhandSlot.setItem(newItemStack);
        player.sendMessage({ translate: "scpdy.msg.cctvMonitor.removedServerLink" });
      }
    });
    return;
  }
  showCameraList2(player, cctvServer, itemStack);
}
function onInteractCctvServerAuthorized2(player, cctvServer, itemStack) {
  const mainhandSlot = player.getComponent("equippable").getEquipmentSlot(mc29.EquipmentSlot.Mainhand);
  if (mainhandSlot.typeId !== TABLET_ITEM_TYPE) return;
  const newItemStack = setCctvServerId2(itemStack, cctvServer.id);
  mainhandSlot.setItem(newItemStack);
  player.sendMessage({
    translate: "scpdy.msg.cctvTablet.connectedToServer",
    with: [cctvServer.id]
  });
}
function onInteractCctvServer2(player, itemStack, cctvServer) {
  const savedServerId = getCctvServerId2(itemStack);
  if (savedServerId !== void 0) {
    player.sendMessage({
      translate: "scpdy.msg.cctvTablet.alreadyConnectedToServer"
    });
    return;
  }
  const password = getPassword(cctvServer);
  if (password === void 0 || password.trim() === "") {
    onInteractCctvServerAuthorized2(player, cctvServer, itemStack);
    return;
  }
  showPasswordForm(player, cctvServer).then((isCorrect) => {
    if (!isCorrect) return;
    try {
      onInteractCctvServerAuthorized2(player, cctvServer, itemStack);
    } catch (error) {
      player.sendMessage({
        translate: "scpdy.msg.misc.error",
        with: [`${error}`]
      });
    }
  });
}
function onUseItem2(arg) {
  if (!arg.itemStack) return;
  if (arg.source.getItemCooldown("scpdy_cctv_tablet_interact") > 0) return;
  onUseCctvTablet(arg.source, arg.itemStack);
  arg.source.startItemCooldown("scpdy_cctv_tablet_use", 2);
}
mc29.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:cctv_tablet", {
    onUse: onUseItem2
  });
});
mc29.world.afterEvents.playerInteractWithEntity.subscribe((event) => {
  if (event.player.isSneaking) return;
  if (event.target.typeId !== SERVER_ENTITY_TYPE) return;
  if (!event.itemStack) return;
  if (event.itemStack.typeId !== TABLET_ITEM_TYPE) return;
  if (event.player.getItemCooldown("scpdy_cctv_tablet_use") > 0) return;
  onInteractCctvServer2(event.player, event.itemStack, event.target);
  event.player.startItemCooldown("scpdy_cctv_tablet_interact", 2);
});

// scripts/logic/components/blocks/blinkingLight.ts
import * as mc30 from "@minecraft/server";
function onTick3(arg) {
  const isLit = arg.block.permutation.getState("lc:is_lit") === true;
  if (isLit) {
    arg.dimension.playSound("scpdy.misc.small_light.off", arg.block.center());
  } else {
    arg.dimension.playSound("scpdy.misc.small_light.on", arg.block.center());
  }
  arg.block.setPermutation(arg.block.permutation.withState("lc:is_lit", !isLit));
}
mc30.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:blinking_light", {
    onTick: onTick3
  });
});

// scripts/logic/components/blocks/cannotBePlacedByPlayer.ts
import * as mc31 from "@minecraft/server";
function beforeOnPlayerPlace4(arg) {
  arg.cancel = true;
}
mc31.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:cannot_be_placed_by_player", {
    beforeOnPlayerPlace: beforeOnPlayerPlace4
  });
});

// scripts/logic/components/blocks/sittable.ts
import * as mc32 from "@minecraft/server";
var SITTABLE_ANCHOR_TYPE = "lc:scpdy_sittable_anchor";
function onPlayerInteract8(arg) {
  const { block, dimension, player } = arg;
  if (!player) return;
  const center = block.center();
  if (distance(center, player.location) > 2.5) {
    if (block.hasTag("chair")) {
      player.onScreenDisplay.setActionBar({
        translate: "scpdy.actionHint.furniture.chair.tooFar"
      });
    }
    return;
  }
  const sitLoc = {
    x: center.x,
    y: center.y - 0.2,
    z: center.z
  };
  const sittableAnchorAlreadyExists = dimension.getEntities({
    type: SITTABLE_ANCHOR_TYPE,
    closest: 1,
    maxDistance: 0.5,
    location: sitLoc
  }).length > 0;
  if (sittableAnchorAlreadyExists) return;
  const blockDirection = block.permutation.getState("minecraft:cardinal_direction");
  let sittableAnchorSpawnEvent;
  switch (blockDirection) {
    case "north":
      sittableAnchorSpawnEvent = `sittable_anchor:r180`;
      break;
    case "east":
      sittableAnchorSpawnEvent = `sittable_anchor:r270`;
      break;
    case "west":
      sittableAnchorSpawnEvent = `sittable_anchor:r90`;
      break;
    default:
      sittableAnchorSpawnEvent = "minecraft:entity_spawned";
      break;
  }
  const sittableAnchor = dimension.spawnEntity(SITTABLE_ANCHOR_TYPE, sitLoc, {
    spawnEvent: sittableAnchorSpawnEvent
  });
  mc32.system.run(() => {
    const rideable = sittableAnchor.getComponent("rideable");
    if (!rideable) return;
    rideable.addRider(player);
  });
}
mc32.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:sittable", {
    onPlayerInteract: onPlayerInteract8
  });
});

// scripts/logic/components/items/dealMaxDamageOnHit.ts
import * as mc33 from "@minecraft/server";
function onHitEntity(arg) {
  try {
    const maxHealth = arg.hitEntity.getComponent("health")?.effectiveMax;
    arg.hitEntity.applyDamage(maxHealth ?? 99999999, {
      cause: mc33.EntityDamageCause.override
    });
  } catch {
  }
}
mc33.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:deal_max_damage_on_hit", {
    onHitEntity
  });
});

// scripts/logic/components/items/dontGetDurabilityDamageOnHit.ts
import * as mc34 from "@minecraft/server";
mc34.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:dont_get_durability_damage_on_hit", {
    onBeforeDurabilityDamage(arg) {
      arg.durabilityDamage = 0;
    }
  });
});

// scripts/logic/components/items/killTargetOnHit.ts
import * as mc35 from "@minecraft/server";
function onHitEntity2(arg) {
  try {
    arg.hitEntity.kill();
  } catch {
  }
}
mc35.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:kill_target_on_hit", {
    onHitEntity: onHitEntity2
  });
});

// scripts/logic/components/items/notUsableForNowMsg.ts
import * as mc36 from "@minecraft/server";
function onUse2(arg) {
  arg.source.sendMessage({
    translate: "scpdy.msg.misc.itemNotUsableForNow"
  });
}
mc36.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:not_usable_for_now_msg", {
    onUse: onUse2
  });
});

// scripts/logic/components/items/removeTargetOnHit.ts
import * as mc37 from "@minecraft/server";
function onHitEntity3(arg) {
  if (arg.hitEntity.typeId === "minecraft:player") {
    arg.hitEntity.kill();
    return;
  }
  try {
    arg.hitEntity.remove();
  } catch {
  }
}
mc37.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:remove_target_on_hit", {
    onHitEntity: onHitEntity3
  });
});

// scripts/logic/config/config.ts
import * as mc38 from "@minecraft/server";
import { ModalFormData as ModalFormData3 } from "@minecraft/server-ui";
var ConfigBuilder = class {
  elements = [];
  divider() {
    this.elements.push({ type: "divider" });
    return this;
  }
  header(value) {
    this.elements.push({ type: "header", value });
    return this;
  }
  label(value) {
    this.elements.push({ type: "label", value });
    return this;
  }
  toggle(name, option) {
    const dynamicPropId = this.dynamicPropertyIdOf(name);
    const realOption = {
      ...option,
      type: "boolean",
      mode: "toggle",
      get: () => {
        const value = mc38.world.getDynamicProperty(dynamicPropId);
        return typeof value === "boolean" ? value : option.default;
      },
      set: (value) => {
        let newValue = void 0;
        if (typeof value === "boolean") newValue = value;
        mc38.world.setDynamicProperty(dynamicPropId, newValue);
      }
    };
    this.elements.push({
      type: "option",
      name,
      option: realOption
    });
    return this;
  }
  dropdown(name, option) {
    const dynamicPropId = this.dynamicPropertyIdOf(name);
    const realOption = {
      ...option,
      type: "number",
      mode: "dropdown",
      get: () => {
        const value = mc38.world.getDynamicProperty(dynamicPropId);
        return typeof value === "number" ? value : option.default;
      },
      set: (value) => {
        let newValue = void 0;
        if (typeof value === "number") newValue = value;
        mc38.world.setDynamicProperty(dynamicPropId, newValue);
      }
    };
    this.elements.push({
      type: "option",
      name,
      option: realOption
    });
    return this;
  }
  dynamicPropertyIdOf(name) {
    return `config_${name}`;
  }
  make() {
    let returnProps = {};
    for (const element of this.elements) {
      if (element.type !== "option") continue;
      const name = element.name;
      const option = element.option;
      Object.defineProperty(returnProps, name, {
        get() {
          return option.get();
        },
        set(value) {
          option.set(value);
        },
        enumerable: false,
        configurable: true
      });
    }
    return returnProps;
  }
  resetToDefaults() {
    for (const element of this.elements) {
      if (element.type !== "option") continue;
      element.option.set(element.option.default);
    }
  }
  async showEditorForm(player) {
    mc38.system.run(() => {
      player.playSound("random.pop");
    });
    const formData = new ModalFormData3().title({ translate: "scpdy.configV2.title" }).submitButton({ translate: "scpdy.configV2.submitButton" }).label({ translate: "scpdy.configV2.description" }).divider().label({ translate: "scpdy.configV2.resetOnExit.label" }).toggle({ translate: "scpdy.configV2.resetOnExit.toggle" }, { defaultValue: false }).divider();
    const optionCallbacks = [];
    for (const element of this.elements) {
      switch (element.type) {
        case "divider":
          formData.divider();
          break;
        case "header":
          formData.header(element.value);
          break;
        case "label":
          formData.label(element.value);
          break;
        case "option":
          const callback = this.addOptionToConfigEditorForm(formData, element.name, element.option);
          optionCallbacks.push(callback);
          break;
      }
    }
    const response = await formData.show(player);
    if (response.canceled) return;
    if (!response.formValues) return;
    for (let i = 0; i < response.formValues.length; i++) {
      const formValue = response.formValues[i];
      if (i === 3 && formValue === true) {
        this.resetToDefaults();
        return;
      }
      if (i < 5) continue;
      const optionIndex = i - 5;
      const callback = optionCallbacks[optionIndex];
      callback?.(formValue);
    }
  }
  addOptionToConfigEditorForm(formData, name, option) {
    const mode = option.mode;
    switch (mode) {
      case "toggle":
        formData.toggle(option.label ?? name, { defaultValue: option.get() });
        break;
      case "dropdown":
        formData.dropdown(option.label ?? name, option.choices, {
          defaultValueIndex: option.get()
        });
        break;
      default:
        throw new Error(`Unhandled option mode: ${mode}`);
    }
    return option.set;
  }
};
var CONFIG_BUILDER = new ConfigBuilder().toggle("disableExpensiveGoreEffects", {
  label: { translate: "scpdy.configV2.disableExpensiveGoreEffects" },
  default: false
}).toggle("disableAdvanced096Movement", {
  label: { translate: "scpdy.configV2.disableAdvancedSCP096Movement" },
  default: false
}).toggle("disableCameraFadeOnBlink", {
  label: { translate: "scpdy.configV2.disableCameraFadeOnBlink" },
  default: false
}).toggle("disableKnockbackOfCertainBullets", {
  label: { translate: "scpdy.configV2.disableKnockbackOfCertainBullets" },
  default: false
}).dropdown("gunTacReloadTrigger", {
  label: { translate: "scpdy.configV2.gunTacReloadTrigger.label" },
  choices: [
    { translate: "scpdy.configV2.gunTacReloadTrigger.0" },
    { translate: "scpdy.configV2.gunTacReloadTrigger.1" },
    { translate: "scpdy.configV2.gunTacReloadTrigger.2" }
  ],
  default: 2
});
var PROPS = CONFIG_BUILDER.make();
var CONFIG = PROPS;
async function showConfigEditorForm(player) {
  await CONFIG_BUILDER.showEditorForm(player);
}
mc38.system.afterEvents.scriptEventReceive.subscribe((event) => {
  if (event.id !== "scpdy:reset_config") return;
  CONFIG_BUILDER.resetToDefaults();
});

// scripts/logic/config/configItem.ts
import * as mc39 from "@minecraft/server";
function onUse3(arg) {
  const player = arg.source;
  if (player.hasTag("scpdy_world_leader") || player.playerPermissionLevel === mc39.PlayerPermissionLevel.Operator) {
    showConfigEditorForm(player);
    return;
  }
  player.sendMessage({ translate: "scpdy.msg.config.restricted" });
  player.playSound("note.bass");
}
mc39.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:config_item", {
    onUse: onUse3
  });
});

// scripts/logic/controlDevices/keypad/keypad.ts
import * as mc42 from "@minecraft/server";
import { ModalFormData as ModalFormData5 } from "@minecraft/server-ui";

// scripts/logic/controlDevices/utils.ts
import * as mc40 from "@minecraft/server";
import "@minecraft/server-ui";
function addControlDeviceModeDropdownToForm(formData) {
  formData.dropdown(
    { translate: "scpdy.form.controlDevice.modeDropdown.label" },
    [
      { translate: "scpdy.machinery.input.mode.text.powerRelayDoors" },
      { translate: "scpdy.machinery.input.mode.text.ctrlBlastDoor" },
      { translate: "scpdy.machinery.input.mode.text.placeRbBelow" },
      { translate: "scpdy.machinery.input.mode.text.placeRbBehind" }
    ],
    { defaultValueIndex: 0 }
  );
}
function getControlDeviceMode(controlDevice) {
  let value;
  try {
    if (controlDevice instanceof mc40.Block) {
      value = controlDevice.permutation.getState("lc:control_device_mode");
    } else if (controlDevice instanceof mc40.Entity) {
      value = controlDevice.getProperty("lc:control_device_mode");
    }
  } catch {
  }
  if (typeof value !== "number") {
    throw new Error("Control device mode property/state is not defined");
  }
  return value;
}
function setControlDeviceMode(controlDevice, value) {
  try {
    if (controlDevice instanceof mc40.Block) {
      controlDevice.setPermutation(controlDevice.permutation.withState("lc:control_device_mode", value));
    } else if (controlDevice instanceof mc40.Entity) {
      controlDevice.setProperty("lc:control_device_mode", value);
    }
  } catch {
    throw new Error("Failed to set control device mode");
  }
}

// scripts/features/machinery/input/events.ts
import "@minecraft/server";

// scripts/utils/EventEmitter.ts
var EventEmitter = class {
  subscribers = /* @__PURE__ */ new Map();
  idCounter = 0;
  /**
   * Subscribe to an event
   * @param eventName - Name of the event to subscribe to
   * @param handler - Handler function to be called when the event is emitted
   * @param options - Subscription options
   * @returns Subscription object for controlling the subscription
   */
  on(eventName, handler, options = {}) {
    const { once = false, priority = 0 } = options;
    const id = this.idCounter++;
    const subscriber = {
      handler,
      once,
      priority,
      active: true,
      id
    };
    if (!this.subscribers.has(eventName)) {
      this.subscribers.set(eventName, []);
    }
    const eventSubscribers = this.subscribers.get(eventName);
    let insertIndex = eventSubscribers.findIndex((sub2) => sub2.priority < priority);
    if (insertIndex === -1) {
      insertIndex = eventSubscribers.length;
    }
    eventSubscribers.splice(insertIndex, 0, subscriber);
    return {
      unsubscribe: () => this.off(eventName, id),
      pause: () => this.pauseSubscription(eventName, id),
      resume: () => this.resumeSubscription(eventName, id),
      isActive: () => this.isSubscriptionActive(eventName, id)
    };
  }
  /**
   * Subscribe to an event and automatically unsubscribe after the first emission
   * @param eventName - Name of the event to subscribe to
   * @param handler - Handler function to be called when the event is emitted
   * @param priority - Priority of the handler (higher numbers execute first)
   * @returns Subscription object for controlling the subscription
   */
  once(eventName, handler, priority = 0) {
    return this.on(eventName, handler, { once: true, priority });
  }
  /**
   * Emit an event synchronously and collect results
   * @param eventName - Name of the event to emit
   * @param data - Data to pass to handlers
   * @returns Array of results from all handlers
   */
  emit(eventName, data) {
    if (!this.subscribers.has(eventName)) {
      return [];
    }
    const subscribers = this.subscribers.get(eventName);
    const toRemove = [];
    const results = [];
    const activeSubscribers = [...subscribers].filter((sub2) => sub2.active);
    for (const subscriber of activeSubscribers) {
      try {
        const result = subscriber.handler(data);
        results.push(result);
      } catch (error) {
        console.error(`Error in event handler for "${String(eventName)}":`, error);
      }
      if (subscriber.once) {
        toRemove.push(subscriber.id);
      }
    }
    if (toRemove.length > 0) {
      this.subscribers.set(
        eventName,
        subscribers.filter((sub2) => !toRemove.includes(sub2.id))
      );
    }
    return results;
  }
  /**
   * Emit an event with data and collect results, including from async handlers
   * @param eventName - Name of the event to emit
   * @param data - Data to pass to handlers
   * @returns Promise that resolves with an array of results from all handlers
   */
  async emitAsync(eventName, data) {
    if (!this.subscribers.has(eventName)) {
      return [];
    }
    const subscribers = this.subscribers.get(eventName);
    const results = [];
    const toRemove = [];
    const activeSubscribers = [...subscribers].filter((sub2) => sub2.active);
    for (const subscriber of activeSubscribers) {
      try {
        const result = subscriber.handler(data);
        results.push(result);
      } catch (error) {
        console.error(`Error in event handler for "${String(eventName)}":`, error);
      }
      if (subscriber.once) {
        toRemove.push(subscriber.id);
      }
    }
    if (toRemove.length > 0) {
      this.subscribers.set(
        eventName,
        subscribers.filter((sub2) => !toRemove.includes(sub2.id))
      );
    }
    return Promise.all(results);
  }
  /**
   * Remove a specific subscriber by ID or all subscribers for an event
   * @param eventName - Name of the event
   * @param handlerId - Optional ID of the handler to remove
   */
  off(eventName, handlerId) {
    if (!this.subscribers.has(eventName)) {
      return;
    }
    if (handlerId === void 0) {
      this.subscribers.delete(eventName);
    } else {
      const eventSubscribers = this.subscribers.get(eventName);
      const filteredSubscribers = eventSubscribers.filter((sub2) => sub2.id !== handlerId);
      if (filteredSubscribers.length === 0) {
        this.subscribers.delete(eventName);
      } else {
        this.subscribers.set(eventName, filteredSubscribers);
      }
    }
  }
  /**
   * Remove all event subscriptions
   */
  removeAllListeners() {
    this.subscribers.clear();
  }
  /**
   * Get the number of subscribers for a specific event
   * @param eventName - Name of the event
   * @returns Number of subscribers
   */
  listenerCount(eventName) {
    if (!this.subscribers.has(eventName)) {
      return 0;
    }
    return this.subscribers.get(eventName).length;
  }
  /**
   * Check if an event has any subscribers
   * @param eventName - Name of the event
   * @returns True if the event has subscribers
   */
  hasListeners(eventName) {
    return this.listenerCount(eventName) > 0;
  }
  /**
   * Pause a subscription
   * @param eventName - Name of the event
   * @param handlerId - ID of the handler to pause
   */
  pauseSubscription(eventName, handlerId) {
    this.setSubscriptionState(eventName, handlerId, false);
  }
  /**
   * Resume a paused subscription
   * @param eventName - Name of the event
   * @param handlerId - ID of the handler to resume
   */
  resumeSubscription(eventName, handlerId) {
    this.setSubscriptionState(eventName, handlerId, true);
  }
  /**
   * Check if a subscription is active
   * @param eventName - Name of the event
   * @param handlerId - ID of the handler to check
   * @returns True if the subscription is active
   */
  isSubscriptionActive(eventName, handlerId) {
    if (!this.subscribers.has(eventName)) {
      return false;
    }
    const subscriber = this.subscribers.get(eventName).find((sub2) => sub2.id === handlerId);
    return subscriber ? subscriber.active : false;
  }
  /**
   * Set the active state of a subscription
   * @param eventName - Name of the event
   * @param handlerId - ID of the handler
   * @param active - Active state to set
   */
  setSubscriptionState(eventName, handlerId, active) {
    if (!this.subscribers.has(eventName)) {
      return;
    }
    const eventSubscribers = this.subscribers.get(eventName);
    const subscriber = eventSubscribers.find((sub2) => sub2.id === handlerId);
    if (subscriber) {
      subscriber.active = active;
    }
  }
};

// scripts/features/machinery/input/events.ts
var MachineryInputEvents = new EventEmitter();

// scripts/features/machinery/input/mode.ts
function _getModeStringFromIndex(index) {
  switch (index) {
    case 0:
      return "powerRelayDoors";
    case 1:
      return "ctrlBlastDoor";
    case 2:
      return "placeRbBelow";
    case 3:
      return "placeRbBehind";
    default:
      throw new Error(`Unknown index: ${index}`);
  }
}
function _isModeString(value) {
  switch (value) {
    case "powerRelayDoors":
    case "ctrlBlastDoor":
    case "placeRbBelow":
    case "placeRbBehind":
      return true;
    default:
      return false;
  }
}

// scripts/logic/controlDevices/keypad/keypad.ts
var KEYPAD_ENTITY_TYPE_ID = "lc:scpdy_keypad";
mc42.world.afterEvents.playerInteractWithEntity.subscribe(({ player, target: keypad }) => {
  if (keypad.typeId !== KEYPAD_ENTITY_TYPE_ID) return;
  if (keypad.getProperty("lc:is_on") === true) return;
  const o5Clearance = getKeypadO5Clearance(keypad);
  if (o5Clearance && getClearanceLevel(player) < 6) {
    player.onScreenDisplay.setActionBar({
      translate: "scpdy.actionHint.misc.o5PermissionRequired"
    });
    playIncorrectBuzzer(keypad, player);
    return;
  }
  keypadInteractionAsync(player, keypad).catch(() => {
    player.sendMessage({ translate: "scpdy.msg.misc.somethingWentWrong" });
  });
});
async function keypadInteractionAsync(player, keypad) {
  const formData = new ModalFormData5().title({ translate: "scpdy.form.keypad.use.title" }).label({ translate: "scpdy.form.keypad.use.note" }).textField(
    { translate: "scpdy.form.keypad.use.passwordField.label" },
    { translate: "scpdy.form.keypad.use.passwordField.placeholder" }
  ).submitButton({ translate: "scpdy.form.keypad.use.submitButton" });
  const hint = getKeypadHint(keypad);
  if (hint.trim() !== "") {
    formData.label({ translate: "scpdy.form.keypad.use.hint", with: [hint] });
  }
  const response = await formData.show(player);
  if (!keypad.isValid) return;
  if (response.canceled) return;
  if (!response.formValues) return;
  const enteredPassword = String(response.formValues[1]);
  const realPassword = getKeypadPassword(keypad);
  if (enteredPassword !== realPassword) {
    player.onScreenDisplay.setActionBar({ translate: "scpdy.actionHint.misc.accessDenied" });
    playIncorrectBuzzer(keypad, player);
    return;
  }
  player.onScreenDisplay.setActionBar({ translate: "scpdy.actionHint.misc.accessGranted" });
  keypad.dimension.playSound("scpdy.interact.keypad.correct", keypad.location, {
    volume: 1.2
  });
  turnOn(keypad, player);
}
function turnOn(keypad, player) {
  keypad.triggerEvent("turn_on");
  MachineryInputEvents.emit("onActivate", {
    mode: _getModeStringFromIndex(getControlDeviceMode(keypad)),
    dimension: keypad.dimension,
    location: keypad.location,
    entity: keypad,
    clearanceLevel: getKeypadO5Clearance(keypad) ? 6 : -1,
    pulseDirection: rotationToDirection({ x: 0, y: Number(keypad.getProperty("lc:rotation_y")) }),
    source: player
  });
}
function playIncorrectBuzzer(keypad, player) {
  const loudBuzzer = getKeypadLoudIncorrectBuzzer(keypad);
  if (loudBuzzer) {
    keypad.dimension.playSound("scpdy.interact.keypad.incorrect_loud", keypad.location, {
      volume: 1.5
    });
    player.runCommand("camerashake add @s 0.16 0.9 rotational");
  } else {
    keypad.dimension.playSound("scpdy.interact.keypad.incorrect", keypad.location, {
      volume: 1.1
    });
  }
}
function getKeypadSourceId(keypad) {
  const sourceId = keypad.getDynamicProperty("sourceId");
  if (typeof sourceId !== "string") return;
  return sourceId;
}
function setKeypadSourceId(keypad, value) {
  keypad.setDynamicProperty("sourceId", value);
}
function getKeypadPassword(keypad) {
  return String(keypad.getDynamicProperty("password"));
}
function setKeypadPassword(keypad, value) {
  keypad.setDynamicProperty("password", value);
}
function getKeypadHint(keypad) {
  return String(keypad.getDynamicProperty("hint"));
}
function setKeypadHint(keypad, value) {
  keypad.setDynamicProperty("hint", value);
}
function getKeypadLoudIncorrectBuzzer(keypad) {
  return keypad.getDynamicProperty("loudIncorrectBuzzer") === true;
}
function setKeypadLoudIncorrectBuzzer(keypad, value) {
  keypad.setDynamicProperty("loudIncorrectBuzzer", value);
}
function getKeypadO5Clearance(keypad) {
  return keypad.getDynamicProperty("o5Clearance") === true;
}
function setKeypadO5Clearance(keypad, value) {
  keypad.setDynamicProperty("o5Clearance", value);
}
function attachNewKeypadEntityTo(block, blockFace, source) {
  const location = getRelativeLocation(
    block.bottomCenter(),
    {
      x: 0,
      y: 0.15,
      z: -0.617
    },
    blockFace
  );
  const entity = block.dimension.spawnEntity(KEYPAD_ENTITY_TYPE_ID, location);
  const rot = directionToRotation(reversedDirection(blockFace));
  entity.setProperty("lc:rotation_y", rot.y);
  setKeypadSourceId(entity, source?.id);
  return entity;
}
mc42.world.afterEvents.entityDie.subscribe(
  ({ deadEntity: keypad }) => {
    removeKeypad(keypad);
  },
  { entityTypes: [KEYPAD_ENTITY_TYPE_ID] }
);
mc42.world.afterEvents.entityHitEntity.subscribe(
  ({ damagingEntity, hitEntity: keypad }) => {
    if (keypad.typeId !== KEYPAD_ENTITY_TYPE_ID) return;
    if (!keypad.isValid) return;
    if (!(damagingEntity instanceof mc42.Player)) return;
    if (damagingEntity.getGameMode() !== mc42.GameMode.Creative && getKeypadSourceId(keypad) !== damagingEntity.id)
      return;
    keypad.dimension.playSound("scpdy.misc.computer.hit", keypad.location);
  },
  { entityTypes: ["minecraft:player"] }
);
function removeKeypad(keypad) {
  try {
    if (mc42.world.gameRules.doMobLoot) {
      const placerItem = new mc42.ItemStack("lc:scpdy_keypad_placer");
      keypad.dimension.spawnItem(placerItem, keypad.location);
    }
    keypad.remove();
  } catch {
  }
}

// scripts/logic/controlDevices/keypad/placer.ts
import * as mc43 from "@minecraft/server";
import { ModalFormData as ModalFormData6 } from "@minecraft/server-ui";
var KEYPAD_PLACER_ITEM_TYPE_ID = "lc:scpdy_keypad_placer";
mc43.system.beforeEvents.startup.subscribe(({ itemComponentRegistry }) => {
  itemComponentRegistry.registerCustomComponent("scpdy:keypad_placer", {
    onUseOn
  });
});
function onUseOn(arg) {
  const player = arg.source;
  if (!(player instanceof mc43.Player)) return;
  if (arg.blockFace === mc43.Direction.Down || arg.blockFace === mc43.Direction.Up) return;
  processPlacementAsync(player, arg).catch(() => {
    player.sendMessage({ translate: "scpdy.msg.misc.somethingWentWrong" });
  });
}
async function processPlacementAsync(player, event) {
  const formData = new ModalFormData6().title({ translate: "scpdy.form.keypad.placementOpts.title" }).label({ translate: "scpdy.form.keypad.placementOpts.warning" }).textField(
    { translate: "scpdy.form.keypad.placementOpts.passwordField.label" },
    { translate: "scpdy.form.keypad.placementOpts.passwordField.placeholder" }
  ).textField(
    { translate: "scpdy.form.keypad.placementOpts.hintField.label" },
    { translate: "scpdy.form.keypad.placementOpts.hintField.placeholder" }
  ).toggle({ translate: "scpdy.form.keypad.placementOpts.loudIncorrectBuzzerToggle.label" }, { defaultValue: false }).divider();
  addControlDeviceModeDropdownToForm(formData);
  formData.toggle({ translate: "scpdy.form.keypad.placementOpts.O5ClearanceToggle.label" }).submitButton({ translate: "scpdy.form.keypad.placementOpts.submitButton" });
  const response = await formData.show(player);
  if (response.canceled) return;
  if (!response.formValues) return;
  const password = String(response.formValues[1]).trim();
  const hint = String(response.formValues[2]);
  const loudIncorrectBuzzer = response.formValues[3] === true;
  const controlDeviceMode = Number(response.formValues[5]);
  const o5Clearance = response.formValues[6] === true;
  if (password === "") {
    player.playSound("note.bass");
    player.sendMessage({ translate: "scpdy.msg.keypad.placement.passwordRequired" });
    return;
  }
  if (password.length > 15) {
    player.playSound("note.bass");
    player.sendMessage({ translate: "scpdy.msg.keypad.placement.passwordTooLong" });
    return;
  }
  if (player.getGameMode() !== mc43.GameMode.Creative) {
    const equippable = player.getComponent("equippable");
    if (!equippable) return;
    const mainhandSlot = equippable.getEquipmentSlot(mc43.EquipmentSlot.Mainhand);
    if (!mainhandSlot.hasItem()) return;
    if (mainhandSlot.typeId !== KEYPAD_PLACER_ITEM_TYPE_ID) return;
    const itemAmount = mainhandSlot.amount;
    if (itemAmount === 1) {
      mainhandSlot.setItem();
    } else {
      mainhandSlot.amount = itemAmount - 1;
    }
  }
  const keypad = attachNewKeypadEntityTo(event.block, event.blockFace, player);
  setKeypadPassword(keypad, password);
  setKeypadHint(keypad, hint);
  setKeypadLoudIncorrectBuzzer(keypad, loudIncorrectBuzzer);
  setControlDeviceMode(keypad, controlDeviceMode);
  setKeypadO5Clearance(keypad, o5Clearance);
}

// scripts/features/gore/gibs.ts
import * as mc45 from "@minecraft/server";

// scripts/features/gore/blood.ts
import * as mc44 from "@minecraft/server";

// scripts/features/gore/shared.ts
var BLOOD_ENTITY_TYPE = "lc:scpdy_blood";
var GIB_SMALL_ENTITY_TYPE = "lc:scpdy_gib_small";
var GIB_MEDIUM_ENTITY_TYPE = "lc:scpdy_gib_medium";

// scripts/features/gore/blood.ts
function spreadBlood(dimension, location, fast) {
  let spawnLoc;
  const raycastHit = dimension.getBlockFromRay(location, DOWN, { maxDistance: 10 });
  if (raycastHit) {
    spawnLoc = add(raycastHit.block.location, raycastHit.faceLocation);
    spawnLoc = add(spawnLoc, { x: 0, y: 0.05, z: 0 });
  } else {
    spawnLoc = {
      x: location.x,
      y: location.y - 9.5,
      z: location.z
    };
  }
  const bloodEntity = dimension.spawnEntity(BLOOD_ENTITY_TYPE, spawnLoc);
  if (fast === true) {
    bloodEntity.triggerEvent("blood:start_spreading_fast");
  } else {
    bloodEntity.triggerEvent("blood:start_spreading_slow");
  }
}
mc44.world.afterEvents.entityDie.subscribe(
  (event) => {
    event.deadEntity.remove();
  },
  {
    entityTypes: [BLOOD_ENTITY_TYPE]
  }
);

// scripts/features/gore/gibs.ts
function spawnGoreExplosion(dimension, location) {
  dimension.playSound("scpdy.gore.explode", location, { volume: 1.6 });
  dimension.spawnParticle("lc:scpdy_body_explosion_particle", location);
  dimension.spawnParticle("lc:scpdy_blood_splash_emitter", location);
  for (let i2 = 0; i2 < 3; i2++) {
    mc45.system.runTimeout(() => {
      dimension.spawnParticle("lc:scpdy_blood_splash_emitter", {
        x: location.x + (Math.random() - 0.5),
        y: location.y + 0.1,
        z: location.z + (Math.random() - 0.5)
      });
    }, i2);
  }
  if (CONFIG.disableExpensiveGoreEffects) return;
  spreadBlood(dimension, location, true);
  const existingGibCount = dimension.getEntities({ families: ["gib"] }).length;
  for (let i2 = 0; i2 < 3; i2++) {
    spawnGib("medium", dimension, location, {
      x: (Math.random() - 0.5) / 1.5,
      y: 0.1,
      z: (Math.random() - 0.5) / 1.5
    });
  }
  let i = 0;
  let runId = -1;
  function spawnSmallGib() {
    spawnGib("small", dimension, location, {
      x: (Math.random() - 0.5) / 6,
      y: 0.25 * (i / 3.5) + Math.random() / 2.7,
      z: (Math.random() - 0.5) / 6
    });
  }
  function spawnMediumGib() {
    spawnGib("medium", dimension, location, {
      x: (Math.random() - 0.5) / 4,
      y: 0.2 * (i / 5) + Math.random() / 3,
      z: (Math.random() - 0.5) / 4
    });
  }
  try {
    runId = mc45.system.runInterval(() => {
      if (existingGibCount < 20) {
        if (i < 7) {
          spawnSmallGib();
        } else if (i < 11) {
          spawnMediumGib();
        } else {
          mc45.system.clearRun(runId);
          return;
        }
      } else if (existingGibCount < 40) {
        if (i < 4) {
          spawnSmallGib();
        } else if (i < 7) {
          spawnMediumGib();
        } else {
          mc45.system.clearRun(runId);
          return;
        }
      } else if (existingGibCount < 60) {
        if (i < 2) {
          spawnSmallGib();
        } else if (i < 4) {
          spawnMediumGib();
        } else {
          mc45.system.clearRun(runId);
          return;
        }
      } else {
        mc45.system.clearRun(runId);
      }
      i++;
    }, 1);
  } catch {
    mc45.system.clearRun(runId);
  }
}
function spawnGib(type, dimension, location, impulse) {
  const entityType = type === "medium" ? "lc:scpdy_gib_medium" : "lc:scpdy_gib_small";
  const gibEntity = dimension.spawnEntity(entityType, location);
  if (impulse) {
    gibEntity.applyImpulse(impulse);
  }
}
function onGibEntityDie(gibEntity) {
  try {
    const particleLoc = {
      x: gibEntity.location.x,
      y: gibEntity.location.y + 0.1,
      z: gibEntity.location.z
    };
    gibEntity.dimension.spawnParticle("lc:scpdy_blood_splash_small_emitter", particleLoc);
  } finally {
    gibEntity.remove();
  }
}
mc45.world.afterEvents.entityDie.subscribe(
  (event) => {
    if (event.damageSource.cause === mc45.EntityDamageCause.selfDestruct) return;
    if (event.damageSource.cause === mc45.EntityDamageCause.lava) return;
    onGibEntityDie(event.deadEntity);
  },
  {
    entityTypes: [GIB_SMALL_ENTITY_TYPE, GIB_MEDIUM_ENTITY_TYPE]
  }
);
mc45.world.afterEvents.playerInteractWithEntity.subscribe((event) => {
  if (!event.target.typeId.startsWith("lc:scpdy_gib")) return;
  event.player.dimension.playSound("random.eat", event.player.getHeadLocation());
  event.target.remove();
  event.player.addEffect("saturation", 2, { amplifier: 1, showParticles: false });
  event.player.addEffect("regeneration", 5, { amplifier: 1, showParticles: true });
});
mc45.system.afterEvents.scriptEventReceive.subscribe((event) => {
  if (event.id !== "scpdy:gore_explosion") return;
  const sourceDim = event.sourceEntity?.dimension ?? event.sourceBlock?.dimension;
  const sourceLoc = event.sourceEntity?.location ?? event.sourceBlock?.location;
  if (!sourceDim || !sourceLoc) return;
  spawnGoreExplosion(sourceDim, sourceLoc);
});

// scripts/logic/humanMobs/corpse.ts
import * as mc47 from "@minecraft/server";

// scripts/logic/humanMobs/shared.ts
import * as mc46 from "@minecraft/server";
var HUMAN_MOB_TYPES = {
  f_researcher: "lc:scpdy_f_researcher",
  f_tro: "lc:scpdy_f_tro",
  f_mtf_epsilon11: "lc:scpdy_f_mtf_epsilon11"
};
var HUMAN_MOB_TYPE_ARRAY = Array.from(Object.values(HUMAN_MOB_TYPES));
var HUMAN_MOB_CORPSE_TYPES = {
  f_tro_dead: "lc:scpdy_f_tro_dead",
  f_researcher: "lc:scpdy_f_researcher_dead",
  f_mtf_epsilon11_dead: "lc:scpdy_f_mtf_epsilon11_dead"
};
var HUMAN_MOB_CORPSE_TYPE_ARRAY = Array.from(Object.values(HUMAN_MOB_CORPSE_TYPES));
var DAMAGE_CAUSES_IGNORE_BODY_EXPLOSION = /* @__PURE__ */ new Set([
  mc46.EntityDamageCause.selfDestruct,
  mc46.EntityDamageCause.lava,
  mc46.EntityDamageCause.magma,
  mc46.EntityDamageCause.fireTick,
  mc46.EntityDamageCause.drowning,
  mc46.EntityDamageCause.freezing,
  mc46.EntityDamageCause.magic,
  mc46.EntityDamageCause.suffocation,
  mc46.EntityDamageCause.wither,
  mc46.EntityDamageCause.soulCampfire,
  mc46.EntityDamageCause.campfire
]);
function canDamageCauseBodyExplosion(damageCause) {
  return !DAMAGE_CAUSES_IGNORE_BODY_EXPLOSION.has(damageCause);
}

// scripts/logic/humanMobs/corpse.ts
function onCorpseEntityDie(entity) {
  spawnGoreExplosion(entity.dimension, entity.location);
  entity.remove();
}
mc47.world.afterEvents.entityDie.subscribe(
  (event) => {
    if (!canDamageCauseBodyExplosion(event.damageSource.cause)) return;
    onCorpseEntityDie(event.deadEntity);
  },
  {
    entityTypes: HUMAN_MOB_CORPSE_TYPE_ARRAY
  }
);

// scripts/lib/utils/entityUtils.ts
import * as mc48 from "@minecraft/server";
function getEntitiesInAllDimensions(options) {
  return [
    ...mc48.world.getDimension("overworld").getEntities(options),
    ...mc48.world.getDimension("nether").getEntities(options),
    ...mc48.world.getDimension("the_end").getEntities(options)
  ];
}
function isEntityDead(entity) {
  if (!entity.isValid) return true;
  const comp = entity.getComponent("health");
  if (!comp) return false;
  return comp.currentValue <= 0;
}
function getEntityName_typeId(typeId) {
  const entityTypeNamespace = typeId.split(":")[0];
  const entityTypeId = entityTypeNamespace === "minecraft" ? typeId.replace("minecraft:", "") : typeId;
  return { rawtext: [{ translate: `entity.${entityTypeId}.name` }] };
}
function getEntityName(value) {
  if (typeof value === "string") {
    return getEntityName_typeId(value);
  }
  if (value instanceof mc48.Player) {
    return {
      rawtext: [{ text: value.name }]
    };
  }
  if (value.nameTag.trim() !== "") return { rawtext: [{ text: value.nameTag }] };
  return getEntityName_typeId(value.typeId);
}
function getModifiedDamageNumber(damage, entity) {
  if (damage <= 0) return 0;
  const resistanceEffect = entity.getEffect("resistance");
  if (resistanceEffect) {
    const resistanceMultiplier = 1 - resistanceEffect.amplifier * 0.2;
    damage = Math.floor(damage * resistanceMultiplier);
  }
  if (damage <= 0) return 0;
  const equippableComponent = entity.getComponent("equippable");
  if (!equippableComponent) return damage;
  let defensePoints = 0;
  let armorToughness = 0;
  let epf = 0;
  const feetEq = equippableComponent.getEquipment(mc48.EquipmentSlot.Feet);
  if (feetEq) {
    epf += feetEq.getComponent("enchantable")?.getEnchantment("protection")?.level ?? 0;
    epf += (feetEq.getComponent("enchantable")?.getEnchantment("projectile_protection")?.level ?? 0) * 2;
  }
  switch (feetEq?.typeId) {
    case "minecraft:leather_boots":
    case "minecraft:golden_boots":
    case "minecraft:chainmail_boots":
      defensePoints += 1;
      break;
    case "minecraft:iron_boots":
      defensePoints += 2;
      break;
    case "minecraft:diamond_boots":
      defensePoints += 3;
      armorToughness += 2;
      break;
    case "minecraft:netherite_boots":
      defensePoints += 3;
      armorToughness += 3;
      break;
  }
  const headEq = equippableComponent.getEquipment(mc48.EquipmentSlot.Head);
  if (headEq) {
    epf += headEq.getComponent("enchantable")?.getEnchantment("protection")?.level ?? 0;
    epf += (headEq.getComponent("enchantable")?.getEnchantment("projectile_protection")?.level ?? 0) * 2;
  }
  switch (headEq?.typeId) {
    case "minecraft:leather_helmet":
      defensePoints += 1;
      break;
    case "minecraft:golden_helmet":
    case "minecraft:chainmail_helmet":
    case "minecraft:iron_helmet":
    case "minecraft:turtle_helmet":
      defensePoints += 2;
      break;
    case "minecraft:diamond_helmet":
      defensePoints += 3;
      armorToughness += 2;
      break;
    case "minecraft:netherite_helmet":
      defensePoints += 3;
      armorToughness += 3;
      break;
  }
  const legsEq = equippableComponent.getEquipment(mc48.EquipmentSlot.Legs);
  if (legsEq) {
    epf += legsEq.getComponent("enchantable")?.getEnchantment("protection")?.level ?? 0;
    epf += (legsEq.getComponent("enchantable")?.getEnchantment("projectile_protection")?.level ?? 0) * 2;
  }
  switch (legsEq?.typeId) {
    case "minecraft:leather_leggings":
      defensePoints += 2;
      break;
    case "minecraft:golden_leggings":
      defensePoints += 3;
      break;
    case "minecraft:chainmail_leggings":
      defensePoints += 4;
      break;
    case "minecraft:iron_leggings":
      defensePoints += 5;
      break;
    case "minecraft:diamond_leggings":
      defensePoints += 6;
      armorToughness += 2;
      break;
    case "minecraft:netherite_leggings":
      defensePoints += 6;
      armorToughness += 3;
      break;
  }
  const chestEq = equippableComponent.getEquipment(mc48.EquipmentSlot.Chest);
  if (chestEq) {
    epf += chestEq.getComponent("enchantable")?.getEnchantment("protection")?.level ?? 0;
    epf += (chestEq.getComponent("enchantable")?.getEnchantment("projectile_protection")?.level ?? 0) * 2;
  }
  switch (chestEq?.typeId) {
    case "minecraft:leather_chestplate":
      defensePoints += 3;
      break;
    case "minecraft:golden_chestplate":
    case "minecraft:chainmail_chestplate":
      defensePoints += 5;
      break;
    case "minecraft:iron_chestplate":
      defensePoints += 6;
      break;
    case "minecraft:diamond_chestplate":
      defensePoints += 8;
      armorToughness += 2;
      break;
    case "minecraft:netherite_chestplate":
      defensePoints += 8;
      armorToughness += 3;
      break;
  }
  damage = damage * (1 - Math.min(20, Math.max(defensePoints / 5, defensePoints - 4 * damage / (armorToughness + 8))) / 25);
  damage -= Math.min(20, epf) / 25;
  damage = Math.floor(damage);
  if (damage <= 0) return 0;
  return damage;
}

// scripts/logic/scps/scp096/shared.ts
var SCP096_ENTITY_TYPE = "lc:scpdy_scp096";
var SCP096_1_TAG = "scp096_1";

// scripts/logic/scps/scp173/shared.ts
var SCP173_ENTITY_TYPE = "lc:scpdy_scp173";

// scripts/logic/humanMobs/death.ts
import * as mc50 from "@minecraft/server";

// scripts/logic/humanMobs/loot.ts
import * as mc49 from "@minecraft/server";
var LOOT_MAP = /* @__PURE__ */ new Map([
  [
    HUMAN_MOB_TYPES.f_researcher,
    {
      getItemStacks: () => []
    }
  ],
  [
    HUMAN_MOB_TYPES.f_tro,
    {
      getItemStacks() {
        return [new mc49.ItemStack("lc:scpdy_ammo_9mm", randomInt(7, 10))];
      }
    }
  ],
  [
    HUMAN_MOB_TYPES.f_mtf_epsilon11,
    {
      getItemStacks() {
        return [new mc49.ItemStack("lc:scpdy_ammo_556mm", randomInt(11, 14))];
      }
    }
  ]
]);
function getHumanMobLootData(entityTypeId) {
  return LOOT_MAP.get(entityTypeId);
}

// scripts/logic/humanMobs/death.ts
function onHumanMobDie(entity, damage, cause, lootData, damagingEntity) {
  if (mc50.world.gameRules.doMobLoot) {
    for (const itemStack of lootData.getItemStacks()) {
      entity.dimension.spawnItem(itemStack, entity.location);
    }
  }
  const isDamageBig = damage > Math.min(30, entity.getComponent("health").effectiveMax);
  const isExplosionDamage = [mc50.EntityDamageCause.entityExplosion, mc50.EntityDamageCause.blockExplosion].includes(cause);
  const shouldGoreExplode = damagingEntity?.typeId !== SCP173_ENTITY_TYPE && (isDamageBig || isExplosionDamage || damagingEntity?.typeId === SCP096_ENTITY_TYPE);
  if (shouldGoreExplode) {
    spawnGoreExplosion(entity.dimension, add(entity.location, UP));
    entity.remove();
    return;
  }
  entity.triggerEvent("human:turn_into_corpse");
}
mc50.world.afterEvents.entityHurt.subscribe(
  (event) => {
    if (event.damageSource.cause === mc50.EntityDamageCause.selfDestruct) return;
    if (!event.hurtEntity.isValid) return;
    const lootData = getHumanMobLootData(event.hurtEntity.typeId);
    if (!lootData) return;
    if (!isEntityDead(event.hurtEntity)) return;
    onHumanMobDie(
      event.hurtEntity,
      event.damage,
      event.damageSource.cause,
      lootData,
      event.damageSource.damagingEntity
    );
  },
  {
    entityTypes: HUMAN_MOB_TYPE_ARRAY
  }
);
mc50.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    spawnGoreExplosion(event.entity.dimension, event.entity.location);
    event.entity.remove();
  },
  {
    entityTypes: HUMAN_MOB_TYPE_ARRAY,
    eventTypes: ["human:gib"]
  }
);

// scripts/logic/humanMobs/humanTypes/f/mtf/epsilon11.ts
import * as mc51 from "@minecraft/server";
var PROP_ID = {
  state: "lc:state",
  canDeployFragGrenade: "lc:can_deploy_frag_grenade"
};
function updateCombatMode(mtfEntity, target) {
  const previousState = mtfEntity.getProperty(PROP_ID.state);
  switch (previousState) {
    case 1:
      mtfEntity.triggerEvent("f_mtf_epsilon11:remove_rifle_combat");
      break;
    case 2:
      mtfEntity.triggerEvent("f_mtf_epsilon11:remove_knife_stabbing");
      break;
    case 3:
      mtfEntity.triggerEvent("f_mtf_epsilon11:remove_frag_grenade_deploying");
      break;
    case 4:
      mtfEntity.triggerEvent("f_mtf_epsilon11:remove_gonna_hide_scp096_face");
      break;
    case 5:
      mtfEntity.triggerEvent("f_mtf_epsilon11:remove_hiding_scp096_face");
      break;
  }
  switch (target.typeId) {
    default:
      mtfEntity.triggerEvent("f_mtf_epsilon11:add_rifle_combat");
      mtfEntity.setProperty(PROP_ID.state, 1);
      break;
    case SCP096_ENTITY_TYPE:
      mtfEntity.triggerEvent("f_mtf_epsilon11:add_gonna_hide_scp096_face");
      mtfEntity.setProperty(PROP_ID.state, 4);
      break;
  }
}
function onTargetEscape(mtfEntity) {
  const previousState = mtfEntity.getProperty(PROP_ID.state);
  switch (previousState) {
    case 1:
      mtfEntity.triggerEvent("f_mtf_epsilon11:remove_rifle_combat");
      break;
    case 2:
      mtfEntity.triggerEvent("f_mtf_epsilon11:remove_knife_stabbing");
      break;
    case 3:
      mtfEntity.triggerEvent("f_mtf_epsilon11:remove_frag_grenade_deploying");
      break;
    case 4:
      mtfEntity.triggerEvent("f_mtf_epsilon11:remove_gonna_hide_scp096_face");
      break;
    case 5:
      mtfEntity.triggerEvent("f_mtf_epsilon11:remove_hiding_scp096_face");
      break;
  }
  mtfEntity.setProperty(PROP_ID.state, 0);
  mtfEntity.setProperty(PROP_ID.canDeployFragGrenade, true);
}
function onTargetInsideKnifeRange(mtfEntity, target) {
  mtfEntity.triggerEvent("f_mtf_epsilon11:remove_rifle_combat");
  mtfEntity.triggerEvent("f_mtf_epsilon11:add_knife_stabbing");
  mtfEntity.setProperty(PROP_ID.state, 2);
}
function knifeStab(mtfEntity) {
  const target = mtfEntity.target;
  if (!target) return;
  const targetDist = distance(mtfEntity.location, target.location);
  if (targetDist > 3) return;
  target.applyDamage(6, {
    cause: mc51.EntityDamageCause.override,
    damagingEntity: mtfEntity
  });
  target.addEffect("slowness", 40, {
    amplifier: 0
  });
}
function onFinishKnifeStabbing(mtfEntity) {
  if (!mtfEntity.target) return;
  const lastFragGrenadeDeployDate = ensureType(mtfEntity.getDynamicProperty("lastFragGrenadeDeployDate"), "number") ?? 0;
  const dateNow = Date.now();
  mtfEntity.triggerEvent("f_mtf_epsilon11:remove_knife_stabbing");
  if (dateNow - lastFragGrenadeDeployDate > 5e3 && Math.random() > 0.4) {
    mtfEntity.triggerEvent("f_mtf_epsilon11:add_frag_grenade_deploying");
    mtfEntity.setProperty(PROP_ID.state, 3);
    mtfEntity.setDynamicProperty("lastFragGrenadeDeployDate", Date.now());
  } else {
    mtfEntity.triggerEvent("f_mtf_epsilon11:add_rifle_combat");
    mtfEntity.setProperty(PROP_ID.state, 1);
  }
}
function deployFragGrenade(mtfEntity) {
  const canDeployFragGrenade = mtfEntity.getProperty(PROP_ID.canDeployFragGrenade) === true;
  if (!canDeployFragGrenade) return;
  mtfEntity.setProperty(PROP_ID.canDeployFragGrenade, false);
  mtfEntity.dimension.spawnEntity("lc:scpdy_mtf_frag_grenade", mtfEntity.location);
}
function onFinishDeployingFragGrenade(mtfEntity) {
  mtfEntity.setProperty(PROP_ID.canDeployFragGrenade, true);
  mtfEntity.triggerEvent("f_mtf_epsilon11:remove_frag_grenade_deploying");
  if (!mtfEntity.target) return;
  mtfEntity.triggerEvent("f_mtf_epsilon11:add_rifle_combat");
  mtfEntity.setProperty(PROP_ID.state, 1);
}
function onTargetInsideScp096FhRange(mtfEntity) {
  const target = mtfEntity.target;
  if (!target) return;
  if (target.typeId !== SCP096_ENTITY_TYPE) return;
  const isFaceHidden = target.getProperty("lc:is_face_hidden") === true;
  if (isFaceHidden) {
    mtfEntity.triggerEvent("f_mtf_epsilon11:remove_hiding_scp096_face");
    mtfEntity.setProperty(PROP_ID.state, 0);
    return;
  }
  mtfEntity.triggerEvent("f_mtf_epsilon11:remove_gonna_hide_scp096_face");
  mtfEntity.triggerEvent("f_mtf_epsilon11:add_hiding_scp096_face");
  mtfEntity.setProperty(PROP_ID.state, 5);
}
function hideScp096Face(mtfEntity) {
  const target = mtfEntity.target;
  if (!target) return;
  if (target.typeId !== SCP096_ENTITY_TYPE) return;
  const isFaceHidden = target.getProperty("lc:is_face_hidden") === true;
  if (isFaceHidden) {
    mtfEntity.triggerEvent("f_mtf_epsilon11:remove_hiding_scp096_face");
    mtfEntity.setProperty(PROP_ID.state, 0);
    return;
  }
  const targetDist = distance(mtfEntity.location, target.location);
  if (targetDist > 2.6) {
    mtfEntity.triggerEvent("f_mtf_epsilon11:remove_hiding_scp096_face");
    mtfEntity.triggerEvent("f_mtf_epsilon11:add_gonna_hide_scp096_face");
    mtfEntity.setProperty(PROP_ID.state, 4);
    return;
  }
  target.triggerEvent("scp096:equip_paper_bag");
  mtfEntity.triggerEvent("f_mtf_epsilon11:remove_hiding_scp096_face");
  mtfEntity.setProperty(PROP_ID.state, 0);
}
function onEntityEventTrigger(mtfEntity, eventId) {
  if (!mtfEntity.isValid) return;
  const isDead = isEntityDead(mtfEntity);
  switch (eventId) {
    case "f_mtf_epsilon11:update_combat_mode":
      if (!isDead && mtfEntity.target) {
        updateCombatMode(mtfEntity, mtfEntity.target);
      }
      break;
    case "f_mtf_epsilon11:on_target_escape":
      if (!isDead) {
        onTargetEscape(mtfEntity);
      }
      break;
    case "f_mtf_epsilon11:on_target_inside_knife_range":
      if (mtfEntity.target) {
        onTargetInsideKnifeRange(mtfEntity, mtfEntity.target);
      }
      break;
    case "f_mtf_epsilon11:knife_stab":
      knifeStab(mtfEntity);
      break;
    case "f_mtf_epsilon11:on_finish_knife_stabbing":
      onFinishKnifeStabbing(mtfEntity);
      break;
    case "f_mtf_epsilon11:deploy_frag_grenade":
      deployFragGrenade(mtfEntity);
      break;
    case "f_mtf_epsilon11:on_finish_deploying_frag_grenade":
      onFinishDeployingFragGrenade(mtfEntity);
      break;
    case "f_mtf_epsilon11:on_target_inside_scp096_fh_range":
      onTargetInsideScp096FhRange(mtfEntity);
      break;
    case "f_mtf_epsilon11:hide_scp096_face":
      hideScp096Face(mtfEntity);
      break;
  }
}
mc51.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    onEntityEventTrigger(event.entity, event.eventId);
  },
  {
    entityTypes: [HUMAN_MOB_TYPES.f_mtf_epsilon11]
  }
);

// scripts/logic/scps/scp001_prototype/index.ts
import * as mc52 from "@minecraft/server";
var SCP001P_ENTITY_TYPE = "lc:scpdy_scp001_proto";
var SCP001P_SINGULARITY_ENTITY_TYPE = "lc:scpdy_scp001_proto_singularity";
function testLeadBlockingRay(dimension, from, dir) {
  return dimension.getBlockFromRay(from, dir, {
    includeTypes: ["lc:scpdy_cst_lead_block"],
    maxDistance: 128
  }) !== void 0;
}
function onUpdate(scp001Entity) {
  const isWeakened = scp001Entity.getProperty("lc:is_weakened") === true;
  const target = scp001Entity.target;
  if (!target || isWeakened) {
    scp001Entity.setProperty("lc:is_mouth_opened", false);
    return;
  }
  const dirToTarget = normalize(sub(scp001Entity.target.location, scp001Entity.location));
  const isLeadBlockingRay = testLeadBlockingRay(scp001Entity.dimension, scp001Entity.location, dirToTarget);
  scp001Entity.setProperty("lc:is_mouth_opened", !isLeadBlockingRay && !isWeakened);
  const ticksUntilSigularity = ensureType(scp001Entity.getDynamicProperty("ticksUntilSingularity"), "number") ?? 0;
  scp001Entity.setDynamicProperty(
    "ticksUntilSingularity",
    ticksUntilSigularity > 0 ? ticksUntilSigularity - 1 : target.isOnGround ? randomInt(20, 30) : randomInt(5, 10)
  );
  if (isLeadBlockingRay) return;
  if (ticksUntilSigularity > 0) return;
  const sigularityLoc = target.location;
  scp001Entity.tryTeleport(sigularityLoc, {
    keepVelocity: false,
    dimension: target.dimension,
    facingLocation: target.location
  });
  scp001Entity.dimension.playSound("scpdy.scp001_proto.jumpscare", sigularityLoc, {
    volume: 1.5
  });
  scp001Entity.dimension.playSound("scpdy.scp001_proto.singularity", sigularityLoc, {
    volume: 2,
    pitch: randomFloat(0.9, 1.1)
  });
  scp001Entity.dimension.spawnEntity(SCP001P_SINGULARITY_ENTITY_TYPE, add(target.location, UP));
  target.applyDamage(12, {
    cause: mc52.EntityDamageCause.fall
  });
}
function onAttack(scp001Entity) {
  const isWeakened = scp001Entity.getProperty("lc:is_weakened") === true;
  let attackLoc;
  if (scp001Entity.target) {
    attackLoc = add(
      scp001Entity.getHeadLocation(),
      normalize(sub(scp001Entity.target.location, scp001Entity.location))
    );
  } else {
    attackLoc = add(scp001Entity.getHeadLocation(), scp001Entity.getViewDirection());
  }
  const entitiesToAttack = scp001Entity.dimension.getEntities({
    closest: isWeakened ? 3 : 10,
    location: attackLoc,
    maxDistance: 2.7,
    excludeFamilies: ["inanimate", "scp001_proto"]
  });
  if (entitiesToAttack.length <= 0) {
    scp001Entity.dimension.playSound("scpdy.misc.woosh", attackLoc);
    return;
  }
  scp001Entity.dimension.playSound("scpdy.misc.monster.attack", attackLoc, {
    pitch: randomFloat(0.9, 1.1),
    volume: 1.4
  });
  for (let i = 0; i < entitiesToAttack.length; i++) {
    const entity = entitiesToAttack[i];
    let damage = isWeakened ? 6 : 9;
    damage = getModifiedDamageNumber(damage, entity);
    const appliedDamage = entity.applyDamage(damage, {
      cause: mc52.EntityDamageCause.override,
      damagingEntity: scp001Entity
    });
    if (appliedDamage && i === 0) {
      entity.dimension.spawnParticle("minecraft:critical_hit_emitter", attackLoc);
    }
  }
}
function onSingularityInhale(singularityEntity) {
  const inhaleLives = singularityEntity.getProperty("lc:inhale_lives");
  if (inhaleLives === 0) {
    singularityEntity.remove();
    return;
  }
  singularityEntity.setProperty("lc:inhale_lives", inhaleLives - 1);
  const origin = singularityEntity.location;
  const entitiesToInhale = singularityEntity.dimension.getEntities({
    maxDistance: 18,
    closest: 15,
    location: origin,
    excludeFamilies: ["blood", "scp001_proto", "scp001_proto_singularity"]
  });
  for (let i = 0; i < entitiesToInhale.length; i++) {
    const entity = entitiesToInhale[i];
    if (entity.typeId === "minecraft:ender_dragon") return;
    const dist = distance(entity.location, origin);
    const dir = sub(origin, entity.location);
    const dirNormalized = normalize(dir);
    const force = mul(dirNormalized, 1.2 - dist / 18);
    if (inhaleLives === 1) {
      entity.addEffect("poison", 200, {
        amplifier: 1
      });
    }
    if (entity instanceof mc52.Player) {
      entity.runCommand("camerashake add @s 0.2 0.3 positional");
      if ([mc52.GameMode.Creative, mc52.GameMode.Spectator].includes(entity.getGameMode())) continue;
    }
    entity.applyImpulse(force);
  }
}
mc52.system.afterEvents.scriptEventReceive.subscribe((event) => {
  if (!event.sourceEntity) return;
  switch (event.sourceEntity.typeId) {
    case SCP001P_ENTITY_TYPE:
      if (event.id === "scpdy:scp001_proto_script_update") {
        onUpdate(event.sourceEntity);
      }
      break;
    case SCP001P_SINGULARITY_ENTITY_TYPE:
      if (event.id === "scpdy:scp001_proto_singularity_inhale") {
        onSingularityInhale(event.sourceEntity);
      }
      break;
  }
});
mc52.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    onAttack(event.entity);
  },
  {
    entityTypes: [SCP001P_ENTITY_TYPE],
    eventTypes: ["scp001_proto:on_attack"]
  }
);
mc52.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    event.entity.getComponent("health").setCurrentValue(100);
  },
  {
    entityTypes: [SCP001P_ENTITY_TYPE],
    eventTypes: ["scp001_proto:get_back_from_neutralized"]
  }
);
mc52.world.afterEvents.entitySpawn.subscribe((event) => {
  if (event.entity.typeId === SCP001P_ENTITY_TYPE) {
    event.entity.setDynamicProperty("ticksUntilSingularity", 20);
  } else if (event.entity.typeId === SCP001P_SINGULARITY_ENTITY_TYPE) {
    event.entity.dimension.spawnParticle("lc:scpdy_singularity_energy_emitter", event.entity.location);
  }
});
mc52.world.afterEvents.entityHurt.subscribe(
  (event) => {
    const ticksUntilSigularity = ensureType(event.hurtEntity.getDynamicProperty("ticksUntilSingularity"), "number") ?? 0;
    if (ticksUntilSigularity <= 6) return;
    event.hurtEntity.setDynamicProperty(
      "ticksUntilSingularity",
      Math.max(6, ticksUntilSigularity - Math.max(5, Math.floor(event.damage * 0.5)))
    );
  },
  {
    entityTypes: [SCP001P_ENTITY_TYPE]
  }
);
mc52.world.afterEvents.entityDie.subscribe((event) => {
  const scp001Entity = event.damageSource.damagingEntity;
  if (!scp001Entity) return;
  if (!scp001Entity.isValid) return;
  if (scp001Entity.typeId !== SCP001P_ENTITY_TYPE) return;
  if (!event.deadEntity.isValid) return;
  const deadEntityHealthComp = event.deadEntity.getComponent("health");
  if (!deadEntityHealthComp) return;
  const scp001HealthComp = scp001Entity.getComponent("health");
  const healAmount = scp001HealthComp.currentValue + Math.min(2, Math.floor(deadEntityHealthComp.effectiveMax / 20));
  const newHealth = Math.min(scp001HealthComp.effectiveMax, healAmount);
  scp001HealthComp.setCurrentValue(newHealth);
  const molangVarMap = new mc52.MolangVariableMap();
  molangVarMap.setFloat("particle_amount", Math.min(1, Math.max(5, Math.floor(healAmount * 0.5))));
  scp001Entity.dimension.spawnParticle("lc:scpdy_heal_emitter", scp001Entity.location, molangVarMap);
});

// scripts/logic/scps/scp096/scp096.ts
import * as mc53 from "@minecraft/server";
function onUpdateScp096(scp096Entity) {
  const scp096State = scp096Entity.getProperty("lc:state");
  const healthComp = scp096Entity.getComponent("health");
  const target = scp096Entity.target;
  scp096Entity.addEffect("speed", 20, {
    amplifier: scp096State === 2 ? 23 : 18,
    showParticles: false
  });
  if (healthComp.currentValue < healthComp.effectiveMax - 700) {
    healthComp.setCurrentValue(healthComp.effectiveMax - 699);
  } else if (healthComp.currentValue < healthComp.effectiveMax) {
    const healAmount = scp096State === 0 ? 4 : 1;
    const newHealth = Math.floor(healthComp.currentValue + healAmount);
    healthComp.setCurrentValue(newHealth);
  }
  if (!target) return;
  target.addTag(SCP096_1_TAG);
  if (scp096State !== 2) return;
  const dirToTarget = sub(target.location, scp096Entity.location);
  let yRotation = Math.atan2(dirToTarget.x, dirToTarget.z) * (180 / Math.PI);
  if (yRotation > 180) {
    yRotation -= 360;
  } else if (yRotation < -180) {
    yRotation += 360;
  }
  scp096Entity.setProperty("lc:chase_body_y_rotation", -yRotation);
  const distanceBetweenTarget = distance(scp096Entity.location, target.location);
  if (distanceBetweenTarget < 2.9 && target.typeId === "minecraft:wither") {
    target.applyDamage(33, {
      cause: mc53.EntityDamageCause.entityAttack,
      damagingEntity: scp096Entity
    });
    onScp096HitWither(scp096Entity, target);
  }
  if (CONFIG.disableAdvanced096Movement) return;
  let ticksUntilNextLocCheck = ensureType(scp096Entity.getDynamicProperty("ticksUntilNextLocCheck"), "number");
  if (ticksUntilNextLocCheck === void 0) {
    ticksUntilNextLocCheck = 2;
  }
  if (ticksUntilNextLocCheck <= 0) {
    const lastCheckedLoc = ensureType(scp096Entity.getDynamicProperty("lastCheckedLoc"), "Vector3");
    if (lastCheckedLoc) {
      const dist = distance(
        {
          x: scp096Entity.location.x,
          y: 0,
          z: scp096Entity.location.z
        },
        {
          x: lastCheckedLoc.x,
          y: 0,
          z: lastCheckedLoc.z
        }
      );
      const detectedStuck = dist < 0.8;
      if (detectedStuck) {
        onDetectedScp096Stuck(scp096Entity);
      }
    }
    scp096Entity.setDynamicProperty("lastCheckedLoc", scp096Entity.location);
    ticksUntilNextLocCheck = randomInt(1, 3);
  }
  scp096Entity.setDynamicProperty("ticksUntilNextLocCheck", ticksUntilNextLocCheck - 1);
}
function onDetectedScp096Stuck(scp096Entity) {
  const dirToTarget = normalize(sub(scp096Entity.target.location, add(scp096Entity.location, UP)));
  const isNearInXZ = distance(
    {
      x: scp096Entity.location.x,
      y: 0,
      z: scp096Entity.location.z
    },
    {
      x: scp096Entity.target.location.x,
      y: 0,
      z: scp096Entity.target.location.z
    }
  ) < 10;
  if (scp096Entity.location.y > scp096Entity.target.location.y - 3) {
    if (mc53.world.gameRules.mobGriefing) {
      scp096Entity.runCommand("setblock ~ ~-1 ~ air destroy");
      scp096Entity.runCommand("setblock ~1 ~-1 ~ air destroy");
      scp096Entity.runCommand("setblock ~-1 ~-1 ~ air destroy");
      scp096Entity.runCommand("setblock ~ ~-1 ~1 air destroy");
      scp096Entity.runCommand("setblock ~ ~-1 ~-1 air destroy");
      scp096Entity.runCommand("setblock ~1 ~-1 ~-1 air destroy");
      scp096Entity.runCommand("setblock ~-1 ~-1 ~1 air destroy");
      scp096Entity.runCommand("setblock ~-1 ~-1 ~1 air destroy");
      scp096Entity.runCommand("setblock ~1 ~-1 ~-1 air destroy");
    }
  } else if (scp096Entity.location.y < scp096Entity.target.location.y + 3) {
    if (isNearInXZ && scp096Entity.location.y < 350) {
      mc53.system.run(() => {
        scp096Entity.addEffect("levitation", 5, {
          amplifier: 17,
          showParticles: false
        });
        scp096Entity.addEffect("slow_falling", 20, {
          amplifier: 1,
          showParticles: false
        });
      });
    }
    if (mc53.world.gameRules.mobGriefing) {
      const raycastHit = scp096Entity.dimension.getBlockFromRay(scp096Entity.location, dirToTarget, {
        maxDistance: 10
      });
      if (raycastHit) {
        const locStr = toString2(raycastHit.block.center());
        const locStr2 = toString2(add(raycastHit.block.center(), UP));
        scp096Entity.runCommand(`setblock ${locStr} air destroy`);
        scp096Entity.runCommand(`setblock ${locStr2} air destroy`);
      }
      mc53.system.run(() => {
        scp096Entity.runCommand("setblock ~ ~2 ~ air destroy");
        scp096Entity.runCommand("setblock ~ ~3 ~ air destroy");
        scp096Entity.runCommand("setblock ~1 ~2 ~ air destroy");
        scp096Entity.runCommand("setblock ~-1 ~2 ~ air destroy");
        scp096Entity.runCommand("setblock ~ ~1 ~2 air destroy");
        scp096Entity.runCommand("setblock ~ ~1 ~-2 air destroy");
        scp096Entity.runCommand("setblock ~1 ~2 ~-1 air destroy");
        scp096Entity.runCommand("setblock ~-1 ~2 ~1 air destroy");
        scp096Entity.runCommand("setblock ~-1 ~2 ~1 air destroy");
        scp096Entity.runCommand("setblock ~1 ~2 ~-1 air destroy");
      });
    }
  }
  if (mc53.world.gameRules.mobGriefing) {
    scp096Entity.runCommand("setblock ~ ~ ~ air destroy");
    scp096Entity.runCommand("setblock ~ ~1 ~ air destroy");
    scp096Entity.runCommand("setblock ~1 ~ ~ air destroy");
    scp096Entity.runCommand("setblock ~1 ~1 ~ air destroy");
    scp096Entity.runCommand("setblock ~-1 ~ ~ air destroy");
    scp096Entity.runCommand("setblock ~-1 ~1 ~ air destroy");
    scp096Entity.runCommand("setblock ~ ~ ~1 air destroy");
    scp096Entity.runCommand("setblock ~ ~1 ~1 air destroy");
    scp096Entity.runCommand("setblock ~ ~ ~-1 air destroy");
    scp096Entity.runCommand("setblock ~ ~1 ~-1 air destroy");
  }
  scp096Entity.dimension.spawnParticle("minecraft:wind_explosion_emitter", scp096Entity.location);
  const smokeParticleVarMap = new mc53.MolangVariableMap();
  smokeParticleVarMap.setVector3("direction", dirToTarget);
  scp096Entity.dimension.spawnParticle(
    "lc:scpdy_scp096_leap_smoke_emitter",
    scp096Entity.location,
    smokeParticleVarMap
  );
  scp096Entity.applyImpulse(changeDir(mul(FORWARD, 2), dirToTarget));
}
function onScp096HitWither(scp096Entity, wither) {
  const dimension = scp096Entity.dimension;
  const raycastHit = dimension.getBlockFromRay(
    wither.getHeadLocation(),
    {
      x: 0,
      y: -1,
      z: 0
    },
    {
      maxDistance: 11
    }
  );
  let witherStunLocation;
  if (!raycastHit) {
    witherStunLocation = {
      x: wither.location.x,
      y: wither.location.y - 9,
      z: wither.location.z
    };
  } else {
    witherStunLocation = add(raycastHit.block.bottomCenter(), UP);
  }
  wither.teleport(witherStunLocation);
  let ticksUntilStunEnd = 20;
  const stunRunId = mc53.system.runInterval(() => {
    try {
      if (ticksUntilStunEnd <= 0) {
        mc53.system.clearRun(stunRunId);
        return;
      }
      wither.teleport(witherStunLocation);
      ticksUntilStunEnd -= 1;
    } catch {
      mc53.system.clearRun(stunRunId);
    }
  }, 1);
}
function emitScp096AttackParticle(dimension, location, playSound) {
  dimension.spawnParticle("minecraft:critical_hit_emitter", add(location, UP));
  if (!playSound) return;
  dimension.playSound("scpdy.scp096.attack", location, {
    volume: 8
  });
}
function onScp096Attack(scp096Entity) {
  const target = scp096Entity.target;
  if (!target) return;
  const targetDist = distance(target.location, scp096Entity.location);
  if (targetDist > 7) return;
  if (target.isOnGround) {
    scp096Entity.setDynamicProperty("ticksUntilNextLocCheck", 13);
  }
  emitScp096AttackParticle(target.dimension, target.location, true);
  target.getComponent("rideable")?.ejectRiders();
  const targetHealthComp = target.getComponent("health");
  let targetHealthCurrent = 1;
  let targetHealthMax = 2;
  if (targetHealthComp) {
    targetHealthCurrent = targetHealthComp.currentValue;
    targetHealthMax = targetHealthComp.effectiveMax;
  }
  const targetHealthBeforeAttack = targetHealthCurrent;
  const dmg = Math.floor(Math.max(34, targetHealthMax / randomFloat(10, 12)));
  let damagedTarget = false;
  damagedTarget = target.applyDamage(dmg, {
    cause: mc53.EntityDamageCause.override,
    damagingEntity: scp096Entity
  });
  if (!damagedTarget) {
    damagedTarget = targetHealthCurrent < targetHealthBeforeAttack;
  }
  if (target.typeId === "minecraft:wither") {
    onScp096HitWither(target, target);
    return;
  }
  if (!damagedTarget) {
    let explode3 = function(entity) {
      entity.dimension.playSound("scpdy.gore.explode", entity.location, { volume: 1.6 });
      entity.dimension.spawnParticle("lc:scpdy_blood_splash_emitter", particleLoc);
      entity.dimension.spawnParticle("lc:scpdy_body_explosion_particle", particleLoc);
      if (entity.typeId === "minecraft:ender_dragon") {
        entity.kill();
      } else {
        entity.remove();
      }
    };
    var explode2 = explode3;
    const particleLoc = {
      x: target.location.x,
      y: target.location.y + 0.7,
      z: target.location.z
    };
    const attackFailCount = ensureType(target.getDynamicProperty("scp096AttackFailCount"), "number") ?? 0;
    if (attackFailCount > 69) {
      explode3(target);
      return;
    }
    target.setDynamicProperty("scp096AttackFailCount", attackFailCount + 1);
  }
}
function onScp096Die(oldScp096Entity, damageSource) {
  if (damageSource.cause === mc53.EntityDamageCause.selfDestruct) return;
  if (damageSource.damagingEntity?.typeId === SCP173_ENTITY_TYPE) return;
  const dimension = oldScp096Entity.dimension;
  const location = oldScp096Entity.location;
  const velocity = oldScp096Entity.getVelocity();
  const hadTarget = oldScp096Entity.target !== void 0;
  const oldHealthComp = oldScp096Entity.getComponent("health");
  const newHealthWhenRevive = oldHealthComp.effectiveMax - 1e3;
  const wasFaceHidden = oldScp096Entity.getProperty("lc:is_face_hidden") === true;
  oldScp096Entity.remove();
  let newScp096Type = SCP096_ENTITY_TYPE;
  if (hadTarget) {
    newScp096Type += "<scp096:spawn_angry>";
  }
  const newScp096Entity = dimension.spawnEntity(newScp096Type, location);
  newScp096Entity.applyImpulse(velocity);
  newScp096Entity.getComponent("health").setCurrentValue(newHealthWhenRevive);
  newScp096Entity.setProperty("lc:is_face_hidden", wasFaceHidden);
  if (damageSource.damagingEntity) {
    const damagerDist = distance(damageSource.damagingEntity.location, oldScp096Entity.location);
    if (damagerDist < 5) {
      damageSource.damagingEntity.applyDamage(randomInt(350, 400), {
        cause: mc53.EntityDamageCause.entityAttack,
        damagingEntity: newScp096Entity
      });
    }
  }
}
function isCreativeOrSpectator(entity) {
  return entity instanceof mc53.Player && [mc53.GameMode.Creative, mc53.GameMode.Spectator].includes(entity.getGameMode());
}
mc53.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    event.entity.dimension.playSound("scpdy.scp096.triggered_scream", event.entity.location, {
      volume: 10
    });
  },
  {
    entityTypes: [SCP096_ENTITY_TYPE],
    eventTypes: ["scp096:target_acquired_while_calm"]
  }
);
mc53.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    onScp096Attack(event.entity);
  },
  {
    entityTypes: [SCP096_ENTITY_TYPE],
    eventTypes: ["scp096:on_attack"]
  }
);
mc53.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    event.entity.remove();
  },
  {
    entityTypes: [SCP096_ENTITY_TYPE],
    eventTypes: ["scp096:force_remove"]
  }
);
mc53.system.afterEvents.scriptEventReceive.subscribe(
  (event) => {
    if (event.id !== "scpdy:scp096_update") return;
    if (!event.sourceEntity) return;
    if (event.sourceEntity.typeId !== SCP096_ENTITY_TYPE) return;
    onUpdateScp096(event.sourceEntity);
  },
  {
    namespaces: ["scpdy"]
  }
);
mc53.world.afterEvents.entityDie.subscribe(
  (event) => {
    onScp096Die(event.deadEntity, event.damageSource);
  },
  {
    entityTypes: [SCP096_ENTITY_TYPE]
  }
);
mc53.world.afterEvents.entityDie.subscribe(
  (event) => {
    event.deadEntity.removeTag(SCP096_1_TAG);
  },
  {
    entityTypes: ["minecraft:player"]
  }
);
mc53.world.afterEvents.entityHurt.subscribe(
  (event) => {
    if (!event.damageSource.damagingEntity) return;
    if (isCreativeOrSpectator(event.damageSource.damagingEntity)) return;
    event.damageSource.damagingEntity.addTag(SCP096_1_TAG);
  },
  {
    entityTypes: [SCP096_ENTITY_TYPE]
  }
);
mc53.world.afterEvents.playerGameModeChange.subscribe((event) => {
  event.player.removeTag(SCP096_1_TAG);
});

// scripts/logic/scps/scp096/wand.ts
import * as mc54 from "@minecraft/server";
function onUse4(arg) {
  const player = arg.source;
  const raycastHit = player.getEntitiesFromViewDirection({
    maxDistance: 100
  })[0];
  if (!raycastHit) {
    player.playSound("note.bass");
    player.onScreenDisplay.setActionBar({
      translate: "scpdy.actionHint.scp096_1_wand.noValidEntity"
    });
    return;
  }
  if (raycastHit.entity.addTag(SCP096_1_TAG)) {
    player.playSound("note.hat");
    player.onScreenDisplay.setActionBar({
      translate: "scpdy.actionHint.scp096_1_wand.taggedEntity",
      with: {
        rawtext: [getEntityName(raycastHit.entity)]
      }
    });
  } else {
    player.playSound("note.bass");
    player.onScreenDisplay.setActionBar({
      translate: "scpdy.actionHint.scp096_1_wand.entityAlreadyTagged"
    });
  }
}
mc54.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:scp096_1_wand", {
    onUse: onUse4
  });
});

// scripts/logic/scps/scp106/corrosionProjectile.ts
import * as mc56 from "@minecraft/server";

// scripts/logic/scps/scp106/shared.ts
import "@minecraft/server";
var SCP106_ENTITY_TYPE_ID = "lc:scpdy_scp106";
var SCP106_TRAIL_ENTITY_TYPE_ID = "lc:scpdy_scp106_trail";
var SCP106_STATE = {
  default: 0,
  diving: 10,
  hidden: 20,
  emergingSlow: 30,
  emergingFast: 40,
  throwingRight: 50,
  throwingLeft: 60,
  retreating: 100
};
function calculateCombatEmergeLocation(scp106, targetArg) {
  const target = targetArg ?? scp106.target;
  if (!target) {
    const block = scp106.dimension.getTopmostBlock({
      x: scp106.location.x,
      z: scp106.location.z
    });
    if (!block) return { x: scp106.location.x, y: scp106.location.y + 0.6, z: scp106.location.z };
    return block.above()?.bottomCenter() ?? scp106.location;
  }
  const test = (loc) => {
    try {
      const block1 = scp106.dimension.getBlockBelow(loc, { maxDistance: 5 });
      if (!block1) return;
      if (!block1.isSolid) return;
      const block2 = block1.above();
      if (!block2) return;
      if (!block2.isAir) return;
      return block2.bottomCenter();
    } catch {
    }
  };
  const targetLoc = target.location;
  return test(add(targetLoc, { x: 1, y: 0, z: 0 })) ?? test(add(targetLoc, { x: 0, y: 0, z: 1 })) ?? test(add(targetLoc, { x: -1, y: 0, z: 0 })) ?? test(add(targetLoc, { x: 0, y: 0, z: -1 })) ?? targetLoc;
}
function getState(scp106) {
  return Number(scp106.getProperty("lc:state"));
}
function setState(scp106, value) {
  scp106.setProperty("lc:state", value);
}
function getCorrosionRight(scp106) {
  return scp106.getProperty("lc:corrosion_right") === true;
}
function setCorrosionRight(scp106, value) {
  scp106.setProperty("lc:corrosion_right", value);
}
function getCorrosionLeft(scp106) {
  return scp106.getProperty("lc:corrosion_left") === true;
}
function setCorrosionLeft(scp106, value) {
  scp106.setProperty("lc:corrosion_left", value);
}
function getCorrosionAcquisitionCooldown(scp106) {
  const value = Number(scp106.getDynamicProperty("corrosionAcquisitionCooldown"));
  return isNaN(value) ? 0 : value;
}
function setCorrosionAcquisitionCooldown(scp106, value) {
  scp106.setDynamicProperty("corrosionAcquisitionCooldown", value);
}
function getCorrosionThrowCooldown(scp106) {
  const value = Number(scp106.getDynamicProperty("corrosionThrowCooldown"));
  return isNaN(value) ? 0 : value;
}
function setCorrosionThrowCooldown(scp106, value) {
  scp106.setDynamicProperty("corrosionThrowCooldown", value);
}
function getLastLocation(scp106) {
  return scp106.getDynamicProperty("lastLocation");
}
function setLastLocation(scp106, value) {
  scp106.setDynamicProperty("lastLocation", value);
}
function getStuckDuration(scp106) {
  const value = Number(scp106.getDynamicProperty("stuckDuration"));
  return isNaN(value) ? 0 : value;
}
function setStuckDuration(scp106, value) {
  scp106.setDynamicProperty("stuckDuration", value);
}
function getHideContext(scp106) {
  return scp106.getDynamicProperty("hideContext");
}
function setHideContext(scp106, value) {
  scp106.setDynamicProperty("hideContext", value);
}
function getHidingTick(scp106) {
  const value = Number(scp106.getDynamicProperty("hidingTick"));
  return isNaN(value) ? 0 : value;
}
function setHidingTick(scp106, value) {
  scp106.setDynamicProperty("hidingTick", value);
}

// scripts/logic/scps/scp106/corrosionProjectile.ts
var CORROSION_PROJECTILE_ENTITY_TYPE_ID = "lc:scpdy_corrosion_projectile";
mc56.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  ({ entity }) => {
    try {
      explode(entity, entity.location);
    } catch {
    }
  },
  {
    entityTypes: [CORROSION_PROJECTILE_ENTITY_TYPE_ID],
    eventTypes: ["bom"]
  }
);
mc56.world.afterEvents.projectileHitEntity.subscribe((event) => {
  if (event.projectile.typeId !== CORROSION_PROJECTILE_ENTITY_TYPE_ID) return;
  if (!event.projectile.isValid) return;
  try {
    explode(event.projectile, event.location, event.getEntityHit().entity, event.source);
  } catch {
  }
});
mc56.world.afterEvents.projectileHitBlock.subscribe((event) => {
  if (event.projectile.typeId !== CORROSION_PROJECTILE_ENTITY_TYPE_ID) return;
  if (!event.projectile.isValid) return;
  try {
    explode(event.projectile, event.location);
  } catch {
  }
});
function explode(projectile, location, hitEntity, source) {
  projectile.dimension.spawnParticle("lc:scpdy_corrosion_burst_emitter", location);
  try {
    const damage = mc56.world.getDifficulty() === mc56.Difficulty.Hard ? 9 : 6;
    if (hitEntity && hitEntity.typeId !== SCP106_ENTITY_TYPE_ID) {
      hitEntity.applyDamage(damage, {
        cause: mc56.EntityDamageCause.override,
        damagingProjectile: projectile,
        damagingEntity: source
      });
      hitEntity.addEffect("blindness", 140, { amplifier: 0 });
    }
  } catch {
  }
  const entities = projectile.dimension.getEntities({
    closest: 10,
    location,
    maxDistance: 2.4,
    excludeTypes: [SCP106_ENTITY_TYPE_ID, CORROSION_PROJECTILE_ENTITY_TYPE_ID]
  });
  for (let i = 0; i < entities.length; i++) {
    const entity = entities[i];
    try {
      entity.addEffect("wither", 100, { amplifier: 1 });
    } catch {
    }
  }
  projectile.remove();
}

// scripts/logic/scps/scp106/scp106.ts
import * as mc57 from "@minecraft/server";
mc57.system.afterEvents.scriptEventReceive.subscribe(
  (event) => {
    if (!event.sourceEntity) return;
    if (!event.sourceEntity.isValid) return;
    if (event.sourceEntity.typeId !== SCP106_ENTITY_TYPE_ID) return;
    const scp106 = event.sourceEntity;
    switch (event.id) {
      case "scpdy_scp106:update":
        onUpdate2(scp106);
        break;
      case "scpdy_scp106:finish_dive":
        onFinishDive(scp106);
        break;
      case "scpdy_scp106:finish_throw_right":
        onFinishThrowingCorrosionRight(scp106);
        break;
      case "scpdy_scp106:finish_throw_left":
        onFinishThrowingCorrosionLeft(scp106);
        break;
      case "scpdy_scp106:finish_emerge_slow":
        onFinishEmergingSlow(scp106);
        break;
      case "scpdy_scp106:finish_emerge_fast":
        onFinishEmergingFast(scp106);
        break;
      case "scpdy_scp106:finish_retreat":
        onFinishRetreatDive(scp106);
        break;
    }
  },
  {
    namespaces: ["scpdy_scp106"]
  }
);
mc57.world.afterEvents.entitySpawn.subscribe((event) => {
  if (event.entity.typeId !== SCP106_ENTITY_TYPE_ID) return;
  onSpawn(event.entity);
});
function onSpawn(scp106) {
  setCorrosionAcquisitionCooldown(scp106, 7);
  setCorrosionThrowCooldown(scp106, 7);
}
mc57.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  ({ entity }) => {
    onSpawnRetreating(entity);
  },
  {
    entityTypes: [SCP106_ENTITY_TYPE_ID],
    eventTypes: ["lc:spawn_retreating"]
  }
);
function onSpawnRetreating(scp106) {
  setHideContext(scp106, "retreat");
  scp106.removeTag("scp096_1");
  const healthComp = scp106.getComponent("health");
  healthComp.setCurrentValue(20);
}
mc57.world.afterEvents.entityDie.subscribe(
  ({ deadEntity }) => {
    try {
      deathExplosion(deadEntity);
    } catch {
    }
  },
  {
    entityTypes: [SCP106_ENTITY_TYPE_ID]
  }
);
function deathExplosion(scp106) {
  scp106.dimension.spawnParticle("lc:scpdy_corrosion_burst_emitter", sub(scp106.getHeadLocation(), UP));
  scp106.remove();
}
function onUpdate2(scp106) {
  const state = getState(scp106);
  if (state === SCP106_STATE.hidden) {
    onUpdateHiddenState(scp106);
    return;
  }
  const entitiesInContact = scp106.dimension.getEntities({
    closest: 20,
    maxDistance: 1.55,
    location: add(scp106.location, UP),
    excludeTypes: [SCP106_ENTITY_TYPE_ID]
  });
  for (const entity of entitiesInContact) {
    try {
      entity.addEffect("wither", 100, { amplifier: 1 });
    } catch {
    }
  }
  if (mc57.world.getDifficulty() === mc57.Difficulty.Hard) {
    scp106.addEffect("speed", 100, { amplifier: 0, showParticles: false });
  }
  if (state === SCP106_STATE.default) {
    onUpdateDefaultState(scp106);
  }
  if (scp106.isOnGround) {
    mc57.system.run(() => {
      scp106.dimension.spawnEntity(SCP106_TRAIL_ENTITY_TYPE_ID, scp106.location);
    });
  }
}
function onUpdateDefaultState(scp106) {
  if (!scp106.target) return;
  const isOnMagnet = isOnMagnetBlock(scp106);
  if (!isOnMagnet && isStuck(scp106)) {
    startDive(scp106, "combat");
    return;
  }
  if (isOnMagnet) {
    scp106.addEffect("slowness", 40, { amplifier: 1, showParticles: true });
  }
  updateCorrosionAcquisitionCooldown(scp106);
  if (!getCorrosionRight(scp106) && !getCorrosionLeft(scp106)) return;
  updateCorrosionThrowCooldown(scp106);
}
function isStuck(scp106) {
  const isTargetVeryClose = scp106.target && distance(scp106.location, scp106.target.location) <= 0.6;
  return !isTargetVeryClose && updateStuckDuration(scp106) > 2;
}
function updateStuckDuration(scp106) {
  const lastLocation = getLastLocation(scp106);
  setLastLocation(scp106, scp106.location);
  if (!lastLocation || distance(lastLocation, scp106.location) > 0.3) {
    setStuckDuration(scp106, 0);
    return 0;
  }
  const stuckDuration = getStuckDuration(scp106);
  setStuckDuration(scp106, stuckDuration + 1);
  return stuckDuration;
}
function startDive(scp106, context) {
  setState(scp106, SCP106_STATE.diving);
  setHideContext(scp106, context);
  setStuckDuration(scp106, 0);
  scp106.triggerEvent("lc:disable_free_movement");
}
function updateCorrosionAcquisitionCooldown(scp106) {
  const corrosionAcquisitionCooldown = getCorrosionAcquisitionCooldown(scp106);
  if (corrosionAcquisitionCooldown > 0) {
    setCorrosionAcquisitionCooldown(scp106, corrosionAcquisitionCooldown - 1);
    return;
  }
  const newCooldown = mc57.world.getDifficulty() === mc57.Difficulty.Hard ? randomInt(3, 5) : randomInt(8, 12);
  setCorrosionAcquisitionCooldown(scp106, newCooldown);
  acquireCorrosion(scp106);
}
function acquireCorrosion(scp106) {
  let right = !getCorrosionRight(scp106) && Math.random() > 0.5;
  if (right) {
    setCorrosionRight(scp106, true);
  } else {
    setCorrosionLeft(scp106, true);
  }
}
function updateCorrosionThrowCooldown(scp106) {
  const corrosionThrowCooldown = getCorrosionThrowCooldown(scp106);
  if (corrosionThrowCooldown > 0) {
    setCorrosionThrowCooldown(scp106, corrosionThrowCooldown - 1);
    return;
  }
  if (!scp106.target) return;
  if (distance(scp106.location, scp106.target.location) < 3.5) return;
  const raycastHitsInTargetDir = scp106.dimension.getEntitiesFromRay(
    scp106.getHeadLocation(),
    normalize(sub(scp106.target.getHeadLocation(), scp106.getHeadLocation())),
    { maxDistance: 32, type: scp106.target.typeId }
  );
  const isSeeingTarget = raycastHitsInTargetDir.some((x) => x.entity === scp106.target);
  if (!isSeeingTarget) return;
  const newCooldown = mc57.world.getDifficulty() === mc57.Difficulty.Hard ? randomInt(1, 2) : randomInt(2, 4);
  setCorrosionThrowCooldown(scp106, newCooldown);
  throwCorrosion(scp106);
}
function throwCorrosion(scp106) {
  if (!scp106.target) return;
  const rightCorrosion = getCorrosionRight(scp106);
  const leftCorrosion = getCorrosionLeft(scp106);
  if (!rightCorrosion && !leftCorrosion) return;
  let doRight = false;
  if (rightCorrosion && leftCorrosion) {
    doRight = Math.random() > 0.5;
  } else {
    doRight = rightCorrosion;
  }
  if (doRight) {
    setState(scp106, SCP106_STATE.throwingRight);
    setCorrosionRight(scp106, false);
  } else {
    setState(scp106, SCP106_STATE.throwingLeft);
    setCorrosionLeft(scp106, false);
  }
  scp106.lookAt(scp106.target.getHeadLocation());
  scp106.triggerEvent("lc:disable_free_movement");
  scp106.triggerEvent("lc:add_throwing_corrosion");
}
function onFinishThrowingCorrosionRight(scp106) {
  onFinishThrowingCorrosion(scp106);
}
function onFinishThrowingCorrosionLeft(scp106) {
  onFinishThrowingCorrosion(scp106);
}
function onFinishThrowingCorrosion(scp106) {
  setState(scp106, SCP106_STATE.default);
  scp106.triggerEvent("lc:enable_free_movement");
  scp106.triggerEvent("lc:remove_throwing_corrosion");
}
function onFinishDive(scp106) {
  enterHiddenState(scp106);
  scp106.tryTeleport({ x: scp106.location.x, y: scp106.location.y - 0.6, z: scp106.location.z });
}
function enterHiddenState(scp106) {
  setState(scp106, SCP106_STATE.hidden);
  scp106.clearVelocity();
  scp106.triggerEvent("lc:hide");
  scp106.triggerEvent("lc:disable_ambient_sound");
}
function onUpdateHiddenState(scp106) {
  scp106.addEffect("invisibility", 30, { showParticles: false });
  const hideCtx = getHideContext(scp106);
  const hidingTick = getHidingTick(scp106);
  setHidingTick(scp106, hidingTick + 1);
  if (hideCtx === "combat") {
    onUpdateCombatHiding(scp106, hidingTick);
  }
  if (hideCtx === "retreat") {
    onUpdateRetreatHiding(scp106, hidingTick);
  }
}
function onUpdateCombatHiding(scp106, hidingTick) {
  let emergeLoc = scp106.getDynamicProperty("emergeLocation");
  if (!emergeLoc) {
    emergeLoc = calculateCombatEmergeLocation(scp106);
    scp106.setDynamicProperty("emergeLocation", emergeLoc);
  }
  if (hidingTick === 1) {
    scp106.tryTeleport(sub(emergeLoc, UP));
    return;
  }
  if (hidingTick === 3) {
    stopHiding(scp106);
    switch (mc57.world.getDifficulty()) {
      default:
        setState(scp106, SCP106_STATE.emergingSlow);
        break;
      case mc57.Difficulty.Normal:
        if (Math.random() > 0.5) {
          setState(scp106, SCP106_STATE.emergingFast);
        } else {
          setState(scp106, SCP106_STATE.emergingSlow);
        }
        break;
      case mc57.Difficulty.Hard:
        setState(scp106, SCP106_STATE.emergingFast);
        break;
    }
    scp106.tryTeleport(emergeLoc);
    if (scp106.target) {
      scp106.lookAt(scp106.target.getHeadLocation());
    }
    scp106.setDynamicProperty("emergeLocation", void 0);
    return;
  }
}
function onUpdateRetreatHiding(scp106, hidingTick) {
  const healthComp = scp106.getComponent("health");
  const healAmount = mc57.world.getDifficulty() !== mc57.Difficulty.Hard ? randomInt(4, 7) : randomInt(1, 2);
  healthComp.setCurrentValue(
    Math.floor(clamp2(healthComp.currentValue + healAmount, healthComp.effectiveMin, healthComp.effectiveMax))
  );
  if (healthComp.currentValue < 300) return;
  if (hidingTick % 3 !== 0) return;
  let emergeLoc = scp106.getDynamicProperty("emergeLocation");
  if (!emergeLoc) {
    const nearbyPossibleTargets = scp106.dimension.getEntities({
      closest: 10,
      maxDistance: 400,
      location: scp106.location,
      excludeFamilies: ["inanimate"],
      excludeTypes: [SCP106_ENTITY_TYPE_ID]
    }).filter((entity) => {
      if (entity instanceof mc57.Player) {
        if ([mc57.GameMode.Creative, mc57.GameMode.Spectator].includes(entity.getGameMode())) return;
      } else if (!entity.matches({ families: ["mob"] })) return;
      return entity;
    });
    const newTarget = nearbyPossibleTargets[randomInt(0, nearbyPossibleTargets.length - 1)];
    emergeLoc = calculateCombatEmergeLocation(scp106, newTarget);
    scp106.setDynamicProperty("emergeLocation", emergeLoc);
    scp106.tryTeleport({ x: emergeLoc.x, y: emergeLoc.y - 0.6, z: emergeLoc.z });
    return;
  }
  stopHiding(scp106);
  setState(scp106, SCP106_STATE.emergingSlow);
  scp106.tryTeleport(emergeLoc);
  scp106.setRotation({ x: 0, y: randomFloat(-180, 180) });
  scp106.setDynamicProperty("emergeLocation", void 0);
}
function stopHiding(scp106) {
  scp106.removeEffect("invisibility");
  scp106.triggerEvent("lc:show");
  scp106.triggerEvent("lc:enable_ambient_sound");
  setHidingTick(scp106, 0);
}
function onFinishEmergingSlow(scp106) {
  onFinishEmerging(scp106);
}
function onFinishEmergingFast(scp106) {
  onFinishEmerging(scp106);
}
function onFinishEmerging(scp106) {
  setState(scp106, SCP106_STATE.default);
  scp106.triggerEvent("lc:enable_free_movement");
}
function onFinishRetreatDive(scp106) {
  setHideContext(scp106, "retreat");
  enterHiddenState(scp106);
  scp106.triggerEvent("lc:join_default_family");
  scp106.tryTeleport({ x: scp106.location.x, y: -63.5, z: scp106.location.z });
}
function isOnMagnetBlock(scp106) {
  const block = scp106.dimension.getBlockBelow(scp106.location, {
    maxDistance: 2,
    includeTypes: ["lc:scpdy_cst_magnet_block"]
  });
  return block != void 0 && block.typeId === "lc:scpdy_cst_magnet_block";
}

// scripts/logic/scps/scp106/trail.ts
import * as mc58 from "@minecraft/server";
mc58.system.afterEvents.scriptEventReceive.subscribe(
  (event) => {
    if (!event.sourceEntity) return;
    if (!event.sourceEntity.isValid) return;
    if (event.sourceEntity.typeId !== SCP106_TRAIL_ENTITY_TYPE_ID) return;
    onUpdate3(event.sourceEntity);
  },
  {
    namespaces: ["scpdy_scp106_trail"]
  }
);
function onUpdate3(entity) {
  entity.dimension.spawnParticle("lc:scpdy_scp106_trail_emitter", entity.location);
  const entities = entity.dimension.getEntities({
    closest: 10,
    maxDistance: 0.8,
    location: entity.location,
    excludeFamilies: ["inanimate"],
    excludeTypes: [SCP106_TRAIL_ENTITY_TYPE_ID, SCP106_ENTITY_TYPE_ID]
  });
  for (const entity2 of entities) {
    try {
      entity2.addEffect("wither", 60, { amplifier: 0 });
    } catch {
    }
  }
}

// scripts/logic/scps/scp173/index.ts
import * as mc60 from "@minecraft/server";

// scripts/logic/scps/scp173/blinking.ts
import * as mc59 from "@minecraft/server";
var BLINK_START_TICK = 50;
var CAMERA_FADE_START_TICK = 52;
var TIMER_WRAP_TICK = 58;
var BLINK_CAMERA_FADE_OPTIONS = {
  fadeColor: { red: 0.02, green: 0.02, blue: 0.02 },
  fadeTime: {
    fadeInTime: 0.05,
    holdTime: 0.5,
    fadeOutTime: 0.11
  }
};
var SCP173_EQO_REUSE = {
  type: SCP173_ENTITY_TYPE,
  closest: 1,
  maxDistance: 30
};
var blinkTimer = 0;
function isBlinkTime() {
  return blinkTimer >= BLINK_START_TICK;
}
function updateBlinkTimer() {
  if (blinkTimer >= TIMER_WRAP_TICK) {
    blinkTimer = 0;
    return;
  }
  blinkTimer++;
}
function playerCameraBlinking() {
  if (blinkTimer !== CAMERA_FADE_START_TICK) return;
  const players = mc59.world.getPlayers();
  if (players.length <= 0) return;
  for (let i = 0; i < players.length; i++) {
    const player = players[i];
    if (player.getGameMode() === mc59.GameMode.Creative) continue;
    SCP173_EQO_REUSE.location = player.location;
    const scp173 = player.dimension.getEntities(SCP173_EQO_REUSE)[0];
    if (!scp173) continue;
    if (!scp173.target) continue;
    player.camera.fade(BLINK_CAMERA_FADE_OPTIONS);
  }
}
mc59.world.afterEvents.worldLoad.subscribe(() => {
  mc59.system.runInterval(() => {
    updateBlinkTimer();
    if (CONFIG.disableCameraFadeOnBlink) return;
    playerCameraBlinking();
  }, 2);
});

// scripts/logic/scps/scp173/index.ts
mc60.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    if (!event.entity.isValid) return;
    const isMobile = event.entity.getProperty("lc:is_mobile") === true;
    if (isBlinkTime()) {
      if (!isMobile) {
        event.entity.triggerEvent("scp173:become_mobile");
      }
      return;
    }
    const isLookedAtBase = event.entity.getProperty("lc:is_looked_at_base") === true;
    if (isLookedAtBase && isMobile) {
      event.entity.triggerEvent("scp173:become_immobile");
    } else if (!isLookedAtBase && !isMobile) {
      event.entity.triggerEvent("scp173:become_mobile");
    }
  },
  {
    entityTypes: [SCP173_ENTITY_TYPE],
    eventTypes: ["scp173:update_script"]
  }
);
mc60.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    event.entity.target?.applyDamage(45451919, {
      cause: mc60.EntityDamageCause.override,
      damagingEntity: event.entity
    });
    event.entity.dimension.playSound("scpdy.scp173.attack", event.entity.location, {
      volume: 1.2
    });
  },
  {
    entityTypes: [SCP173_ENTITY_TYPE],
    eventTypes: ["scp173:attack_target"]
  }
);

// scripts/logic/scps/scp427/scp427.ts
import * as mc62 from "@minecraft/server";

// scripts/logic/scps/scp427/scp427_1.ts
import * as mc61 from "@minecraft/server";
var SCP427_1_ENTITY_TYPE = "lc:scpdy_scp427_1";
mc61.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    const { entity } = event;
    const itemStack = new mc61.ItemStack("lc:scpdy_scp427_1_flesh_raw", randomInt(1, 3));
    entity.dimension.spawnItem(itemStack, entity.location);
    spawnGoreExplosion(entity.dimension, add(entity.location, UP));
    entity.remove();
  },
  {
    entityTypes: [SCP427_1_ENTITY_TYPE],
    eventTypes: ["gib"]
  }
);
mc61.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    const { entity } = event;
    if (!entity.isValid) return;
    if (length(entity.getVelocity()) > 1.5) return;
    const chainsawStunTick = entity.getProperty("lc:chainsaw_stun_tick");
    if (chainsawStunTick > 0) return;
    if (!entity.target) return;
    const dist = distance(entity.target.location, entity.location);
    const impulse = mul(sub(entity.target.location, entity.location), dist / 8);
    entity.applyImpulse(impulse);
  },
  {
    entityTypes: [SCP427_1_ENTITY_TYPE],
    eventTypes: ["scp427_1:on_target_inside_melee_range"]
  }
);
mc61.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    const { entity } = event;
    const chainsawStunTick = entity.getProperty("lc:chainsaw_stun_tick");
    if (chainsawStunTick === 1) {
      entity.triggerEvent("scp427_1:become_free");
      entity.setProperty("lc:chainsaw_stun_tick", 0);
    } else if (chainsawStunTick > 1) {
      entity.setProperty("lc:chainsaw_stun_tick", chainsawStunTick - 1);
    }
  },
  {
    entityTypes: [SCP427_1_ENTITY_TYPE],
    eventTypes: ["scp427_1:decrease_chainsaw_stun_tick"]
  }
);

// scripts/logic/scps/scp427/scp427.ts
var SCP427_CLOSED_ITEM_TYPE_ID = "lc:scpdy_scp427";
var SCP427_OPEN_ITEM_TYPE_ID = "lc:scpdy_scp427_open";
var SCP427_TICK_INTERVAL = 8;
var SCP427_1_PROGRESS_OMEN_TICKS = [60, 90, 140, 180, 220];
var SCP427_1_PROGRESS_COMPLETE_TICK = 250;
var scp427CurrentTick = 0;
mc62.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:scp427", {
    onUse(arg) {
      if (!arg.itemStack) return;
      const newItemTypeId = arg.itemStack.typeId === SCP427_CLOSED_ITEM_TYPE_ID ? SCP427_OPEN_ITEM_TYPE_ID : SCP427_CLOSED_ITEM_TYPE_ID;
      const newItemStack = new mc62.ItemStack(newItemTypeId, 1);
      newItemStack.nameTag = arg.itemStack.nameTag;
      const equippable = arg.source.getComponent("equippable");
      equippable.setEquipment(mc62.EquipmentSlot.Mainhand, newItemStack);
      arg.source.dimension.playSound(
        "scpdy.interact.pick_item.2",
        add(arg.source.getHeadLocation(), arg.source.getViewDirection())
      );
    }
  });
});
subscribe((player, { healthComp, mainhandSlot }) => {
  if (mc62.system.currentTick % SCP427_TICK_INTERVAL !== 0) return;
  if ([mc62.GameMode.Creative, mc62.GameMode.Spectator].includes(player.getGameMode())) return;
  scp427CurrentTick++;
  if (healthComp.currentValue <= 0) return;
  const scp427TotalTime = player.getDynamicProperty("scp427TotalTime") ?? 0;
  const scp427_1_progress = player.getDynamicProperty("scp427_1_progress") ?? 0;
  if (scp427TotalTime >= 200) {
    if (scp427TotalTime > 220 && !player.getEffect("speed")) {
      player.addEffect("speed", 9999, { amplifier: 1, showParticles: false });
    }
    if (scp427TotalTime > 300 && !player.getEffect("resistance")) {
      player.addEffect("resistance", 9999, { amplifier: 1, showParticles: false });
    }
    if (scp427TotalTime > 350 && !player.getEffect("health_boost")) {
      player.addEffect("health_boost", 9999, { amplifier: 1, showParticles: false });
    }
    if (scp427TotalTime > 400 && !player.getEffect("strength")) {
      player.addEffect("strength", 9999, { amplifier: 0, showParticles: false });
    }
    if (scp427TotalTime > 450 && !player.getEffect("saturation")) {
      player.addEffect("saturation", 10, { amplifier: 0, showParticles: false });
    }
  }
  if (scp427TotalTime > 50) {
    const progressionInterval = scp427TotalTime < 100 ? 10 : scp427TotalTime < 250 ? 6 : scp427TotalTime < 350 ? 4 : scp427TotalTime < 400 ? 3 : scp427TotalTime < 450 ? 2 : 1;
    if (scp427CurrentTick % progressionInterval === 0) {
      const newProgress = scp427_1_progress + 1;
      player.setDynamicProperty("scp427_1_progress", newProgress);
      if (SCP427_1_PROGRESS_OMEN_TICKS.includes(newProgress)) {
        player.playSound("scpdy.scp427.heartbeat");
        player.addEffect("darkness", 20, {
          showParticles: false
        });
      } else if (newProgress === SCP427_1_PROGRESS_COMPLETE_TICK - 6) {
        player.camera.fade({
          fadeColor: { red: 0.8, green: 0, blue: 0 },
          fadeTime: {
            fadeInTime: 1.7,
            holdTime: 1,
            fadeOutTime: 0.8
          }
        });
      } else if (newProgress === SCP427_1_PROGRESS_COMPLETE_TICK) {
        player.dimension.playSound("scpdy.gore.explode", player.getHeadLocation(), {
          volume: 1.2,
          pitch: 0.9
        });
        player.dimension.spawnEntity(SCP427_1_ENTITY_TYPE, player.location, {
          initialRotation: player.getRotation().y
        });
        player.kill();
      }
    }
  }
  const isHoldingOpenScp427 = mainhandSlot.hasItem() && mainhandSlot.typeId === SCP427_OPEN_ITEM_TYPE_ID;
  if (!isHoldingOpenScp427) return;
  player.setDynamicProperty("scp427TotalTime", scp427TotalTime + 1);
  if (healthComp.currentValue < healthComp.effectiveMax && !player.getEffect("regeneration")) {
    const amplifier = scp427TotalTime < 150 ? 1 : scp427TotalTime < 300 ? 2 : scp427TotalTime < 500 ? 3 : 4;
    player.addEffect("regeneration", 40, { amplifier });
  }
});
mc62.world.afterEvents.entityDie.subscribe(
  (event) => {
    clearScp427RelatedDynamicProperties(event.deadEntity);
  },
  {
    entityTypes: ["minecraft:player"]
  }
);
mc62.world.beforeEvents.playerGameModeChange.subscribe((event) => {
  if (![mc62.GameMode.Creative, mc62.GameMode.Spectator].includes(event.toGameMode)) return;
  clearScp427RelatedDynamicProperties(event.player);
});
function clearScp427RelatedDynamicProperties(player) {
  player.setDynamicProperty("scp427TotalTime", void 0);
  player.setDynamicProperty("scp427_1_progress", void 0);
}

// scripts/logic/scps/scp427/scp427_1_flesh.ts
import * as mc63 from "@minecraft/server";
mc63.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:scp427_1_flesh_cooked", {
    onConsume(arg) {
      arg.source.addEffect("strength", 1200, { amplifier: 1 });
    }
  });
});
mc63.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:scp427_1_flesh_raw", {
    onConsume(arg) {
      arg.source.addEffect("poison", 600, { amplifier: 0 });
    }
  });
});

// scripts/logic/theUnknown/unknownCore/unknownCore.ts
import * as mc66 from "@minecraft/server";

// scripts/logic/theUnknown/shared.ts
import "@minecraft/server";
var UNKNOWN_BREEZE_ENTITY_TYPE = "lc:scpdy_unknown_breeze";
var UNKNOWN_CORE_ENTITY_TYPE = "lc:scpdy_unknown_core";
var UNKNOWN_GOLEM_ENTITY_TYPE = "lc:scpdy_unknown_golem";
var UNKNOWN_SPIDER_ENTITY_TYPE = "lc:scpdy_unknown_spider";
var UNKNOWN_ZOMBIE_ENTITY_TYPE = "lc:scpdy_unknown_zombie";
var UNKNOWN_RACE_MOB_TYPES = [
  UNKNOWN_BREEZE_ENTITY_TYPE,
  UNKNOWN_CORE_ENTITY_TYPE,
  UNKNOWN_GOLEM_ENTITY_TYPE,
  UNKNOWN_SPIDER_ENTITY_TYPE,
  UNKNOWN_ZOMBIE_ENTITY_TYPE
];
function isUnknownRace(entity) {
  return UNKNOWN_RACE_MOB_TYPES.includes(entity.typeId);
}

// scripts/logic/theUnknown/unknownCore/meteorite.ts
import * as mc65 from "@minecraft/server";
var METEORITE_ENTITY_TYPE = "lc:scpdy_unknown_core_meteorite";
function getFallLocation(meteorite) {
  const fallbackFallLoc = ensureType(meteorite.getDynamicProperty("fallbackFallLoc"), "Vector3") ?? meteorite.location;
  const targetEntityId = ensureType(meteorite.getDynamicProperty("targetEntityId"), "string");
  if (targetEntityId === void 0) return fallbackFallLoc;
  const target = mc65.world.getEntity(targetEntityId);
  if (!target) return fallbackFallLoc;
  let velocity = target.getVelocity();
  const velocityLength = length(velocity);
  velocity.y = 0;
  velocity = normalize(velocity);
  velocity = mul(velocity, velocityLength * randomFloat(3, 10));
  const fallLocXZ = add(add(target.location, velocity), mul(random(), randomFloat(-3, 3)));
  const raycastHit = meteorite.dimension.getBlockFromRay(
    {
      x: fallLocXZ.x,
      y: 330,
      z: fallLocXZ.z
    },
    DOWN,
    {
      excludeTypes: ["minecraft:barrier"],
      maxDistance: 500
    }
  );
  if (!raycastHit) {
    return target.location;
  }
  return add(raycastHit.block.location, raycastHit.faceLocation);
}
function onSpawnDebug(meteorite) {
  const nearestPlayer = meteorite.dimension.getPlayers({
    closest: 1,
    location: meteorite.location
  })[0];
  if (!nearestPlayer) return;
  meteorite.setDynamicProperty("targetEntityId", nearestPlayer.id);
}
function onStartFallDelayTimer(meteorite) {
  const fallLoc = getFallLocation(meteorite);
  meteorite.tryTeleport(fallLoc);
  meteorite.dimension.spawnParticle("lc:scpdy_unknown_core_meteorite_marker_particle", fallLoc);
}
mc65.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    switch (event.eventId) {
      case "unknown_core_meteorite:spawn_debug":
        onSpawnDebug(event.entity);
        break;
      case "unknown_core_meteorite:start_fall_delay_timer":
        onStartFallDelayTimer(event.entity);
        break;
    }
  },
  {
    entityTypes: [METEORITE_ENTITY_TYPE]
  }
);

// scripts/logic/theUnknown/unknownCore/unknownCore.ts
var PROP_ID2 = {
  stage: "lc:stage",
  s1State: "lc:s1_state",
  s2State: "lc:s2_state",
  s3State: "lc:s3_state",
  canBeDamaged: "lc:can_be_damaged"
};
var STAGE_STATE_IDX = {
  s1: {
    fireball: 10,
    shield: 20,
    toS2: 100
  },
  s2: {
    plasmaRapid: 10,
    plasmaPowerful: 20,
    summonAllies: 30,
    exposeEye: 40,
    toS3: 100
  },
  s3: {
    swordSlash1: 10,
    swordSlash2: 20,
    fireball: 30,
    exposeEye: 40,
    fireMeteorites: 50,
    summonAllies: 60,
    heal: 70,
    plasmaUltra: 80,
    defeat: 100
  }
};
var CombatStateArrays;
((CombatStateArrays2) => {
  CombatStateArrays2.S2_1 = [
    STAGE_STATE_IDX.s2.exposeEye,
    "setDelay_1",
    STAGE_STATE_IDX.s2.summonAllies,
    STAGE_STATE_IDX.s2.plasmaPowerful,
    STAGE_STATE_IDX.s2.plasmaRapid,
    STAGE_STATE_IDX.s2.plasmaRapid,
    "setDelay_1",
    STAGE_STATE_IDX.s2.exposeEye,
    "setDelay_1",
    STAGE_STATE_IDX.s2.plasmaPowerful,
    STAGE_STATE_IDX.s2.plasmaRapid,
    STAGE_STATE_IDX.s2.plasmaRapid,
    "setDelay_1"
  ];
  CombatStateArrays2.S2_2 = [
    STAGE_STATE_IDX.s2.summonAllies,
    STAGE_STATE_IDX.s2.plasmaPowerful,
    STAGE_STATE_IDX.s2.plasmaPowerful,
    STAGE_STATE_IDX.s2.plasmaRapid,
    STAGE_STATE_IDX.s2.plasmaRapid,
    STAGE_STATE_IDX.s2.plasmaRapid,
    "setDelay_3",
    STAGE_STATE_IDX.s2.exposeEye,
    "setDelay_1",
    STAGE_STATE_IDX.s2.plasmaPowerful,
    STAGE_STATE_IDX.s2.plasmaPowerful,
    STAGE_STATE_IDX.s2.plasmaRapid,
    STAGE_STATE_IDX.s2.plasmaRapid,
    STAGE_STATE_IDX.s2.plasmaRapid,
    "setDelay_3",
    STAGE_STATE_IDX.s2.exposeEye,
    "setDelay_1"
  ];
  CombatStateArrays2.S3_1 = [
    STAGE_STATE_IDX.s3.summonAllies,
    STAGE_STATE_IDX.s3.swordSlash1,
    STAGE_STATE_IDX.s3.swordSlash2,
    STAGE_STATE_IDX.s3.fireball,
    "setDelay_3",
    STAGE_STATE_IDX.s3.exposeEye,
    "setDelay_1",
    STAGE_STATE_IDX.s3.swordSlash1,
    STAGE_STATE_IDX.s3.swordSlash2,
    STAGE_STATE_IDX.s3.fireball,
    "setDelay_3",
    STAGE_STATE_IDX.s3.exposeEye,
    "setDelay_1"
  ];
  CombatStateArrays2.S3_2 = [
    STAGE_STATE_IDX.s3.summonAllies,
    STAGE_STATE_IDX.s3.plasmaUltra,
    STAGE_STATE_IDX.s3.swordSlash1,
    STAGE_STATE_IDX.s3.swordSlash2,
    STAGE_STATE_IDX.s3.fireball,
    "setDelay_3",
    STAGE_STATE_IDX.s3.exposeEye,
    "setDelay_1",
    STAGE_STATE_IDX.s3.fireMeteorites,
    STAGE_STATE_IDX.s3.exposeEye,
    "setDelay_1",
    STAGE_STATE_IDX.s3.plasmaUltra,
    STAGE_STATE_IDX.s3.swordSlash1,
    STAGE_STATE_IDX.s3.swordSlash2,
    STAGE_STATE_IDX.s3.fireball,
    "setDelay_3",
    STAGE_STATE_IDX.s3.exposeEye,
    "setDelay_1"
  ];
  CombatStateArrays2.S3_3 = [
    STAGE_STATE_IDX.s3.summonAllies,
    STAGE_STATE_IDX.s3.fireMeteorites,
    STAGE_STATE_IDX.s3.plasmaUltra,
    "setDelay_1",
    STAGE_STATE_IDX.s3.exposeEye,
    STAGE_STATE_IDX.s3.swordSlash1,
    STAGE_STATE_IDX.s3.swordSlash2,
    STAGE_STATE_IDX.s3.fireball,
    "setDelay_1",
    STAGE_STATE_IDX.s3.exposeEye,
    STAGE_STATE_IDX.s3.swordSlash1,
    STAGE_STATE_IDX.s3.swordSlash2,
    STAGE_STATE_IDX.s3.fireball,
    "setDelay_1"
  ];
})(CombatStateArrays || (CombatStateArrays = {}));
function getDifficulty(unknownCore) {
  const difficulty = unknownCore.getProperty("lc:difficulty");
  return difficulty;
}
function getStageState(unknownCore, stage) {
  const stageState = stage === 1 ? unknownCore.getProperty("lc:s1_state") : stage === 2 ? unknownCore.getProperty("lc:s2_state") : stage === 3 ? unknownCore.getProperty("lc:s3_state") : 0;
  return stageState;
}
function updateLocationLock(unknownCore) {
  const locationLock = ensureType(unknownCore.getDynamicProperty("locationLock"), "Vector3");
  if (!locationLock) return;
  const dist = distance(locationLock, unknownCore.location);
  if (dist <= 0.1) return;
  unknownCore.tryTeleport(locationLock, {
    keepVelocity: false
  });
}
function getNextCombatBehaviorTick(unknownCore) {
  return ensureType(unknownCore.getDynamicProperty("ticksUntilDecideNextCombatBehavior"), "number") ?? 0;
}
function setNextCombatBehaviorTick(unknownCore, value) {
  unknownCore.setDynamicProperty("ticksUntilDecideNextCombatBehavior", value);
}
function decideNextCombatBehavior(unknownCore, stage) {
  const healthComp = unknownCore.getComponent("health");
  switch (stage) {
    case 1: {
      const didGuard = unknownCore.getDynamicProperty("s1_didGuard") === true;
      if (!didGuard && Math.random() > 0.5 && healthComp.currentValue < healthComp.effectiveMax / 2) {
        unknownCore.setProperty(PROP_ID2.s1State, STAGE_STATE_IDX.s1.shield);
        unknownCore.setDynamicProperty("s1_didGuard", true);
        setNextCombatBehaviorTick(unknownCore, 2);
        break;
      }
      unknownCore.setProperty(PROP_ID2.s1State, STAGE_STATE_IDX.s1.fireball);
      unknownCore.setDynamicProperty("s1_didGuard", void 0);
      break;
    }
    case 2: {
      const decisionCountPropId = healthComp.currentValue < healthComp.effectiveMax / 2 ? "s2MidCombatDecisionCount" : "s2CombatDecisionCount";
      const decisionCount = ensureType(unknownCore.getDynamicProperty(decisionCountPropId), "number") ?? 0;
      const combatStateArray = healthComp.currentValue < healthComp.effectiveMax / 2 ? CombatStateArrays.S2_2 : CombatStateArrays.S2_1;
      const nextThing = combatStateArray[decisionCount % combatStateArray.length];
      unknownCore.setDynamicProperty(decisionCountPropId, decisionCount + 1);
      if (typeof nextThing === "string") {
        if (nextThing.startsWith("setDelay")) {
          const delay = +nextThing[nextThing.length - 1];
          setNextCombatBehaviorTick(unknownCore, delay);
        } else {
          throw new Error(`Unrecognized next behavior: ${nextThing}`);
        }
        return;
      }
      if (nextThing === STAGE_STATE_IDX.s2.summonAllies) {
        const nearbyAllies = unknownCore.dimension.getEntities({
          closest: 6,
          maxDistance: 20,
          location: unknownCore.location,
          families: ["the_unknown"],
          excludeTypes: [UNKNOWN_CORE_ENTITY_TYPE]
        });
        if (nearbyAllies.length < 5) {
          unknownCore.setProperty(PROP_ID2.s2State, STAGE_STATE_IDX.s2.summonAllies);
        } else {
          unknownCore.setProperty(PROP_ID2.s2State, STAGE_STATE_IDX.s2.plasmaRapid);
        }
        return;
      }
      unknownCore.setProperty(PROP_ID2.s2State, nextThing ?? 0);
      break;
    }
    case 3: {
      const decisionCountPropId = healthComp.currentValue < healthComp.effectiveMax / 5 ? "s3LowCombatDecisionCount" : healthComp.currentValue < healthComp.effectiveMax / 2 ? "s3MidCombatDecisionCount" : "s3CombatDecisionCount";
      const decisionCount = ensureType(unknownCore.getDynamicProperty(decisionCountPropId), "number") ?? 0;
      const combatStateArray = healthComp.currentValue < healthComp.effectiveMax / 5 ? CombatStateArrays.S3_3 : healthComp.currentValue < healthComp.effectiveMax / 2 ? CombatStateArrays.S3_2 : CombatStateArrays.S3_1;
      const nextThing = combatStateArray[decisionCount % combatStateArray.length];
      unknownCore.setDynamicProperty(decisionCountPropId, decisionCount + 1);
      if (typeof nextThing === "string") {
        if (nextThing.startsWith("setDelay")) {
          const delay = +nextThing[nextThing.length - 1];
          setNextCombatBehaviorTick(unknownCore, delay);
        } else {
          throw new Error(`Unrecognized next behavior: ${nextThing}`);
        }
        return;
      }
      if (nextThing === STAGE_STATE_IDX.s3.summonAllies) {
        const nearbyAllies = unknownCore.dimension.getEntities({
          closest: 11,
          maxDistance: 20,
          location: unknownCore.location,
          families: ["the_unknown"],
          excludeTypes: [UNKNOWN_CORE_ENTITY_TYPE]
        });
        if (nearbyAllies.length < 10) {
          unknownCore.setProperty(PROP_ID2.s3State, STAGE_STATE_IDX.s3.summonAllies);
        } else {
          unknownCore.setProperty(PROP_ID2.s3State, STAGE_STATE_IDX.s3.swordSlash1);
        }
        return;
      }
      unknownCore.setProperty(PROP_ID2.s3State, nextThing ?? 0);
      break;
    }
  }
}
function onUpdate4(unknownCore) {
  updateLocationLock(unknownCore);
  const target = unknownCore.target;
  if (!target) return;
  const dirToTarget = sub(target.location, unknownCore.location);
  let yRotation = Math.atan2(dirToTarget.x, dirToTarget.z) * (180 / Math.PI);
  if (yRotation > 180) {
    yRotation -= 360;
  } else if (yRotation < -180) {
    yRotation += 360;
  }
  unknownCore.setProperty("lc:combat_y_body_rot", -yRotation);
  const stage = unknownCore.getProperty(PROP_ID2.stage);
  const stageState = getStageState(unknownCore, stage);
  if (stageState !== 0) return;
  let ticksLeft = getNextCombatBehaviorTick(unknownCore);
  if (ticksLeft === 0) {
    decideNextCombatBehavior(unknownCore, stage);
    return;
  }
  const next = ticksLeft > 0 ? ticksLeft - 1 : 0;
  setNextCombatBehaviorTick(unknownCore, next);
}
function resetHealth(unknownCore) {
  const healthComp = unknownCore.getComponent("health");
  if (!healthComp) return;
  healthComp.setCurrentValue(healthComp.effectiveMax);
}
function onFatalDamage(unknownCore) {
  unknownCore.getComponent("health")?.setCurrentValue(1);
  const stage = unknownCore.getProperty(PROP_ID2.stage);
  switch (stage) {
    case 1:
      unknownCore.setProperty(PROP_ID2.s1State, STAGE_STATE_IDX.s1.toS2);
      break;
    case 2:
      unknownCore.setProperty(PROP_ID2.s2State, STAGE_STATE_IDX.s2.toS3);
      break;
    case 3:
      unknownCore.setProperty(PROP_ID2.s3State, STAGE_STATE_IDX.s3.defeat);
      break;
  }
  setNextCombatBehaviorTick(unknownCore, 3);
}
function summonAlliesS2(unknownCore) {
  const difficulty = getDifficulty(unknownCore);
  const ally1Pos = add(unknownCore.location, { x: 0, y: 0, z: 1 });
  const ally1 = unknownCore.dimension.spawnEntity(UNKNOWN_ZOMBIE_ENTITY_TYPE, ally1Pos);
  ally1.setDynamicProperty("parentUnknownCoreId", unknownCore.id);
  const ally2Pos = add(unknownCore.location, { x: 0, y: 0, z: -1 });
  const ally2 = unknownCore.dimension.spawnEntity(UNKNOWN_BREEZE_ENTITY_TYPE, ally2Pos);
  ally2.setDynamicProperty("parentUnknownCoreId", unknownCore.id);
  if (difficulty <= mc66.Difficulty.Easy) return;
  const ally3Pos = add(unknownCore.location, { x: 1, y: 0, z: 0 });
  const ally3 = unknownCore.dimension.spawnEntity(UNKNOWN_GOLEM_ENTITY_TYPE, ally3Pos);
  ally3.setDynamicProperty("parentUnknownCoreId", unknownCore.id);
  if (difficulty <= mc66.Difficulty.Normal) return;
  const ally4Pos = add(unknownCore.location, { x: -1, y: 0, z: 0 });
  const ally4 = unknownCore.dimension.spawnEntity(UNKNOWN_SPIDER_ENTITY_TYPE, ally4Pos);
  ally4.setDynamicProperty("parentUnknownCoreId", unknownCore.id);
}
function summonAlliesS3(unknownCore) {
  const difficulty = getDifficulty(unknownCore);
  const ally1Pos = add(unknownCore.location, { x: 0, y: 0, z: 1 });
  const ally1 = unknownCore.dimension.spawnEntity(UNKNOWN_ZOMBIE_ENTITY_TYPE, ally1Pos);
  ally1.setDynamicProperty("parentUnknownCoreId", unknownCore.id);
  const ally2Pos = add(unknownCore.location, { x: 0, y: 0, z: -1 });
  const ally2 = unknownCore.dimension.spawnEntity(UNKNOWN_ZOMBIE_ENTITY_TYPE, ally2Pos);
  ally2.setDynamicProperty("parentUnknownCoreId", unknownCore.id);
  if (difficulty <= mc66.Difficulty.Easy) return;
  const ally3Pos = add(unknownCore.location, { x: 1, y: 0, z: 0 });
  const ally3 = unknownCore.dimension.spawnEntity(UNKNOWN_BREEZE_ENTITY_TYPE, ally3Pos);
  ally3.setDynamicProperty("parentUnknownCoreId", unknownCore.id);
  const ally4Pos = add(unknownCore.location, { x: -1, y: 0, z: 0 });
  const ally4 = unknownCore.dimension.spawnEntity(UNKNOWN_GOLEM_ENTITY_TYPE, ally4Pos);
  ally4.setDynamicProperty("parentUnknownCoreId", unknownCore.id);
  if (difficulty <= mc66.Difficulty.Normal) return;
  const ally5Pos = add(unknownCore.location, { x: -1, y: 0, z: 1 });
  const ally5 = unknownCore.dimension.spawnEntity(UNKNOWN_SPIDER_ENTITY_TYPE, ally5Pos);
  ally5.setDynamicProperty("parentUnknownCoreId", unknownCore.id);
}
function onSwingSword(unknownCore) {
  const target = unknownCore.target;
  if (!target) return;
  const dist = distance(unknownCore.location, target.location);
  if (dist > 3.8) return;
  target.applyDamage(getModifiedDamageNumber(13, target), {
    cause: mc66.EntityDamageCause.override,
    damagingEntity: unknownCore
  });
}
function fireMeteorite(unknownCore) {
  const meteoriteEntity = unknownCore.dimension.spawnEntity(METEORITE_ENTITY_TYPE, unknownCore.location);
  meteoriteEntity.setDynamicProperty("targetEntityId", unknownCore.target?.id);
  const fallbackFallLoc = unknownCore.target ? add(unknownCore.target.location, mul(random(), randomFloat(-2, 2))) : add(unknownCore.location, mul(random(), randomFloat(-10, 10)));
  meteoriteEntity.setDynamicProperty("fallbackFallLoc", fallbackFallLoc);
}
function onDefeat(unknownCore) {
  const childAllies = unknownCore.dimension.getEntities({
    families: ["the_unknown"],
    excludeTypes: [UNKNOWN_CORE_ENTITY_TYPE],
    maxDistance: 100,
    location: unknownCore.location
  });
  for (const childAlly of childAllies) {
    const parentUnknownCoreId = childAlly.getDynamicProperty("parentUnknownCoreId");
    if (typeof parentUnknownCoreId !== "string") continue;
    if (parentUnknownCoreId !== unknownCore.id) continue;
    childAlly.kill();
  }
  mc66.system.run(() => {
    unknownCore.remove();
  });
}
mc66.world.afterEvents.dataDrivenEntityTrigger.subscribe((event) => {
  if (event.entity.typeId !== UNKNOWN_CORE_ENTITY_TYPE) return;
  switch (event.eventId) {
    case "unknown_core:set_can_be_damaged:true":
      event.entity.removeTag("scpdy_ignore_slasher_capture");
      break;
    case "unknown_core:set_can_be_damaged:false":
      event.entity.addTag("scpdy_ignore_slasher_capture");
      break;
    case "unknown_core:reset_health":
      resetHealth(event.entity);
      break;
    case "unknown_core:script_update":
      onUpdate4(event.entity);
      break;
    case "unknown_core:on_fatal_damage":
      onFatalDamage(event.entity);
      break;
    case "unknown_core:on_defeat":
      onDefeat(event.entity);
      break;
    case "unknown_core:s2:summon_allies":
      summonAlliesS2(event.entity);
      break;
    case "unknown_core:s3:summon_allies":
      summonAlliesS3(event.entity);
      break;
    case "unknown_core:s3:on_swing_sword":
      onSwingSword(event.entity);
      break;
    case "unknown_core:s3:fire_meteorite":
      fireMeteorite(event.entity);
      break;
  }
});
mc66.world.afterEvents.entitySpawn.subscribe((event) => {
  if (event.entity.typeId !== UNKNOWN_CORE_ENTITY_TYPE) return;
  const locationLock = {
    x: Math.floor(event.entity.location.x) + 0.5,
    y: event.entity.location.y,
    z: Math.floor(event.entity.location.z) + 0.5
  };
  event.entity.tryTeleport(locationLock);
  event.entity.setDynamicProperty("locationLock", locationLock);
});

// scripts/logic/theUnknown/unknownCore/projectiles.ts
import * as mc67 from "@minecraft/server";
mc67.world.afterEvents.projectileHitEntity.subscribe((event) => {
  const hitEntity = event.getEntityHit().entity;
  if (!hitEntity) return;
  if (isUnknownRace(hitEntity)) return;
  switch (event.projectile.typeId) {
    case "lc:scpdy_unknown_core_plasma_small": {
      hitEntity.applyDamage(2, {
        cause: mc67.EntityDamageCause.override
      });
      hitEntity.addEffect("slowness", 40, {
        amplifier: 0
      });
      break;
    }
    case "lc:scpdy_unknown_core_plasma_large": {
      hitEntity.applyDamage(3, {
        cause: mc67.EntityDamageCause.override
      });
      hitEntity.addEffect("slowness", 40, {
        amplifier: 1
      });
      break;
    }
    case "lc:scpdy_unknown_core_plasma_ultra": {
      hitEntity.applyDamage(2, {
        cause: mc67.EntityDamageCause.override
      });
      hitEntity.addEffect("slowness", 100, {
        amplifier: 1
      });
      break;
    }
    case "lc:scpdy_unknown_core_sword_magic": {
      hitEntity.addEffect("poison", 100, {
        amplifier: 0
      });
      if (hitEntity instanceof mc67.Player) {
        hitEntity.runCommand("stopsound @s scpdy.unknown_core.sword_magic_fly");
      }
      break;
    }
    case "lc:scpdy_unknown_core_fireball": {
      hitEntity.addEffect("wither", 100, {
        amplifier: 1
      });
      break;
    }
  }
});
mc67.world.afterEvents.projectileHitBlock.subscribe((event) => {
  switch (event.projectile.typeId) {
    case "lc:scpdy_unknown_core_plasma_small":
    case "lc:scpdy_unknown_core_plasma_large":
    case "lc:scpdy_unknown_core_plasma_ultra": {
      event.projectile.remove();
      const hitInfo = event.getBlockHit();
      const hitLoc = add(hitInfo.block, hitInfo.faceLocation);
      const particleLoc = add(hitLoc, mul(event.hitVector, -1));
      event.dimension.spawnParticle("lc:scpdy_unknown_core_plasma_blast_small_emitter", particleLoc);
      break;
    }
    case "lc:scpdy_unknown_core_sword_magic": {
      event.projectile.remove();
      const hitInfo = event.getBlockHit();
      const hitLoc = add(hitInfo.block, hitInfo.faceLocation);
      const particleLoc = add(hitLoc, mul(event.hitVector, -1));
      event.dimension.spawnParticle("lc:scpdy_unknown_core_sword_magic_2_emitter", particleLoc);
      break;
    }
  }
});

// scripts/logic/theUnknown/waveSpawner.ts
import * as mc69 from "@minecraft/server";

// scripts/logic/theUnknown/waves.ts
import * as mc68 from "@minecraft/server";
var lastDifficulty;
var lastWaveCount;
var waveEnemyArrayCache;
function shuffleArray(array) {
  for (let i = array.length - 1; i >= 0; i--) {
    let j = Math.floor(Math.random() * (i + 1));
    let temp = array[i];
    array[i] = array[j];
    array[j] = temp;
  }
}
function createWaveEnemyArray_easy(waveCount) {
  const array = [];
  if (waveCount >= 10) {
    array.push(UNKNOWN_GOLEM_ENTITY_TYPE, UNKNOWN_SPIDER_ENTITY_TYPE, UNKNOWN_CORE_ENTITY_TYPE);
  } else {
    for (let i = 0; i < waveCount; i++) {
      array.push(UNKNOWN_ZOMBIE_ENTITY_TYPE);
      if (i >= 5) {
        array.push(UNKNOWN_SPIDER_ENTITY_TYPE);
      }
      if (i >= 8) {
        array.push(UNKNOWN_GOLEM_ENTITY_TYPE);
      }
    }
  }
  lastDifficulty = mc68.Difficulty.Easy;
  lastWaveCount = waveCount;
  waveEnemyArrayCache = array;
  return array;
}
function createWaveEnemyArray_normal(waveCount) {
  const array = [];
  if (waveCount >= 10) {
    array.push(UNKNOWN_CORE_ENTITY_TYPE);
  } else {
    if (waveCount >= 5) {
      const spiderAmount = clamp2(waveCount - 5, 1, 4);
      for (let i = 0; i < spiderAmount; i++) {
        array.push(UNKNOWN_SPIDER_ENTITY_TYPE);
      }
      const golemAmount = clamp2(waveCount - 5, 1, 2);
      for (let i = 0; i < golemAmount; i++) {
        array.push(UNKNOWN_GOLEM_ENTITY_TYPE);
      }
    }
    if (waveCount >= 8) {
      const breezeAmount = clamp2(waveCount - 8, 1, 2);
      for (let i = 0; i < breezeAmount; i++) {
        array.push(UNKNOWN_BREEZE_ENTITY_TYPE);
      }
    }
    shuffleArray(array);
    const zombieAmount = clamp2(Math.floor(waveCount * 1.3), 1, 15);
    for (let i = 0; i < zombieAmount; i++) {
      array.push(UNKNOWN_ZOMBIE_ENTITY_TYPE);
    }
  }
  lastDifficulty = mc68.Difficulty.Normal;
  lastWaveCount = waveCount;
  waveEnemyArrayCache = array;
  return array;
}
function createWaveEnemyArray_hard(waveCount) {
  const array = [];
  if (waveCount >= 10) {
    array.push(
      UNKNOWN_GOLEM_ENTITY_TYPE,
      UNKNOWN_GOLEM_ENTITY_TYPE,
      UNKNOWN_GOLEM_ENTITY_TYPE,
      UNKNOWN_CORE_ENTITY_TYPE
    );
  } else {
    {
      const spiderAmount = clamp2(waveCount, 1, 6);
      for (let i = 0; i < spiderAmount; i++) {
        array.push(UNKNOWN_SPIDER_ENTITY_TYPE);
      }
    }
    if (waveCount >= 5) {
      const golemAmount = clamp2(waveCount, 1, 3);
      for (let i = 0; i < golemAmount; i++) {
        array.push(UNKNOWN_GOLEM_ENTITY_TYPE);
      }
    }
    if (waveCount >= 8) {
      const breezeAmount = 2 + (waveCount - 8);
      for (let i = 0; i < breezeAmount; i++) {
        array.push(UNKNOWN_BREEZE_ENTITY_TYPE);
      }
    }
    shuffleArray(array);
    const zombieAmount = clamp2(Math.floor(waveCount * 1.6), 1, 15);
    for (let i = 0; i < zombieAmount; i++) {
      array.push(UNKNOWN_ZOMBIE_ENTITY_TYPE);
    }
  }
  lastDifficulty = mc68.Difficulty.Hard;
  lastWaveCount = waveCount;
  waveEnemyArrayCache = array;
  return array;
}
function getWaveEnemyArray(waveCount, difficulty) {
  if (lastDifficulty === difficulty && lastWaveCount === waveCount) return waveEnemyArrayCache;
  switch (difficulty) {
    case mc68.Difficulty.Easy:
      return createWaveEnemyArray_easy(waveCount);
    case mc68.Difficulty.Normal:
      return createWaveEnemyArray_normal(waveCount);
    case mc68.Difficulty.Hard:
      return createWaveEnemyArray_hard(waveCount);
  }
  return [];
}

// scripts/logic/theUnknown/waveSpawner.ts
var UNKNOWN_WAVE_SPAWNER_ENTITY_TYPE = "lc:scpdy_the_unknown_wave_spawner";
function getNearbyPlayers(waveSpawner) {
  return waveSpawner.dimension.getPlayers({
    maxDistance: 80,
    location: waveSpawner.location
  });
}
function onNearbyPlayers(waveSpawner, ...func) {
  const players = getNearbyPlayers(waveSpawner);
  for (let i = 0; i < players.length; i++) {
    const player = players[i];
    for (let j = 0; j < func.length; j++) {
      const f = func[j];
      f(player);
    }
  }
}
function getDifficulty2(waveSpawner) {
  return waveSpawner.getProperty("lc:difficulty");
}
function onSpawn2(waveSpawner) {
  const difficulty = getDifficulty2(waveSpawner);
  if (difficulty === mc69.Difficulty.Peaceful) {
    onNearbyPlayers(waveSpawner, (player) => {
      player.sendMessage({
        translate: "scpdy.msg.theUnknownWaveSpawner.peacefulNotSupported"
      });
    });
    waveSpawner.triggerEvent("despawn");
    return;
  }
  waveSpawner.triggerEvent("the_unknown_wave_spawner:start_wave_break");
  const scoreboardObjective = mc69.world.scoreboard.addObjective(waveSpawner.id, "Scores");
  mc69.world.scoreboard.setObjectiveAtDisplaySlot(mc69.DisplaySlotId.Sidebar, {
    objective: scoreboardObjective,
    sortOrder: mc69.ObjectiveSortOrder.Descending
  });
}
function getWaveMusicId(difficulty, waveCount) {
  const music1 = "scpdy.music.the_unknown_battle_1";
  const music2 = "scpdy.music.the_unknown_battle_2";
  const music3 = "scpdy.music.the_unknown_battle_3";
  return waveCount < 5 ? music1 : waveCount < 10 ? music2 : music3;
}
function onStartWaveBreakEvent(waveSpawner) {
  const currentWave = waveSpawner.getProperty("lc:wave_count");
  const nextWave = currentWave + 1;
  const difficulty = getDifficulty2(waveSpawner);
  const waveBreakSeconds = difficulty === mc69.Difficulty.Hard ? 3 : difficulty === mc69.Difficulty.Normal ? 5 : 7;
  const msg = {
    translate: "scpdy.msg.theUnknownWaveSpawner.waveBreakStarted",
    with: [`${currentWave}`, `${nextWave}`]
  };
  onNearbyPlayers(waveSpawner, (player) => {
    const newMusic = getWaveMusicId(difficulty, nextWave);
    const oldMusic = player.getDynamicProperty("theUnknownBattleMusic");
    if (oldMusic !== newMusic) {
      player.playMusic(newMusic, {
        fade: waveBreakSeconds,
        loop: true
      });
    }
    player.setDynamicProperty("theUnknownBattleMusic", newMusic);
    if (currentWave === 0) return;
    player.sendMessage(msg);
  });
}
function onEndWaveBreakEvent(waveSpawner) {
  const currentWave = waveSpawner.getProperty("lc:wave_count");
  const nextWave = currentWave + 1;
  waveSpawner.setProperty("lc:wave_count", nextWave);
  mc69.system.runTimeout(() => {
    waveSpawner.triggerEvent("the_unknown_wave_spawner:on_tick_wave_battle");
  }, 1);
  const msg = {
    translate: "scpdy.msg.theUnknownWaveSpawner.waveXStarted",
    with: [`${nextWave}`]
  };
  onNearbyPlayers(waveSpawner, (player) => {
    player.sendMessage(msg);
  });
}
function getLastWaveCount(difficulty) {
  return 10;
}
function setNextWaveEnemyIndex(waveSpawner, value) {
  waveSpawner.setDynamicProperty("nextWaveEnemyIndex", value);
}
function getNextWaveEnemyIndex(waveSpawner) {
  let value = waveSpawner.getDynamicProperty("nextWaveEnemyIndex");
  if (typeof value !== "number") {
    value = 0;
    setNextWaveEnemyIndex(waveSpawner, 0);
  }
  return value;
}
function getNearbyWaveEnemies(waveSpawner, waveCount) {
  return waveSpawner.dimension.getEntities({
    families: ["the_unknown"],
    maxDistance: 80,
    location: waveSpawner.location
  }).filter((entity) => entity.getDynamicProperty("waveId") === `${waveSpawner.id}_${waveCount}`);
}
function onFinishAllWaves(waveSpawner) {
  try {
    const scoreboardObjective = mc69.world.scoreboard.getObjective(waveSpawner.id);
    if (!scoreboardObjective) return;
    const winners = [];
    const participants = scoreboardObjective.getParticipants();
    for (const participant of participants) {
      if (participant.type === mc69.ScoreboardIdentityType.FakePlayer) continue;
      const score = scoreboardObjective.getScore(participant);
      winners.push({
        identity: participant,
        score
      });
    }
    winners.sort((a, b) => b.score - a.score);
    const congratulationMsg = {
      rawtext: [
        { translate: "scpdy.msg.theUnknownWaveSpawner.finished" },
        { text: "\n" },
        { translate: "scpdy.msg.theUnknownWaveSpawner.congratsSep1" },
        { text: "\n" }
      ]
    };
    for (const winner of winners) {
      congratulationMsg.rawtext.push({ text: `${winner.identity.displayName} - ${winner.score}` }, { text: "\n" });
      const entity = winner.identity.getEntity();
      if (entity && entity instanceof mc69.Player) {
        mc69.system.run(() => {
          entity.sendMessage({
            translate: "scpdy.msg.theUnknownWaveSpawner.playerScore",
            with: [`${winner.score}`]
          });
          const xp = Math.max(1, Math.ceil(winner.score * 0.8));
          entity.addExperience(xp);
        });
      }
    }
    congratulationMsg.rawtext.push({ translate: "scpdy.msg.theUnknownWaveSpawner.congratsSep2" });
    mc69.world.sendMessage(congratulationMsg);
  } finally {
    mc69.system.run(() => {
      waveSpawner.kill();
    });
  }
}
function onTickWaveBattle(waveSpawner) {
  const waveCount = waveSpawner.getProperty("lc:wave_count");
  const difficulty = getDifficulty2(waveSpawner);
  const waveEnemyArray = getWaveEnemyArray(waveCount, difficulty);
  const nextWaveEnemyIndex = getNextWaveEnemyIndex(waveSpawner);
  if (nextWaveEnemyIndex > waveEnemyArray.length - 1) {
    const nearbyWaveEnemies = getNearbyWaveEnemies(waveSpawner, waveCount);
    if (nearbyWaveEnemies.length > 0) return;
    if (waveCount >= getLastWaveCount(difficulty)) {
      onFinishAllWaves(waveSpawner);
      return;
    }
    setNextWaveEnemyIndex(waveSpawner, 0);
    waveSpawner.triggerEvent("the_unknown_wave_spawner:start_wave_break");
    return;
  }
  const enemyTypeId = waveEnemyArray[nextWaveEnemyIndex];
  const spawnLoc = enemyTypeId === UNKNOWN_CORE_ENTITY_TYPE ? {
    x: waveSpawner.location.x,
    y: Math.floor(waveSpawner.location.y) + 1.6,
    z: waveSpawner.location.z
  } : waveSpawner.location;
  const spawnedMob = waveSpawner.dimension.spawnEntity(enemyTypeId, spawnLoc);
  spawnedMob.setDynamicProperty("waveId", `${waveSpawner.id}_${waveCount}`);
  spawnedMob.setDynamicProperty("waveSpawnerId", waveSpawner.id);
  setNextWaveEnemyIndex(waveSpawner, nextWaveEnemyIndex + 1);
}
function stopBattleMusic(player) {
  const battleMusic = player.getDynamicProperty("theUnknownBattleMusic");
  if (battleMusic === void 0) return false;
  player.runCommand("music stop 5");
  player.setDynamicProperty("theUnknownBattleMusic", void 0);
  return true;
}
function afterRemove(id) {
  try {
    mc69.world.scoreboard.removeObjective(id);
  } catch {
  }
  for (const player of mc69.world.getPlayers()) {
    stopBattleMusic(player);
  }
}
function onPlayerDie(player) {
  const waveSpawnerNearby = player.dimension.getEntities({
    closest: 1,
    type: UNKNOWN_WAVE_SPAWNER_ENTITY_TYPE,
    maxDistance: 80,
    location: player.location
  })[0];
  if (!waveSpawnerNearby) {
    stopBattleMusic(player);
    return;
  }
  const scoreboardObjective = mc69.world.scoreboard.getObjective(waveSpawnerNearby.id);
  if (!scoreboardObjective) return;
  if (!scoreboardObjective.hasParticipant(player)) return;
  scoreboardObjective.addScore(player, -100);
}
mc69.world.afterEvents.worldLoad.subscribe(() => {
  for (const player of mc69.world.getPlayers()) {
    stopBattleMusic(player);
  }
});
mc69.world.beforeEvents.playerLeave.subscribe((event) => {
  stopBattleMusic(event.player);
});
mc69.world.afterEvents.entitySpawn.subscribe((event) => {
  if (event.entity.typeId !== UNKNOWN_WAVE_SPAWNER_ENTITY_TYPE) return;
  mc69.system.run(() => {
    onSpawn2(event.entity);
  });
});
mc69.world.afterEvents.entityRemove.subscribe(
  (event) => {
    afterRemove(event.removedEntityId);
  },
  {
    entityTypes: [UNKNOWN_WAVE_SPAWNER_ENTITY_TYPE]
  }
);
mc69.world.afterEvents.entityDie.subscribe(
  (event) => {
    onPlayerDie(event.deadEntity);
  },
  {
    entityTypes: ["minecraft:player"]
  }
);
mc69.world.afterEvents.entityHurt.subscribe((event) => {
  try {
    if (!event.damageSource.damagingEntity) return;
    if (!isUnknownRace(event.hurtEntity)) return;
    const waveSpawnerId = ensureType(event.hurtEntity.getDynamicProperty("waveSpawnerId"), "string");
    if (waveSpawnerId === void 0) return;
    const scoreboardObjective = mc69.world.scoreboard.getObjective(waveSpawnerId);
    if (!scoreboardObjective) return;
    const score = clamp2(event.damage, 0, event.hurtEntity.getComponent("health").effectiveMax);
    scoreboardObjective.addScore(event.damageSource.damagingEntity, score);
  } catch {
  }
});
mc69.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    switch (event.eventId) {
      case "the_unknown_wave_spawner:start_wave_break":
        onStartWaveBreakEvent(event.entity);
        break;
      case "the_unknown_wave_spawner:on_finish_wave_break":
        onEndWaveBreakEvent(event.entity);
        break;
      case "the_unknown_wave_spawner:on_tick_wave_battle":
        onTickWaveBattle(event.entity);
        break;
    }
  },
  {
    entityTypes: [UNKNOWN_WAVE_SPAWNER_ENTITY_TYPE],
    eventTypes: [
      "the_unknown_wave_spawner:start_wave_break",
      "the_unknown_wave_spawner:on_finish_wave_break",
      "the_unknown_wave_spawner:on_tick_wave_battle"
    ]
  }
);

// scripts/logic/uncategorized/mobBullet.ts
import { world as world33 } from "@minecraft/server";
function isBlockBreakable(type) {
  if (type === "lc:scpdy_bulletproof_glass") return false;
  if (type.endsWith("glass")) return true;
  if (type.endsWith("glass_pane")) return true;
  if (type.endsWith("sign")) return true;
  return false;
}
world33.afterEvents.entitySpawn.subscribe((event) => {
  if (!event.entity.typeId.startsWith("lc:scpdy_bullet")) return;
  event.entity.dimension.spawnParticle("lc:scpdy_muzzle_flash_particle", event.entity.location);
  event.entity.dimension.spawnParticle("lc:scpdy_muzzle_smoke_emitter", event.entity.location);
});
world33.afterEvents.projectileHitBlock.subscribe((event) => {
  if (!event.projectile.typeId.startsWith("lc:scpdy_bullet")) return;
  if (event.dimension.getPlayers({ location: event.location, closest: 1, maxDistance: 40 })[0] !== void 0) {
    spawnBulletRicochetParticle(event.dimension, event.location, event.hitVector);
  }
  const hitBlock = event.getBlockHit().block;
  if (!world33.gameRules.mobGriefing) return;
  if (!isBlockBreakable(hitBlock.typeId)) return;
  event.dimension.runCommand(`setblock ${toString2(hitBlock.location)} air destroy`);
});

// scripts/logic/uncategorized/testingEquipment.ts
import * as mc70 from "@minecraft/server";
function giveCombatEquipment(player) {
  const armorHead = new mc70.ItemStack("minecraft:diamond_helmet");
  {
    const enchantable = armorHead.getComponent("enchantable");
    enchantable?.addEnchantment({
      level: 1,
      type: new mc70.EnchantmentType("mending")
    });
    enchantable?.addEnchantment({
      level: 3,
      type: new mc70.EnchantmentType("unbreaking")
    });
    enchantable?.addEnchantment({
      level: 4,
      type: new mc70.EnchantmentType("protection")
    });
  }
  const armorChest = new mc70.ItemStack("minecraft:diamond_chestplate");
  {
    const enchantable = armorChest.getComponent("enchantable");
    enchantable?.addEnchantment({
      level: 1,
      type: new mc70.EnchantmentType("mending")
    });
    enchantable?.addEnchantment({
      level: 3,
      type: new mc70.EnchantmentType("unbreaking")
    });
    enchantable?.addEnchantment({
      level: 4,
      type: new mc70.EnchantmentType("blast_protection")
    });
  }
  const armorLegs = new mc70.ItemStack("minecraft:diamond_leggings");
  {
    const enchantable = armorLegs.getComponent("enchantable");
    enchantable?.addEnchantment({
      level: 1,
      type: new mc70.EnchantmentType("mending")
    });
    enchantable?.addEnchantment({
      level: 3,
      type: new mc70.EnchantmentType("unbreaking")
    });
    enchantable?.addEnchantment({
      level: 4,
      type: new mc70.EnchantmentType("projectile_protection")
    });
    enchantable?.addEnchantment({
      level: 2,
      type: new mc70.EnchantmentType("swift_sneak")
    });
  }
  const armorFeet = new mc70.ItemStack("minecraft:diamond_boots");
  {
    const enchantable = armorFeet.getComponent("enchantable");
    enchantable?.addEnchantment({
      level: 1,
      type: new mc70.EnchantmentType("mending")
    });
    enchantable?.addEnchantment({
      level: 3,
      type: new mc70.EnchantmentType("unbreaking")
    });
    enchantable?.addEnchantment({
      level: 4,
      type: new mc70.EnchantmentType("fire_protection")
    });
    enchantable?.addEnchantment({
      level: 4,
      type: new mc70.EnchantmentType("feather_falling")
    });
  }
  const equippable = player.getComponent("equippable");
  equippable.setEquipment(mc70.EquipmentSlot.Head, armorHead);
  equippable.setEquipment(mc70.EquipmentSlot.Chest, armorChest);
  equippable.setEquipment(mc70.EquipmentSlot.Legs, armorLegs);
  equippable.setEquipment(mc70.EquipmentSlot.Feet, armorFeet);
}
mc70.system.afterEvents.scriptEventReceive.subscribe((event) => {
  if (event.id !== "scpdy:get_testing_armors") return;
  if (!event.sourceEntity) return;
  if (!(event.sourceEntity instanceof mc70.Player)) return;
  giveCombatEquipment(event.sourceEntity);
});

// scripts/logic/weapons/guns/mag/index.ts
import * as mc72 from "@minecraft/server";

// scripts/logic/weapons/guns/mag/mag.ts
import * as mc71 from "@minecraft/server";
function onUse5(arg) {
  const { itemStack, source: player } = arg;
  if (!itemStack) return;
  if (player.getItemCooldown("scpdy_mag_swap") > 0) return;
  const equippable = player.getComponent("equippable");
  const mainhandItem = equippable.getEquipment(mc71.EquipmentSlot.Mainhand);
  const offhandItem = equippable.getEquipment(mc71.EquipmentSlot.Offhand);
  if (offhandItem) {
    mc71.system.run(() => {
      equippable.setEquipment(mc71.EquipmentSlot.Mainhand, offhandItem);
    });
  }
  equippable.setEquipment(mc71.EquipmentSlot.Offhand, mainhandItem);
  equippable.setEquipment(mc71.EquipmentSlot.Mainhand, void 0);
  player.dimension.playSound("scpdy.gun.magazine.equip", player.getHeadLocation());
  player.startItemCooldown("scpdy_mag_swap", 5);
}
mc71.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:mag", {
    onUse: onUse5
  });
});

// scripts/logic/weapons/guns/mag/index.ts
function equipMag(args) {
  const { player, inventoryContainer, offhandSlot, magItemTypeId, force } = args;
  if (offhandSlot.hasItem() && offhandSlot.typeId === magItemTypeId) return true;
  const magItemSlot = getContainerSlot(inventoryContainer, (slot) => {
    return slot.hasItem() && slot.typeId === magItemTypeId;
  });
  if (!magItemSlot) return false;
  const offhandItem = offhandSlot.getItem();
  if (!force && offhandItem && !offhandItem.hasTag("scpdy:mag")) return false;
  if (offhandItem) {
    mc72.system.run(() => {
      player.getComponent("inventory")?.container?.addItem(offhandItem);
    });
  }
  const magItem = magItemSlot.getItem();
  offhandSlot.setItem(magItem);
  magItemSlot.setItem();
  player.dimension.playSound("scpdy.gun.magazine.equip", player.location);
  return true;
}

// scripts/logic/weapons/guns/MP5-SD.ts
import * as mc74 from "@minecraft/server";

// scripts/logic/weapons/guns/bullet.ts
import * as mc73 from "@minecraft/server";
var BULLET_TYPE_OBJ = {
  default: "lc:scpdy_scriptable_bullet",
  large: "lc:scpdy_scriptable_bullet_large"
};
var FLYING_BULLET_MAP = /* @__PURE__ */ new Map();
function shootBullet(bulletType, shootOptions) {
  const entity = shootOptions.dimension.spawnEntity(BULLET_TYPE_OBJ[bulletType], shootOptions.initialLocation, {
    initialRotation: shootOptions.initialRotation
  });
  entity.applyImpulse(shootOptions.initialVelocity);
  FLYING_BULLET_MAP.set(entity.id, {
    bulletType,
    shootOptions,
    isInvalid: false,
    sharedState: {
      hitBlockCount: 0,
      hitEntityCount: 0,
      stopCurrentEventSequence: false
    }
  });
}
mc73.world.afterEvents.projectileHitBlock.subscribe((hitEvent) => {
  const flyingBulletInfo = FLYING_BULLET_MAP.get(hitEvent.projectile.id);
  if (!flyingBulletInfo) return;
  if (flyingBulletInfo.isInvalid) {
    try {
      hitEvent.projectile.remove();
    } catch {
    }
    return;
  }
  try {
    flyingBulletInfo.sharedState.stopCurrentEventSequence = false;
    flyingBulletInfo.sharedState.hitBlockCount++;
    const events = flyingBulletInfo.shootOptions.onHitBlock;
    if (!events) return;
    for (let i = 0; i < events.length; i++) {
      const event = events[i];
      if (event.condition !== void 0) {
        if (!event.condition(hitEvent, flyingBulletInfo.sharedState)) continue;
      }
      if (flyingBulletInfo.sharedState.stopCurrentEventSequence) return;
      if (event.type === "callback") {
        event.callback(hitEvent, flyingBulletInfo.sharedState);
      } else if (event.type === "removeBullet") {
        hitEvent.projectile.remove();
        flyingBulletInfo.isInvalid = true;
        return;
      } else if (event.type === "spawnRicochet") {
        spawnBulletRicochetParticle(hitEvent.dimension, hitEvent.location, hitEvent.hitVector);
      }
    }
  } catch (err) {
    flyingBulletInfo.isInvalid = true;
    hitEvent.projectile.remove();
    throw err;
  }
});
mc73.world.afterEvents.projectileHitEntity.subscribe((hitEvent) => {
  const flyingBulletInfo = FLYING_BULLET_MAP.get(hitEvent.projectile.id);
  if (!flyingBulletInfo) return;
  if (flyingBulletInfo.isInvalid) {
    try {
      hitEvent.projectile.remove();
    } catch {
    }
    return;
  }
  try {
    flyingBulletInfo.sharedState.stopCurrentEventSequence = false;
    const hitEntity = hitEvent.getEntityHit().entity;
    if (!hitEntity) return;
    if (hitEntity === flyingBulletInfo.shootOptions.sourceEntity) return;
    flyingBulletInfo.sharedState.hitEntityCount++;
    const events = flyingBulletInfo.shootOptions.onHitEntity;
    if (!events) return;
    for (let i = 0; i < events.length; i++) {
      const event = events[i];
      if (event.condition !== void 0) {
        if (!event.condition(hitEvent, hitEntity, flyingBulletInfo.sharedState)) continue;
      }
      if (flyingBulletInfo.sharedState.stopCurrentEventSequence) return;
      if (event.type === "callback") {
        event.callback(hitEvent, hitEntity, flyingBulletInfo.sharedState);
      } else if (event.type === "removeBullet") {
        hitEvent.projectile.remove();
        flyingBulletInfo.isInvalid = true;
        return;
      } else if (event.type === "damageEntity") {
        const oldVelocity = hitEntity.getVelocity();
        const damage = event.canDamageBeModified === true ? Math.max(1, getModifiedDamageNumber(event.damage, hitEntity)) : event.damage;
        hitEntity.applyDamage(damage, {
          cause: event.damageCause,
          damagingEntity: flyingBulletInfo.shootOptions.sourceEntity
        });
        if (CONFIG.disableKnockbackOfCertainBullets) {
          hitEntity.clearVelocity();
          const revelo = {
            x: clamp2(oldVelocity.x / 3, -0.5, 0.5),
            y: 0,
            z: clamp2(oldVelocity.z / 3, -0.5, 0.5)
          };
          hitEntity.applyImpulse(revelo);
          if (event.knockbackPower === void 0) continue;
          const impulse = chain(FORWARD).scale(event.knockbackPower).changeDir(hitEvent.hitVector).done();
          hitEntity.applyImpulse(impulse);
        }
      } else if (event.type === "spawnRicochet") {
        spawnBulletRicochetParticle(hitEvent.dimension, hitEvent.location, hitEvent.hitVector);
      }
    }
  } catch (err) {
    flyingBulletInfo.isInvalid = true;
    hitEvent.projectile.remove();
    throw err;
  }
});
mc73.world.afterEvents.worldLoad.subscribe(() => {
  const scriptableBullets = getEntitiesInAllDimensions({
    families: ["scriptable_bullet"]
  });
  for (let i = 0; i < scriptableBullets.length; i++) {
    const bullet = scriptableBullets[i];
    bullet.remove();
  }
});
mc73.world.afterEvents.entityLoad.subscribe((event) => {
  if (!FLYING_BULLET_MAP.has(event.entity.id)) return;
  event.entity.remove();
});
mc73.world.afterEvents.entityRemove.subscribe((event) => {
  FLYING_BULLET_MAP.delete(event.removedEntityId);
});

// scripts/logic/weapons/guns/commonBulletHitEvents.ts
var BREAK_GLASS_AND_END_SEQUENCE = {
  type: "callback",
  condition(event, sharedState) {
    const hitBlock = event.getBlockHit().block;
    if (hitBlock.typeId === "minecraft:glass") return true;
    if (hitBlock.typeId === "minecraft:glass_pane") return true;
    if (hitBlock.typeId.startsWith("minecraft:") && (hitBlock.typeId.endsWith("stained_glass") || hitBlock.typeId.endsWith("stained_glass_pane"))) {
      return true;
    }
    return false;
  },
  callback(event, sharedState) {
    const hitBlock = event.getBlockHit().block;
    const xyz = toString2(hitBlock.location);
    event.dimension.runCommand(`setblock ${xyz} air destroy`);
    sharedState.stopCurrentEventSequence = true;
    event.projectile.remove();
  }
};

// scripts/logic/weapons/guns/shared.ts
function getAmmoDisplayText(magAmmo, magAmmoMax, invAmmo) {
  const lowAmmoCount = Math.floor(Math.max(2, Math.min(15, magAmmoMax / 3.5)));
  const magAmmoColor = magAmmo <= 0 ? "\xA7c" : magAmmo < magAmmoMax / lowAmmoCount ? "\xA7e" : "\xA7f";
  const invAmmoColor = invAmmo <= 0 ? "\xA77" : "\xA7f";
  const text = `${magAmmoColor}${magAmmo}\xA7f | ${invAmmoColor}${invAmmo}`;
  return text;
}

// scripts/logic/weapons/guns/MP5-SD.ts
var MAG_ITEM_TYPE_ID = "lc:scpdy_gun_mp5_mag";
var PICK_DURATION = 10;
var AIM_DURATION = 3;
var COOLDOWN_IDS = {
  pick: "scpdy_gun_mp5sd_pick",
  shootScript: "scpdy_gun_mp5sd_shoot_script",
  shoot1: "scpdy_gun_mp5sd_shoot_1",
  shoot2: "scpdy_gun_mp5sd_shoot_2",
  reload: "scpdy_gun_mp5sd_reload",
  reloadTac: "scpdy_gun_mp5sd_reload_tac"
};
registerAdvancedItemProfile({
  itemTypeId: "lc:scpdy_gun_mp5sd",
  createInstance: (args) => new MP5SD(args)
});
var MP5SD = class extends AdvancedItem {
  playerInventoryContainer;
  aimTick = 0;
  reloadData;
  wasCrosshairHidden = false;
  isUsingToShoot = false;
  shotsFired = 0;
  tpAnimVars = {
    isADS: false,
    wasADS: false,
    ticksUntilADSTransitionEnd: 0,
    ticksUntilStopADS: 0,
    ticksUntilSpecialAnimTimeEnd: 0
  };
  /**
   * Enabling this flag will start gun reload on next tick if it meets condition
   */
  tryReloadingNextTick = false;
  constructor(args) {
    super(args);
    this.playerInventoryContainer = args.player.getComponent("inventory").container;
    args.player.startItemCooldown(COOLDOWN_IDS.pick, 2);
    equipMag({
      player: this.player,
      inventoryContainer: this.playerInventoryContainer,
      offhandSlot: this.playerOffhand,
      magItemTypeId: MAG_ITEM_TYPE_ID,
      force: false
    });
    args.player.playAnimation("animation.scpdy_player.mp5.pick");
    this.tpAnimVars.ticksUntilSpecialAnimTimeEnd = 8;
  }
  onTick(mainhandItemStack) {
    this.onTick_1();
    this.onTick_TPAnim();
  }
  onTick_TPAnim() {
    if (this.tpAnimVars.ticksUntilSpecialAnimTimeEnd > 0) {
      this.tpAnimVars.ticksUntilSpecialAnimTimeEnd--;
    }
    if (this.tpAnimVars.ticksUntilADSTransitionEnd > 0) {
      this.tpAnimVars.ticksUntilADSTransitionEnd--;
    }
    if (this.tpAnimVars.ticksUntilStopADS > 0) {
      this.tpAnimVars.ticksUntilStopADS--;
      if (this.tpAnimVars.ticksUntilStopADS <= 0) this.tpAnimVars.isADS = false;
      else if (!this.tpAnimVars.isADS) this.tpAnimVars.isADS = true;
    }
    if (this.tpAnimVars.isADS && !this.tpAnimVars.wasADS) {
      this.playTPAnimIfNotDuringSpecialAnimTime("animation.scpdy_player.mp5.ready_to_aim");
      this.tpAnimVars.ticksUntilADSTransitionEnd = 4;
      this.tpAnimVars.wasADS = true;
    } else if (!this.tpAnimVars.isADS && this.tpAnimVars.wasADS) {
      this.playTPAnimIfNotDuringSpecialAnimTime("animation.scpdy_player.mp5.aim_to_ready");
      this.tpAnimVars.ticksUntilADSTransitionEnd = 4;
      this.tpAnimVars.wasADS = false;
    }
    if (this.tpAnimVars.ticksUntilADSTransitionEnd <= 0) {
      this.playTPAnimIfNotDuringSpecialAnimTime(
        this.tpAnimVars.isADS ? "animation.scpdy_player.mp5.aim" : "animation.scpdy_player.mp5.ready"
      );
    }
  }
  playTPAnimIfNotDuringSpecialAnimTime(id) {
    if (this.tpAnimVars.ticksUntilSpecialAnimTimeEnd > 0) return;
    this.player.playAnimation(id);
  }
  onTick_1() {
    const cdReload = this.player.getItemCooldown(COOLDOWN_IDS.reload);
    const cdReloadTac = this.player.getItemCooldown(COOLDOWN_IDS.reloadTac);
    const dontAim = cdReload > 0 || cdReloadTac > 0;
    if (dontAim) {
      this.aimTick = this.aimTick > 0 ? this.aimTick - 1 : 0;
      if (this.tpAnimVars.isADS) this.tpAnimVars.ticksUntilStopADS = 1;
    } else {
      this.aimTick = this.player.isSneaking ? this.aimTick < AIM_DURATION ? this.aimTick + 1 : AIM_DURATION : this.aimTick > 0 ? this.aimTick - 1 : 0;
      if (this.tpAnimVars.ticksUntilSpecialAnimTimeEnd <= 0) {
        if (this.player.isSneaking) {
          this.tpAnimVars.isADS = true;
          this.tpAnimVars.ticksUntilStopADS = 0;
        } else if (this.tpAnimVars.isADS) {
          this.tpAnimVars.ticksUntilStopADS = 1;
        }
      }
    }
    const isADS = this.aimTick >= AIM_DURATION;
    if (isADS) {
      this.player.addEffect("slowness", 3, {
        amplifier: 0,
        showParticles: false
      });
      if (!this.wasCrosshairHidden) {
        this.player.onScreenDisplay.setHudVisibility(mc74.HudVisibility.Hide, [mc74.HudElement.Crosshair]);
        this.wasCrosshairHidden = true;
      }
    } else if (this.wasCrosshairHidden) {
      this.player.onScreenDisplay.setHudVisibility(mc74.HudVisibility.Reset, [mc74.HudElement.Crosshair]);
      this.wasCrosshairHidden = false;
    }
    const magItemStack = this.playerOffhand.getItem();
    if (!magItemStack || magItemStack.typeId !== MAG_ITEM_TYPE_ID) {
      if (this.reloadData) {
        this.reloadData = void 0;
        this.player.startItemCooldown(COOLDOWN_IDS.reload, 0);
        this.player.startItemCooldown(COOLDOWN_IDS.reloadTac, 0);
      }
      this.player.onScreenDisplay.setActionBar({
        translate: "scpdy.actionHint.gun.noMagInInventory"
      });
      return;
    }
    const magAmmoType = getAmmoType(magItemStack);
    const magDurabilityComp = magItemStack.getComponent("durability");
    const magAmmoCountNow = magDurabilityComp.maxDurability - magDurabilityComp.damage;
    const invAmmoCountNow = getTotalAmmoCount(this.playerInventoryContainer, magAmmoType);
    const ammoDisplayText = getAmmoDisplayText(magAmmoCountNow, magDurabilityComp.maxDurability, invAmmoCountNow);
    this.player.onScreenDisplay.setActionBar(`${ammoDisplayText}`);
    if (this.currentTick < PICK_DURATION) return;
    if (this.player.getItemCooldown(COOLDOWN_IDS.shootScript) > 0) return;
    if (this.isUsingToShoot) {
      if (magAmmoCountNow <= 0) {
        this.isUsingToShoot = false;
        this.shotsFired = 0;
        this.tryReloadingNextTick = true;
        return;
      }
      if (this.player.getItemCooldown(COOLDOWN_IDS.shootScript) > 0) return;
      this.shotsFired++;
      this.shoot(isADS);
      magDurabilityComp.damage++;
      this.playerOffhand.setItem(magItemStack);
      return;
    }
    if (this.tryReloadingNextTick) {
      this.tryReloadingNextTick = false;
      if (this.reloadData) return;
      if (invAmmoCountNow <= 0) return;
      if (magDurabilityComp.damage <= 0) return;
      this.reloadData = {
        tac: magAmmoCountNow > 0,
        tick: 0
      };
    }
    if (this.reloadData) {
      if (magAmmoCountNow <= 0 && invAmmoCountNow <= 0) {
        this.reloadData = void 0;
        this.player.startItemCooldown(COOLDOWN_IDS.reload, 0);
        this.player.startItemCooldown(COOLDOWN_IDS.reloadTac, 0);
        return;
      }
      if (this.reloadData.tac) {
        if (this.reloadData.tick === 0) {
          this.player.startItemCooldown(COOLDOWN_IDS.reloadTac, 32);
          this.player.playAnimation("animation.scpdy_player.mp5.reload_tac");
          this.tpAnimVars.ticksUntilSpecialAnimTimeEnd = 32;
        }
        if (this.reloadData.tick === 4) {
          this.player.runCommand("camerashake add @s 0.01 0.2 rotational");
          this.playSoundAtHead("scpdy.gun.mp5sd.mag_remove", { volume: 1.2 });
        }
        if (this.reloadData.tick === 18) {
          this.player.runCommand("camerashake add @s 0.02 0.08 rotational");
          this.playSoundAtHead("scpdy.gun.mp5sd.mag_attach", { volume: 1.2 });
          const reloadAmount = removeAmmo(this.playerInventoryContainer, magAmmoType, magDurabilityComp.damage);
          magDurabilityComp.damage -= reloadAmount;
          this.playerOffhand.setItem(magItemStack);
        }
        if (this.reloadData.tick === 32) {
          this.reloadData = void 0;
          return;
        }
      } else {
        if (this.reloadData.tick === 0) {
          this.player.startItemCooldown(COOLDOWN_IDS.reload, 52);
          this.player.playAnimation("animation.scpdy_player.mp5.reload");
          this.tpAnimVars.ticksUntilSpecialAnimTimeEnd = 52;
        }
        if (this.reloadData.tick === 2) {
          this.player.runCommand("camerashake add @s 0.01 0.1 rotational");
          this.playSoundAtHead("scpdy.gun.mp5sd.ch_pull", { volume: 1.2 });
        }
        if (this.reloadData.tick === 10) {
          this.player.runCommand("camerashake add @s 0.01 0.2 rotational");
          this.playSoundAtHead("scpdy.gun.mp5sd.mag_remove", { volume: 1.2 });
        }
        if (this.reloadData.tick === 27) {
          this.player.runCommand("camerashake add @s 0.015 0.1 rotational");
          this.playSoundAtHead("scpdy.gun.mp5sd.mag_attach", { volume: 1.2 });
        }
        if (this.reloadData.tick === 37) {
          this.player.runCommand("camerashake add @s 0.02 0.1 rotational");
          this.playSoundAtHead("scpdy.gun.mp5sd.ch_slap", { volume: 1.2 });
          const reloadAmount = removeAmmo(this.playerInventoryContainer, magAmmoType, magDurabilityComp.damage);
          magDurabilityComp.damage -= reloadAmount;
          this.playerOffhand.setItem(magItemStack);
        }
        if (this.reloadData.tick === 53) {
          this.reloadData = void 0;
          return;
        }
      }
      this.reloadData.tick++;
      return;
    }
  }
  onRemove() {
    this.player.onScreenDisplay.setHudVisibility(mc74.HudVisibility.Reset, [mc74.HudElement.Crosshair]);
    this.player.playAnimation("animation.scpdy_player.mp5.remove");
  }
  isUsable() {
    if (this.currentTick < PICK_DURATION) return false;
    if (this.tryReloadingNextTick) return false;
    if (this.reloadData) return false;
    if (this.player.getItemCooldown(COOLDOWN_IDS.shootScript) > 0) return false;
    return true;
  }
  onStartUse(event) {
    const equipMagResult = equipMag({
      player: this.player,
      inventoryContainer: this.playerInventoryContainer,
      offhandSlot: this.playerOffhand,
      magItemTypeId: MAG_ITEM_TYPE_ID,
      force: true
    });
    if (!equipMagResult) return;
    this.player.playSound("scpdy.gun.trigger_click");
    const magItemStack = this.playerOffhand.getItem();
    if (!magItemStack) return;
    const magDurabilityComp = magItemStack.getComponent("durability");
    if (!magDurabilityComp) return;
    const magAmmoCountNow = magDurabilityComp.maxDurability - magDurabilityComp.damage;
    if (magAmmoCountNow <= 0) {
      this.tryReloadingNextTick = true;
      return;
    }
    this.isUsingToShoot = true;
  }
  onStopUse(event) {
    this.isUsingToShoot = false;
    this.shotsFired = 0;
  }
  onSwingArm() {
    if (CONFIG.gunTacReloadTrigger !== 2) return;
    if (this.tryReloadingNextTick) return;
    this.tryReloadingNextTick = true;
  }
  onHitEntity(event) {
    if (CONFIG.gunTacReloadTrigger !== 1) return;
    if (this.tryReloadingNextTick) return;
    this.tryReloadingNextTick = true;
  }
  onHitBlock(event) {
    if (CONFIG.gunTacReloadTrigger !== 1) return;
    if (this.tryReloadingNextTick) return;
    this.tryReloadingNextTick = true;
  }
  playSoundAtHead(soundId, options) {
    this.player.dimension.playSound(
      soundId,
      add(this.player.getHeadLocation(), this.player.getViewDirection()),
      options
    );
  }
  getMuzzleLocation(ads) {
    const move = ads ? {
      x: 0,
      y: -0.12,
      z: 1.3
    } : {
      x: 0.13,
      y: 0,
      z: 1.6
    };
    const muzzleLoc = add(
      getRelativeToHead(this.player.getHeadLocation(), this.player.getViewDirection(), move),
      this.player.getVelocity()
    );
    return muzzleLoc;
  }
  shoot(ads) {
    const shotsFired = this.shotsFired;
    this.player.startItemCooldown(COOLDOWN_IDS.shootScript, 1.5);
    if (this.player.getItemCooldown(COOLDOWN_IDS.shoot1) > 0) {
      this.player.startItemCooldown(COOLDOWN_IDS.shoot1, 0);
      this.player.startItemCooldown(COOLDOWN_IDS.shoot2, 15);
    } else {
      this.player.startItemCooldown(COOLDOWN_IDS.shoot2, 0);
      this.player.startItemCooldown(COOLDOWN_IDS.shoot1, 15);
    }
    this.player.playAnimation("animation.scpdy_player.mp5.shoot");
    this.tpAnimVars.ticksUntilSpecialAnimTimeEnd = 9;
    this.tpAnimVars.ticksUntilStopADS = 600;
    const bulletSpread = shotsFired === 1 ? 0 : (0.01 + Math.min(0.15, (ads ? 0.02 : 0.03) * shotsFired) + Math.min(0.1, length(this.player.getVelocity()) / 4)) * 0.23;
    const shootBulletVelocity = chain(FORWARD).scale(9.3).changeDir(this.player.getViewDirection()).rotateRad(random(), randomFloat(-bulletSpread, bulletSpread)).done();
    shootBullet("default", {
      dimension: this.player.dimension,
      initialLocation: this.player.getHeadLocation(),
      initialVelocity: shootBulletVelocity,
      sourceEntity: this.player,
      onHitBlock: [
        BREAK_GLASS_AND_END_SEQUENCE,
        {
          type: "spawnRicochet"
        },
        {
          type: "removeBullet"
        }
      ],
      onHitEntity: [
        {
          type: "damageEntity",
          damage: 2,
          damageCause: mc74.EntityDamageCause.override,
          canDamageBeModified: true,
          condition(event, hitEntity, sharedState) {
            sharedState.stopCurrentEventSequence = true;
            if (!mc74.world.gameRules.pvp && hitEntity instanceof mc74.Player) return false;
            sharedState.stopCurrentEventSequence = false;
            return true;
          }
        },
        {
          type: "removeBullet"
        }
      ]
    });
    const muzzleLoc = this.getMuzzleLocation(ads);
    this.player.dimension.spawnParticle("lc:scpdy_muzzle_smoke_emitter", muzzleLoc);
    {
      const particleLoc = getRelativeToHead(
        this.player.getHeadLocation(),
        this.player.getViewDirection(),
        ads ? {
          x: 0.05,
          y: 0.02,
          z: 0.67
        } : {
          x: 0.3,
          y: -0.1,
          z: 1
        }
      );
      const viewDirection = this.player.getViewDirection();
      const rightDirection = cross(viewDirection, UP);
      const particleDirection = add(
        add(
          scale(viewDirection, -randomFloat(1.3, 1.5)),
          // Go backwards
          scale(rightDirection, randomFloat(0.6, 0.7))
          // Go right
        ),
        fromPartial({
          x: randomFloat(-0.1, 0.1),
          y: randomFloat(1.1, 1.3),
          // Go up
          z: randomFloat(-0.1, 0.1)
        })
      );
      const molangVarMap = new mc74.MolangVariableMap();
      molangVarMap.setFloat("speed", 6);
      molangVarMap.setVector3("direction", particleDirection);
      this.player.dimension.spawnParticle("lc:scpdy_bullet_casing_drop_var0_particle", particleLoc, molangVarMap);
    }
    this.playSoundAtHead("scpdy.gun.mp5sd.shoot", {
      volume: 1.4,
      pitch: randomFloat(0.95, 1.05)
    });
    const camShakeAmount = Math.min(0.1, 0.01 + (ads ? 5e-4 : 1e-3) * this.shotsFired);
    this.player.runCommand(`camerashake add @s ${camShakeAmount} 0.05 rotational`);
  }
};

// scripts/logic/weapons/guns/T-5000.ts
import * as mc75 from "@minecraft/server";
var MAG_ITEM_TYPE_ID2 = "lc:scpdy_gun_t5000_mag";
var PICK_DURATION2 = 15;
var AIM_DURATION2 = 5;
var COOLDOWN_IDS2 = {
  pick: "scpdy_gun_t5000_pick",
  shoot: "scpdy_gun_t5000_shoot",
  boltCycle: "scpdy_gun_t5000_bolt_cycle",
  reload: "scpdy_gun_t5000_reload",
  reloadTac: "scpdy_gun_t5000_reload_tac"
};
registerAdvancedItemProfile({
  itemTypeId: "lc:scpdy_gun_t5000",
  createInstance: (args) => new T5000(args)
});
var T5000 = class extends AdvancedItem {
  playerInventoryContainer;
  aimTick = 0;
  boltCycleData;
  reloadData;
  tpAnimVars = {
    isADS: false,
    wasADS: false,
    ticksUntilADSTransitionEnd: 0,
    ticksUntilStopADS: 0,
    ticksUntilSpecialAnimTimeEnd: 0
  };
  /**
   * Enabling this flag will start gun reload on next tick if it meets condition
   */
  tryReloadingNextTick = false;
  constructor(args) {
    super(args);
    this.playerInventoryContainer = args.player.getComponent("inventory").container;
    args.player.startItemCooldown(COOLDOWN_IDS2.pick, 2);
    args.player.onScreenDisplay.setHudVisibility(mc75.HudVisibility.Hide, [mc75.HudElement.Crosshair]);
    equipMag({
      player: this.player,
      inventoryContainer: this.playerInventoryContainer,
      offhandSlot: this.playerOffhand,
      magItemTypeId: MAG_ITEM_TYPE_ID2,
      force: false
    });
    args.player.playAnimation("animation.scpdy_player.t5000.pick");
    this.tpAnimVars.ticksUntilSpecialAnimTimeEnd = 8;
  }
  onTick(mainhandItemStack) {
    this.onTick_1(mainhandItemStack);
    this.onTick_TPAnim();
  }
  onTick_TPAnim() {
    if (this.tpAnimVars.ticksUntilSpecialAnimTimeEnd > 0) {
      this.tpAnimVars.ticksUntilSpecialAnimTimeEnd--;
    }
    if (this.tpAnimVars.ticksUntilADSTransitionEnd > 0) {
      this.tpAnimVars.ticksUntilADSTransitionEnd--;
    }
    if (this.tpAnimVars.ticksUntilStopADS > 0) {
      this.tpAnimVars.ticksUntilStopADS--;
      if (this.tpAnimVars.ticksUntilStopADS <= 0) this.tpAnimVars.isADS = false;
      else if (!this.tpAnimVars.isADS) this.tpAnimVars.isADS = true;
    }
    if (this.tpAnimVars.isADS && !this.tpAnimVars.wasADS) {
      this.playTPAnimIfNotDuringSpecialAnimTime("animation.scpdy_player.t5000.ready_to_aim");
      this.tpAnimVars.ticksUntilADSTransitionEnd = 4;
      this.tpAnimVars.wasADS = true;
    } else if (!this.tpAnimVars.isADS && this.tpAnimVars.wasADS) {
      this.playTPAnimIfNotDuringSpecialAnimTime("animation.scpdy_player.t5000.aim_to_ready");
      this.tpAnimVars.ticksUntilADSTransitionEnd = 4;
      this.tpAnimVars.wasADS = false;
    }
    if (this.tpAnimVars.ticksUntilADSTransitionEnd <= 0) {
      this.playTPAnimIfNotDuringSpecialAnimTime(
        this.tpAnimVars.isADS ? "animation.scpdy_player.t5000.aim" : "animation.scpdy_player.t5000.ready"
      );
    }
  }
  playTPAnimIfNotDuringSpecialAnimTime(id) {
    if (this.tpAnimVars.ticksUntilSpecialAnimTimeEnd > 0) return;
    this.player.playAnimation(id);
  }
  onTick_1(mainhandItemStack) {
    const cdReload = this.player.getItemCooldown(COOLDOWN_IDS2.reload);
    const cdReloadTac = this.player.getItemCooldown(COOLDOWN_IDS2.reloadTac);
    const dontAim = cdReload > 0 || cdReloadTac > 0;
    if (dontAim) {
      this.aimTick = this.aimTick > 0 ? this.aimTick - 1 : 0;
      if (this.tpAnimVars.isADS) this.tpAnimVars.ticksUntilStopADS = 1;
    } else {
      this.aimTick = this.player.isSneaking ? this.aimTick < AIM_DURATION2 ? this.aimTick + 1 : AIM_DURATION2 : this.aimTick > 0 ? this.aimTick - 1 : 0;
      if (this.tpAnimVars.ticksUntilSpecialAnimTimeEnd <= 0) {
        if (this.player.isSneaking) {
          this.tpAnimVars.isADS = true;
          this.tpAnimVars.ticksUntilStopADS = 0;
        } else if (this.tpAnimVars.isADS) {
          this.tpAnimVars.ticksUntilStopADS = 1;
        }
      }
    }
    const isADS = this.aimTick >= AIM_DURATION2;
    if (isADS) {
      const movementVector = this.player.inputInfo.getMovementVector();
      const length2 = Math.sqrt(movementVector.x ** 2 + movementVector.y ** 2);
      this.player.addEffect("slowness", 3, {
        amplifier: length2 > 0.3 ? 3 : 8,
        showParticles: false
      });
    }
    const magItemStack = this.playerOffhand.getItem();
    if (!magItemStack || magItemStack.typeId !== MAG_ITEM_TYPE_ID2) {
      if (this.reloadData) {
        this.reloadData = void 0;
        this.player.startItemCooldown(COOLDOWN_IDS2.reload, 0);
        this.player.startItemCooldown(COOLDOWN_IDS2.reloadTac, 0);
      }
      this.player.onScreenDisplay.setActionBar({
        translate: "scpdy.actionHint.gun.noMagInInventory"
      });
      return;
    }
    const magAmmoType = getAmmoType(magItemStack);
    const magDurabilityComp = magItemStack.getComponent("durability");
    const magAmmoCountNow = magDurabilityComp.maxDurability - magDurabilityComp.damage;
    const invAmmoCountNow = getTotalAmmoCount(this.playerInventoryContainer, magAmmoType);
    const ammoDisplayText = getAmmoDisplayText(magAmmoCountNow, magDurabilityComp.maxDurability, invAmmoCountNow);
    this.player.onScreenDisplay.setActionBar(`${ammoDisplayText}`);
    if (this.currentTick < PICK_DURATION2) return;
    if (this.player.getItemCooldown(COOLDOWN_IDS2.shoot) > 0) return;
    if (!this.boltCycleData && this.getIsBoltCycleNeeded(mainhandItemStack)) {
      this.boltCycleData = {
        tick: 0
      };
    }
    if (this.boltCycleData) {
      if (this.boltCycleData.tick === 0) {
        this.player.startItemCooldown(COOLDOWN_IDS2.boltCycle, 18);
        this.player.playAnimation("animation.scpdy_player.t5000.bolt");
        this.tpAnimVars.ticksUntilSpecialAnimTimeEnd = 20;
        this.tpAnimVars.ticksUntilStopADS = 600;
      }
      if (this.boltCycleData.tick === 3) {
        this.playSoundAtHead("scpdy.gun.t5000.bolt_open");
      }
      if (this.boltCycleData.tick === 7) {
        this.player.runCommand("camerashake add @s 0.025 0.08 rotational");
      }
      if (this.boltCycleData.tick === 9) {
        this.playSoundAtHead("scpdy.gun.t5000.bolt_close");
      }
      if (this.boltCycleData.tick === 12) {
        this.player.runCommand("camerashake add @s 0.025 0.11 rotational");
      }
      if (this.boltCycleData.tick === 18) {
        this.boltCycleData = void 0;
        this.setIsBoltCycleNeeded(mainhandItemStack, void 0);
        this.playerMainhand.setItem(mainhandItemStack);
        return;
      }
      this.boltCycleData.tick++;
      return;
    }
    if (this.tryReloadingNextTick) {
      this.tryReloadingNextTick = false;
      if (this.reloadData) return;
      if (invAmmoCountNow <= 0) return;
      if (magDurabilityComp.damage <= 0) return;
      this.reloadData = {
        tac: magAmmoCountNow > 0,
        tick: 0
      };
    }
    if (this.reloadData) {
      if (magAmmoCountNow <= 0 && invAmmoCountNow <= 0) {
        this.reloadData = void 0;
        this.player.startItemCooldown(COOLDOWN_IDS2.reload, 0);
        this.player.startItemCooldown(COOLDOWN_IDS2.reloadTac, 0);
        return;
      }
      if (this.reloadData.tac) {
        if (this.reloadData.tick === 0) {
          this.player.startItemCooldown(COOLDOWN_IDS2.reloadTac, 32);
          this.player.playAnimation("animation.scpdy_player.t5000.reload_tac");
          this.tpAnimVars.ticksUntilSpecialAnimTimeEnd = 32;
        }
        if (this.reloadData.tick === 6) {
          this.playSoundAtHead("scpdy.gun.t5000.mag_remove");
        }
        if (this.reloadData.tick === 18) {
          this.playSoundAtHead("scpdy.gun.t5000.mag_attach");
        }
        if (this.reloadData.tick === 20) {
          this.player.runCommand("camerashake add @s 0.02 0.08 rotational");
          const reloadAmount = removeAmmo(this.playerInventoryContainer, magAmmoType, magDurabilityComp.damage);
          magDurabilityComp.damage -= reloadAmount;
          this.playerOffhand.setItem(magItemStack);
        }
        if (this.reloadData.tick === 32) {
          this.reloadData = void 0;
          return;
        }
      } else {
        if (this.reloadData.tick === 0) {
          this.player.startItemCooldown(COOLDOWN_IDS2.reload, 52);
          this.player.playAnimation("animation.scpdy_player.t5000.reload");
          this.tpAnimVars.ticksUntilSpecialAnimTimeEnd = 52;
        }
        if (this.reloadData.tick === 1) {
          this.playSoundAtHead("scpdy.gun.t5000.bolt_open");
        }
        if (this.reloadData.tick === 6) {
          this.player.runCommand("camerashake add @s 0.01 0.1 rotational");
        }
        if (this.reloadData.tick === 14) {
          this.playSoundAtHead("scpdy.gun.t5000.mag_remove");
        }
        if (this.reloadData.tick === 16) {
          this.player.runCommand("camerashake add @s 0.01 0.2 rotational");
        }
        if (this.reloadData.tick === 27) {
          this.playSoundAtHead("scpdy.gun.t5000.mag_attach");
        }
        if (this.reloadData.tick === 28) {
          this.player.runCommand("camerashake add @s 0.015 0.1 rotational");
        }
        if (this.reloadData.tick === 36) {
          this.playSoundAtHead("scpdy.gun.t5000.bolt_close");
        }
        if (this.reloadData.tick === 39) {
          this.player.runCommand("camerashake add @s 0.02 0.2 rotational");
        }
        if (this.reloadData.tick === 44) {
          const reloadAmount = removeAmmo(this.playerInventoryContainer, magAmmoType, magDurabilityComp.damage);
          magDurabilityComp.damage -= reloadAmount;
          this.playerOffhand.setItem(magItemStack);
        }
        if (this.reloadData.tick === 52) {
          this.reloadData = void 0;
          return;
        }
      }
      this.reloadData.tick++;
      return;
    }
  }
  onRemove() {
    this.player.onScreenDisplay.setHudVisibility(mc75.HudVisibility.Reset, [mc75.HudElement.Crosshair]);
    this.player.playAnimation("animation.scpdy_player.t5000.remove");
  }
  isUsable(event) {
    if (this.currentTick < PICK_DURATION2) return false;
    if (this.tryReloadingNextTick) return false;
    if (this.boltCycleData) return false;
    if (this.reloadData) return false;
    if (this.getIsBoltCycleNeeded(event.itemStack)) return false;
    if (this.player.getItemCooldown(COOLDOWN_IDS2.shoot) > 0) return false;
    return true;
  }
  onStartUse(event) {
    const equipMagResult = equipMag({
      player: this.player,
      inventoryContainer: this.playerInventoryContainer,
      offhandSlot: this.playerOffhand,
      magItemTypeId: MAG_ITEM_TYPE_ID2,
      force: true
    });
    if (!equipMagResult) return;
    const magItemStack = this.playerOffhand.getItem();
    const magDurabilityComp = magItemStack.getComponent("durability");
    const magAmmoCountNow = magDurabilityComp.maxDurability - magDurabilityComp.damage;
    if (magAmmoCountNow <= 0) {
      this.tryReloadingNextTick = true;
      return;
    }
    this.shoot(this.aimTick >= AIM_DURATION2);
    if (magAmmoCountNow > 1) {
      this.setIsBoltCycleNeeded(event.itemStack, true);
      this.playerMainhand.setItem(event.itemStack);
    }
    magDurabilityComp.damage++;
    this.playerOffhand.setItem(magItemStack);
  }
  onStopUse(event) {
  }
  onSwingArm() {
    if (CONFIG.gunTacReloadTrigger !== 2) return;
    if (this.tryReloadingNextTick) return;
    this.tryReloadingNextTick = true;
  }
  onHitEntity(event) {
    if (CONFIG.gunTacReloadTrigger !== 1) return;
    if (this.tryReloadingNextTick) return;
    this.tryReloadingNextTick = true;
  }
  onHitBlock(event) {
    if (CONFIG.gunTacReloadTrigger !== 1) return;
    if (this.tryReloadingNextTick) return;
    this.tryReloadingNextTick = true;
  }
  playSoundAtHead(soundId, options) {
    this.player.dimension.playSound(
      soundId,
      add(this.player.getHeadLocation(), this.player.getViewDirection()),
      options
    );
  }
  getIsBoltCycleNeeded(itemStack) {
    return itemStack.getDynamicProperty("isBoltCycleNeeded") === true;
  }
  setIsBoltCycleNeeded(itemStack, value) {
    itemStack.setDynamicProperty("isBoltCycleNeeded", value);
  }
  getMuzzleLocation(ads) {
    const move = ads ? {
      x: 0,
      y: -0.1,
      z: 1.3
    } : {
      x: 0.13,
      y: 0.026,
      z: 1.2
    };
    const muzzleLoc = add(
      getRelativeToHead(this.player.getHeadLocation(), this.player.getViewDirection(), move),
      this.player.getVelocity()
    );
    return muzzleLoc;
  }
  shoot(ads) {
    this.player.startItemCooldown(COOLDOWN_IDS2.boltCycle, 0);
    this.player.startItemCooldown(COOLDOWN_IDS2.shoot, 5);
    this.player.playAnimation("animation.scpdy_player.t5000.shoot");
    this.tpAnimVars.ticksUntilSpecialAnimTimeEnd = 9;
    this.tpAnimVars.ticksUntilStopADS = 600;
    const bulletSpread = ads ? 0 : (0.1 + Math.min(0.5, length(this.player.getVelocity()))) * 0.5;
    const shootBulletVelocity = chain(FORWARD).scale(15).changeDir(this.player.getViewDirection()).rotateRad(random(), randomFloat(-bulletSpread, bulletSpread)).done();
    shootBullet("default", {
      dimension: this.player.dimension,
      initialLocation: this.player.getHeadLocation(),
      initialVelocity: shootBulletVelocity,
      sourceEntity: this.player,
      onHitBlock: [
        BREAK_GLASS_AND_END_SEQUENCE,
        {
          type: "spawnRicochet"
        },
        {
          type: "removeBullet"
        }
      ],
      onHitEntity: [
        {
          type: "damageEntity",
          damage: 24,
          damageCause: mc75.EntityDamageCause.override,
          canDamageBeModified: true,
          knockbackPower: 1,
          condition(event, hitEntity, sharedState) {
            sharedState.stopCurrentEventSequence = true;
            if (!mc75.world.gameRules.pvp && hitEntity instanceof mc75.Player) return false;
            sharedState.stopCurrentEventSequence = false;
            return true;
          }
        },
        {
          type: "removeBullet"
        }
      ]
    });
    const muzzleLoc = this.getMuzzleLocation(ads);
    this.player.dimension.spawnParticle("lc:scpdy_muzzle_flash_particle", muzzleLoc);
    const soundListeners = this.player.dimension.getPlayers({
      closest: 20,
      location: this.player.location,
      maxDistance: 80
    });
    const soundOrigin = muzzleLoc;
    for (let i = 0; i < soundListeners.length; i++) {
      const listener = soundListeners[i];
      const dist = distance(soundOrigin, listener.location);
      if (dist < 40) {
        listener.playSound("scpdy.gun.t5000.shoot_nearby", {
          location: soundOrigin,
          volume: 3
        });
      } else {
        listener.playSound("scpdy.gun.t5000.shoot_distant", {
          location: soundOrigin,
          volume: 5
        });
      }
    }
    this.player.runCommand("camerashake add @s 0.06 0.1 rotational");
  }
};

// scripts/logic/weapons/throwables/fragGrenade.ts
import * as mc76 from "@minecraft/server";
var FRAG_GRENADE_ITEM_TYPE = "lc:scpdy_frag_grenade";
var FRAG_GRENADE_THROWN_ENTITY_TYPE = "lc:scpdy_player_frag_grenade_thrown";
var FRAG_GRENADE_DROPPED_ENTITY_TYPE = "lc:scpdy_player_frag_grenade_dropped";
function throwFragGrenade(player, ticksSinceRemovedPin) {
  const playerHeadLoc = player.getHeadLocation();
  if (ticksSinceRemovedPin >= 100) {
    player.applyDamage(37, {
      cause: mc76.EntityDamageCause.entityExplosion,
      damagingEntity: player
    });
    player.dimension.spawnEntity(FRAG_GRENADE_DROPPED_ENTITY_TYPE, playerHeadLoc, {
      spawnEvent: "frag_grenade:explode"
    });
    return;
  }
  player.dimension.playSound("scpdy.frag_grenade.throw", playerHeadLoc, { volume: 1.1 });
  const throwForceMultiplierMax = player.isSneaking ? 0.9 : 1.9;
  const throwForceMultiplier = Math.min(throwForceMultiplierMax, Math.max(1, ticksSinceRemovedPin / 10));
  const impulse = chain(FORWARD).mul(throwForceMultiplier).changeDir(player.getViewDirection()).done();
  const grenadeEntity = player.dimension.spawnEntity(FRAG_GRENADE_THROWN_ENTITY_TYPE, playerHeadLoc);
  grenadeEntity.setDynamicProperty("sourcePlayerId", player.id);
  const ticksUntilExplode = Math.floor(Math.max(0, 40 - ticksSinceRemovedPin / 1.75));
  if (ticksUntilExplode > 0) {
    grenadeEntity.applyImpulse(impulse);
  }
  grenadeEntity.setProperty("lc:ticks_until_explode", ticksUntilExplode);
}
function turnThrownFragGrenadeIntoDropped(thrownGrenadeEntity, keepVelocity) {
  if (!thrownGrenadeEntity.isValid) return;
  thrownGrenadeEntity.dimension.playSound("scpdy.frag_grenade.hit", thrownGrenadeEntity.location, {
    volume: 1.15
  });
  const ticksUntilExplode = thrownGrenadeEntity.getProperty("lc:ticks_until_explode");
  if (ticksUntilExplode <= 0) {
    thrownGrenadeEntity.triggerEvent("frag_grenade:explode");
    return;
  }
  const sourcePlayerId = thrownGrenadeEntity.getDynamicProperty("sourcePlayerId");
  const thrownGrenadeVelocity = thrownGrenadeEntity.getVelocity();
  const droppedGrenadeEntity = thrownGrenadeEntity.dimension.spawnEntity(FRAG_GRENADE_DROPPED_ENTITY_TYPE, {
    x: thrownGrenadeEntity.location.x,
    y: thrownGrenadeEntity.location.y + 0.1,
    z: thrownGrenadeEntity.location.z
  });
  if (keepVelocity) {
    droppedGrenadeEntity.applyImpulse({
      x: thrownGrenadeVelocity.x,
      y: 0,
      z: thrownGrenadeVelocity.z
    });
  }
  droppedGrenadeEntity.applyImpulse({
    x: 0,
    y: -thrownGrenadeVelocity.y * 0.5,
    z: 0
  });
  droppedGrenadeEntity.setProperty("lc:ticks_until_explode", ticksUntilExplode);
  droppedGrenadeEntity.setDynamicProperty("sourcePlayerId", sourcePlayerId);
  thrownGrenadeEntity.remove();
}
mc76.world.afterEvents.projectileHitBlock.subscribe((event) => {
  if (event.projectile.typeId !== FRAG_GRENADE_THROWN_ENTITY_TYPE) return;
  turnThrownFragGrenadeIntoDropped(event.projectile, true);
});
mc76.world.afterEvents.projectileHitEntity.subscribe((event) => {
  if (event.projectile.typeId !== FRAG_GRENADE_THROWN_ENTITY_TYPE) return;
  if (!event.projectile.isValid) return;
  const hitEntity = event.getEntityHit().entity;
  if (hitEntity && hitEntity.typeId === FRAG_GRENADE_THROWN_ENTITY_TYPE) return;
  const sourcePlayerId = ensureType(event.projectile.getDynamicProperty("sourcePlayerId"), "string");
  if (sourcePlayerId !== void 0 && sourcePlayerId === hitEntity?.id) return;
  turnThrownFragGrenadeIntoDropped(event.projectile, false);
  if (hitEntity) {
    hitEntity.applyDamage(2, {
      cause: mc76.EntityDamageCause.entityAttack,
      damagingEntity: sourcePlayerId !== void 0 ? mc76.world.getEntity(sourcePlayerId) : void 0
    });
  }
});
mc76.world.afterEvents.itemStartUse.subscribe((event) => {
  if (event.itemStack.typeId !== FRAG_GRENADE_ITEM_TYPE) return;
  const player = event.source;
  if (player.getItemCooldown("scpdy_frag_grenade_throw") > 0) return;
  player.setDynamicProperty("isThrowingFragGrenade", true);
  const throwTriggerIndex = ensureType(player.getDynamicProperty("nextFragGrenadeThrowTriggerIndex"), "number") ?? 0;
  player.startItemCooldown(`scpdy_frag_grenade_use_trigger_${throwTriggerIndex + 1}`, 2);
  const nextThrowTriggerIndex = throwTriggerIndex === 0 ? 1 : 0;
  player.setDynamicProperty("nextFragGrenadeThrowTriggerIndex", nextThrowTriggerIndex);
});
mc76.world.afterEvents.itemStopUse.subscribe((event) => {
  if (!event.itemStack) return;
  if (event.itemStack.typeId !== FRAG_GRENADE_ITEM_TYPE) return;
  const player = event.source;
  if (player.getDynamicProperty("isThrowingFragGrenade") === false) return;
  player.setDynamicProperty("isThrowingFragGrenade", false);
  const equippableComp = player.getComponent("equippable");
  if (!equippableComp) return;
  const mainhandSlot = equippableComp.getEquipmentSlot(mc76.EquipmentSlot.Mainhand);
  if (!mainhandSlot.hasItem()) return;
  if (mainhandSlot.typeId !== FRAG_GRENADE_ITEM_TYPE) return;
  const heldTicks = 72e3 - event.useDuration;
  player.startItemCooldown("scpdy_frag_grenade_throw", 3);
  if (heldTicks <= 9) {
    player.playSound("scpdy.frag_grenade.pull");
  }
  throwFragGrenade(player, Math.floor(Math.max(0, heldTicks - 10)));
  if (player.getGameMode() === mc76.GameMode.Creative) return;
  mc76.system.runTimeout(() => {
    if (mainhandSlot.typeId !== FRAG_GRENADE_ITEM_TYPE) {
      player.runCommand(`clear @s ${FRAG_GRENADE_ITEM_TYPE} 0 1`);
      return;
    }
    if (mainhandSlot.amount > 1) {
      mainhandSlot.amount--;
    } else {
      mainhandSlot.setItem();
    }
  }, 1);
});

// scripts/features/config/commands.ts
import * as mc78 from "@minecraft/server";

// scripts/features/config/config.ts
import * as mc77 from "@minecraft/server";

// scripts/utils/math.ts
function clamp3(value, min, max) {
  return Math.max(min, Math.min(max, value));
}
function randi(min, max) {
  return Math.floor(Math.random() * (max - min + 1)) + min;
}
function randf(min, max) {
  return Math.random() * (max - min) + min;
}
function flattenCoordinates(major, minor, minorRange = 4) {
  return minor + major * minorRange;
}
function unflattenToCoordinates(flatIndex, minorRange = 4) {
  return {
    major: Math.floor(flatIndex / minorRange),
    minor: flatIndex % minorRange
  };
}

// scripts/features/config/ConfigBuilder.ts
import "@minecraft/server";
import { ModalFormData as ModalFormData7 } from "@minecraft/server-ui";
var ConfigBuilder2 = class {
  constructor(readAndWriteTo) {
    this.readAndWriteTo = readAndWriteTo;
  }
  elements = [];
  toggle(name, label, defaultValue = false, tooltip) {
    this.elements.push({
      type: "toggle",
      name,
      get: () => {
        const value = this.getDynamicProperty(name);
        return typeof value === "boolean" ? value : defaultValue;
      },
      set: (valueArg) => {
        const value = valueArg === defaultValue ? void 0 : valueArg;
        this.setDynamicProperty(name, value);
      },
      reset: () => {
        this.setDynamicProperty(name, void 0);
      },
      defaultValue,
      label,
      tooltip
    });
    return this;
  }
  slider(name, label, minValue = 0, maxValue = 1, defaultValue = 0, valueStep = 1, tooltip) {
    const fixValue = (valueArg) => {
      const value = Math.round(valueArg / valueStep) * valueStep;
      return clamp3(value, minValue, maxValue);
    };
    const fixedDefaultValue = fixValue(defaultValue);
    this.elements.push({
      type: "slider",
      name,
      get: () => {
        const value = this.getDynamicProperty(name);
        return typeof value === "number" ? fixValue(value) : fixedDefaultValue;
      },
      set: (valueArg) => {
        const fixedValueArg = fixValue(valueArg);
        const value = fixedValueArg === fixedDefaultValue ? void 0 : fixedValueArg;
        this.setDynamicProperty(name, value);
      },
      reset: () => {
        this.setDynamicProperty(name, void 0);
      },
      minValue,
      maxValue,
      defaultValue,
      valueStep,
      label,
      tooltip
    });
    return this;
  }
  textField(name, label, placeholderText, defaultValue = "", tooltip) {
    this.elements.push({
      type: "textField",
      name,
      get: () => {
        const value = this.getDynamicProperty(name);
        return typeof value === "string" ? value : defaultValue;
      },
      set: (valueArg) => {
        const value = valueArg === defaultValue ? void 0 : valueArg;
        this.setDynamicProperty(name, value);
      },
      reset: () => {
        this.setDynamicProperty(name, void 0);
      },
      defaultValue,
      placeholderText,
      label,
      tooltip
    });
    return this;
  }
  dropdown(name, label, choices, defaultValue, tooltip) {
    const choicesObjEntries = Object.entries(choices);
    const choiceArray = [];
    for (const [k, v] of choicesObjEntries) {
      choiceArray.push({ name: k, text: v.text });
    }
    if (choiceArray.length <= 0) return this;
    const defaultValueIndex = choiceArray.findIndex((choice) => choice.name === defaultValue);
    this.elements.push({
      type: "dropdown",
      name,
      get: () => {
        const indexProp = this.getDynamicProperty(name);
        const index = typeof indexProp === "number" ? indexProp : defaultValueIndex;
        const value = choiceArray[index]?.name ?? String(defaultValue);
        return value;
      },
      getIndex: () => {
        const indexProp = this.getDynamicProperty(name);
        const index = typeof indexProp === "number" ? indexProp : defaultValueIndex;
        return index;
      },
      set: (valueArg) => {
        const foundIndex = choiceArray.findIndex((choice) => choice.name === valueArg);
        const index = foundIndex !== -1 ? foundIndex : defaultValueIndex;
        const result = index === defaultValueIndex ? void 0 : index;
        this.setDynamicProperty(name, result);
      },
      setIndex: (indexArg) => {
        const result = indexArg === defaultValueIndex ? void 0 : indexArg;
        this.setDynamicProperty(name, result);
      },
      reset: () => {
        this.setDynamicProperty(name, void 0);
      },
      choiceArray,
      defaultValue: String(defaultValue),
      defaultValueIndex,
      label,
      tooltip
    });
    return this;
  }
  header(text) {
    this.elements.push({ type: "header", text });
  }
  label(text) {
    this.elements.push({ type: "label", text });
  }
  divider() {
    this.elements.push({ type: "divider" });
  }
  build() {
    const o = {};
    for (const element of this.elements) {
      switch (element.type) {
        case "toggle":
        case "slider":
        case "textField":
        case "dropdown":
          Object.defineProperty(o, element.name, {
            get() {
              return element.get();
            },
            set(v) {
              return element.set(v);
            },
            configurable: false,
            enumerable: true
          });
          break;
      }
    }
    return o;
  }
  resetAllOptions() {
    for (const element of this.elements) {
      switch (element.type) {
        case "toggle":
        case "slider":
        case "textField":
        case "dropdown":
          element.reset();
          break;
      }
    }
  }
  getDynamicProperty(id) {
    try {
      return this.readAndWriteTo.getDynamicProperty(`config__${id}`);
    } catch {
      return void 0;
    }
  }
  setDynamicProperty(id, value) {
    try {
      this.readAndWriteTo.setDynamicProperty(`config__${id}`, value);
    } catch {
    }
  }
  async showEditorForm(target) {
    const formData = new ModalFormData7().title({ translate: "scpdy.config.text.editorFormTitle" }).label({ translate: "scpdy.config.text.editorFormExplanation" }).submitButton({ translate: "scpdy.misc.text.saveAndExit" });
    formData.divider();
    formData.toggle(
      { translate: "scpdy.config.text.editorFormResetToggle.label" },
      {
        defaultValue: false,
        tooltip: { translate: "scpdy.config.text.editorFormResetToggle.tooltip" }
      }
    );
    formData.divider();
    const handlers = [
      null,
      // Label
      null,
      // Divider
      null,
      // Reset toggle
      null
      // Divider
    ];
    for (const element of this.elements) {
      switch (element.type) {
        case "divider":
          formData.divider();
          handlers.push(null);
          break;
        case "header":
          formData.header(element.text);
          handlers.push(null);
          break;
        case "label":
          formData.label(element.text);
          handlers.push(null);
          break;
        case "toggle":
          formData.toggle(element.label, {
            defaultValue: element.get(),
            tooltip: element.tooltip
          });
          handlers.push((value) => {
            if (typeof value !== "boolean") return;
            element.set(value);
          });
          break;
        case "slider":
          formData.slider(element.label, element.minValue, element.maxValue, {
            valueStep: element.valueStep,
            defaultValue: element.get(),
            tooltip: element.tooltip
          });
          handlers.push((value) => {
            if (typeof value !== "number") return;
            element.set(value);
          });
          break;
        case "textField":
          formData.textField(element.label, element.placeholderText, {
            defaultValue: element.get(),
            tooltip: element.tooltip
          });
          handlers.push((value) => {
            if (typeof value !== "string") return;
            element.set(value);
          });
          break;
        case "dropdown":
          formData.dropdown(
            element.label,
            element.choiceArray.map((x) => x.text),
            {
              defaultValueIndex: element.getIndex(),
              tooltip: element.tooltip
            }
          );
          handlers.push((value) => {
            if (typeof value !== "number") return;
            element.setIndex(value);
          });
          break;
      }
    }
    const response = await formData.show(target);
    if (response.canceled) return;
    if (!response.formValues) return;
    for (let i = 0; i < response.formValues.length; i++) {
      const formValue = response.formValues[i];
      if (i === 2 && formValue === true) {
        this.resetAllOptions();
        target.sendMessage({ translate: "scpdy.config.text.messageOnReset" });
        return;
      }
      const handler = handlers[i];
      handler?.(formValue);
    }
    target.sendMessage({ translate: "scpdy.config.text.messageOnSaveAndExit" });
  }
};

// scripts/features/config/config.ts
var BUILDER = new ConfigBuilder2(mc77.world);
var CONFIG_INTERNAL = BUILDER.build();
async function showConfigEditorForm2(target) {
  await BUILDER.showEditorForm(target);
}

// scripts/features/config/commands.ts
mc78.system.beforeEvents.startup.subscribe((event) => {
  event.customCommandRegistry.registerCommand(
    {
      name: "scpdy:configeditor",
      description: "Shows the SCP: Dystopia addon config editor form",
      permissionLevel: mc78.CommandPermissionLevel.Host,
      optionalParameters: [
        {
          name: "target",
          type: mc78.CustomCommandParamType.PlayerSelector
        }
      ]
    },
    (origin, players) => {
      const targets = Array.isArray(players) ? players : [origin.sourceEntity];
      let showCount = 0;
      for (const target of targets) {
        if (!(target instanceof mc78.Player)) continue;
        mc78.system.run(() => {
          showConfigEditorForm2(target);
        });
        showCount++;
      }
      if (showCount > 0) {
        return {
          status: mc78.CustomCommandStatus.Success,
          message: `Successfully showed the config editor form to ${showCount} players.`
        };
      }
      return {
        status: mc78.CustomCommandStatus.Failure,
        message: "Target player must be specified."
      };
    }
  );
});

// scripts/features/player_loop.ts
import * as mc79 from "@minecraft/server";
var PlayerLoopEvent = new EventEmitter();
mc79.world.afterEvents.worldLoad.subscribe(() => {
  mc79.system.runInterval(() => {
    const players = mc79.world.getPlayers();
    for (let i = 0; i < players.length; i++) {
      const player = players[i];
      onTickPlayer2(player);
    }
  }, 1);
});
function onTickPlayer2(player) {
  if (!player.isValid) return;
  const health = player.getComponent("health");
  if (!health) throw new Error("Failed to get player health component");
  const equippable = player.getComponent("equippable");
  if (!equippable) throw new Error("Failed to get player equippable component");
  const inventory = player.getComponent("inventory");
  if (!inventory) throw new Error("Failed to get player inventory component");
  PlayerLoopEvent.emit("onTick", {
    player,
    health,
    equippable,
    inventory
  });
}

// scripts/features/blast_door/blast_door.ts
import * as mc81 from "@minecraft/server";

// scripts/features/fnet/fnet.ts
import * as mc80 from "@minecraft/server";
var LkdnEvents = new EventEmitter();
var MAX_FNET_COUNT = 5;
var MAX_FZONE_COUNT = 10;
var fnets = void 0;
mc80.world.afterEvents.worldLoad.subscribe(() => {
  fnets = Array.from({ length: MAX_FNET_COUNT }, (_, index) => new Fnet(index));
});
function getFnet(index) {
  if (!fnets) throw new Error("fnets are not available yet");
  index = Math.floor(index);
  throwIfFnetIndexIsInvalid(index);
  const fnet = fnets[index];
  return fnet;
}
function getAllFnets() {
  if (!fnets) throw new Error("fnets are not available yet");
  return fnets;
}
function throwIfFnetIndexIsInvalid(index) {
  if (index < 0) throw new Error(`Provided index (${index}) is less than 0.`);
  if (index > MAX_FNET_COUNT)
    throw new RangeError(`Provided index (${index}) exceeds the maximum facility network count.`);
}
function throwIfFzoneIndexIsInvalid(index) {
  if (index < 0) throw new Error(`Provided index (${index}) is less than 0.`);
  if (index > MAX_FZONE_COUNT)
    throw new RangeError(`Provided index (${index}) exceeds the maximum facility zone count.`);
}
mc80.world.afterEvents.worldLoad.subscribe(() => {
  mc80.system.runInterval(() => {
    for (let i = 0; i < MAX_FNET_COUNT; i++) {
      const fnet = getFnet(i);
      for (let i2 = 0; i2 < MAX_FZONE_COUNT; i2++) {
        const fzone = fnet.getZone(i2);
        updateZoneStates(fzone);
      }
    }
  }, 20);
});
function updateZoneStates(zone) {
  const lkdnDuration = zone.getLkdnDuration();
  const scheduledLkdnDelay = zone.getScheduledLkdnDelay();
  if (scheduledLkdnDelay != void 0 && scheduledLkdnDelay > 0) {
    const next = scheduledLkdnDelay - 1;
    zone.setScheduledLkdnDelay(next);
    if (next <= 0) {
      zone.startLkdn(lkdnDuration);
    }
    return;
  }
  if (lkdnDuration != void 0 && lkdnDuration > 0) {
    const next = lkdnDuration - 1;
    zone.setLkdnDuration(next);
  }
  if (lkdnDuration == void 0) {
    zone.stopLkdn();
  }
}
var Fnet = class {
  index;
  cachedZones = new Array(MAX_FZONE_COUNT);
  /** @param index - The index of the facility network (zero-based). */
  constructor(index) {
    index = Math.floor(index);
    throwIfFnetIndexIsInvalid(index);
    this.index = index;
  }
  get name() {
    const customName = this.getCustomName();
    let value;
    if (customName == void 0) {
      value = {
        translate: "scpdy.fnet.nonameX",
        with: [`${this.index + 1}`]
      };
    } else {
      value = {
        translate: "scpdy.fnet.customNameX",
        with: [`${this.index + 1}`, customName]
      };
    }
    return value;
  }
  prefix(value) {
    return `fnet_${this.index}_${value}`;
  }
  getCustomName() {
    const propId = this.prefix("name");
    const value = mc80.world.getDynamicProperty(propId);
    return value == void 0 ? void 0 : String(value).trim();
  }
  setCustomName(value) {
    const propId = this.prefix("name");
    if (value == void 0 || value.trim() === "") value = void 0;
    mc80.world.setDynamicProperty(propId, value);
  }
  /**
   * Creates an {@link Fzone} (facility zone) object or retrieves a cached one from previous call.
   * @param index - The index of the facility zone (zero-based).
   * @returns Facility zone.
   */
  getZone(index) {
    index = Math.floor(index);
    throwIfFzoneIndexIsInvalid(index);
    let fzone = this.cachedZones[index];
    if (!fzone) {
      fzone = new Fzone(index, this);
      this.cachedZones[index] = fzone;
    }
    return fzone;
  }
  /**
   * Retrieves all available Fzones (facility zones) that belong to this facility network.
   * @returns Array of Fzone objects.
   */
  getAllZones() {
    for (let i = 0; i < MAX_FZONE_COUNT; i++) {
      this.getZone(i);
    }
    return this.cachedZones;
  }
};
var Fzone = class {
  index;
  fnet;
  /**
   * @param index - The index of the facility network (zero-based).
   * @param fnet - The facility network this zone belongs to.
   **/
  constructor(index, fnet) {
    index = Math.floor(index);
    throwIfFzoneIndexIsInvalid(index);
    this.index = index;
    this.fnet = fnet;
  }
  /** Alias for {@link getLkdnActive}. */
  get isLkdnActive() {
    return this.getLkdnActive() === true;
  }
  /** Whether the value returned from {@link getScheduledLkdnDelay} is greater than 0. */
  get isLkdnScheduled() {
    return Boolean(this.getScheduledLkdnDelay());
  }
  get name() {
    const customName = this.getCustomName();
    let value;
    if (customName == void 0) {
      value = {
        translate: "scpdy.fzone.nonameX",
        with: [`${this.index + 1}`]
      };
    } else {
      value = {
        translate: "scpdy.fzone.customNameX",
        with: [`${this.index + 1}`, customName]
      };
    }
    return value;
  }
  prefix(value) {
    return this.fnet.prefix(`fzone_${this.index}_${value}`);
  }
  getCustomName() {
    const propId = this.prefix("name");
    const value = mc80.world.getDynamicProperty(propId);
    return value == void 0 ? void 0 : String(value).trim();
  }
  setCustomName(value) {
    const propId = this.prefix("name");
    if (value == void 0 || value.trim() === "") value = void 0;
    mc80.world.setDynamicProperty(propId, value);
  }
  /** Gets whether the lockdown is active in this facility zone. */
  getLkdnActive() {
    const propId = this.prefix("isLkdnActive");
    const value = mc80.world.getDynamicProperty(propId);
    return typeof value == "boolean" ? value : void 0;
  }
  /**
   * Sets whether the lockdown is active in this facility zone.
   *
   * **Do not call this method directly!**
   * Use {@link startLkdn} instead.
   **/
  setLkdnActive(value) {
    const propId = this.prefix("isLkdnActive");
    if (value == void 0 || value == false) value = void 0;
    mc80.world.setDynamicProperty(propId, value);
  }
  /** Gets the scheduled delay (in seconds) before this facility zone is locked down. */
  getScheduledLkdnDelay() {
    const propId = this.prefix("lkdnDelay");
    const value = mc80.world.getDynamicProperty(propId);
    return typeof value == "number" ? value === 45451919 ? Infinity : value : void 0;
  }
  /**
   * Sets the scheduled delay (in seconds) before this facility zone is locked down.
   *
   * **Do not call this method directly!**
   * Use {@link scheduleLkdn} instead.
   **/
  setScheduledLkdnDelay(value) {
    const propId = this.prefix("lkdnDelay");
    if (value == void 0 || value <= 0) value = void 0;
    else if (value === Infinity) value = 45451919;
    mc80.world.setDynamicProperty(propId, value);
  }
  /** Gets the duration (in seconds) for which this facility zone will be locked down. */
  getLkdnDuration() {
    const propId = this.prefix("lkdnDuration");
    const value = mc80.world.getDynamicProperty(propId);
    return typeof value == "number" ? value === 45451919 ? Infinity : value : void 0;
  }
  /**
   * Sets the duration (in seconds) for which this facility zone will be locked down.
   *
   * **Do not call this method directly!**
   * Use {@link startLkdn} or {@link scheduleLkdn} instead.
   **/
  setLkdnDuration(value) {
    const propId = this.prefix("lkdnDuration");
    if (value == void 0 || value <= 0) value = void 0;
    else if (value === Infinity) value = 45451919;
    mc80.world.setDynamicProperty(propId, value);
  }
  /**
   * Starts the lockdown of this facility zone immediately.
   *
   * If `duration` is set, the lockdown will automatically stop once that period has elapsed.
   * You can also stop it manually by calling {@link stopLkdn}.
   *
   * @param duration - Duration (in seconds) for which this facility zone will be locked down.
   * @returns Whether the lockdown was successfully started.
   */
  startLkdn(duration = Infinity) {
    if (this.getLkdnActive()) return false;
    if (duration != void 0 && duration > 0) {
      this.setLkdnDuration(duration);
    }
    this.cancelScheduledLkdn();
    this.setLkdnActive(true);
    LkdnEvents.emit("onStart", {
      fzone: this,
      duration
    });
    return true;
  }
  /**
   * Stops the lockdown of this facility zone.
   * @returns Whether the lockdown was successfully stopped.
   */
  stopLkdn() {
    if (!this.getLkdnActive()) return false;
    this.setLkdnActive(void 0);
    this.setScheduledLkdnDelay(void 0);
    this.setLkdnDuration(void 0);
    LkdnEvents.emit("onStop", {
      fzone: this
    });
    return true;
  }
  /**
   * Schedules a future lockdown.
   * @param delay - Delay (in seconds) before the lockdown begins..
   * @param duration - Duration (in seconds) for which this facility zone will be locked down.
   * @returns Whether the lockdown was scheduled successfully.
   */
  scheduleLkdn(delay, duration = Infinity) {
    if (this.getLkdnActive()) return false;
    this.setScheduledLkdnDelay(delay);
    if (duration != void 0 && duration > 0) {
      this.setLkdnDuration(duration);
    }
    LkdnEvents.emit("onScheduled", {
      fzone: this,
      delay
    });
    return true;
  }
  /**
   * Clears the currently scheduled lockdown.
   * @returns Whether the scheduled lockdown was cleared successfully.
   */
  cancelScheduledLkdn() {
    if (this.getLkdnActive()) return false;
    const delay = this.getScheduledLkdnDelay();
    if (delay == void 0 || delay <= 0) return false;
    this.setScheduledLkdnDelay(void 0);
    LkdnEvents.emit("onScheduleCanceled", {
      fzone: this
    });
    return true;
  }
};

// scripts/features/blast_door/blast_door.ts
var BLAST_DOOR_INFO_MAP = /* @__PURE__ */ new Map([
  [
    "lc:scpdy_blast_door_1",
    {
      size: { x: 5, y: 3 },
      lockdownO5AccessTime: 6
    }
  ],
  [
    "lc:scpdy_blast_door_2",
    {
      size: { x: 5, y: 3 },
      lockdownO5AccessTime: 6
    }
  ]
]);
var BLAST_DOOR_TYPES = Array.from(BLAST_DOOR_INFO_MAP.keys());
function getBlastDoorInfo(blastDoor) {
  const info = BLAST_DOOR_INFO_MAP.get(blastDoor.typeId);
  if (!info) throw new Error(`Blast door info of ${blastDoor.typeId} is undefined`);
  return info;
}
function getBlastDoorFzone(blastDoor) {
  const belongsToFacilityNetwork = blastDoor.getProperty("lc:belongs_to_facility_network") === true;
  if (!belongsToFacilityNetwork) return void 0;
  const facilityNetworkIndex = blastDoor.getProperty("lc:facility_network_index");
  const facilityZoneIndex = blastDoor.getProperty("lc:facility_zone_index");
  const facilityNetwork = getFnet(facilityNetworkIndex);
  const facilityZone = facilityNetwork.getZone(facilityZoneIndex);
  return facilityZone;
}
function setIsOpen(blastDoor, mode) {
  switch (mode) {
    case "open":
      blastDoor.setProperty("lc:is_open", true);
      break;
    case "close":
      blastDoor.setProperty("lc:is_open", false);
      break;
    case "switch":
      const isOpen = blastDoor.getProperty("lc:is_open") === true;
      blastDoor.setProperty("lc:is_open", !isOpen);
      break;
  }
}
function controlBlastDoor(blastDoor, mode, isO5Access) {
  const blastDoorInfo = getBlastDoorInfo(blastDoor);
  const isOpen = blastDoor.getProperty("lc:is_open") === true;
  if (isOpen && mode === "open") return false;
  if (!isOpen && mode === "close") return false;
  const facilityZone = getBlastDoorFzone(blastDoor);
  if (facilityZone && facilityZone.isLkdnActive) {
    if (!isO5Access) {
      return false;
    }
    blastDoor.setDynamicProperty("lockdownO5AccessTime", blastDoorInfo.lockdownO5AccessTime);
    return true;
  }
  const detectRedstone = blastDoor.getProperty("lc:detect_redstone") === true;
  if (detectRedstone) return false;
  setIsOpen(blastDoor, mode);
  return true;
}
function getNearestBlastDoor(dimension, origin, maxDistance = 7) {
  return dimension.getEntities({
    closest: 1,
    maxDistance: maxDistance === Infinity ? void 0 : maxDistance,
    location: origin,
    families: ["scpdy_blast_door"]
  })[0];
}
function removeBlastDoor(blastDoor) {
  const info = getBlastDoorInfo(blastDoor);
  const horSizeDivided = Math.floor((info.size.x - 1) / 2);
  blastDoor.setDynamicProperty("dontHandleRemoval", true);
  try {
    blastDoor.runCommand(
      `fill ^${horSizeDivided} ^${info.size.y} ^0  ^${-horSizeDivided} ^0 ^0 air replace lc:scpdy_blast_door_dummy`
    );
    const item = new mc81.ItemStack(`${blastDoor.typeId}_preview_placer`, 1);
    blastDoor.dimension.spawnItem(item, blastDoor.location);
  } finally {
    blastDoor.remove();
  }
}
function removeBlastDoorAtBlock(block) {
  const blastDoor = block.dimension.getEntities({
    closest: 1,
    maxDistance: 1,
    location: block.center(),
    families: ["scpdy_blast_door"]
  })[0];
  if (!blastDoor) return;
  removeBlastDoor(blastDoor);
}
function isBlastDoor(entity) {
  return BLAST_DOOR_INFO_MAP.has(entity.typeId);
}
function beforeRemoveBlastDoor(blastDoor) {
  const dimension = blastDoor.dimension;
  const location = blastDoor.location;
  const rotation = blastDoor.getRotation();
  const entityTypeId = blastDoor.typeId;
  const isOpen = blastDoor.getProperty("lc:is_open") === true;
  const detectRedstone = blastDoor.getProperty("lc:detect_redstone") === true;
  const belongsToFacilityNetwork = blastDoor.getProperty("lc:belongs_to_facility_network");
  const facilityNetworkIndex = blastDoor.getProperty("lc:facility_network_index");
  const facilityZoneIndex = blastDoor.getProperty("lc:facility_zone_index");
  const oldBlastDoorDynamicProps = blastDoor.getDynamicPropertyIds().map((propId) => {
    return { propId, value: blastDoor.getDynamicProperty(propId) };
  });
  mc81.system.run(() => {
    const newBlastDoor = dimension.spawnEntity(entityTypeId, location);
    newBlastDoor.setRotation(rotation);
    newBlastDoor.setProperty("lc:is_open", isOpen);
    newBlastDoor.setProperty("lc:detect_redstone", detectRedstone);
    newBlastDoor.setProperty("lc:belongs_to_facility_network", belongsToFacilityNetwork);
    newBlastDoor.setProperty("lc:facility_network_index", facilityNetworkIndex);
    newBlastDoor.setProperty("lc:facility_zone_index", facilityZoneIndex);
    for (const prop of oldBlastDoorDynamicProps) {
      newBlastDoor.setDynamicProperty(prop.propId, prop.value);
    }
  });
}
function updateRotationLock(blastDoor) {
  let yRotLock = ensureType(blastDoor.getDynamicProperty("yRotLock"), "number");
  if (yRotLock === void 0) {
    const newYRotLock = Math.abs(blastDoor.getRotation().y % 90) < 45 ? 0 : 90;
    blastDoor.setDynamicProperty("yRotLock", newYRotLock);
    yRotLock = newYRotLock;
  }
  blastDoor.setRotation({ x: 0, y: yRotLock });
  blastDoor.tryTeleport({
    x: Math.floor(blastDoor.location.x) + 0.5,
    y: Math.floor(blastDoor.location.y),
    z: Math.floor(blastDoor.location.z) + 0.5
  });
}
function checkRedstone(blastDoor) {
  try {
    const rbCheckLoc1 = add(blastDoor.location, {
      x: 0,
      y: -2,
      z: 0
    });
    const rbCheckBlock1 = blastDoor.dimension.getBlock(rbCheckLoc1);
    if (rbCheckBlock1?.getRedstonePower() ?? 0 > 0) return true;
    const rbCheckLoc2 = add(blastDoor.location, {
      x: 0,
      y: 4,
      z: 0
    });
    const rbCheckBlock2 = blastDoor.dimension.getBlock(rbCheckLoc2);
    if (rbCheckBlock2?.getRedstonePower() ?? 0 > 0) return true;
    return false;
  } catch {
    return false;
  }
}
function updateState(blastDoor) {
  const isOpen = blastDoor.getProperty("lc:is_open") === true;
  const facilityZone = getBlastDoorFzone(blastDoor);
  if (facilityZone) {
    if (facilityZone.isLkdnActive) {
      const lockdownO5AccessTime = ensureType(blastDoor.getDynamicProperty("lockdownO5AccessTime"), "number") ?? 0;
      if (lockdownO5AccessTime > 0) {
        setIsOpen(blastDoor, "open");
        blastDoor.setDynamicProperty("lockdownO5AccessTime", lockdownO5AccessTime - 1);
        return;
      }
      if (isOpen) {
        setIsOpen(blastDoor, "close");
        blastDoor.setDynamicProperty("wasClosedByLockdown", true);
      }
      return;
    }
    const wasClosedByLockdown = blastDoor.getDynamicProperty("wasClosedByLockdown") === true;
    if (wasClosedByLockdown) {
      setIsOpen(blastDoor, "open");
      blastDoor.setDynamicProperty("wasClosedByLockdown", void 0);
      blastDoor.setDynamicProperty("lockdownO5AccessTime", void 0);
    }
  }
  const detectRedstone = blastDoor.getProperty("lc:detect_redstone") === true;
  if (detectRedstone) {
    const detected = checkRedstone(blastDoor);
    setIsOpen(blastDoor, detected ? "open" : "close");
  }
}
mc81.world.afterEvents.entitySpawn.subscribe((event) => {
  if (!isBlastDoor(event.entity)) return;
  mc81.system.run(() => {
    updateRotationLock(event.entity);
  });
});
mc81.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    if (!isBlastDoor(event.entity)) return;
    switch (event.eventId) {
      case "blast_door:open":
        controlBlastDoor(event.entity, "open", false);
        break;
      case "blast_door:close":
        controlBlastDoor(event.entity, "close", false);
        break;
      case "blast_door:switch":
        controlBlastDoor(event.entity, "switch", false);
        break;
    }
  },
  {
    entityTypes: BLAST_DOOR_TYPES
  }
);
mc81.system.afterEvents.scriptEventReceive.subscribe(
  (event) => {
    if (!event.sourceEntity) return;
    if (!isBlastDoor(event.sourceEntity)) return;
    switch (event.id) {
      case "scpdy:blast_door_update_rotation_lock":
        updateRotationLock(event.sourceEntity);
        break;
      case "scpdy:blast_door_update_state":
        updateState(event.sourceEntity);
        break;
    }
  },
  {
    namespaces: ["scpdy"]
  }
);
mc81.world.beforeEvents.entityRemove.subscribe((event) => {
  if (!isBlastDoor(event.removedEntity)) return;
  if (event.removedEntity.getDynamicProperty("dontHandleRemoval") === true) return;
  beforeRemoveBlastDoor(event.removedEntity);
});

// scripts/features/blast_door/dummy_block.ts
import * as mc82 from "@minecraft/server";
function onPlayerBreak5(arg) {
  removeBlastDoorAtBlock(arg.block);
}
mc82.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:blast_door_dummy", {
    onPlayerBreak: onPlayerBreak5
  });
});

// scripts/features/blast_door/from_input_device.ts
MachineryInputEvents.on("onActivate", (data) => {
  if (data.mode !== "ctrlBlastDoor") return;
  const nearbyBlastDoor = getNearestBlastDoor(data.dimension, data.location, 12);
  if (!nearbyBlastDoor) return;
  controlBlastDoor(nearbyBlastDoor, "switch", data.clearanceLevel === 6);
});

// scripts/features/blast_door/preview.ts
import * as mc83 from "@minecraft/server";
import { ModalFormData as ModalFormData8 } from "@minecraft/server-ui";
var BD_PREVIEW_TYPE_TO_BLAST_DOOR_TYPE = {
  "lc:scpdy_blast_door_1_preview": "lc:scpdy_blast_door_1",
  "lc:scpdy_blast_door_2_preview": "lc:scpdy_blast_door_2"
};
var BD_PREVIEW_TYPES = Array.from(Object.keys(BD_PREVIEW_TYPE_TO_BLAST_DOOR_TYPE));
function isBlastDoorPreviewEntity(entity) {
  return entity.typeId in BD_PREVIEW_TYPE_TO_BLAST_DOOR_TYPE;
}
function createBlastDoor(previewEntity, args) {
  const typeId = BD_PREVIEW_TYPE_TO_BLAST_DOOR_TYPE[previewEntity.typeId];
  if (typeId === void 0) throw new Error("typeId is undefined");
  const rotate = previewEntity.getProperty("lc:rotate_door") === true;
  const blastDoorEntity = previewEntity.dimension.spawnEntity(typeId, previewEntity.location);
  blastDoorEntity.setProperty("lc:detect_redstone", args.detectRedstone);
  if (args.facilityNetwork) {
    blastDoorEntity.setProperty("lc:belongs_to_facility_network", true);
    blastDoorEntity.setProperty("lc:facility_network_index", args.facilityNetwork.networkIndex);
    blastDoorEntity.setProperty("lc:facility_zone_index", args.facilityNetwork.zoneIndex);
  }
  if (rotate) {
    blastDoorEntity.setDynamicProperty("yRotLock", 90);
    blastDoorEntity.setRotation({ x: 0, y: 90 });
  }
  blastDoorEntity.dimension.spawnParticle("minecraft:egg_destroy_emitter", {
    x: blastDoorEntity.location.x,
    y: blastDoorEntity.location.y + 0.5,
    z: blastDoorEntity.location.z
  });
  blastDoorEntity.dimension.playSound("scpdy.misc.boom_1", blastDoorEntity.location);
  previewEntity.remove();
}
async function showCreationForm(previewEntity, player) {
  const formData1 = new ModalFormData8();
  formData1.title({ translate: "scpdy.form.blastDoorCreation.title" });
  formData1.toggle(
    { translate: "scpdy.form.blastDoorCreation.redstoneDetectionToggle.label" },
    { defaultValue: false }
  );
  const fnets2 = getAllFnets();
  formData1.dropdown(
    {
      translate: "scpdy.form.blastDoorCreation.facilityNetworkDropdown.label"
    },
    [
      { translate: "scpdy.form.misc.none" },
      ...fnets2.map(
        ({ name, index }) => name ?? {
          translate: "scpdy.form.blastDoorCreation.facilityNetworkDropdown.option",
          with: [(index + 1).toString()]
        }
      )
    ],
    { defaultValueIndex: 0 }
  );
  formData1.submitButton({ translate: "scpdy.form.blastDoorCreation.submitButton" });
  const response1 = await formData1.show(player);
  if (response1.canceled) return;
  if (!response1.formValues) return;
  const creationArgs = {
    detectRedstone: response1.formValues[0] === true
  };
  const facilityNetworkSelection = response1.formValues[1];
  const isFacilityNetworkSpecified = facilityNetworkSelection !== 0;
  if (isFacilityNetworkSpecified) {
    const fnet = fnets2[facilityNetworkSelection - 1];
    const zones = fnet.getAllZones();
    const formData2 = new ModalFormData8();
    formData2.title({ translate: "scpdy.form.blastDoorCreation.setZone.title" });
    formData2.dropdown(
      {
        translate: "scpdy.form.blastDoorCreation.setZone.facilityZoneDropdown.label"
      },
      zones.map(
        ({ name, index }) => name ?? {
          translate: "scpdy.form.blastDoorCreation.setZone.facilityZoneDropdown.option",
          with: [(index + 1).toString()]
        }
      ),
      { defaultValueIndex: 0 }
    );
    formData2.submitButton({ translate: "scpdy.form.blastDoorCreation.setZone.submitButton" });
    const response2 = await formData2.show(player);
    if (response2.canceled) return;
    if (!response2.formValues) return;
    creationArgs.facilityNetwork = {
      networkIndex: facilityNetworkSelection - 1,
      zoneIndex: response2.formValues[0]
    };
  }
  createBlastDoor(previewEntity, creationArgs);
}
mc83.world.afterEvents.playerInteractWithEntity.subscribe((event) => {
  if (!isBlastDoorPreviewEntity(event.target)) return;
  const { player, target } = event;
  if (player.isSneaking) {
    player.addTag("scpdy_read_blast_door_create_tip");
    showCreationForm(target, player);
    return;
  }
  if (player.addTag("scpdy_read_blast_door_create_tip")) {
    mc83.system.runTimeout(() => {
      player.playSound("random.orb");
      player.sendMessage({
        translate: "scpdy.msg.blastDoorPreview.sneakAndInteractToCreate"
      });
    }, 12);
  }
  const rotateDoor = target.getProperty("lc:rotate_door") === true;
  player.playSound("random.click");
  player.onScreenDisplay.setActionBar({
    translate: rotateDoor ? "scpdy.action.interact.blastDoorPreview.changeRotation.false" : "scpdy.action.interact.blastDoorPreview.changeRotation.true"
  });
  target.setProperty("lc:rotate_door", !rotateDoor);
});
mc83.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  (event) => {
    if (!isBlastDoorPreviewEntity(event.entity)) return;
    if (event.eventId === "blast_door_preview:fix_rotation") {
      try {
        const rotateDoor = event.entity.getProperty("lc:rotate_door") === true;
        event.entity.setRotation({
          x: 0,
          y: rotateDoor ? 90 : 0
        });
      } catch {
      }
    }
  },
  {
    entityTypes: BD_PREVIEW_TYPES
  }
);
mc83.world.afterEvents.entitySpawn.subscribe((event) => {
  if (!isBlastDoorPreviewEntity(event.entity)) return;
  event.entity.teleport({
    x: Math.floor(event.entity.location.x) + 0.5,
    y: Math.round(event.entity.location.y),
    z: Math.floor(event.entity.location.z) + 0.5
  });
});
mc83.world.afterEvents.entityDie.subscribe(
  (event) => {
    if (!isBlastDoorPreviewEntity(event.deadEntity)) return;
    try {
      const item = new mc83.ItemStack(`${event.deadEntity.typeId}_placer`, 1);
      event.deadEntity.dimension.spawnItem(item, event.deadEntity.location);
    } finally {
      event.deadEntity.remove();
    }
  },
  {
    entityTypes: BD_PREVIEW_TYPES
  }
);

// scripts/features/doors/cmd_door.ts
import * as mc85 from "@minecraft/server";

// scripts/utils/vec3.ts
import { Direction as Direction6 } from "@minecraft/server";
var ONE = Object.freeze({
  x: 1,
  y: 1,
  z: 1
});
var HALF = Object.freeze({
  x: 0.5,
  y: 0.5,
  z: 0.5
});
var ZERO2 = Object.freeze({
  x: 0,
  y: 0,
  z: 0
});
var UP2 = Object.freeze({
  x: 0,
  y: 1,
  z: 0
});
var DOWN2 = Object.freeze({
  x: 0,
  y: -1,
  z: 0
});
var LEFT = Object.freeze({
  x: -1,
  y: 0,
  z: 0
});
var RIGHT = Object.freeze({
  x: 1,
  y: 0,
  z: 0
});
var FORWARD2 = Object.freeze({
  x: 0,
  y: 0,
  z: 1
});
var BACKWARD = Object.freeze({
  x: 0,
  y: 0,
  z: -1
});
function isVector3(value) {
  if (value == null) return false;
  if (typeof value !== "object") return false;
  if (Array.isArray(value)) return false;
  const hasXYZ = "x" in value && typeof value.x === "number" && "y" in value && typeof value.y === "number" && "z" in value && typeof value.z === "number";
  return hasXYZ;
}
function midpoint2(vecA, vecB) {
  return {
    x: (vecA.x + vecB.x) / 2,
    y: (vecA.y + vecB.y) / 2,
    z: (vecA.z + vecB.z) / 2
  };
}
function toString3(vec) {
  return `(${vec.x}, ${vec.y}, ${vec.z})`;
}
function toString22(vec) {
  return `${vec.x} ${vec.y} ${vec.z}`;
}

// scripts/utils/block.ts
import "@minecraft/server";
function createBlockStatesString(states) {
  const array = [];
  for (const [k, v] of Object.entries(states)) {
    const vStr = typeof v === "string" ? `"${v}"` : `${v}`;
    array.push(`"${k}"=${vStr}`);
  }
  return array.join(",");
}
function destroyBlock(block) {
  const location = `${block.x} ${block.y} ${block.z}`;
  block.dimension.runCommand(`setblock ${location} air destroy`);
}

// scripts/features/doors/cmd_door.ts
var STATE = {
  isBottomPart: "lc:is_bottom_part",
  nextAction: "lc:next_action",
  powerLevelMinor: "lc:power_level_minor",
  stepMajor: "lc:step_major",
  stepMinor: "lc:step_minor"
};
var minStepIndex = 0;
var maxStepIndex = 15;
mc85.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:cmd_door", COMPONENT);
  event.customCommandRegistry.registerEnum("scpdy:cmdDoorCtrlMode", ["close", "open", "switch"]);
  event.customCommandRegistry.registerCommand(
    {
      name: "scpdy:cmddoor",
      description: "Control a Command Door of SCP: Dystopia addon",
      permissionLevel: mc85.CommandPermissionLevel.GameDirectors,
      mandatoryParameters: [
        {
          name: "scpdy:cmdDoorCtrlMode",
          type: mc85.CustomCommandParamType.Enum
        },
        {
          name: "location",
          type: mc85.CustomCommandParamType.Location
        }
      ]
    },
    (origin, unknownMode, location) => {
      const dimension = origin.initiator?.dimension ?? origin.sourceBlock?.dimension ?? origin.sourceEntity?.dimension ?? mc85.world.getDimension("overworld");
      if (!isVector3(location)) return;
      const mode = unknownMode;
      const block = dimension.getBlock(location);
      if (!block)
        return {
          status: mc85.CustomCommandStatus.Failure,
          message: `Failed to get a block at ${toString22(location)}`
        };
      try {
        tryControl(block, mode);
      } catch (error) {
        return {
          status: mc85.CustomCommandStatus.Failure,
          message: `${error}`
        };
      }
      return {
        status: mc85.CustomCommandStatus.Success,
        message: `Successfully sent a control signal to the Command Door at ${toString22(block)}`
      };
    }
  );
});
var COMPONENT = {
  onTick({ block, dimension }, arg1) {
    const params = arg1.params;
    const isBottomPart = Boolean(block.permutation.getState(STATE.isBottomPart));
    if (!isBottomPart) return;
    const otherPartBlock = isBottomPart ? block.above() : block.below();
    if (!otherPartBlock || otherPartBlock.typeId !== block.typeId) return;
    const nextAction = block.permutation.getState(STATE.nextAction);
    const { major: currentStepMajor, minor: currentStepMinor } = getStep(block.permutation);
    const currentStepIndex = flattenCoordinates(currentStepMajor, currentStepMinor);
    const newStepIndex = getUpdatedStepIndex(currentStepIndex, nextAction, minStepIndex, maxStepIndex);
    const newNextAction = (
      // maybe i can make this cleaner but im too lazy
      mc85.system.currentTick % 2 === 0 && (newStepIndex <= minStepIndex || newStepIndex >= maxStepIndex) ? "none" : nextAction
    );
    const newStepUnflat = unflattenToCoordinates(newStepIndex, 4);
    block.setPermutation(
      block.permutation.withState(STATE.nextAction, newNextAction).withState(STATE.stepMajor, newStepUnflat.major).withState(STATE.stepMinor, newStepUnflat.minor)
    );
    otherPartBlock.setPermutation(
      otherPartBlock.permutation.withState(STATE.stepMajor, newStepUnflat.major).withState(STATE.stepMinor, newStepUnflat.minor)
    );
    if (mc85.system.currentTick % 2 !== 0) return currentStepIndex;
    if (params.openSound && nextAction === "open" && newStepIndex === 1) {
      dimension.playSound(params.openSound.id, block.location, {
        volume: params.openSound.volume,
        pitch: params.openSound.pitch
      });
    }
    if (params.closeSound && nextAction === "close" && newStepIndex === 14) {
      dimension.playSound(params.closeSound.id, block.location, {
        volume: params.closeSound.volume,
        pitch: params.closeSound.pitch
      });
    }
  },
  beforeOnPlayerPlace({ player }) {
    if (!player) return;
    mc85.system.run(() => {
      if (!player.addTag("scpdy_sent_cmd_door_beginner_tip")) return;
      mc85.system.runTimeout(() => {
        player.sendMessage({ translate: "scpdy.cmdDoor.text.beginnerTip" });
        player.playSound("random.orb");
      }, 8);
    });
  },
  onPlace({ block }) {
    const isBottomPart = Boolean(block.permutation.getState(STATE.isBottomPart));
    if (!isBottomPart) {
      const blockBelow = block.below();
      if (!blockBelow || blockBelow.typeId === block.typeId) return;
      block.setType("minecraft:air");
      return;
    }
    const blockAbove = block.above();
    if (!blockAbove || !(blockAbove.isAir || block.isLiquid)) {
      destroyBlock(block);
      return;
    }
    const upperPartPermutation = block.permutation.withState(STATE.isBottomPart, false);
    blockAbove.setPermutation(upperPartPermutation);
  },
  onPlayerBreak({ block, brokenBlockPermutation }) {
    const isBottomPart = Boolean(brokenBlockPermutation.getState(STATE.isBottomPart));
    const otherPartBlock = isBottomPart ? block.above() : block.below();
    if (!otherPartBlock || otherPartBlock.typeId !== brokenBlockPermutation.type.id) return;
    destroyBlock(otherPartBlock);
  }
};
function getUpdatedStepIndex(currentStepIndex, nextAction, minStepIndex2, maxStepIndex2) {
  if (mc85.system.currentTick % 2 !== 0) return currentStepIndex;
  if (nextAction === "open" && currentStepIndex < maxStepIndex2) {
    return currentStepIndex + 1;
  }
  if (nextAction === "close" && currentStepIndex > minStepIndex2) {
    return currentStepIndex - 1;
  }
  return currentStepIndex;
}
function getStep(permutation) {
  const major = Number(permutation.getState(STATE.stepMajor));
  const minor = Number(permutation.getState(STATE.stepMinor));
  return { major, minor };
}
function tryControl(cmdDoorBlock, mode) {
  if (!cmdDoorBlock.isValid) throw new Error("Block is invalid");
  if (!cmdDoorBlock.hasTag("lc:cmd_door")) throw new Error("Block is not a Command Door");
  if (cmdDoorBlock.hasTag("lc:door_top_part")) {
    return tryControl(cmdDoorBlock.below(), mode);
  }
  const newNextAction = getAppropriateNextActionForControl(cmdDoorBlock.permutation, mode);
  mc85.system.run(() => {
    cmdDoorBlock.setPermutation(cmdDoorBlock.permutation.withState(STATE.nextAction, newNextAction));
  });
}
function getAppropriateNextActionForControl(permutation, mode) {
  const { major: currentStepMajor, minor: currentStepMinor } = getStep(permutation);
  const currentStepIndex = flattenCoordinates(currentStepMajor, currentStepMinor);
  const currentNextAction = permutation.getState(STATE.nextAction);
  switch (mode) {
    case "close":
      return currentStepIndex > minStepIndex ? "close" : "none";
    case "open":
      return currentStepIndex < maxStepIndex ? "open" : "none";
    case "switch":
      return currentStepIndex < maxStepIndex || currentNextAction === "close" ? "open" : currentStepIndex > minStepIndex || currentNextAction === "open" ? "close" : "none";
  }
}

// scripts/utils/clearance_level.ts
import * as mc86 from "@minecraft/server";
function getClearanceLevel2(itemStack) {
  if (!itemStack) return -1;
  if (itemStack.hasTag("lc:keycard_o5")) return 6;
  if (itemStack.hasTag("lc:keycard_lvl5")) return 5;
  if (itemStack.hasTag("lc:keycard_lvl4")) return 4;
  if (itemStack.hasTag("lc:keycard_lvl3")) return 3;
  if (itemStack.hasTag("lc:keycard_lvl2")) return 2;
  if (itemStack.hasTag("lc:keycard_lvl1")) return 1;
  if (itemStack.hasTag("lc:keycard_lvl0")) return 0;
  if (itemStack.typeId.startsWith("sra")) {
    switch (itemStack.typeId) {
      case "scp:cf_o5_keycard":
      case "sra:classified_o5_keycard":
        return 6;
      case "scp:cf_omni_keycard":
      case "scp:cf_lvl_5_keycard":
      case "sra:classified_omni_keycard":
      case "sra:classified_lvl_5_keycard":
        return 5;
      case "scp:cf_lvl_4_keycard":
      case "sra:classified_lvl_4_keycard":
        return 4;
      case "scp:cf_lvl_3_keycard":
      case "sra:classified_lvl_3_keycard":
        return 3;
      case "scp:cf_lvl_2_keycard":
      case "sra:classified_lvl_2_keycard":
        return 2;
      case "scp:cf_lvl_1_keycard":
      case "sra:classified_lvl_1_keycard":
        return 1;
      case "scp:cf_lvl_0_keycard":
      case "sra:containment_box_keycard":
        return 0;
      default:
        return -1;
    }
  }
  return -1;
}
function getEntityClearanceLevel(target) {
  if (!target.isValid) return -1;
  if (target instanceof mc86.Player) {
    const inventory = target.getComponent("inventory");
    if (inventory) {
      for (let i = 0; i < inventory.container.size; i++) {
        const slot = inventory.container.getSlot(i);
        const foundScp005 = slot.hasItem() && slot.typeId === "lc:scpdy_scp005";
        if (foundScp005) return 6;
      }
    }
    const equippable = target.getComponent("equippable");
    if (equippable) {
      const mainhandCl = getClearanceLevel2(equippable.getEquipment(mc86.EquipmentSlot.Mainhand));
      const offhandCl = getClearanceLevel2(equippable.getEquipment(mc86.EquipmentSlot.Offhand));
      const maxCl = Math.max(mainhandCl, offhandCl);
      if (maxCl !== -1) return maxCl;
    }
    return -1;
  }
  return -1;
}

// scripts/utils/player.ts
import * as mc87 from "@minecraft/server";
function isCreativeOrSpectator2(player) {
  if (!player.isValid) return false;
  const gameMode = player.getGameMode();
  return gameMode === mc87.GameMode.Creative || gameMode === mc87.GameMode.Spectator;
}
function consumeHandItem(player, options) {
  if (!player.isValid) return 0;
  const equippable = player.getComponent("equippable");
  if (!equippable) return 0;
  const mainhandSlot = equippable.getEquipmentSlot(mc87.EquipmentSlot.Mainhand);
  const mainhandItemStack = mainhandSlot.getItem();
  if (!mainhandItemStack) return 0;
  if (!options?.filter?.(mainhandItemStack)) return 0;
  let amount = mainhandItemStack.amount;
  let consumed = 0;
  const max = options.max ?? 1;
  for (let i = 0; i < max; i++) {
    if (amount <= 0) break;
    amount--;
    consumed++;
  }
  if (amount > 1) {
    mainhandItemStack.amount = amount;
    mainhandSlot.setItem(mainhandItemStack);
  } else {
    mainhandSlot.setItem(void 0);
  }
  return consumed;
}

// scripts/features/doors/easy_door.ts
import * as mc88 from "@minecraft/server";
import { ModalFormData as ModalFormData9 } from "@minecraft/server-ui";
var STATE2 = {
  isBottomPart: "lc:is_bottom_part",
  clearanceLevel: "lc:clearance_level",
  nextAction: "lc:next_action",
  stepMajor: "lc:step_major",
  stepMinor: "lc:step_minor"
};
var MIN_STEP_INDEX = 0;
var MAX_STEP_INDEX = 15;
mc88.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:easy_door", COMPONENT2);
});
var COMPONENT2 = {
  beforeOnPlayerPlace(arg) {
    arg.cancel = true;
    mc88.system.run(() => {
      if (!arg.player) return;
      if (!arg.player.isValid) return;
      showPlacementForm(arg.player, arg);
    });
  },
  onPlace({ block }) {
    const isBottomPart = Boolean(block.permutation.getState(STATE2.isBottomPart));
    if (!isBottomPart) {
      const blockBelow = block.below();
      if (!blockBelow || blockBelow.typeId === block.typeId) return;
      block.setType("minecraft:air");
      return;
    }
    const blockAbove = block.above();
    if (!blockAbove || !(blockAbove.isAir || block.isLiquid)) {
      destroyBlock(block);
      return;
    }
    const upperPartPermutation = block.permutation.withState(STATE2.isBottomPart, false);
    blockAbove.setPermutation(upperPartPermutation);
  },
  onPlayerBreak({ block, brokenBlockPermutation }) {
    const isBottomPart = Boolean(brokenBlockPermutation.getState(STATE2.isBottomPart));
    const otherPartBlock = isBottomPart ? block.above() : block.below();
    if (!otherPartBlock || otherPartBlock.typeId !== brokenBlockPermutation.type.id) return;
    destroyBlock(otherPartBlock);
  },
  onTick({ block, dimension }, arg1) {
    const params = arg1.params;
    const isBottomPart = Boolean(block.permutation.getState(STATE2.isBottomPart));
    if (!isBottomPart) return;
    const otherPartBlock = isBottomPart ? block.above() : block.below();
    if (!otherPartBlock || otherPartBlock.typeId !== block.typeId) return;
    const nextAction = block.permutation.getState(STATE2.nextAction);
    const { major: currentStepMajor, minor: currentStepMinor } = getStep2(block.permutation);
    const currentStepIndex = flattenCoordinates(currentStepMajor, currentStepMinor);
    const newStepIndex = getUpdatedStepIndex2(currentStepIndex, nextAction, MIN_STEP_INDEX, MAX_STEP_INDEX);
    const newNextAction = (
      // idk wtf is going on
      mc88.system.currentTick % 2 === 0 && (newStepIndex <= MIN_STEP_INDEX || newStepIndex >= MAX_STEP_INDEX) ? "none" : nextAction
    );
    const newStepUnflat = unflattenToCoordinates(newStepIndex, 4);
    block.setPermutation(
      block.permutation.withState(STATE2.nextAction, newNextAction).withState(STATE2.stepMajor, newStepUnflat.major).withState(STATE2.stepMinor, newStepUnflat.minor)
    );
    otherPartBlock.setPermutation(
      otherPartBlock.permutation.withState(STATE2.stepMajor, newStepUnflat.major).withState(STATE2.stepMinor, newStepUnflat.minor)
    );
    if (mc88.system.currentTick % 2 !== 0) return currentStepIndex;
    if (params.openSound && nextAction === "open" && newStepIndex === 1) {
      dimension.playSound(params.openSound.id, block.location, {
        volume: params.openSound.volume,
        pitch: params.openSound.pitch
      });
    }
    if (params.closeSound && nextAction === "close" && newStepIndex === 14) {
      dimension.playSound(params.closeSound.id, block.location, {
        volume: params.closeSound.volume,
        pitch: params.closeSound.pitch
      });
    }
  },
  onPlayerInteract({ block, dimension, player }) {
    if (!player) return;
    const minClearanceLevel = Number(block.permutation.getState(STATE2.clearanceLevel));
    const playerClearanceLevel = getEntityClearanceLevel(player);
    const isAccepted = playerClearanceLevel >= minClearanceLevel;
    if (!isAccepted) {
      dimension.playSound("scpdy.interact.keycard_reader.deny", block.center());
      player.onScreenDisplay.setActionBar({
        translate: "scpdy.misc.text.youDontHaveRequiredClearanceLevel",
        with: [String(minClearanceLevel)]
      });
      return;
    }
    mc88.system.run(() => {
      tryControl2(block, "switch");
    });
    if (minClearanceLevel !== -1) {
      dimension.playSound("scpdy.interact.keycard_reader.accept", block.center());
    }
  }
};
async function showPlacementForm(player, e) {
  const formData = new ModalFormData9();
  formData.title({ translate: "scpdy.easyDoor" });
  formData.dropdown(
    { translate: "scpdy.misc.text.clearanceLevel" },
    [
      { translate: "scpdy.misc.text.none" },
      { translate: "scpdy.misc.text.clearanceLevel.level_0" },
      { translate: "scpdy.misc.text.clearanceLevel.level_1" },
      { translate: "scpdy.misc.text.clearanceLevel.level_2" },
      { translate: "scpdy.misc.text.clearanceLevel.level_3" },
      { translate: "scpdy.misc.text.clearanceLevel.level_4" },
      { translate: "scpdy.misc.text.clearanceLevel.level_5" },
      { translate: "scpdy.misc.text.clearanceLevel.o5" }
    ],
    {
      defaultValueIndex: 0,
      tooltip: { translate: "scpdy.misc.text.clearanceLevel.tooltip" }
    }
  );
  const response = await formData.show(player);
  if (response.canceled) return;
  if (!response.formValues) return;
  if (!player.isValid) return;
  const clearanceLevelRes = Number(response.formValues[0]);
  const clearanceLevel = clearanceLevelRes - 1;
  const newPermutation = e.permutationToPlace.withState(STATE2.clearanceLevel, clearanceLevel);
  if (!e.block.isValid) return;
  if (!e.block.isAir && !e.block.isLiquid) return;
  const shouldAbort = !isCreativeOrSpectator2(player) && consumeHandItem(player, {
    filter: (itemStack) => itemStack.typeId === e.permutationToPlace.getItemStack()?.typeId,
    max: 1
  }) <= 0;
  if (shouldAbort) return;
  e.block.setPermutation(newPermutation);
  e.dimension.playSound("place.iron", e.block.center(), { pitch: 0.81 });
}
function getUpdatedStepIndex2(currentStepIndex, nextAction, minStepIndex2, maxStepIndex2) {
  if (mc88.system.currentTick % 2 !== 0) return currentStepIndex;
  if (nextAction === "open" && currentStepIndex < maxStepIndex2) {
    return currentStepIndex + 1;
  }
  if (nextAction === "close" && currentStepIndex > minStepIndex2) {
    return currentStepIndex - 1;
  }
  return currentStepIndex;
}
function getStep2(permutation) {
  const major = Number(permutation.getState(STATE2.stepMajor));
  const minor = Number(permutation.getState(STATE2.stepMinor));
  return { major, minor };
}
function tryControl2(easyDoorBlock, mode) {
  if (!easyDoorBlock.isValid) throw new Error("Block is invalid");
  if (!easyDoorBlock.hasTag("lc:easy_door")) throw new Error("Block is not a Command Door");
  if (easyDoorBlock.hasTag("lc:door_top_part")) {
    return tryControl2(easyDoorBlock.below(), mode);
  }
  const newNextAction = getAppropriateNextActionForControl2(easyDoorBlock.permutation, mode);
  easyDoorBlock.setPermutation(easyDoorBlock.permutation.withState(STATE2.nextAction, newNextAction));
}
function getAppropriateNextActionForControl2(permutation, mode) {
  const { major: currentStepMajor, minor: currentStepMinor } = getStep2(permutation);
  const currentStepIndex = flattenCoordinates(currentStepMajor, currentStepMinor);
  const currentNextAction = permutation.getState(STATE2.nextAction);
  switch (mode) {
    case "close":
      return currentStepIndex > MIN_STEP_INDEX ? "close" : "none";
    case "open":
      return currentStepIndex < MAX_STEP_INDEX ? "open" : "none";
    case "switch":
      return currentStepIndex < MAX_STEP_INDEX || currentNextAction === "close" ? "open" : currentStepIndex > MIN_STEP_INDEX || currentNextAction === "open" ? "close" : "none";
  }
}

// scripts/features/doors/relay_door.ts
import * as mc91 from "@minecraft/server";

// scripts/utils/direction.ts
import * as mc89 from "@minecraft/server";
function reverseDirection(direction) {
  switch (direction) {
    case mc89.Direction.Up:
      return mc89.Direction.Down;
    case mc89.Direction.Down:
      return mc89.Direction.Up;
    case mc89.Direction.North:
      return mc89.Direction.South;
    case mc89.Direction.South:
      return mc89.Direction.North;
    case mc89.Direction.West:
      return mc89.Direction.East;
    case mc89.Direction.East:
      return mc89.Direction.West;
    default:
      throw new Error(`Unknown direction: ${direction}`);
  }
}
function getRelativeBlockAtDirection(origin, direction, steps = 1) {
  switch (direction) {
    case mc89.Direction.Up:
      return origin.above(steps);
    case mc89.Direction.Down:
      return origin.below(steps);
    case mc89.Direction.North:
      return origin.north(steps);
    case mc89.Direction.South:
      return origin.south(steps);
    case mc89.Direction.West:
      return origin.west(steps);
    case mc89.Direction.East:
      return origin.east(steps);
    default:
      throw new Error(`Unknown direction: ${direction}`);
  }
}
function getBlockCardinalDirection2(permutation) {
  const blockDir = permutation.getState("minecraft:cardinal_direction");
  switch (blockDir) {
    case "north":
      return mc89.Direction.North;
    case "south":
      return mc89.Direction.South;
    case "west":
      return mc89.Direction.West;
    case "east":
      return mc89.Direction.East;
  }
  return void 0;
}
function getBlockFace(permutation) {
  const blockDir = permutation.getState("minecraft:block_face");
  switch (blockDir) {
    case "up":
      return mc89.Direction.Up;
    case "down":
      return mc89.Direction.Down;
    case "north":
      return mc89.Direction.North;
    case "south":
      return mc89.Direction.South;
    case "west":
      return mc89.Direction.West;
    case "east":
      return mc89.Direction.East;
  }
  return void 0;
}

// scripts/utils/wrench.ts
import * as mc90 from "@minecraft/server";
function isHoldingWrench2(player, checkOffhand = true) {
  if (!player.isValid) return false;
  const equippable = player.getComponent("equippable");
  if (!equippable) return false;
  const test = (itemStack) => {
    if (!itemStack) return false;
    if (itemStack.hasTag("lc:wrench")) return true;
    if (itemStack.typeId === "sra:hammer") return true;
    return false;
  };
  if (test(equippable.getEquipment(mc90.EquipmentSlot.Mainhand))) return true;
  if (checkOffhand && test(equippable.getEquipment(mc90.EquipmentSlot.Offhand))) return true;
  return false;
}

// scripts/features/doors/relay_door.ts
var STATE3 = {
  isBottomPart: "lc:is_bottom_part",
  detectRedstone: "lc:detect_redstone",
  powerLevelMajor: "lc:power_level_major",
  powerLevelMinor: "lc:power_level_minor",
  stepMajor: "lc:step_major",
  stepMinor: "lc:step_minor"
};
var DEFAULT_MIN_STEP_INDEX = 0;
var DEFAULT_MAX_STEP_INDEX = 15;
var POWER_LEVEL_DECREASE_INTERVAL = 12;
mc91.system.beforeEvents.startup.subscribe(({ blockComponentRegistry }) => {
  blockComponentRegistry.registerCustomComponent("scpdy:relay_door", COMPONENT3);
});
var COMPONENT3 = {
  onTick({ block, dimension }, { params: unknownParams }) {
    if (!block.hasTag("lc:relay_door")) return;
    const params = unknownParams;
    const minStepIndex2 = DEFAULT_MIN_STEP_INDEX;
    const maxStepIndex2 = DEFAULT_MAX_STEP_INDEX;
    const isBottomPart = Boolean(block.permutation.getState(STATE3.isBottomPart));
    if (!isBottomPart) return;
    const otherPartBlock = isBottomPart ? block.above() : block.below();
    if (!otherPartBlock || otherPartBlock.typeId !== block.typeId) return;
    const { major: currentPowerLevelMajor, minor: currentPowerLevelMinor } = getPowerLevel(block.permutation);
    const currentPowerLevel = flattenCoordinates(currentPowerLevelMajor, currentPowerLevelMinor, 3);
    const detectRedstone = Boolean(block.permutation.getState(STATE3.detectRedstone));
    const isRedstoneDetected = detectRedstone && checkRedstonePower(block);
    let newPowerLevel = getUpdatedPowerLevel(currentPowerLevel, detectRedstone, block, isRedstoneDetected);
    const { major: currentStepMajor, minor: currentStepMinor } = getStep3(block.permutation);
    const currentStepIndex = flattenCoordinates(currentStepMajor, currentStepMinor);
    const newStepIndex = getUpdatedStepIndex3(currentStepIndex, currentPowerLevel, minStepIndex2, maxStepIndex2);
    const newPowerLevelUnflat = unflattenToCoordinates(newPowerLevel, 3);
    const newStepUnflat = unflattenToCoordinates(newStepIndex, 4);
    block.setPermutation(
      block.permutation.withState(STATE3.powerLevelMajor, newPowerLevelUnflat.major).withState(STATE3.powerLevelMinor, newPowerLevelUnflat.minor).withState(STATE3.stepMajor, newStepUnflat.major).withState(STATE3.stepMinor, newStepUnflat.minor)
    );
    otherPartBlock.setPermutation(
      otherPartBlock.permutation.withState(STATE3.stepMajor, newStepUnflat.major).withState(STATE3.stepMinor, newStepUnflat.minor)
    );
    if (mc91.system.currentTick % 2 !== 0) return currentStepIndex;
    if (params.openSound && currentPowerLevel > 0 && newStepIndex === 1) {
      dimension.playSound(params.openSound.id, block.location, {
        volume: params.openSound.volume,
        pitch: params.openSound.pitch
      });
    }
    if (params.closeSound && !isRedstoneDetected && currentPowerLevel <= 0 && newStepIndex === 14) {
      dimension.playSound(params.closeSound.id, block.location, {
        volume: params.closeSound.volume,
        pitch: params.closeSound.pitch
      });
    }
  },
  beforeOnPlayerPlace({ player }) {
    if (!player) return;
    mc91.system.run(() => {
      if (!player.addTag("scpdy_sent_relay_door_tip")) return;
      mc91.system.runTimeout(() => {
        if (!player || !player.isValid) return;
        player.sendMessage({ translate: "scpdy.relayDoor.tip" });
        player.playSound("random.orb");
      }, 8);
    });
  },
  onPlace({ block }) {
    const isBottomPart = Boolean(block.permutation.getState(STATE3.isBottomPart));
    if (!isBottomPart) {
      const blockBelow = block.below();
      if (!blockBelow || blockBelow.typeId === block.typeId) return;
      block.setType("minecraft:air");
      return;
    }
    const blockAbove = block.above();
    if (!blockAbove || !(blockAbove.isAir || block.isLiquid)) {
      destroyBlock(block);
      return;
    }
    const upperPartPermutation = block.permutation.withState(STATE3.isBottomPart, false);
    blockAbove.setPermutation(upperPartPermutation);
  },
  onPlayerBreak({ block, brokenBlockPermutation }) {
    const isBottomPart = Boolean(brokenBlockPermutation.getState(STATE3.isBottomPart));
    const otherPartBlock = isBottomPart ? block.above() : block.below();
    if (!otherPartBlock || otherPartBlock.typeId !== brokenBlockPermutation.type.id) return;
    destroyBlock(otherPartBlock);
  },
  onPlayerInteract({ player, block: blockUnchecked }) {
    if (!player) return;
    if (!isHoldingWrench2(player)) return;
    const block = blockUnchecked.hasTag("lc:door_top_part") ? blockUnchecked.below() : blockUnchecked;
    const detectRedstone = Boolean(block.permutation.getState(STATE3.detectRedstone));
    block.setPermutation(block.permutation.withState(STATE3.detectRedstone, !detectRedstone));
    player.playSound("random.click");
    if (detectRedstone) {
      player.onScreenDisplay.setActionBar({
        translate: "scpdy.relayDoor.text.redstoneDetection.off"
      });
    } else {
      player.onScreenDisplay.setActionBar({
        translate: "scpdy.relayDoor.text.redstoneDetection.on"
      });
      if (player.addTag("scpdy_warned_relay_door_redstone_detection_lag")) {
        mc91.system.runTimeout(() => {
          if (!player.isValid) return;
          player.sendMessage({
            translate: "scpdy.relayDoor.text.redstoneDetection.warn"
          });
          player.playSound("note.bass");
        }, 15);
      }
    }
  }
};
function getUpdatedPowerLevel(currentPowerLevel, detectRedstone, block, isRedstoneDetected) {
  let newPowerLevel = currentPowerLevel;
  if (currentPowerLevel > 0 && mc91.system.currentTick % POWER_LEVEL_DECREASE_INTERVAL === 0) {
    newPowerLevel--;
  }
  if (isRedstoneDetected && newPowerLevel === 0) {
    newPowerLevel = 1;
  }
  return newPowerLevel;
}
function getUpdatedStepIndex3(currentStepIndex, currentPowerLevel, minStepIndex2, maxStepIndex2) {
  if (mc91.system.currentTick % 2 !== 0) return currentStepIndex;
  if (currentPowerLevel > 0 && currentStepIndex < maxStepIndex2) {
    return currentStepIndex + 1;
  }
  if (currentPowerLevel <= 0 && currentStepIndex > minStepIndex2) {
    return currentStepIndex - 1;
  }
  return currentStepIndex;
}
function getPowerLevel(permutation) {
  const major = Number(permutation.getState(STATE3.powerLevelMajor));
  const minor = Number(permutation.getState(STATE3.powerLevelMinor));
  return { major, minor };
}
function getStep3(permutation) {
  const major = Number(permutation.getState(STATE3.stepMajor));
  const minor = Number(permutation.getState(STATE3.stepMinor));
  return { major, minor };
}
function checkRedstonePower(block) {
  if (Number(block.getRedstonePower()) > 0) return true;
  if (Number(block.below(1)?.getRedstonePower()) > 0) return true;
  if (Number(block.below(2)?.getRedstonePower()) > 0) return true;
  return false;
}
MachineryInputEvents.on("onActivate", (data) => {
  if (data.mode !== "powerRelayDoors") return;
  const block = data.block ?? data.dimension.getBlock(data.location);
  if (!block) return;
  const blockBehind = getRelativeBlockAtDirection(block, data.pulseDirection ?? mc91.Direction.North);
  if (!blockBehind) return;
  tryPowerRelayDoorsFrom(blockBehind);
});
function tryPowerRelayDoorsFrom(block) {
  setPowerLevel(block);
  const north = block.north();
  if (north && setPowerLevel(north)) {
    const north2 = north.north();
    if (north2) setPowerLevel(north2);
  }
  const south = block.south();
  if (south && setPowerLevel(south)) {
    const south2 = south.south();
    if (south2) setPowerLevel(south2);
  }
  const west = block.west();
  if (west && setPowerLevel(west)) {
    const west2 = west.west();
    if (west2) setPowerLevel(west2);
  }
  const east = block.east();
  if (east && setPowerLevel(east)) {
    const east2 = east.east();
    if (east2) setPowerLevel(east2);
  }
  const below2 = block.below(2);
  below2 && setPowerLevel(below2);
}
function setPowerLevel(relayDoorBlock, powerLevel = 6) {
  if (!relayDoorBlock.isValid) return false;
  if (!relayDoorBlock.hasTag("lc:relay_door")) return false;
  if (relayDoorBlock.hasTag("lc:door_top_part")) {
    return setPowerLevel(relayDoorBlock.below());
  }
  const powerLevelUnflat = unflattenToCoordinates(powerLevel, 3);
  relayDoorBlock.setPermutation(
    relayDoorBlock.permutation.withState(STATE3.powerLevelMajor, powerLevelUnflat.major).withState(STATE3.powerLevelMinor, powerLevelUnflat.minor)
  );
  return true;
}

// scripts/features/fnet/fnet_manager_item.ts
import * as mc92 from "@minecraft/server";
import { ActionFormData as ActionFormData4, ModalFormData as ModalFormData10 } from "@minecraft/server-ui";
mc92.system.beforeEvents.startup.subscribe((event) => {
  event.itemComponentRegistry.registerCustomComponent("scpdy:fnet_manager", {
    onUse(arg0) {
      mc92.system.run(() => {
        showFnetSelectionForm(arg0.source);
      });
    }
  });
});
async function showFnetSelectionForm(player) {
  const fnets2 = getAllFnets();
  const formData = new ActionFormData4();
  formData.title({ translate: "scpdy.fnetManager" });
  formData.body({ translate: "scpdy.fnetManager.selectFnetToEdit" });
  formData.divider();
  fnets2.forEach((fnet) => formData.button(fnet.name));
  const response = await formData.show(player);
  if (!player.isValid) return;
  if (response.canceled) return;
  if (response.selection == void 0) return;
  const selected = fnets2[response.selection];
  await showFnetEditForm(player, selected);
}
async function showFnetEditForm(player, fnet) {
  const formData = new ActionFormData4();
  formData.title({ translate: "scpdy.fnetManager" });
  formData.body({ translate: "scpdy.fnetManager.selectedX", with: { rawtext: [fnet.name] } });
  formData.divider();
  formData.button({ translate: "scpdy.fnetManager.changeName" });
  formData.button({ translate: "scpdy.fnetManager.changeZoneNames" });
  const response = await formData.show(player);
  if (!player.isValid) return;
  if (response.canceled) return;
  if (response.selection === 0) {
    await showChangeFnetNameForm(player, fnet);
  } else if (response.selection === 1) {
    await showChangeFzoneNamesForm(player, fnet);
  }
}
async function showChangeFnetNameForm(player, fnet) {
  const formData = new ModalFormData10();
  formData.title({ translate: "scpdy.fnetManager" });
  formData.submitButton({ translate: "scpdy.misc.text.saveAndExit" });
  formData.textField(
    { translate: "scpdy.misc.text.newName" },
    { translate: "scpdy.misc.text.enterNewName" },
    {
      defaultValue: fnet.getCustomName(),
      tooltip: { translate: "scpdy.misc.text.leaveThisEmptyToReset" }
    }
  );
  const response = await formData.show(player);
  if (!player.isValid) return;
  if (response.canceled) return;
  if (!response.formValues) return;
  const newName = String(response.formValues[0]).trim();
  if (newName === "") {
    fnet.setCustomName(void 0);
    return;
  }
  fnet.setCustomName(newName);
}
async function showChangeFzoneNamesForm(player, fnet) {
  const zones = fnet.getAllZones();
  const formData = new ModalFormData10();
  formData.title({ translate: "scpdy.fnetManager" });
  formData.submitButton({ translate: "scpdy.misc.text.saveAndExit" });
  formData.label({ translate: "scpdy.fnetManager.youCanEditZoneNames" });
  formData.divider();
  for (const zone of zones) {
    formData.textField(
      zone.name,
      { translate: "scpdy.misc.text.enterNewName" },
      {
        defaultValue: zone.getCustomName(),
        tooltip: { translate: "scpdy.misc.text.leaveThisEmptyToReset" }
      }
    );
  }
  const response = await formData.show(player);
  if (!player.isValid) return;
  if (response.canceled) return;
  if (!response.formValues) return;
  for (let i = 0; i < response.formValues.length; i++) {
    const formValue = response.formValues[i];
    if (i < 2) continue;
    const zoneIndex = i - 2;
    const zone = zones[zoneIndex];
    const newName = String(formValue).trim();
    if (newName === "") {
      zone.setCustomName(void 0);
      continue;
    }
    zone.setCustomName(newName);
  }
}

// scripts/features/fnet/upgrade_old_fnet_stuff.ts
import { system as system56, world as world45 } from "@minecraft/server";
world45.afterEvents.worldLoad.subscribe(() => {
  for (let i = 0; i < MAX_FNET_COUNT; i++) {
    const newNet = getFnet(i);
    const oldNetName = world45.getDynamicProperty(`scpdy_facilityNetwork_${i}_name`);
    if (typeof oldNetName === "string") newNet.setCustomName(oldNetName);
    system56.run(() => {
      world45.setDynamicProperty(`scpdy_facilityNetwork_${i}_name`, void 0);
    });
    for (let j = 0; j < MAX_FZONE_COUNT; j++) {
      const newZone = newNet.getZone(j);
      const oldZoneName = world45.getDynamicProperty(`scpdy_facilityZone_${i}_${j}_name`);
      const oldZoneLkdnActive = world45.getDynamicProperty(`scpdy_facilityZone_${i}_${j}_lockdown`);
      const oldZoneLkdnDelay = world45.getDynamicProperty(`scpdy_facilityZone_${i}_${j}_lockdownDelay`);
      const oldZoneLkdnDuration = world45.getDynamicProperty(`scpdy_facilityZone_${i}_${j}_lockdownDuration`);
      if (typeof oldZoneName === "string") newZone.setCustomName(oldZoneName);
      if (oldZoneLkdnActive === true) newZone.setLkdnActive(true);
      if (typeof oldZoneLkdnDelay === "number") newZone.setScheduledLkdnDelay(oldZoneLkdnDelay);
      if (typeof oldZoneLkdnDuration === "number") newZone.setLkdnDuration(oldZoneLkdnDuration);
      system56.run(() => {
        world45.setDynamicProperty(`scpdy_facilityZone_${i}_${j}_name`, void 0);
        world45.setDynamicProperty(`scpdy_facilityZone_${i}_${j}_lockdown`, void 0);
        world45.setDynamicProperty(`scpdy_facilityZone_${i}_${j}_lockdownDelay`, void 0);
        world45.setDynamicProperty(`scpdy_facilityZone_${i}_${j}_lockdownDuration`, void 0);
      });
    }
  }
});

// scripts/features/furniture/panel_fan.ts
import * as mc93 from "@minecraft/server";
var STATE4 = {
  fanRotation: "lc:fan_rotation"
};
mc93.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:panel_fan", {
    onTick({ block }) {
      tickFanRotation(block);
    }
  });
  event.blockComponentRegistry.registerCustomComponent("scpdy:panel_fan_static", {
    onPlace({ block }) {
      randomizeFanRotation(block);
    }
  });
});
function randomizeFanRotation(block) {
  const nextFanRotation = randi(0, 4);
  block.setPermutation(block.permutation.withState(STATE4.fanRotation, nextFanRotation));
}
function tickFanRotation(block) {
  const fanRotationUnflat = Number(block.permutation.getState(STATE4.fanRotation));
  const nextFanRotation = (fanRotationUnflat + 1) % 5;
  block.setPermutation(block.permutation.withState(STATE4.fanRotation, nextFanRotation));
  if (mc93.system.currentTick % 20 === 0) {
    block.dimension.playSound("scpdy.misc.fan_rotation.moderate", block.center(), {
      pitch: randf(0.98, 1.02),
      volume: 1
    });
  }
}

// scripts/features/gore/gib_event.ts
import * as mc94 from "@minecraft/server";
mc94.world.afterEvents.entityHurt.subscribe((e) => {
  if (!e.hurtEntity.isValid) return;
  if (!e.hurtEntity.matches({ families: ["gibbable_on_death"] })) return;
  const health = e.hurtEntity.getComponent("health");
  if (!health) return;
  if (health.currentValue > 0) return;
  const doesDamageCausePreventGib = e.damageSource.cause === mc94.EntityDamageCause.selfDestruct || e.damageSource.cause === mc94.EntityDamageCause.void || e.damageSource.cause === mc94.EntityDamageCause.fire || e.damageSource.cause === mc94.EntityDamageCause.fireTick || e.damageSource.cause === mc94.EntityDamageCause.campfire;
  if (doesDamageCausePreventGib) return;
  const isExplosionDamage = e.damageSource.cause === mc94.EntityDamageCause.blockExplosion || e.damageSource.cause === mc94.EntityDamageCause.entityExplosion;
  const instantGib = e.hurtEntity.matches({ families: ["instant_gib_on_death"] });
  const isBigDamage = Math.max(15, health.effectiveMax) <= e.damage;
  const doGib = isExplosionDamage || instantGib || isBigDamage;
  if (doGib) {
    try {
      e.hurtEntity.triggerEvent("lc:gib");
    } catch {
    }
    return;
  }
  tryTransformIntoCorpseOrGib(e.hurtEntity);
});
function tryTransformIntoCorpseOrGib(entity) {
  try {
    entity.triggerEvent("lc:transform_into_dead");
  } catch {
    try {
      entity.triggerEvent("lc:gib");
    } catch {
    }
  }
}
mc94.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  ({ entity }) => {
    try {
      const location = midpoint2(entity.location, entity.getHeadLocation());
      spawnGoreExplosion(entity.dimension, location);
      entity.remove();
    } catch {
    }
  },
  {
    eventTypes: ["lc:gib"]
  }
);

// scripts/features/lkdn/lkdn_alarm.ts
import * as mc95 from "@minecraft/server";
import { ActionFormData as ActionFormData5 } from "@minecraft/server-ui";
var STATE5 = {
  fnetIndex: "lc:fnet_index",
  fzoneIndex: "lc:fzone_index",
  isActive: "lc:is_active",
  soundLoop: "lc:sound_loop"
};
mc95.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:lkdn_alarm", COMPONENT4);
});
var COMPONENT4 = {
  beforeOnPlayerPlace(e) {
    e.cancel = true;
    mc95.system.run(() => {
      beforeOnPlayerPlaceAsync(e);
    });
  },
  onPlayerInteract(e) {
    if (!e.player) return;
    if (!isHoldingWrench2(e.player)) return;
    const soundLoop = Boolean(e.block.permutation.getState(STATE5.soundLoop));
    if (soundLoop) {
      e.player.onScreenDisplay.setActionBar({ translate: "scpdy.machinery.alarm.soundLoop.off" });
    } else {
      e.player.onScreenDisplay.setActionBar({ translate: "scpdy.machinery.alarm.soundLoop.on" });
    }
    e.block.setPermutation(e.block.permutation.withState(STATE5.soundLoop, !soundLoop));
    e.player.playSound("random.click");
  },
  onTick({ block, dimension }) {
    const soundLoop = Boolean(block.permutation.getState(STATE5.soundLoop));
    const fzone = getFzone(block.permutation);
    const shouldBeActive = fzone.isLkdnActive || fzone.isLkdnScheduled;
    const isCurrentlyActive = Boolean(block.permutation.getState(STATE5.isActive));
    if (shouldBeActive && !isCurrentlyActive) {
      block.setPermutation(block.permutation.withState(STATE5.isActive, true));
    } else if (!shouldBeActive && isCurrentlyActive) {
      block.setPermutation(block.permutation.withState(STATE5.isActive, false));
    }
    const shouldPlaySound = shouldBeActive && soundLoop || shouldBeActive && !isCurrentlyActive;
    if (shouldPlaySound) {
      dimension.playSound("scpdy.misc.alarm", block.center(), {
        volume: 1.1
      });
    }
  }
};
function getFzone(permutation) {
  const fnetIndex = Number(permutation.getState(STATE5.fnetIndex));
  const fzoneIndex = Number(permutation.getState(STATE5.fzoneIndex));
  const fnet = getFnet(fnetIndex);
  const fzone = fnet.getZone(fzoneIndex);
  return fzone;
}
async function beforeOnPlayerPlaceAsync(e) {
  if (!e.player) return;
  const fnets2 = getAllFnets();
  const formData1 = new ActionFormData5();
  formData1.title({ translate: "scpdy.lkdnAlarm" });
  formData1.body({ translate: "scpdy.fnetManager.selectFnet" });
  fnets2.forEach((fnet2) => formData1.button(fnet2.name));
  const response1 = await formData1.show(e.player);
  if (!e.player.isValid) return;
  if (response1.canceled) return;
  if (response1.selection == void 0) return;
  const fnet = getFnet(response1.selection);
  const zones = fnet.getAllZones();
  const formData2 = new ActionFormData5();
  formData2.title({ translate: "scpdy.lkdnAlarm" });
  formData2.body({ translate: "scpdy.fnetManager.selectFzone" });
  zones.forEach((zone2) => formData2.button(zone2.name));
  const response2 = await formData2.show(e.player);
  if (!e.player.isValid) return;
  if (response2.canceled) return;
  if (response2.selection == void 0) return;
  const zone = zones[response2.selection];
  if (!e.block.isValid) return;
  if (!e.block.isAir && !e.block.isLiquid) return;
  const shouldAbort = !isCreativeOrSpectator2(e.player) && consumeHandItem(e.player, {
    filter: (itemStack) => itemStack.typeId === e.permutationToPlace.getItemStack()?.typeId,
    max: 1
  }) <= 0;
  if (shouldAbort) return;
  const newPermutation = e.permutationToPlace.withState(STATE5.fnetIndex, fnet.index).withState(STATE5.fzoneIndex, zone.index);
  e.block.setPermutation(newPermutation);
  e.dimension.playSound("place.iron", e.block.center(), { pitch: 0.81 });
}

// scripts/features/lkdn/lkdn_door.ts
import * as mc96 from "@minecraft/server";
import { ActionFormData as ActionFormData6 } from "@minecraft/server-ui";
var STATE6 = {
  isBottomPart: "lc:is_bottom_part",
  nextAction: "lc:next_action",
  fnetIndex: "lc:fnet_index",
  fzoneIndexMinor: "lc:fzone_index_minor",
  fzoneIndexMajor: "lc:fzone_index_major",
  stepMajor: "lc:step_major",
  stepMinor: "lc:step_minor"
};
var MIN_STEP_INDEX2 = 0;
var MAX_STEP_INDEX2 = 15;
mc96.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:lkdn_door", COMPONENT5);
});
var COMPONENT5 = {
  onPlace({ block }) {
    const isBottomPart = Boolean(block.permutation.getState(STATE6.isBottomPart));
    if (!isBottomPart) {
      const blockBelow = block.below();
      if (!blockBelow || blockBelow.typeId === block.typeId) return;
      block.setType("minecraft:air");
      return;
    }
    const blockAbove = block.above();
    if (!blockAbove || !(blockAbove.isAir || block.isLiquid)) {
      destroyBlock(block);
      return;
    }
    const upperPartPermutation = block.permutation.withState(STATE6.isBottomPart, false);
    blockAbove.setPermutation(upperPartPermutation);
  },
  onPlayerBreak({ block, brokenBlockPermutation }) {
    const isBottomPart = Boolean(brokenBlockPermutation.getState(STATE6.isBottomPart));
    const otherPartBlock = isBottomPart ? block.above() : block.below();
    if (!otherPartBlock || otherPartBlock.typeId !== brokenBlockPermutation.type.id) return;
    destroyBlock(otherPartBlock);
  },
  beforeOnPlayerPlace(e) {
    e.cancel = true;
    mc96.system.run(() => {
      if (!e.player) return;
      showPlacementForm2(e.player, e);
    });
  },
  onTick(arg0, arg1) {
    onTick4(arg0, arg1.params);
  }
};
function getZone(block) {
  const fnet = getFnet(Number(block.permutation.getState(STATE6.fnetIndex)));
  const zoneIndexUnflat = getZoneIndexUnflat(block.permutation);
  const zoneIndex = flattenCoordinates(zoneIndexUnflat.major, zoneIndexUnflat.minor);
  const zone = fnet.getZone(zoneIndex);
  return zone;
}
function getZoneIndexUnflat(permutation) {
  const major = Number(permutation.getState(STATE6.fzoneIndexMajor));
  const minor = Number(permutation.getState(STATE6.fzoneIndexMinor));
  return { major, minor };
}
function getStep4(permutation) {
  const major = Number(permutation.getState(STATE6.stepMajor));
  const minor = Number(permutation.getState(STATE6.stepMinor));
  return { major, minor };
}
async function showPlacementForm2(player, e) {
  const fnets2 = getAllFnets();
  const formData1 = new ActionFormData6();
  formData1.title({ translate: "scpdy.lkdnDoor" });
  formData1.body({ translate: "scpdy.fnetManager.selectFnet" });
  fnets2.forEach((fnet2) => formData1.button(fnet2.name));
  const response1 = await formData1.show(player);
  if (!player.isValid) return;
  if (response1.canceled) return;
  if (response1.selection == void 0) return;
  const fnet = getFnet(response1.selection);
  const zones = fnet.getAllZones();
  const formData2 = new ActionFormData6();
  formData2.title({ translate: "scpdy.lkdnDoor" });
  formData2.body({ translate: "scpdy.fnetManager.selectFzone" });
  zones.forEach((zone2) => formData2.button(zone2.name));
  const response2 = await formData2.show(player);
  if (!player.isValid) return;
  if (response2.canceled) return;
  if (response2.selection == void 0) return;
  const zone = zones[response2.selection];
  if (!e.block.isValid) return;
  if (!e.block.isAir && !e.block.isLiquid) return;
  const shouldAbort = !isCreativeOrSpectator2(player) && consumeHandItem(player, {
    filter: (itemStack) => itemStack.typeId === e.permutationToPlace.getItemStack()?.typeId,
    max: 1
  }) <= 0;
  if (shouldAbort) return;
  const zoneIndexUnflat = unflattenToCoordinates(zone.index, 3);
  const newPermutation = e.permutationToPlace.withState(STATE6.fnetIndex, fnet.index).withState(STATE6.fzoneIndexMajor, zoneIndexUnflat.major).withState(STATE6.fzoneIndexMinor, zoneIndexUnflat.minor);
  e.block.setPermutation(newPermutation);
  e.dimension.playSound("place.iron", e.block.center(), { pitch: 0.81 });
}
function onTick4({ block, dimension }, params) {
  const isBottomPart = Boolean(block.permutation.getState(STATE6.isBottomPart));
  if (!isBottomPart) return;
  const otherPartBlock = isBottomPart ? block.above() : block.below();
  if (!otherPartBlock || otherPartBlock.typeId !== block.typeId) return;
  const { major: currentStepMajor, minor: currentStepMinor } = getStep4(block.permutation);
  const currentStepIndex = flattenCoordinates(currentStepMajor, currentStepMinor);
  const currentNextAction = block.permutation.getState(STATE6.nextAction);
  const newNextAction = updateNextActionBasedOnZoneState(block, currentStepIndex, currentNextAction);
  const newStepIndex = getUpdatedStepIndex4(currentStepIndex, newNextAction);
  const newStepUnflat = unflattenToCoordinates(newStepIndex, 4);
  block.setPermutation(
    block.permutation.withState(STATE6.nextAction, newNextAction).withState(STATE6.stepMajor, newStepUnflat.major).withState(STATE6.stepMinor, newStepUnflat.minor)
  );
  otherPartBlock.setPermutation(
    otherPartBlock.permutation.withState(STATE6.stepMajor, newStepUnflat.major).withState(STATE6.stepMinor, newStepUnflat.minor)
  );
  if (mc96.system.currentTick % 2 !== 0) return;
  if (params.openSound && newNextAction === "open" && newStepIndex === 1) {
    dimension.playSound(params.openSound.id, block.location, {
      volume: params.openSound.volume,
      pitch: params.openSound.pitch
    });
  }
  if (params.closeSound && newNextAction === "close" && newStepIndex === 14) {
    dimension.playSound(params.closeSound.id, block.location, {
      volume: params.closeSound.volume,
      pitch: params.closeSound.pitch
    });
  }
}
function getUpdatedStepIndex4(currentStepIndex, nextAction) {
  if (mc96.system.currentTick % 2 !== 0) return currentStepIndex;
  if (nextAction === "open" && currentStepIndex < MAX_STEP_INDEX2) {
    return currentStepIndex + 1;
  }
  if (nextAction === "close" && currentStepIndex > MIN_STEP_INDEX2) {
    return currentStepIndex - 1;
  }
  return currentStepIndex;
}
function updateNextActionBasedOnZoneState(block, currentStepIndex, currentNextAction) {
  const zone = getZone(block);
  let newNextAction;
  if (zone.isLkdnActive && currentStepIndex > MIN_STEP_INDEX2) {
    newNextAction = "close";
  } else if (!zone.isLkdnActive && currentStepIndex < MAX_STEP_INDEX2) {
    newNextAction = "open";
  } else {
    newNextAction = "none";
  }
  if (currentNextAction !== newNextAction) {
    block.setPermutation(block.permutation.withState(STATE6.nextAction, newNextAction));
  }
  return newNextAction;
}

// scripts/features/lkdn/lkdn_panel.ts
import * as mc97 from "@minecraft/server";
import { ActionFormData as ActionFormData7, ModalFormData as ModalFormData11 } from "@minecraft/server-ui";
var STATE7 = {
  isLkdnActive: "lc:is_lkdn_active",
  detectedRedstone: "lc:detected_redstone",
  fnetIndex: "lc:fnet_index",
  fzoneIndex: "lc:fzone_index"
};
mc97.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:lkdn_panel", COMPONENT6);
});
var COMPONENT6 = {
  beforeOnPlayerPlace(e) {
    e.cancel = true;
    mc97.system.run(() => {
      if (!e.player) return;
      showPlacementForm3(e.player, e);
    });
  },
  onPlayerInteract(e) {
    mc97.system.run(() => {
      onInteract3(e);
    });
  },
  onTick({ block }) {
    mc97.system.run(() => {
      if (!block.isValid) return;
      updateRedstonePowering(block);
      updateVisual(block);
    });
  }
};
function getZone2(panelBlock) {
  const fnet = getFnet(Number(panelBlock.permutation.getState(STATE7.fnetIndex)));
  const zone = fnet.getZone(Number(panelBlock.permutation.getState(STATE7.fzoneIndex)));
  return zone;
}
function updateVisual(panelBlock) {
  const zone = getZone2(panelBlock);
  const isPanelActive = Boolean(panelBlock.permutation.getState(STATE7.isLkdnActive));
  if (isPanelActive == zone.isLkdnActive) return;
  panelBlock.setPermutation(panelBlock.permutation.withState(STATE7.isLkdnActive, zone.isLkdnActive));
  panelBlock.dimension.playSound("scpdy.interact.lever.flip", panelBlock.center(), { volume: 1.1 });
}
function updateRedstonePowering(panelBlock) {
  const zone = getZone2(panelBlock);
  const previouslyDetectedRedstone = Boolean(panelBlock.permutation.getState(STATE7.detectedRedstone));
  const currentlyDetectedRedstone = checkRedstonePower2(panelBlock);
  if (previouslyDetectedRedstone && !currentlyDetectedRedstone) {
    zone.stopLkdn();
    updateVisual(panelBlock);
  }
  if (currentlyDetectedRedstone) {
    zone.startLkdn(Infinity);
    updateVisual(panelBlock);
  }
  panelBlock.setPermutation(panelBlock.permutation.withState(STATE7.detectedRedstone, currentlyDetectedRedstone));
}
function checkRedstonePower2(panelBlock) {
  const cardinalDir = getBlockCardinalDirection2(panelBlock.permutation);
  if (!cardinalDir) throw new Error("Failed to get block direction");
  const cardinalDirReversed = reverseDirection(cardinalDir);
  const backBlock = getRelativeBlockAtDirection(panelBlock, cardinalDirReversed, 1);
  const redstonePower = backBlock?.getRedstonePower();
  if (redstonePower == void 0 || redstonePower <= 0) return false;
  return true;
}
async function showPlacementForm3(player, e) {
  const fnets2 = getAllFnets();
  const formData1 = new ActionFormData7();
  formData1.title({ translate: "scpdy.lkdnPanel" });
  formData1.body({ translate: "scpdy.fnetManager.selectFnet" });
  fnets2.forEach((fnet2) => formData1.button(fnet2.name));
  const response1 = await formData1.show(player);
  if (!player.isValid) return;
  if (response1.canceled) return;
  if (response1.selection == void 0) return;
  const fnet = getFnet(response1.selection);
  const zones = fnet.getAllZones();
  const formData2 = new ActionFormData7();
  formData2.title({ translate: "scpdy.lkdnPanel" });
  formData2.body({ translate: "scpdy.fnetManager.selectFzone" });
  zones.forEach((zone2) => formData2.button(zone2.name));
  const response2 = await formData2.show(player);
  if (!player.isValid) return;
  if (response2.canceled) return;
  if (response2.selection == void 0) return;
  const zone = zones[response2.selection];
  if (!e.block.isValid) return;
  if (!e.block.isAir && !e.block.isLiquid) return;
  const shouldAbort = !isCreativeOrSpectator2(player) && consumeHandItem(player, {
    filter: (itemStack) => itemStack.typeId === e.permutationToPlace.getItemStack()?.typeId,
    max: 1
  }) <= 0;
  if (shouldAbort) return;
  const newPermutation = e.permutationToPlace.withState(STATE7.fnetIndex, fnet.index).withState(STATE7.fzoneIndex, zone.index);
  e.block.setPermutation(newPermutation);
  e.dimension.playSound("place.iron", e.block.center(), { pitch: 0.81 });
}
async function onInteract3(e) {
  if (!e.player) return;
  if (!e.player.isValid) return;
  const zone = getZone2(e.block);
  if (checkRedstonePower2(e.block)) {
    e.player.onScreenDisplay.setActionBar({ translate: "scpdy.misc.text.accessDenied" });
    e.dimension.playSound("scpdy.interact.button.click", e.block.center(), {
      pitch: 0.7,
      volume: 0.9
    });
    return;
  }
  if (zone.isLkdnActive) {
    await showStopLkdnForm(e.player, e.block);
    return;
  }
  if (zone.isLkdnScheduled) {
    await showCancelLkdnScheduleForm(e.player, e.block);
    return;
  }
  await showStartLkdnForm(e.player, e.block);
}
async function showStartLkdnForm(player, panelBlock) {
  const zone = getZone2(panelBlock);
  const formData = new ModalFormData11();
  formData.title({ translate: "scpdy.lkdnPanel" });
  formData.label({ translate: "scpdy.lkdnPanel.networkX", with: { rawtext: [zone.fnet.name] } });
  formData.label({ translate: "scpdy.lkdnPanel.targetZoneX", with: { rawtext: [zone.name] } });
  formData.divider();
  formData.textField(
    { translate: "scpdy.misc.text.delay" },
    { translate: "scpdy.misc.text.none" },
    {
      tooltip: { translate: "scpdy.misc.text.enterNumber" }
    }
  );
  formData.textField(
    { translate: "scpdy.misc.text.duration" },
    { translate: "scpdy.misc.text.infinity" },
    {
      tooltip: { translate: "scpdy.misc.text.enterNumber" }
    }
  );
  const response = await formData.show(player);
  if (!player.isValid) return;
  if (response.canceled) return;
  if (!response.formValues) return;
  const delay = Number(response.formValues[3]);
  const duration = Number(response.formValues[4]);
  const durationMaybeInf = duration <= 0 ? Infinity : duration;
  if (delay > 0) {
    zone.scheduleLkdn(delay, durationMaybeInf);
    player.sendMessage({ translate: "scpdy.lkdnPanel.scheduledLkdn" });
  } else {
    zone.startLkdn(durationMaybeInf);
    player.sendMessage({ translate: "scpdy.lkdnPanel.startedLkdn" });
  }
  updateVisual(panelBlock);
}
async function showCancelLkdnScheduleForm(player, panelBlock) {
  const zone = getZone2(panelBlock);
  const formData = new ActionFormData7();
  formData.title({ translate: "scpdy.lkdnPanel" });
  formData.body({ translate: "scpdy.lkdnPanel.lkdnIsScheduled" });
  formData.label({ translate: "scpdy.lkdnPanel.networkX", with: { rawtext: [zone.fnet.name] } });
  formData.label({ translate: "scpdy.lkdnPanel.targetZoneX", with: { rawtext: [zone.name] } });
  formData.divider();
  formData.label({ translate: "scpdy.lkdnPanel.wannaCancelSchedule" });
  formData.button({ translate: "scpdy.misc.text.yes" });
  formData.button({ translate: "scpdy.misc.text.no" });
  const response = await formData.show(player);
  if (!player.isValid) return;
  if (response.canceled) return;
  if (response.selection !== 0) return;
  if (zone.isLkdnActive) {
    player.sendMessage({ translate: "scpdy.misc.text.somethingWentWrong" });
    return;
  }
  zone.cancelScheduledLkdn();
  updateVisual(panelBlock);
  player.sendMessage({ translate: "scpdy.lkdnPanel.canceledSchedule" });
}
async function showStopLkdnForm(player, panelBlock) {
  const zone = getZone2(panelBlock);
  const formData = new ActionFormData7();
  formData.title({ translate: "scpdy.lkdnPanel" });
  formData.body({ translate: "scpdy.lkdnPanel.lkdnIsActive" });
  formData.label({ translate: "scpdy.lkdnPanel.networkX", with: { rawtext: [zone.fnet.name] } });
  formData.label({ translate: "scpdy.lkdnPanel.targetZoneX", with: { rawtext: [zone.name] } });
  formData.divider();
  formData.label({ translate: "scpdy.lkdnPanel.wannaStopLkdn" });
  formData.button({ translate: "scpdy.misc.text.yes" });
  formData.button({ translate: "scpdy.misc.text.no" });
  const response = await formData.show(player);
  if (!player.isValid) return;
  if (response.canceled) return;
  if (response.selection !== 0) return;
  if (!zone.isLkdnActive) {
    player.sendMessage({ translate: "scpdy.misc.text.somethingWentWrong" });
    return;
  }
  zone.stopLkdn();
  updateVisual(panelBlock);
  player.sendMessage({ translate: "scpdy.lkdnPanel.stoppedLkdn" });
}

// scripts/features/machinery/input/btn_component.ts
import * as mc98 from "@minecraft/server";
import { ModalFormData as ModalFormData12 } from "@minecraft/server-ui";
var STATE8 = {
  isPressed: "lc:is_pressed",
  mode: "lc:mode",
  rbLifespan: "lc:rb_lifespan"
};
mc98.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:btn", COMPONENT7);
});
var COMPONENT7 = {
  beforeOnPlayerPlace(arg) {
    arg.cancel = true;
    mc98.system.run(async () => {
      if (!arg.player) return;
      const formData = new ModalFormData12();
      formData.title({ translate: "scpdy.machinery.input.btn.text" });
      formData.dropdown(
        { translate: "scpdy.machinery.input.mode.text" },
        [
          { translate: "scpdy.machinery.input.mode.text.powerRelayDoors" },
          { translate: "scpdy.machinery.input.mode.text.ctrlBlastDoor" },
          { translate: "scpdy.machinery.input.mode.text.placeRbBelow" },
          { translate: "scpdy.machinery.input.mode.text.placeRbBehind" }
        ],
        {
          defaultValueIndex: 0,
          tooltip: { translate: "scpdy.machinery.input.mode.text.tooltip" }
        }
      );
      formData.slider({ translate: "scpdy.machinery.input.rbLifespan" }, 1, 5, {
        defaultValue: 5,
        valueStep: 1,
        tooltip: "scpdy.machinery.input.rbLifespan.tooltip"
      });
      const response = await formData.show(arg.player);
      if (response.canceled) return;
      if (!response.formValues) return;
      const modeIndex = Number(response.formValues[0]);
      const mode = _getModeStringFromIndex(modeIndex);
      const rbLifespan = Number(response.formValues[1]);
      const newPermutation = arg.permutationToPlace.withState(STATE8.mode, mode).withState(STATE8.rbLifespan, rbLifespan);
      if (!arg.block.isAir && !arg.block.isLiquid) return;
      const abort = !isCreativeOrSpectator2(arg.player) && consumeHandItem(arg.player, {
        filter: (itemStack) => itemStack.typeId === arg.permutationToPlace.getItemStack()?.typeId,
        max: 1
      }) <= 0;
      if (abort) return;
      arg.block.setPermutation(newPermutation);
      arg.dimension.playSound("dig.copper", arg.block.center(), { pitch: randf(0.8, 1) });
      if (mode !== "placeRbBehind" && mode !== "placeRbBelow") return;
      if (!arg.player.addTag("scpdy_sent_input_device_rb_tip")) return;
      mc98.system.runTimeout(() => {
        if (!arg.player || !arg.player.isValid) return;
        arg.player.sendMessage({ translate: "scpdy.machinery.input.rbMode.tip" });
        arg.player.playSound("random.orb");
      }, 8);
    });
  },
  onPlayerInteract({ block, dimension, player }) {
    if (!player) return;
    const isPressed = Boolean(block.permutation.getState(STATE8.isPressed));
    if (isPressed) return;
    block.setPermutation(block.permutation.withState(STATE8.isPressed, true));
    dimension.playSound("scpdy.interact.button.click", block.center());
    const mode = String(block.permutation.getState(STATE8.mode));
    if (!_isModeString(mode)) {
      throw new Error(`Unknown mode: ${mode}`);
    }
    const dir1 = getBlockCardinalDirection2(block.permutation);
    if (!dir1) {
      throw new Error("Failed to get block direction");
    }
    const dir2 = reverseDirection(dir1);
    MachineryInputEvents.emit("onActivate", {
      mode,
      dimension,
      location: block.center(),
      clearanceLevel: -1,
      source: player,
      block,
      pulseDirection: dir2,
      rbLifespan: Number(block.permutation.getState(STATE8.rbLifespan))
    });
  },
  onTick({ block, dimension }) {
    const isPressed = Boolean(block.permutation.getState(STATE8.isPressed));
    if (!isPressed) return;
    block.setPermutation(block.permutation.withState(STATE8.isPressed, false));
    dimension.playSound("scpdy.interact.button.click", block.center(), {
      pitch: 0.23,
      volume: 0.19
    });
  }
};

// scripts/features/machinery/input/kr_component.ts
import * as mc99 from "@minecraft/server";
import { ModalFormData as ModalFormData13 } from "@minecraft/server-ui";
var STATE9 = {
  isOn: "lc:is_on",
  clearanceLevel: "lc:clearance_level",
  mode: "lc:mode",
  rbLifespan: "lc:rb_lifespan"
};
mc99.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:kr", COMPONENT8);
});
var COMPONENT8 = {
  beforeOnPlayerPlace(arg) {
    arg.cancel = true;
    mc99.system.run(async () => {
      if (!arg.player) return;
      const formData = new ModalFormData13();
      formData.title({ translate: "scpdy.machinery.input.kr.text" });
      formData.dropdown(
        { translate: "scpdy.misc.text.clearanceLevel" },
        [
          { translate: "scpdy.misc.text.clearanceLevel.level_0" },
          { translate: "scpdy.misc.text.clearanceLevel.level_1" },
          { translate: "scpdy.misc.text.clearanceLevel.level_2" },
          { translate: "scpdy.misc.text.clearanceLevel.level_3" },
          { translate: "scpdy.misc.text.clearanceLevel.level_4" },
          { translate: "scpdy.misc.text.clearanceLevel.level_5" },
          { translate: "scpdy.misc.text.clearanceLevel.o5" }
        ],
        {
          defaultValueIndex: 0,
          tooltip: { translate: "scpdy.misc.text.clearanceLevel.tooltip" }
        }
      );
      formData.dropdown(
        { translate: "scpdy.machinery.input.mode.text" },
        [
          { translate: "scpdy.machinery.input.mode.text.powerRelayDoors" },
          { translate: "scpdy.machinery.input.mode.text.ctrlBlastDoor" },
          { translate: "scpdy.machinery.input.mode.text.placeRbBelow" },
          { translate: "scpdy.machinery.input.mode.text.placeRbBehind" }
        ],
        {
          defaultValueIndex: 0,
          tooltip: { translate: "scpdy.machinery.input.mode.text.tooltip" }
        }
      );
      formData.slider({ translate: "scpdy.machinery.input.rbLifespan" }, 1, 5, {
        defaultValue: 5,
        valueStep: 1,
        tooltip: "scpdy.machinery.input.rbLifespan.tooltip"
      });
      const response = await formData.show(arg.player);
      if (response.canceled) return;
      if (!response.formValues) return;
      const clearanceLevel = Number(response.formValues[0]);
      const modeIndex = Number(response.formValues[1]);
      const mode = _getModeStringFromIndex(modeIndex);
      const rbLifespan = Number(response.formValues[2]);
      const newPermutation = arg.permutationToPlace.withState(STATE9.clearanceLevel, clearanceLevel).withState(STATE9.mode, mode).withState(STATE9.rbLifespan, rbLifespan);
      if (!arg.block.isAir && !arg.block.isLiquid) return;
      const abort = !isCreativeOrSpectator2(arg.player) && consumeHandItem(arg.player, {
        filter: (itemStack) => itemStack.typeId === arg.permutationToPlace.getItemStack()?.typeId,
        max: 1
      }) <= 0;
      if (abort) return;
      arg.block.setPermutation(newPermutation);
      arg.dimension.playSound("dig.copper", arg.block.center(), { pitch: randf(0.8, 1) });
      if (mode !== "placeRbBehind" && mode !== "placeRbBelow") return;
      if (!arg.player.addTag("scpdy_sent_input_device_rb_tip")) return;
      mc99.system.runTimeout(() => {
        if (!arg.player || !arg.player.isValid) return;
        arg.player.sendMessage({ translate: "scpdy.machinery.input.rbMode.tip" });
        arg.player.playSound("random.orb");
      }, 8);
    });
  },
  onPlayerInteract({ block, dimension, player }) {
    if (!player) return;
    const isOn = Boolean(block.permutation.getState(STATE9.isOn));
    if (isOn) return;
    const minimumClearanceLevel = Number(block.permutation.getState(STATE9.clearanceLevel));
    const playerClearanceLevel = getEntityClearanceLevel(player);
    const isAccepted = playerClearanceLevel >= minimumClearanceLevel;
    if (!isAccepted) {
      dimension.playSound("scpdy.interact.keycard_reader.deny", block.center());
      player.onScreenDisplay.setActionBar({
        translate: "scpdy.misc.text.youDontHaveRequiredClearanceLevel",
        with: [String(minimumClearanceLevel)]
      });
      return;
    }
    const mode = String(block.permutation.getState(STATE9.mode));
    if (!_isModeString(mode)) {
      throw new Error(`Unknown mode: ${mode}`);
    }
    const dir1 = getBlockCardinalDirection2(block.permutation);
    if (!dir1) {
      throw new Error("Failed to get block direction");
    }
    const dir2 = reverseDirection(dir1);
    MachineryInputEvents.emit("onActivate", {
      mode,
      dimension,
      location: block.center(),
      clearanceLevel: playerClearanceLevel,
      source: player,
      block,
      pulseDirection: dir2,
      rbLifespan: Number(block.permutation.getState(STATE9.rbLifespan))
    });
    block.setPermutation(block.permutation.withState(STATE9.isOn, true));
    dimension.playSound("scpdy.interact.keycard_reader.accept", block.center());
  },
  onTick({ block }) {
    const isOn = Boolean(block.permutation.getState(STATE9.isOn));
    if (!isOn) return;
    block.setPermutation(block.permutation.withState(STATE9.isOn, false));
  }
};

// scripts/features/machinery/rb_anchor/rb_anchor.ts
import * as mc100 from "@minecraft/server";
var RB_ANCHOR_ENTITY_TYPE = "lc:scpdy_rb_anchor";
var RB_PLACEHOLDER_BLOCK_TYPE = "lc:scpdy_rb_placeholder";
function spawnOrReactivateRbAnchor(block, sexTilDeath = 5) {
  if (block.typeId !== RB_PLACEHOLDER_BLOCK_TYPE) return false;
  block.setType("minecraft:redstone_block");
  const alrExistingRbAnchor = block.dimension.getEntities({
    type: RB_ANCHOR_ENTITY_TYPE,
    maxDistance: 1,
    location: block.center(),
    closest: 1
  })[0];
  if (alrExistingRbAnchor) {
    setSexTilDeath(alrExistingRbAnchor, sexTilDeath);
    return true;
  }
  const newRbAnchor = block.dimension.spawnEntity(RB_ANCHOR_ENTITY_TYPE, block.center());
  setSexTilDeath(newRbAnchor, sexTilDeath);
  return true;
}
function getSexTilDeath(rbAnchor) {
  return Number(rbAnchor.getProperty("lc:sex_til_death"));
}
function setSexTilDeath(rbAnchor, value) {
  rbAnchor.setProperty("lc:sex_til_death", value);
}
mc100.world.afterEvents.dataDrivenEntityTrigger.subscribe(
  ({ entity: rbAnchor }) => {
    const sexTilDeath = getSexTilDeath(rbAnchor);
    if (sexTilDeath <= 0) {
      deactivateAt(rbAnchor.dimension, rbAnchor.location);
      rbAnchor.remove();
      return;
    }
    setSexTilDeath(rbAnchor, sexTilDeath - 1);
  },
  {
    eventTypes: ["lc:update_script"],
    entityTypes: [RB_ANCHOR_ENTITY_TYPE]
  }
);
function deactivateAt(dimension, location) {
  const block = dimension.getBlock(location);
  if (!block) return;
  if (block.typeId !== "minecraft:redstone_block") return;
  block.setType(RB_PLACEHOLDER_BLOCK_TYPE);
}

// scripts/features/machinery/rb_anchor/on_input_event.ts
MachineryInputEvents.on("onActivate", (data) => {
  if (data.mode !== "placeRbBehind" && data.mode !== "placeRbBelow") return;
  const block = data.block ?? data.dimension.getBlock(data.location);
  if (!block) return;
  if (!data.pulseDirection) return;
  if (data.mode === "placeRbBehind") {
    const block2 = getRelativeBlockAtDirection(block, data.pulseDirection, 2);
    if (!block2) return;
    spawnOrReactivateRbAnchor(block2, data.rbLifespan);
    return;
  }
  if (data.mode === "placeRbBelow") {
    const block2 = block.below(3);
    if (!block2) return;
    spawnOrReactivateRbAnchor(block2, data.rbLifespan);
  }
});

// scripts/features/machinery/alarm.ts
import * as mc101 from "@minecraft/server";
import { ModalFormData as ModalFormData14 } from "@minecraft/server-ui";
var STATE10 = {
  redstoneDir: "lc:redstone_dir",
  redstoneExtend: "lc:redstone_extend",
  isActive: "lc:is_active"
};
var redstoneDirs = ["backward", "forward", "up", "down"];
mc101.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:alarm", COMPONENT9);
});
var COMPONENT9 = {
  beforeOnPlayerPlace(e) {
    e.cancel = true;
    mc101.system.run(() => {
      beforeOnPlayerPlaceAsync2(e);
    });
  },
  onTick({ block, dimension }) {
    const detectedRedstone = checkForRedstone(block);
    const shouldBeActive = detectedRedstone;
    const isCurrentlyActive = Boolean(block.permutation.getState(STATE10.isActive));
    if (shouldBeActive && !isCurrentlyActive) {
      block.setPermutation(block.permutation.withState(STATE10.isActive, true));
    } else if (!shouldBeActive && isCurrentlyActive) {
      block.setPermutation(block.permutation.withState(STATE10.isActive, false));
    }
    if (shouldBeActive) {
      dimension.playSound("scpdy.misc.alarm", block.center(), {
        volume: 1.1
      });
    }
  }
};
function checkForRedstone(alarm) {
  const redstoneDetectionBlock = getBlockForRedstoneDetection(alarm);
  if (!redstoneDetectionBlock) return false;
  const power = redstoneDetectionBlock.getRedstonePower();
  return power != void 0 && power > 0;
}
function getBlockForRedstoneDetection(alarm) {
  const blockFace = getBlockFace(alarm.permutation);
  if (!blockFace) return;
  const redstoneDir = alarm.permutation.getState(STATE10.redstoneDir);
  const redstoneExtend = Number(alarm.permutation.getState(STATE10.redstoneExtend));
  switch (redstoneDir) {
    case "up":
      return alarm.above(redstoneExtend);
    case "down":
      return alarm.below(redstoneExtend);
    case "backward":
      return getRelativeBlockAtDirection(alarm, reverseDirection(blockFace), redstoneExtend);
    case "forward":
      return getRelativeBlockAtDirection(alarm, blockFace, redstoneExtend);
  }
}
async function beforeOnPlayerPlaceAsync2(e) {
  if (!e.player) return;
  const formData = new ModalFormData14();
  formData.title({ translate: "scpdy.machinery.alarm" });
  formData.dropdown(
    { translate: "scpdy.machinery.alarm.redstoneDetectionDir" },
    [
      { translate: "scpdy.misc.text.backward" },
      { translate: "scpdy.misc.text.forward" },
      { translate: "scpdy.misc.text.up" },
      { translate: "scpdy.misc.text.down" }
    ],
    {
      defaultValueIndex: 0,
      tooltip: { translate: "scpdy.machinery.alarm.redstoneDetectionDir.tip" }
    }
  );
  formData.slider({ translate: "scpdy.machinery.alarm.redstoneDetectionExtend" }, 1, 5, {
    defaultValue: 2,
    valueStep: 1,
    tooltip: { translate: "scpdy.machinery.alarm.redstoneDetectionExtent.tip" }
  });
  const response = await formData.show(e.player);
  if (!e.player.isValid) return;
  if (response.canceled) return;
  if (!response.formValues) return;
  const newPermutation = e.permutationToPlace.withState(STATE10.redstoneDir, redstoneDirs[Number(response.formValues[0])]).withState(STATE10.redstoneExtend, Number(response.formValues[1]));
  if (!e.block.isAir && !e.block.isLiquid) return;
  const abort = !isCreativeOrSpectator2(e.player) && consumeHandItem(e.player, {
    filter: (itemStack) => itemStack.typeId === e.permutationToPlace.getItemStack()?.typeId,
    max: 1
  }) <= 0;
  if (abort) return;
  e.block.setPermutation(newPermutation);
  e.dimension.playSound("dig.copper", e.block.center(), { pitch: randf(0.8, 1) });
}

// scripts/features/uncategorizable/block_components/change_block_type_on_tick.ts
import * as mc102 from "@minecraft/server";
mc102.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:change_block_type_on_tick", COMPONENT10);
});
var COMPONENT10 = {
  onTick({ block }, arg1) {
    const params = arg1.params;
    const newType = String(params.newType).trim();
    if (newType === "") return;
    const oldPermutation = block.permutation;
    block.setType(newType);
    console.log(`Converted ${oldPermutation.type.id} at ${toString3(block)} to ${newType}`);
  }
};

// scripts/features/uncategorizable/block_components/variant_wrench_cyclable.ts
import * as mc103 from "@minecraft/server";
mc103.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:variant_wrench_cyclable", COMPONENT11);
});
var COMPONENT11 = {
  onPlayerInteract(arg0, arg1) {
    onInteract4(arg0, arg1.params);
  }
};
function onInteract4(e, params) {
  if (!e.player) return;
  if (!isHoldingWrench2(e.player)) return;
  let variantValues = params.variantValues;
  if (!Array.isArray(variantValues))
    throw new Error("variantValues parameter is required for scpdy:wrenchable_variant component");
  let variantStateName = String(params.variantStateName).trim();
  if (variantStateName === "")
    throw new Error("variantStateName parameter is required for scpdy:wrenchable_variant component");
  const currentVariant = e.block.permutation.getState(variantStateName);
  const currentVariantIndex = variantValues.indexOf(currentVariant);
  const nextVariantIndex = (currentVariantIndex + 1) % variantValues.length;
  const nextVariant = variantValues[nextVariantIndex];
  e.block.setPermutation(e.block.permutation.withState(variantStateName, nextVariant));
}

// scripts/old_feature_upgrades/v1/blocks/upgrade_v1_boolean_door.ts
import * as mc104 from "@minecraft/server";
mc104.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:upgrade_v1_boolean_door", {
    onTick: onTick5,
    onRandomTick: onTick5
  });
});
function onTick5({ block, dimension }, arg1) {
  const params = arg1.params;
  const isLowerPart = Boolean(block.permutation.getState("lc:is_lower_part"));
  if (!isLowerPart) return;
  const newType = String(params.newType).trim();
  if (newType === "") return;
  try {
    const above = block.above();
    if (above.typeId === block.typeId) {
      above.setType("minecraft:air");
    }
  } catch {
  }
  const cardinalDir = getBlockCardinalDirection2(block.permutation) ?? mc104.Direction.North;
  const isClearanceSet = Boolean(block.permutation.getState("lc:is_clearance_set"));
  const minClearanceLevel = Number(block.permutation.getState("lc:minimum_clearance_level"));
  const statesString = createBlockStatesString({
    "minecraft:cardinal_direction": cardinalDir.toLowerCase(),
    "lc:clearance_level": isClearanceSet ? minClearanceLevel : -1
  });
  const oldPermutation = block.permutation;
  const cmd = `setblock ${toString22(block)} ${newType} [${statesString}]`;
  dimension.runCommand(cmd);
  console.log(`Converted ${oldPermutation.type.id} at ${toString3(block)} to ${newType}`);
}

// scripts/old_feature_upgrades/v1/blocks/upgrade_v1_button.ts
import * as mc105 from "@minecraft/server";
mc105.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:upgrade_v1_button", COMPONENT12);
});
var COMPONENT12 = {
  onTick({ block, dimension }, arg1) {
    const params = arg1.params;
    const newType = String(params.newType).trim();
    if (newType === "") return;
    const modeIndex = Number(block.permutation.getState("lc:mode"));
    const modeString = _getModeStringFromIndex(modeIndex);
    const cardinalDir = getBlockCardinalDirection2(block.permutation) ?? mc105.Direction.North;
    const reversedDir = reverseDirection(cardinalDir);
    const oldPermutation = block.permutation;
    const statesString = createBlockStatesString({
      "minecraft:cardinal_direction": reversedDir.toLowerCase(),
      "lc:mode": modeString
    });
    const cmd = `setblock ${toString22(block)} ${newType} [${statesString}]`;
    dimension.runCommand(cmd);
    console.log(`Converted ${oldPermutation.type.id} at ${toString3(block)} to ${newType}`);
  }
};

// scripts/old_feature_upgrades/v1/blocks/upgrade_v1_command_door.ts
import * as mc106 from "@minecraft/server";
mc106.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:upgrade_v1_command_door", COMPONENT13);
});
var COMPONENT13 = {
  onTick: onTick6,
  onRandomTick: onTick6
};
function onTick6({ block, dimension }, arg1) {
  const params = arg1.params;
  const isLowerPart = Boolean(block.permutation.getState("lc:is_lower_part"));
  if (!isLowerPart) return;
  const newType = String(params.newType).trim();
  if (newType === "") return;
  try {
    const above = block.above();
    if (above.typeId === block.typeId) {
      above.setType("minecraft:air");
    }
  } catch {
  }
  const cardinalDir = getBlockCardinalDirection2(block.permutation) ?? mc106.Direction.North;
  const statesString = createBlockStatesString({
    "minecraft:cardinal_direction": cardinalDir.toLowerCase()
  });
  const oldPermutation = block.permutation;
  const cmd = `setblock ${toString22(block)} ${newType} [${statesString}]`;
  dimension.runCommand(cmd);
  console.log(`Converted ${oldPermutation.type.id} at ${toString3(block)} to ${newType}`);
}

// scripts/old_feature_upgrades/v1/blocks/upgrade_v1_keycard_reader.ts
import * as mc107 from "@minecraft/server";
mc107.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:upgrade_v1_keycard_reader", COMPONENT14);
});
var COMPONENT14 = {
  onTick({ block, dimension }, arg1) {
    const params = arg1.params;
    const newType = String(params.newType).trim();
    const clearanceLevel = Number(params.level);
    if (newType === "") return;
    if (isNaN(clearanceLevel)) return;
    const modeIndex = Number(block.permutation.getState("lc:mode"));
    const modeString = _getModeStringFromIndex(modeIndex);
    const cardinalDir = getBlockCardinalDirection2(block.permutation) ?? mc107.Direction.North;
    const reversedDir = reverseDirection(cardinalDir);
    const oldPermutation = block.permutation;
    const statesString = createBlockStatesString({
      "minecraft:cardinal_direction": reversedDir.toLowerCase(),
      "lc:mode": modeString,
      "lc:clearance_level": clamp3(Math.floor(clearanceLevel), 0, 6)
    });
    const cmd = `setblock ${toString22(block)} ${newType} [${statesString}]`;
    dimension.runCommand(cmd);
    console.log(`Converted ${oldPermutation.type.id} at ${toString3(block)} to ${newType}`);
  }
};

// scripts/old_feature_upgrades/v1/blocks/upgrade_v1_lockdown_alarm.ts
import * as mc108 from "@minecraft/server";
mc108.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:upgrade_v1_lockdown_alarm", {
    onPlayerInteract: (arg0, arg1) => upgrade(arg0.block, arg1.params),
    onTick: (arg0, arg1) => upgrade(arg0.block, arg1.params)
  });
});
function upgrade(block, params) {
  const newType = String(params.newType).trim();
  if (newType === "") return;
  const fnetIndex = Number(params.networkIndex);
  const fzoneIndex = Number(block.permutation.getState("lc:facility_zone_index"));
  const blockFace = getBlockFace(block.permutation) ?? mc108.Direction.North;
  const statesString = createBlockStatesString({
    "minecraft:block_face": blockFace.toLowerCase(),
    "lc:fnet_index": fnetIndex,
    "lc:fzone_index": fzoneIndex
  });
  const oldPermutation = block.permutation;
  const cmd = `setblock ${toString22(block)} ${newType} [${statesString}]`;
  block.dimension.runCommand(cmd);
  console.log(`Converted ${oldPermutation.type.id} at ${toString3(block)} to ${newType}`);
}

// scripts/old_feature_upgrades/v1/blocks/upgrade_v1_lockdown_door.ts
import * as mc109 from "@minecraft/server";
mc109.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:upgrade_v1_lockdown_door", {
    onPlayerInteract: (arg0, arg1) => upgrade2(arg0.block, arg1.params),
    onTick: (arg0, arg1) => upgrade2(arg0.block, arg1.params),
    onRandomTick: (arg0, arg1) => upgrade2(arg0.block, arg1.params)
  });
});
function upgrade2(block, params) {
  const isLowerPart = Boolean(block.permutation.getState("lc:is_lower_part"));
  if (!isLowerPart) return;
  const newType = String(params.newType).trim();
  if (newType === "") return;
  try {
    const above = block.above();
    if (above.typeId === block.typeId) {
      above.setType("minecraft:air");
    }
  } catch {
  }
  const fnetIndex = Number(params.networkIndex);
  const fzoneIndex = Number(block.permutation.getState("lc:facility_zone_index"));
  const fzoneIndexUnflat = unflattenToCoordinates(fzoneIndex);
  const cardinalDir = getBlockCardinalDirection2(block.permutation) ?? mc109.Direction.North;
  const statesString = createBlockStatesString({
    "minecraft:cardinal_direction": cardinalDir.toLowerCase(),
    "lc:fnet_index": fnetIndex,
    "lc:fzone_index_major": fzoneIndexUnflat.major,
    "lc:fzone_index_minor": fzoneIndexUnflat.minor
  });
  const oldPermutation = block.permutation;
  const cmd = `setblock ${toString22(block)} ${newType} [${statesString}]`;
  block.dimension.runCommand(cmd);
  console.log(`Converted ${oldPermutation.type.id} at ${toString3(block)} to ${newType}`);
}

// scripts/old_feature_upgrades/v1/blocks/upgrade_v1_lockdown_panel.ts
import * as mc110 from "@minecraft/server";
mc110.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:upgrade_v1_lockdown_panel", {
    onPlayerInteract: (arg0, arg1) => upgrade3(arg0.block, arg1.params),
    onTick: (arg0, arg1) => upgrade3(arg0.block, arg1.params)
  });
});
function upgrade3(block, params) {
  const newType = String(params.newType).trim();
  if (newType === "") return;
  const fnetIndex = Number(params.networkIndex);
  const fzoneIndex = Number(block.permutation.getState("lc:facility_zone_index"));
  const cardinalDir = getBlockCardinalDirection2(block.permutation) ?? mc110.Direction.North;
  const statesString = createBlockStatesString({
    "minecraft:cardinal_direction": cardinalDir.toLowerCase(),
    "lc:fnet_index": fnetIndex,
    "lc:fzone_index": fzoneIndex
  });
  const oldPermutation = block.permutation;
  const cmd = `setblock ${toString22(block)} ${newType} [${statesString}]`;
  block.dimension.runCommand(cmd);
  console.log(`Converted ${oldPermutation.type.id} at ${toString3(block)} to ${newType}`);
}

// scripts/old_feature_upgrades/v1/blocks/upgrade_v1_mech_door.ts
import * as mc111 from "@minecraft/server";
mc111.system.beforeEvents.startup.subscribe((event) => {
  event.blockComponentRegistry.registerCustomComponent("scpdy:upgrade_mechanical_door_to_relay_door", COMPONENT15);
});
var COMPONENT15 = {
  onTick({ block, dimension }, arg1) {
    const params = arg1.params;
    const isLowerPart = Boolean(block.permutation.getState("lc:is_lower_part"));
    if (!isLowerPart) return;
    const relayDoorType = String(params.relayDoorType);
    if (relayDoorType === "") return;
    const cardinalDirection = getBlockCardinalDirection2(block.permutation) ?? mc111.Direction.North;
    const statesString = createBlockStatesString({
      "minecraft:cardinal_direction": cardinalDirection.toLowerCase()
    });
    const cmd = `setblock ${toString22(block)} ${relayDoorType} [${statesString}]`;
    try {
      const above = block.above();
      if (above.typeId === block.typeId) {
        above.setType("minecraft:air");
      }
    } catch {
    }
    const oldPermutation = block.permutation;
    dimension.runCommand(cmd);
    console.log(`Converted ${oldPermutation.type.id} at ${toString3(block)} to ${relayDoorType}`);
  }
};
